
/*
 * Codes at here are heavily taken from upload.cgi.c which is for large file uploading , but
 * in fact "upload_settings" only need few memory(~16k) so it is not necessary to follow
 * upload.cgi.c at all.
 *
 * YYHuang@Ralink TODO: code size.
 *
 */

#include <unistd.h>	//for unlink
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <linux/reboot.h>
#include <bcmnvram.h>

#include "upload_msg.h"

#define cprintf(fmt, args...) do { \
	FILE *fp = fopen("/dev/console", "w"); \
	if (fp) { \
		fprintf(fp, fmt , ## args); \
		fclose(fp); \
	} \
} while (0)

#define US_DEBUG 1 

#ifdef US_DEBUG
#define uprintf(fmt, args...) cprintf("(%s:%d)=> " fmt, __FUNCTION__, __LINE__, ## args)
#else
#define uprintf(fmt, args...)
#endif /* DEBUG */


#define RFC_ERROR "RFC1867 ...."
#define REFRESH_TIMEOUT		"80000"		/* Modiy by zhengshan at 20110301 60000 = 60 secs*/


void *memmem(const void *buf, size_t buf_len, const void *byte_line, size_t byte_line_len)
{
    unsigned char *bl = (unsigned char *)byte_line;
    unsigned char *bf = (unsigned char *)buf;
    unsigned char *p  = bf;

    while (byte_line_len <= (buf_len - (p - bf))){
        unsigned int b = *bl & 0xff;
        if ((p = (unsigned char *) memchr(p, b, buf_len - (p - bf))) != NULL){
            if ( (memcmp(p, byte_line, byte_line_len)) == 0)
                return p;
            else
                p++;
        }else{
            break;
        }
    }
    return NULL;
}

#define MEM_SIZE	1024
#define MEM_HALT	512
int findStrInFile(char *filename, int offset, unsigned char *str, int str_len)
{
	int pos = 0, rc;
	FILE *fp;
	unsigned char mem[MEM_SIZE];

	if(str_len > MEM_HALT)
		return -1;
	if(offset <0)
		return -1;

	fp = fopen(filename, "rb");
	if(!fp)
		return -1;

	rewind(fp);
	fseek(fp, offset + pos, SEEK_SET);
	rc = fread(mem, 1, MEM_SIZE, fp);
	while(rc){
		unsigned char *mem_offset;
		mem_offset = (unsigned char*)memmem(mem, rc, str, str_len);
		if(mem_offset){
			fclose(fp);	//found it
			return (mem_offset - mem) + pos + offset;
		}

		if(rc == MEM_SIZE){
			pos += MEM_HALT;	// 8
		}else
			break;

		rewind(fp);
		fseek(fp, offset+pos, SEEK_SET);
		rc = fread(mem, 1, MEM_SIZE, fp);
	}

	fclose(fp);
	return -1;
}

/*
 *  ps. callee must free memory...
 */
void *getMemInFile(char *filename, int offset, int len)
{
    void *result;
    FILE *fp;
    if( (fp = fopen(filename, "r")) == NULL ){
        return NULL;
    }
	fseek(fp, offset, SEEK_SET);
    result = malloc(sizeof(unsigned char) * len );
	if(!result)
		return NULL;
    if( fread(result, 1, len, fp) != len){
        free(result);
        return NULL;
    }
    return result;
}

static int isFileChecksumInvalid(char *filename, const char *checksum)
{
	FILE *fp = NULL;
	char fileChecksum[32] = {0};
	char cmd[128] = {0};
	int ret = 0;

	snprintf(cmd, sizeof(cmd), "cksum %s", filename);
	fp = popen(cmd, "r");
	if (fp){
		if ( fgets(fileChecksum, sizeof(fileChecksum)-1, fp) )
			strtok(fileChecksum, " ");
		pclose(fp);
	}
	ret = strncmp(fileChecksum, checksum, sizeof(fileChecksum));
	return ret;
}

#define PROFILE_HEADER		"WDK1"
static int import(char *filename, int offset, int len, const char *checksum)
{
    char cmd[4096];
	int ret = 0;
	char *data;
	FILE *fp, *src;
	char *pname = tempnam("/tmp", "set");
	char header[8];
	unsigned long count;

	snprintf(cmd, 4096, "cp %s /tmp/tmpcgi", filename);
      system(cmd);

    if (!(fp = fopen(pname, "w+"))) {
		return -1;
    }

	if (!(src = fopen(filename, "r"))) {
		fclose(fp);
		return -1;
	}

    if( fseek(src, offset, SEEK_SET) == -1){
        printf("fseek error\n");
    }
	
	while(len > 0) {
		if (!fread(&data, 1, 1, src))
			break;
		fwrite(&data, 1, 1, fp);
		len--;
	}

    fclose(fp);
    fclose(src);

	if ( isFileChecksumInvalid(pname, checksum) ) {
        printf("Checksum invalid.");
        ret = -1;
    } else {
        snprintf(cmd, 4096, "nvram restore %s", pname);
	system(cmd);
	ret = nvram_commit();
    }
	
	unlink(pname);
	
    return ret;
}

int main (int argc, char *argv[])
{
    int file_begin, file_end;
    int line_begin, line_end;
    char *boundary; int boundary_len;
    char *filename = getenv("UPLOAD_FILENAME");
    char checksum[32] = {0};

    uprintf( "enter.\n" );
    if(!filename){
        printf("failed, can't get env var.\n");
        goto err;
    }
    line_begin = 0;
    if((line_end = findStrInFile(filename, line_begin, "\r\n", 2)) == -1){
        printf("%s", RFC_ERROR);
        goto err;
    }
	boundary_len = line_end - line_begin;
    boundary = getMemInFile(filename, line_begin, boundary_len);

    // sth like this..
    // Content-Disposition: form-data; name="filename"; filename="\\192.168.3.171\tftpboot\a.out"
    //
    char *line, *semicolon, *user_filename;
    line_begin = line_end + 2;
    if((line_end = findStrInFile(filename, line_begin, "\r\n", 2)) == -1){
        printf("%s", RFC_ERROR);
        goto err;
    }
    line = getMemInFile(filename, line_begin, line_end - line_begin);
    if(strncasecmp(line, "content-disposition: form-data;", strlen("content-disposition: form-data;"))){
        printf("%s", RFC_ERROR);
        goto err;
    }
    semicolon = line + strlen("content-disposition: form-data;") + 1;
    if(! (semicolon = strchr(semicolon, ';'))  ){
        printf("dont support multi-field upload.\n");
        goto err;
    }
    user_filename = semicolon + 2;
    if( strncasecmp(user_filename, "filename=", strlen("filename="))  ){
        printf("%s", RFC_ERROR);
        goto err;
    }

    user_filename += strlen("filename=");
    //until now we dont care about what  true filename is.
    free(line);

    // We may check a string  "Content-Type: application/octet-stream" here
    // but if our firmware extension name is the same with other known name,
    // the browser will define other content-type itself instead,
    line_begin = line_end + 2;
    if((line_end = findStrInFile(filename, line_begin, "\r\n", 2)) == -1){
        printf("%s", RFC_ERROR);
        goto err;
    }

    line_begin = line_end + 2;
    if((line_end = findStrInFile(filename, line_begin, "\r\n", 2)) == -1){
        printf("%s", RFC_ERROR);
        goto err;
    }

    file_begin = line_end + 2;

    if( (file_end = findStrInFile(filename, file_begin, boundary, boundary_len)) == -1){
        printf("%s", RFC_ERROR);
        goto err;
    }
    file_end -= 2;		// back 2 chars.(\r\n);

/* the line is write by ExportSettings.sh with command 'cksum', format like:
#ID 2788018277 3561 conf
and the first long number after 'ID' is the checksum. */

    /* find checksum begin */
    if( (line_begin = findStrInFile(filename, file_begin, "#ID", 3)) == -1){
        printf("%s", RFC_ERROR);
        goto err;
    }

    /* checksum end */
    if( (line_end = findStrInFile(filename, line_begin, "\x0a", 1)) == -1){
        printf("%s", RFC_ERROR);
        goto err;
    }

    line = getMemInFile(filename, line_begin, line_end - line_begin);
    if (line){
    	/* config file checksum will save to buf */
    	sscanf(line, "#ID %s %*d Default", checksum);
    	free(line);
	line = NULL;
    }else{
	printf("%s", RFC_ERROR);
	goto err;
    }
    file_begin = line_end + 1;
    if (-1 == import(filename, file_begin, file_end - file_begin, checksum) )
    	goto err;

	webwrite(1, NULL);
    /*start of modify when upgrade successful can not jump normal by zengxianzhu at 20110120*/
    javascriptUpdate(1);
    /*start of modify when upgrade successful can not jump normal by zengxianzhu at 20110120*/
    webFoot();
	free(boundary);
	system("sleep 3 && reboot &");
    return 0;
err:

    free(boundary);
	webwrite(0, NULL);
	javascriptUpdate(1);
    webFoot();
	system("sleep 3 && reboot &");
    exit(0);
}

