/*
 *	firewall.c -- Firewall Settings
 *
 *	Copyright (c) Ralink Technology Corporation All Rights Reserved.
 *
 *	$Id: firewall.c,v 1.29.2.1 2009-03-24 08:56:16 yy Exp $
 */

/*
 *	if  WAN or LAN ip changed, we must restart firewall.
 */
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <ctype.h>
#include <dirent.h>
#include <notify.h>
#include <oid.h>
#include <net_conn.h>
#include "bcmnvram.h"
#include "utils.h"
#include "webs.h"
#include "firewall.h"
#include "internet.h"
#include "linux/autoconf.h"
#include "config/autoconf.h"

#define PORT_TRIGGER_MAX_SUBRULES	8
#define PORT_TRIGGER_MAX_APPRULES	10

#define DD printf("---> %d\n", __LINE__);

									// (The actual string is about 7200 bytes.)

int getGoAHeadServerPort(void);
static int getMac_arpCache(char *ipAddr,char *macAddr,int *macFlag);

int isMacValid(char *str)
{
	int i, len = strlen(str);
	if(len != 17)
		return 0;

	for(i=0; i<5; i++){
		if( (!isxdigit( str[i*3])) || (!isxdigit( str[i*3+1])) || (str[i*3+2] != ':') )
			return 0;
	}
	return (isxdigit(str[15]) && isxdigit(str[16])) ? 1: 0;
}

static int isIpValid(char *str)
{
	struct in_addr addr;	// for examination
	if( (! strcmp(T("any"), str)) || (! strcmp(T("any/0"), str)))
		return 1;

	if(! (inet_aton(str, &addr))){
		printf("isIpValid(): %s is not a valid IP address.\n", str);
		return 0;
	}
	return 1;
}

static int isNumOnly(char *str){
	int i, len = strlen(str);
	for(i=0; i<len; i++){
		if((str[i] >= '0' && str[i] <= '9'))
			continue;
		return 0;
	}
	return 1;
}

static int isAllNumAndSlash(char *str){
	int i, len = strlen(str);
	for(i=0; i<len; i++){
		if( (str[i] >= '0' && str[i] <= '9') || str[i] == '.' || str[i] == '/' )
			continue;
		return 0;
	}
	return 1;
}

static int isOnlyOneSlash(char *str)
{
	int i, count=0;
	int len = strlen(str);
	for(i=0; i<len; i++)
		if( str[i] == '/')
			count++;
	return count <= 1 ? 1 : 0;
}

int isIpNetmaskValid(char *s)
{
	char str[32];
	char *slash;
	struct in_addr addr;    // for examination

	if(!s || !strlen(s)){
		return 0;
	}

	strncpy(str, s, sizeof(str));

    if( (!strcmp("any", str)) || (!strcmp("any/0", str)))
        return 1;

	if (!isAllNumAndSlash(str)){
		return 0;
	}

	if(!isOnlyOneSlash(str)){
		return 0;
	}

	slash = strchr(str, '/');
	if(slash){
		int mask;

		*slash = '\0';
		slash++;
		if(!strlen(slash)){
			return 0;
		}

		if(!isNumOnly(slash)){
			return 0;
		}

		mask = atoi(slash);
		if(mask < 0 || mask > 32){
			return 0;
		}
	}

	if(! (inet_aton(str, &addr))){
        printf("isIpNetmaskValid(): %s is not a valid IP address.\n", str);
        return 0;
    }
    return 1;
}

static int getDMZEnableASP(int eid, webs_t wp, int argc, char_t **argv)
{
	int type, value;
	char *dmze = (char *)nvram_safe_get( "dmz_enable");
	if(dmze)
		value = atoi(dmze);
	else
		value = 0;

	if( ejArgs(argc, argv, T("%d"), &type) == 1){
		if(type == value)
			websWrite(wp, T("selected"));
		else
			websWrite(wp, T(" "));
		return 0;
	}
	return -1;
}

static int  getPortForwardEnableASP(int eid, webs_t wp, int argc, char_t **argv)
{
	int type, value;
	char *pfe = (char *)nvram_safe_get( "port_forward_en");

	if(pfe)
		value = atoi(pfe);
	else
		value = 0;

	if( ejArgs(argc, argv, T("%d"), &type) == 1){
		if(type == value)
			websWrite(wp, T("selected"));
		else
			websWrite(wp, T(" "));
		return 0;
	}
	return -1;
}


static int  getIPPortFilterEnableASP(int eid, webs_t wp, int argc, char_t **argv)
{
	int type, value;
	char *pfe = (char *)nvram_safe_get( "ip_filter_enable");

	if(pfe)
		value = atoi(pfe);
	else
		value = 0;

	if( ejArgs(argc, argv, T("%d"), &type) == 1){
		if(type == value)
			websWrite(wp, T("selected"));
		else
			websWrite(wp, T(" "));
		return 0;
	}
	return -1;
}

/* add by radar@20081229. For Anti-DoS Option select string */
#define	NVRAM_ANTI_DOS_EN		"AntiDoSEnable"

#define	NVRAM_ANTI_PORTSCAN_EN	"AntiPortScanEnable"
#define	NVRAM_APS_SENTRY_TCP_PORTS	"AntiPortScanSentryTcpPort"
#define	NVRAM_APS_SENTRY_UDP_PORTS	"AntiPortScanSentryUdpPort"
#define	NVRAM_APS_IGNORE_TCP_PORTS	"AntiPortScanIgnoreTcpPort"
#define	NVRAM_APS_IGNORE_UDP_PORTS	"AntiPortScanIgnoreUdpPort"
#define NVRAM_OPMode "OperationMode"
static int  getAntiDoSEnableASP(int eid, webs_t wp, int argc, char_t **argv)
{
	int type, value;
    char *pfe = (char *)nvram_safe_get( NVRAM_ANTI_DOS_EN);

	if(pfe)
		value = atoi(pfe);
	else
		value = 0;

	if( ejArgs(argc, argv, T("%d"), &type) == 1){
		if(type == value)
			websWrite(wp, T("selected"));
		else
			websWrite(wp, T(" "));
		return 0;
	}
	return -1;
}
/* for Anti-Port scan Option select string */
static int  getAntiPortScanEnableASP(int eid, webs_t wp, int argc, char_t **argv)
{
	int type, value;
	char *pfe = (char *)nvram_safe_get( NVRAM_ANTI_PORTSCAN_EN);

	if(pfe)
		value = atoi(pfe);
	else
		value = 0;

	if( ejArgs(argc, argv, T("%d"), &type) == 1){
		if(type == value)
			websWrite(wp, T("selected"));
		else
			websWrite(wp, T(" "));
		return 0;
	}
	return -1;
}

static int getNums(const char *value, char delimit)
{
	char *pos = (char *)value;
    int count=1;
    if(!pos)
    	return 0;
	while( (pos = strchr(pos, delimit))){
		pos = pos+1;
		count++;
	}
	return count;
}


inline int getRuleNums(const char *rules){
	return getNums(rules, ';');
}

static int getDefaultFirewallPolicyASP(int eid, webs_t wp, int argc, char_t **argv)
{
	int value;
	char *p = (char *)nvram_safe_get( "ip_filter_default_policy");
	int default_policy;
	if(!p)
		default_policy = 0;
	else
		default_policy = atoi(p);

	if( ejArgs(argc, argv, T("%d"), &value) != 1){
		return -1;
	}

	if(default_policy == value )
		websWrite(wp, T(" selected "));
	return 0;
}

static int checkIfUnderBridgeModeASP(int eid, webs_t wp, int argc, char_t **argv)
{
	char *mode = (char *)nvram_safe_get( "sw_mode_ex");
	if(atoi(mode)==2 || atoi(mode)==3 || atoi(mode)==4 ||atoi(mode)==5 ||atoi(mode)==6)
		return -1;	// fatal error, make ASP engine complained.
	if(atoi(mode)==2 || atoi(mode)==3 || atoi(mode)==4 ||atoi(mode)==5 ||atoi(mode)==6)	// bridge mode
		websWrite(wp, T(HTML_NO_FIREWALL_UNDER_BRIDGE_MODE));
	return 0;
}

/*
 * ASP function
 */
static int getPortTriggerRuleNumsASP(int eid, webs_t wp, int argc, char_t **argv)
{
	int irules = 0;
	const char *en_rules = nvram_safe_get( "port_trigger_rules");

	if (en_rules && strlen(en_rules))
		irules += getRuleNums((char *)en_rules);
	websWrite(wp, T("%d"),  irules);
	return 0;
}

/*
 * ASP function
 */
static int getPortForwardRuleNumsASP(int eid, webs_t wp, int argc, char_t **argv)
{
    char *rules = (char *)nvram_safe_get( "port_forward_rules");
	if(!rules || !strlen(rules) ){
		websWrite(wp, T("0"));
		return 0;
	}
	websWrite(wp, T("%d"), getRuleNums(rules));
	return 0;
}


/*
 * ASP function
 */
static int getIPPortRuleNumsASP(int eid, webs_t wp, int argc, char_t **argv)
{
    const char *rules = nvram_safe_get( "ip_filter_rules");
	if(!rules || !strlen(rules) ){
		websWrite(wp, T("0"));
		return 0;
	}

	websWrite(wp, T("%d"), getRuleNums((char *)rules));
	return 0;
}

/* web print single line no include appname */
static int webWriteTriggerSingleRule(webs_t wp, char *ruleline, int j)
{
	char tportS[6], tportE[6], oportS[6], oportE[6];
	char tprotocol[2], oprotocol[2];
	int nport, nprotocol;

/* ҳпͷɱṩ*/
//	websWrite(wp, T("</tr>\n"));

	/* trigger start port */
	if ((getNthValueSafe(0, ruleline, ',', tportS, sizeof(tportS)) == -1))
		return -1;
	nport = atoi(tportS);
	if ( (nport == 0) || (nport > 65535) )
		return -1;

	/* trigger end port */
	if ((getNthValueSafe(1, ruleline, ',', tportE, sizeof(tportE)) == -1))
		return -1;
	nport = atoi(tportE);
	if ( (nport == 0) || (nport > 65535) )
		return -1;

	/* trigger protocol */
	if ((getNthValueSafe(2, ruleline, ',', tprotocol, sizeof(tprotocol)) == -1))
		return -1;
	nprotocol = atoi(tprotocol);
	switch(nprotocol){
		case PROTO_TCP:
		case PROTO_UDP:
		case PROTO_TCP_UDP:
			break;
		default:
			return -1;
	}

	/* open start port */
	if ((getNthValueSafe(3, ruleline, ',', oportS, sizeof(oportS)) == -1))
		return -1;
	nport = atoi(oportS);
	if ( (nport == 0) || (nport > 65535) )
		return -1;

	/* open end port */
	if ((getNthValueSafe(4, ruleline, ',', oportE, sizeof(oportE)) == -1))
		return -1;
	nport = atoi(oportE);
	if ( (nport == 0) || (nport > 65535) )
		return -1;

	/* open protocol */
	if ((getNthValueSafe(5, ruleline, ',', oprotocol, sizeof(oprotocol)) == -1))
		return -1;
	nprotocol = atoi(oprotocol);
	switch(nprotocol){
		case PROTO_TCP:
		case PROTO_UDP:
		case PROTO_TCP_UDP:
			break;
		default:
			return -1;
	}

	/* web printf trigger */
	switch( atoi(tprotocol) ) {
		case PROTO_TCP:
			websWrite(wp, T("<td align=center><a name=\"tprotocol%d\" href=\"\" onClick=\"return false;\" style=\"cursor:default;\">TCP</a></td>\n"),j);
			break;
		case PROTO_UDP:
			websWrite(wp, T("<td align=center><a name=\"tprotocol%d\" href=\"\" onClick=\"return false;\" style=\"cursor:default;\">UDP</a></td>\n"),j);
			break;
		case PROTO_TCP_UDP:
			websWrite(wp, T("<td align=center><a name=\"tprotocol%d\" href=\"\" onClick=\"return false;\" style=\"cursor:default;\">TCP/UDP</a></td>\n"),j);
			break;
	}
	/* start port */
	websWrite(wp, T("<td align=center><a name=\"tports%d\" href=\"\" onClick=\"return false;\" style=\"cursor:default;\">%s</a></td>\n"), j, tportS);
	/* end port */
	websWrite(wp, T("<td align=center><a name=\"tporte%d\" href=\"\" onClick=\"return false;\" style=\"cursor:default;\">%s</a></td>\n"), j, tportE);

	/* web printf trigger */
	switch( atoi(oprotocol) ) {
		case PROTO_TCP:
			websWrite(wp, T("<td align=center><a name=\"oprotocol%d\" href=\"\" onClick=\"return false;\" style=\"cursor:default;\">TCP</a></td>\n"), j);
			break;
		case PROTO_UDP:
			websWrite(wp, T("<td align=center><a name=\"oprotocol%d\" href=\"\" onClick=\"return false;\" style=\"cursor:default;\">UDP</a></td>\n"), j);
			break;
		case PROTO_TCP_UDP:
			websWrite(wp, T("<td align=center><a name=\"oprotocol%d\" href=\"\" onClick=\"return false;\" style=\"cursor:default;\">TCP/UDP</a></td>\n"), j);
			break;
	}
	/* start port */
	websWrite(wp, T("<td align=center><a name=\"oports%d\" href=\"\" onClick=\"return false;\" style=\"cursor:default;\">%s</a></td>\n"), j, oportS);
	/* end port */
	websWrite(wp, T("<td align=center><a name=\"oporte%d\" href=\"\" onClick=\"return false;\" style=\"cursor:default;\">%s</a></td>\n"), j, oportE);
//	websWrite(wp, T("</tr>\n"));
	return 0;
}

/*
 * ASP function
 */
static int showPortTriggerRulesASP(int eid, webs_t wp, int argc, char_t **argv)
{
	int i, j, k;
	char rec[280]; /* app_name + (portS+portE+protol)*2*max_subrules  + many_","== 16 + (5+5+1)*2*8  + (',') */
	char appname[18];
	char enable[2];
	char multi_rules[PORT_TRIGGER_MAX_APPRULES][32];
//	char tportS[6], tportE[6], oportS[6], oportE[6];
//	char tprotocol[2], oprotocol[2];
	char const *rules = nvram_safe_get( "port_trigger_rules");
	char *new_rules;
//	int nport, nprotocol;
	int ret;

	if(!rules)
		return 0;
	if(!strlen(rules))
		return 0;

    new_rules = strdup(rules);
    if (!new_rules)
    {
        return 0;
    }

/* format is :
 * [app_name]|[enable(0,1)]|[trigger_port_start],[trigger_port_end],[trigger_protocol],[open_port_start],[open_port_end],[open_protocol]|[next_tport_start],[next_tport_end],[next...],[]...|;
 */
 	i = 0;
 	while(getNthValueSafe(i, new_rules, ';', rec, sizeof(rec)) != -1 ){
		// get app name
		if((getNthValueSafe(0, rec, '|', appname, sizeof(appname)) == -1)){
			continue;
		}
		// get enable state
		if((getNthValueSafe(1, rec, '|', enable, sizeof(enable)) == -1)){
			continue;
		}
		for (j = 0; j < PORT_TRIGGER_MAX_APPRULES; j++){
			/* get the number 'j+1' rules */
			if((getNthValueSafe(j+2, rec, '|', multi_rules[j], sizeof(multi_rules[j])) == -1))
				break;
			if ( !strlen(multi_rules[j]) )
				break;
		}
		/* empty rule ?! */
		if (!j)
		{
		    free(new_rules);
			return 0;
        }
		websWrite(wp, T("<tr>\n"));
		// output No.
		websWrite(wp, T("<td rowspan=\"%d\" class=\"leftnone\">%d&nbsp;</td>\n"), j, i+1);
		if (enable[0] == '1')
			websWriteWithTranslation(wp, T("<td rowspan=\"%d\" id=\"name%d\">%s(<#Enable#>)</td>\n"), j, i, appname);
		else
			websWriteWithTranslation(wp, T("<td rowspan=\"%d\" id=\"name%d\">%s(<#Disable#>)</td>\n"), j, i, appname);
		/* write the sub-rules */
		if (0 != webWriteTriggerSingleRule(wp, multi_rules[0],i))
			continue;
#ifdef HAVE_WEB_CNS
		;
#else
		websWrite(wp, T("<td rowspan=\"%d\"><a href=\"#edit\" onClick=\"trigger_edit_action(%d,%d); return false;\" ><img src=\"/images/edit.jpg\" width=\"16\" /></a></td>\n"), j, j, i );
#endif
    websWrite(wp, T("<td rowspan=\"%d\" class=\"rightnone\"><input type=\"checkbox\" name=\"actRule%d\" onClick=\"disableDelete(1);\"></td>\n"), j, i );
		websWrite(wp, T("</tr>\n"));

		/* continue next sub-rule */
		for (k = 1; k < j; k++){
			websWrite(wp, T("<tr>\n"));
			ret = webWriteTriggerSingleRule(wp, multi_rules[k],i);
			websWrite(wp, T("</tr>\n"));
			/* if fail to write rule, break loop. */
			if (0 != ret)
				break;
		}

		i++;	/* next rule idx */
	}

	free(new_rules);

	return 0;
}

/*
 * ASP function
 */
static int showPortForwardRulesASP(int eid, webs_t wp, int argc, char_t **argv)
{
	int i=0;
	int prf_int, prt_int, Client_prf_int,Client_prt_int,proto;
	char ip_address[32], prf[8], prt[8], /*Client_ip_address[32],*/ Client_prf[8], Client_prt[8], comment[34], protocol[8];
	char rec[128];
	char *rules = (char *)nvram_safe_get( "port_forward_rules");
	char *new_rules;


	if(!rules)
		return 0;
	if(!strlen(rules))
		return 0;

    new_rules = strdup(rules);
    if (!new_rules)
    {
        return 0;
    }

	/* format is :
	 * [ip],[port_from],[port_to],[protocol],[comment],;
	 */
	while(getNthValueSafe(i++, new_rules, ';', rec, sizeof(rec)) != -1 ){
		// get ip address
		if((getNthValueSafe(0, rec, ',', ip_address, sizeof(ip_address)) == -1)){
			continue;
		}
		if(!isIpValid(ip_address)){
			continue;
		}

		// get port range "from"
		if((getNthValueSafe(1, rec, ',', prf, sizeof(prf)) == -1)){
			continue;
		}
		if( (prf_int = atoi(prf)) == 0 || prf_int > 65535){
			continue;
		}

		// get port range "to"
		if((getNthValueSafe(2, rec, ',', prt, sizeof(prt)) == -1)){
			continue;
		}
		if( (prt_int = atoi(prt)) > 65535){
			continue;
		}


#if 0
	    // get client ip address  zengiqngchu
		if((getNthValueSafe(3, rec, ',', Client_ip_address, sizeof(Client_ip_address)) == -1)){
			continue;
		}
		if(!isIpValid(Client_ip_address)){
			continue;
		}
#endif
		// get client port range "from"
		if((getNthValueSafe(3, rec, ',', Client_prf, sizeof(Client_prf)) == -1)){
			continue;
		}
		if( (Client_prf_int = atoi(Client_prf)) == 0 || Client_prf_int > 65535){
			continue;
		}

		// get client port range "to"
		if((getNthValueSafe(4, rec, ',', Client_prt, sizeof(Client_prt)) == -1)){
			continue;
		}
		if( (Client_prt_int = atoi(Client_prt)) > 65535){
			continue;
		}

		// get protocol
		if((getNthValueSafe(5, rec, ',', protocol, sizeof(protocol)) == -1)){
			continue;
		}
		proto = atoi(protocol);
		switch(proto){
			case PROTO_TCP:
			case PROTO_UDP:
			case PROTO_TCP_UDP:
				break;
			default:
				continue;
		}

		if((getNthValueSafe(6, rec, ',', comment, sizeof(comment)) == -1)){
			continue;
		}
		
#ifdef HAVE_WEB_CNS
		websWrite(wp, T("<tr>\n"));
		// output No.
		websWrite(wp, T("<td> %d&nbsp; <input type=\"checkbox\" name=\"delRule%d\"> </td>"), i, i-1 );

		// output Comment
		if(strlen(comment))
			websWrite(wp, T("<td align=center> %s</td>"), comment);
		else
			websWrite(wp, T("<td align=center> &nbsp; </td>"));


		// output server IP address
		websWrite(wp, T("<td align=center> %s </td>"), ip_address);

		// output server Port Range
		if(prt_int)
			websWrite(wp, T("<td align=center> %d - %d </td>"), prf_int, prt_int);
		else
			websWrite(wp, T("<td align=center> %d </td>"), prf_int);

		// output client IP address
//		websWrite(wp, T("<td align=center> %s </td>"), Client_ip_address);

		// output client Port Range
		if(Client_prt_int)
			websWrite(wp, T("<td align=center> %d - %d </td>"), Client_prf_int, Client_prt_int);
		else
			websWrite(wp, T("<td align=center> %d </td>"), Client_prf_int);

		// output Protocol
        switch(proto){
            case PROTO_TCP:
				websWrite(wp, T("<td align=center> TCP </td>"));
				break;
            case PROTO_UDP:
				websWrite(wp, T("<td align=center> UDP </td>"));
				break;
            case PROTO_TCP_UDP:
				websWrite(wp, T("<td align=center> TCP + UDP </td>"));
				break;
		}


		websWrite(wp, T("</tr>\n"));
#else
		websWrite(wp, T("<tr>\n"));
		// output No.
		/*start of change portforward page minpengwei 20120606*/
		websWrite(wp, T("<td align=center class=\"leftnone\">%d</td>"), i);
        /*end of change portforward page minpengwei 20120606*/

		// output Comment
		if(strlen(comment))
			websWrite(wp, T("<td align=center id=\"cmt%d\">%s</td>"), i-1 , comment);
		else
			websWrite(wp, T("<td align=center id=\"cmt%d\"> &nbsp; </td>"), i-1 );


		// output server IP address
		websWrite(wp, T("<td align=center id=\"ip%d\">%s</td>"), i-1 , ip_address);

		// output server Port Range
		/*start of change portforward page minpengwei 20120606*/
		if(prt_int)
			websWrite(wp, T("<td align=center id=\"sport%d\">%d-%d</td>"), i-1 , prf_int, prt_int);
		else
			websWrite(wp, T("<td align=center id=\"sport%d\">%d</td>"), i-1 , prf_int);
       /*end of change portforward page minpengwei 20120606*/

		// output client IP address
//		websWrite(wp, T("<td align=center> %s </td>"), Client_ip_address);

		// output client Port Range
		/*start of change portforward page minpengwei 20120606*/
		if(Client_prt_int)
			websWrite(wp, T("<td style='display:none;' align=center id=\"cport%d\">%d-%d</td>"), i-1 , Client_prf_int, Client_prt_int);
		else
			websWrite(wp, T("<td style='display:none;' align=center id=\"cport%d\">%d</td>"), i-1 , Client_prf_int);
        /*end of change portforward page minpengwei 20120606*/

		// output Protocol
        switch(proto){
            case PROTO_TCP:
				websWrite(wp, T("<td align=center id=\"proto%d\">TCP</td>"), i-1 );
				break;
            case PROTO_UDP:
				websWrite(wp, T("<td align=center id=\"proto%d\">UDP</td>"), i-1 );
				break;
            case PROTO_TCP_UDP:
				websWrite(wp, T("<td align=center id=\"proto%d\">Tcp+Udp</td>"), i-1 );
				break;
		}

		/*start of change portforward page minpengwei 20120606*/
		websWrite(wp, T("<td align=\"center\"><a href=\"#edit\" onClick=\"forward_edit_action(%d); return false;\" ><img src=\"/images/edit.jpg\" width=\"16\" /></a></td>"), i-1 );
		websWrite(wp, T("<td align=\"center\" class=\"rightnone\"><input type=\"checkbox\" name=\"delRule%d\"></td>"), i-1 );
/*end of change portforward page minpengwei 20120606*/
		
		websWrite(wp, T("</tr>\n"));
#endif
	}

	free(new_rules);

	return 0;
}

static void getRulesPacketCount(webs_t wp, char_t *path, char_t *query)
{
	FILE *fp;
	int i, step_in_chains=0;
	char buf[1024], *default_policy;
	int default_drop_flag;
	int index=0, pkt_count;
	int *result;

	// check if the default policy is "drop"
	default_policy = (char *)nvram_safe_get( "ip_filter_default_policy");
	if(!default_policy)
		default_policy = "0";
	default_drop_flag = atoi(default_policy);

	websWrite(wp, T("HTTP/1.1 200 OK\nContent-type: text/plain\nPragma: no-cache\nCache-Control: no-cache\n\n"));

	result = (int *)malloc(sizeof(int) * 128);
	if(!result)
		goto error;

	fp = popen("iptables -t filter -L -v", "r");
	if(!fp){
		free(result);
		goto error;
	}

	while(fgets(buf, 1024, fp) && index < 128){
		if(step_in_chains){
			if(buf[0] == '\n')
				break;
			if(buf[0] == ' ' && buf[1] == 'p' && buf[2] == 'k' && buf[3] == 't' )
				continue;
			// Skip the first one rule if default policy is drop.
			if(default_drop_flag){
				default_drop_flag = 0;
				continue;
			}
			sscanf(buf, "%d ", &pkt_count);
			result[index++] = pkt_count;
		}

		if(strstr(buf, "Chain " IPPORT_FILTER_CHAIN))
			step_in_chains = 1;
	}
	pclose(fp);

	if(index > 0)
		websWrite(wp, "%d", result[0]);
	for(i=1; i<index; i++)
		websWrite(wp, " %d", result[i]);

	free(result);
error:
	websDone(wp, 200);
	return;
}


/*
 * ASP function
 */
static int showIPPortFilterRulesASP(int eid, webs_t wp, int argc, char_t **argv)
{
	int i;
	int sprf_int, sprt_int, proto;
	char mac_address[32];
	char sip_1[32], sip_2[32], sprf[8], sprt[8], comment[33], protocol[8], action[4];
	char dip_1[32], dip_2[32], dprf[8], dprt[8];
	int dprf_int, dprt_int;
	char rec[256];
	char *default_policy;
	char ac_day_all[4], ac_mon[4], ac_tue[4], ac_wed[4], ac_thu[4], ac_fri[4], ac_sat[4], ac_sun[4], ac_all_hour[4], ac_start[4], ac_end[4];
	char ac_day_list[512], ac_time_list[512];
	char *rules = (char *)nvram_safe_get( "ip_filter_rules");
	if(!rules)
		return 0;

    default_policy = (char *)nvram_safe_get( "ip_filter_default_policy");
    // add the default policy to the end of FORWARD chain
    if(!default_policy)
		return 0;
	if(!strlen(default_policy))
		return 0;

	i=0;
	while(getNthValueSafe(i, rules, ';', rec, sizeof(rec)) != -1 && strlen(rec)){
		printf("i=%d, rec=%s, strlen(rec)=%d\n", i, rec, strlen(rec));
		// get ip 1
        if((getNthValueSafe(0, rec, ',', sip_1, sizeof(sip_1)) == -1)){
			i++;
			continue;
		}
        if(!isIpNetmaskValid(sip_1)){
			i++;
			continue;
		}
		// translate "any/0" to "any" for readable reason
		if( !strcmp(sip_1, "any/0"))
			strcpy(sip_1, "-");

		// get ip 2
        // get ip address
        if((getNthValueSafe(1, rec, ',', sip_2, sizeof(sip_2)) == -1)){
			i++;
			continue;
		}
		// dont verify cause we dont have ip range support
		//if(!isIpValid(sip_2))
        //    continue;

		// get port range "from"
		if((getNthValueSafe(2, rec, ',', sprf, sizeof(sprf)) == -1)){
			i++;
			continue;
		}
		if( (sprf_int = atoi(sprf)) > 65535){
			i++;
			continue;
		}

		// get port range "to"
		if((getNthValueSafe(3, rec, ',', sprt, sizeof(sprt)) == -1)){
			i++;
			continue;
		}
		if( (sprt_int = atoi(sprt)) > 65535){
			i++;
			continue;
		}

		// get ip 1
        if((getNthValueSafe(4, rec, ',', dip_1, sizeof(dip_1)) == -1)){
			i++;
            continue;
		}
        if(!isIpNetmaskValid(dip_1)){
			i++;
            continue;
		}
		// translate "any/0" to "any" for readable reason
		if( !strcmp(dip_1, "any/0"))
			strcpy(dip_1, "-");

		// get ip 2
        if((getNthValueSafe(5, rec, ',', dip_2, sizeof(dip_2)) == -1)){
			i++;
            continue;
		}
		// dont verify cause we dont have ip range support
		//if(!isIpValid(dip_2))
        //    continue;

		// get protocol
		if((getNthValueSafe(8, rec, ',', protocol, sizeof(protocol)) == -1)){
			i++;
			continue;
		}
		proto = atoi(protocol);
		switch(proto){
			case PROTO_TCP:
			case PROTO_UDP:
			case PROTO_NONE:
			case PROTO_ICMP:
				break;
			default:
				continue;
		}

		// get port range "from"
		if((getNthValueSafe(6, rec, ',', dprf, sizeof(dprf)) == -1)){
			i++;
			continue;
		}
		if( (dprf_int = atoi(dprf)) > 65535){
			i++;
			continue;
		}

		// get port range "to"
		if((getNthValueSafe(7, rec, ',', dprt, sizeof(dprt)) == -1)){
			i++;
			continue;
		}
		if( (dprt_int = atoi(dprt)) > 65535){
			i++;
			continue;
		}

		// get action
		if((getNthValueSafe(9, rec, ',', action, sizeof(action)) == -1)){
			i++;
			continue;
		}

		// get comment
		if((getNthValueSafe(10, rec, ',', comment, sizeof(comment)) == -1)){
			i++;
			continue;
		}

		// get mac address
		if((getNthValueSafe(11, rec, ',', mac_address, sizeof(mac_address)) == -1)){
			i++;
			continue;
		}
		if(!strlen(mac_address))
			gstrcpy(mac_address, T("-"));

		// get day all
		if((getNthValueSafe(12, rec, ',', ac_day_all, sizeof(ac_day_all)) == -1)){
			i++;
			continue;
		}
		if( atoi(ac_day_all) == 7 )
		{
			gstrcpy(ac_day_list, T("<#dayweekall#>"));
		}
		else
		{
			gstrcpy(ac_day_list, T(" "));
			if((getNthValueSafe(13, rec, ',', ac_mon, sizeof(ac_mon)) == -1)){
				i++;
				continue;
			}
			if( atoi(ac_mon) )
				strcat(ac_day_list, T("<#weekdaymon#>"));

			if((getNthValueSafe(14, rec, ',', ac_tue, sizeof(ac_tue)) == -1)){
				i++;
				continue;
			}
			if( atoi(ac_tue) )
				strcat(ac_day_list, T(" <#weekdaytue#>"));

			if((getNthValueSafe(15, rec, ',', ac_wed, sizeof(ac_wed)) == -1)){
				i++;
				continue;
			}
			if( atoi(ac_wed) )
				strcat(ac_day_list, T(" <#weekdaywed#>"));

			if((getNthValueSafe(16, rec, ',', ac_thu, sizeof(ac_thu)) == -1)){
				i++;
				continue;
			}
			if( atoi(ac_thu) )
				strcat(ac_day_list, T(" <#weekdaythu#>"));

			if((getNthValueSafe(17, rec, ',', ac_fri, sizeof(ac_fri)) == -1)){
				i++;
				continue;
			}
			if( atoi(ac_fri) )
				strcat(ac_day_list, T(" <#weekdayfri#>"));

			if((getNthValueSafe(18, rec, ',', ac_sat, sizeof(ac_sat)) == -1)){
				i++;
				continue;
			}
			if( atoi(ac_sat) )
				strcat(ac_day_list, T(" <#weekdaysat#>"));

			if((getNthValueSafe(19, rec, ',', ac_sun, sizeof(ac_sun)) == -1)){
				i++;
				continue;
			}
			if( atoi(ac_sun) )
				strcat(ac_day_list, T(" <#weekdaysun#>"));
		}
		// get day time
		if((getNthValueSafe(20, rec, ',', ac_all_hour, sizeof(ac_all_hour)) == -1)){
			i++;
			continue;
		}
		if( atoi(ac_all_hour) == 0 )
		{
			gstrcpy(ac_time_list, T("<#daytimeall#>"));
		}
		else
		{
			gstrcpy(ac_time_list, T(" "));
			if((getNthValueSafe(21, rec, ',', ac_start, sizeof(ac_start)) == -1)){
				i++;
				continue;
			}
			if( strlen(ac_start) )
			{
				strcat(ac_time_list, ac_start);
				strcat(ac_time_list, T("-"));
			}

			if((getNthValueSafe(22, rec, ',', ac_end, sizeof(ac_end)) == -1)){
				i++;
				continue;
			}
			if( strlen(ac_end) )
				strcat(ac_time_list, ac_end);

		}
		
#ifdef HAVE_WEB_CNS
		websWrite(wp, T("<tr>\n"));
		// output No.
		websWrite(wp, T("<td> %d&nbsp; <input type=\"checkbox\" id=\"delRule%d\" name=\"delRule%d\"> </td>"), i+1, i,i );

		// output Mac address
		websWrite(wp, T("<td align=center> %s </td>"), mac_address);

		// output DIP
		websWrite(wp, T("<td align=center> %s </td>"), dip_1);
		// we dont support ip range
		// websWrite(wp, T("<td align=center> %s-%s </td>"), ip_1, ip_2);

		// output SIP
		websWrite(wp, T("<td align=center> %s </td>"), sip_1);
		// we dont support ip range
		// websWrite(wp, T("<td align=center> %s-%s </td>"), ip_1, ip_2);

		// output Protocol
		switch(proto){
			case PROTO_TCP:
				websWrite(wp, T("<td align=center> TCP </td>"));
				break;
			case PROTO_UDP:
				websWrite(wp, T("<td align=center> UDP </td>"));
				break;
			case PROTO_ICMP:
				websWrite(wp, T("<td align=center> ICMP </td>"));
				break;
			case PROTO_NONE:
				websWrite(wp, T("<td align=center> - </td>"));
				break;
		}

		// output dest Port Range
		if(dprt_int)
			websWrite(wp, T("<td align=center> %d - %d </td>"), dprf_int, dprt_int);
		else{
			// we re-descript the port number here because
			// "any" word is more meanful than "0"
			if(!dprf_int){
				websWrite(wp, T("<td align=center> - </td>"), dprf_int);
			}else{
				websWrite(wp, T("<td align=center> %d </td>"), dprf_int);
			}
		}

		// output Source Port Range
		if(sprt_int)
			websWrite(wp, T("<td align=center> %d - %d </td>"), sprf_int, sprt_int);
		else{
			// we re-descript the port number here because
			// "any" word is more meanful than "0"
			if(!sprf_int){
				websWrite(wp, T("<td align=center> - </td>"), sprf_int);
			}else{
				websWrite(wp, T("<td align=center> %d </td>"), sprf_int);
			}
		}


		// output action
		switch(atoi(action)){
			case ACTION_DROP:
				websWriteWithTranslation(wp, T("<td align=center id=portFilterAction%d><#Dropped#></td>"), i);
				break;
			case ACTION_ACCEPT:
				websWriteWithTranslation(wp, T("<td align=center id=portFilterAction%d><#Accepted#></td>"), i);
				break;
		}

		// output Comment
		if(strlen(comment))
			websWrite(wp, T("<td align=center> %s</td>"), comment);
		else
			websWrite(wp, T("<td align=center> &nbsp; </td>"));

		// output the id of "packet count"
		websWriteWithTranslation(wp, T("<td align=center id=pktCnt%d title='%s %s'><#daytime#></td>"), i, ac_day_list, ac_time_list);

		websWrite(wp, T("</tr>\n"));

		i++;
	}

	switch(atoi(default_policy)){
		case 0:
			websWriteWithTranslation(wp, T("<tr><td align=center colspan=11 id=portCurrentFilterDefaultAccept><#PortFilter_Body_23#></td><td align=center id=pktCnt%d>-</td></tr>"), i);
			break;
		case 1:
			websWriteWithTranslation(wp, T("<tr><td align=center colspan=11 id=portCurrentFilterDefaultDrop><#PortFilter_Body_24#></td><td align=center id=pktCnt%d>-</td></tr>"), i);
			break;
	}
		
#else
		websWrite(wp, T("<tr>\n"));
		// output No.
		websWrite(wp, T("<td class=\"leftnone\">%d</td>"), i+1);

		// output Mac address
		websWrite(wp, T("<td align=center id=\"mac%d%d\">%s</td>"), i,i, mac_address);

		// output DIP
		websWrite(wp, T("<td align=center id=\"dip%d%d\">%s</td>"), i,i, dip_1);
		// we dont support ip range
		// websWrite(wp, T("<td align=center> %s-%s </td>"), ip_1, ip_2);

		// output SIP
		websWrite(wp, T("<td align=center id=\"sip%d%d\">%s</td>"), i,i, sip_1);
		// we dont support ip range
		// websWrite(wp, T("<td align=center> %s-%s </td>"), ip_1, ip_2);

		// output Protocol
        switch(proto){
            case PROTO_TCP:
				websWrite(wp, T("<td align=center id=\"proto%d\">TCP</td>"), i);
				break;
            case PROTO_UDP:
				websWrite(wp, T("<td align=center id=\"proto%d\">UDP</td>"), i);
				break;
            case PROTO_ICMP:
				websWrite(wp, T("<td align=center id=\"proto%d\">ICMP</td>"), i);
				break;
            case PROTO_NONE:
				websWrite(wp, T("<td align=center id=\"proto%d\">-</td>"), i);
				break;
		}

		// output dest Port Range
		if(dprt_int)
			websWrite(wp, T("<td align=center id=\"dport%d\">%d-%d</td>"), i, dprf_int, dprt_int);
		else{
			// we re-descript the port number here because
			// "any" word is more meanful than "0"
			if(!dprf_int){
				websWrite(wp, T("<td align=center id=\"dport%d\">-</td>"), i, dprf_int);
			}else{
				websWrite(wp, T("<td align=center id=\"dport%d\">%d</td>"), i, dprf_int);
			}
		}

		// output Source Port Range
		if(sprt_int)
			websWrite(wp, T("<td align=center id=\"sport%d\">%d-%d</td>"), i, sprf_int, sprt_int);
		else{
			// we re-descript the port number here because
			// "any" word is more meanful than "0"
			if(!sprf_int){
				websWrite(wp, T("<td align=center id=\"sport%d\">-</td>"), i, sprf_int);
			}else{
				websWrite(wp, T("<td align=center id=\"sport%d\">%d</td>"), i, sprf_int);
			}
		}


		// output action
        switch(atoi(action)){
            case ACTION_DROP:
				websWriteWithTranslation(wp, T("<td align=center id=portFilterAction%d><#Dropped#></td>"), i);
				break;
            case ACTION_ACCEPT:
				websWriteWithTranslation(wp, T("<td align=center id=portFilterAction%d><#Accepted#></td>"), i);
				break;
		}

		// output Comment
		if(strlen(comment))
			websWrite(wp, T("<td align=center id=\"cmt%d\">%s</td>"), i, comment);
		else
			websWrite(wp, T("<td align=center id=\"cmt%d\">&nbsp;</td>"), i);

		// output the id of "packet count"
		websWrite(wp, T("<td><a href=\"#edit\" onClick=\"filter_edit_action(%d); return false;\" ><img src=\"/images/edit.jpg\" width=\"16\" /></a></td>"), i );
		websWrite(wp, T("<td class=\"rightnone\"><input type=\"checkbox\" id=\"delRule%d\" name=\"delRule%d\" onClick=\"disableDelete(1);\"> </td>"), i,i );

		websWrite(wp, T("</tr>\n"));

		i++;
	}

	switch(atoi(default_policy)){
		case 0:
			websWriteWithTranslation(wp, T("<tr><td class=\"none\" align=center colspan=11 id=portCurrentFilterDefaultAccept><#PortFilter_Body_23#></td></tr>"), i);
			break;
		case 1:
			websWriteWithTranslation(wp, T("<tr><td class=\"none\" align=center colspan=11 id=portCurrentFilterDefaultDrop><#PortFilter_Body_24#></td></tr>"), i);
			break;
	}
#endif
	return 0;
}



static int showARPRulesASP(int eid, webs_t wp, int argc, char_t **argv)
	//static void  showARPRulesASP(webs_t wp, char_t *path, char_t *query)
{
	int i;

 	char mac_addr[32], ip_addr[32],  rec[256];

	char const *rules = nvram_safe_get( "ARPRules");
	if(!rules)
		return 0;



	i=0;
	while(getNthValueSafe(i, rules, ';', rec, sizeof(rec)) != -1 && (strlen(rec))){
		printf("i=%d, rec=%s, strlen(rec)=%d\n", i, rec, strlen(rec));
		// get ip 1
        if((getNthValueSafe(0, rec, ',', ip_addr, sizeof(ip_addr)) == -1)){
			i++;
			continue;
		}
        if(!isIpNetmaskValid(ip_addr)){
			i++;
			continue;
		}
			// get mac address
        if((getNthValueSafe(1, rec, ',', mac_addr, sizeof(mac_addr)) == -1)){
			i++;
			continue;
		}

		if(!strlen(mac_addr))
			gstrcpy(mac_addr, T("-"));

		websWrite(wp, T("<tr>\n"));
		// output No.
		websWrite(wp, T("<td> %d&nbsp; <input type=\"checkbox\" name=\"delRule%d\"> </td>"), i+1, i);

		// output  IP
		websWrite(wp, T("<td align=center> %s </td>"), ip_addr);

		// Mac address
		websWrite(wp, T("<td align=center> %s </td>"), mac_addr);

		websWrite(wp, T("</tr>\n"));

		i++;
	}

	return 0;
}

static int showDMZIPAddressASP(int eid, webs_t wp, int argc, char_t **argv)
{
	char *DMZIPAddress = (char *)nvram_safe_get( "dmz_host_addr");
	if(!DMZIPAddress)
		return 0;
	if(!strlen(DMZIPAddress))
		return 0;

	websWrite(wp, T("%s"), DMZIPAddress);
	return 0;
}
static int showSuperDMZMACAddressASP(int eid, webs_t wp, int argc, char_t **argv)
{
	char *SperDMZMACAddress = (char *)nvram_safe_get( "SuperDMZMACAddress");
	if(!SperDMZMACAddress)
		return 0;
	if(!strlen(SperDMZMACAddress))
		return 0;

	websWrite(wp, T("%s"), SperDMZMACAddress);
	return 0;
}
/* add by radar@20081229. for Firewall Sentry Port list. */
static int showFWSentryPortASP(int eid, webs_t wp, int argc, char_t **argv)
{
	char *type = NULL;
	char *sentryPort = NULL;
	if (ejArgs(argc, argv, T("%s"), &type) < 1) {
		websWrite(wp, T(" "));
		return 0;
	}
	if (0 == strcmp(type, "tcp")){
		sentryPort = (char *)nvram_safe_get( NVRAM_APS_SENTRY_TCP_PORTS);
	}else if (0 == strcmp(type, "udp")){
		sentryPort = (char *)nvram_safe_get( NVRAM_APS_SENTRY_UDP_PORTS);
	}

	if (!sentryPort || !strlen(sentryPort))
		return 0;

	websWrite(wp, T("%s"), sentryPort);
	return 0;
}

static int showFWIgnorePortASP(int eid, webs_t wp, int argc, char_t **argv)
{
	char *type = NULL;
	char *ignorePort = NULL;
	if (ejArgs(argc, argv, T("%s"), &type) < 1) {
		websWrite(wp, T(" "));
		return 0;
	}
	if (0 == strcmp(type, "tcp")){
		ignorePort = (char *)nvram_safe_get( NVRAM_APS_IGNORE_TCP_PORTS);
	}else if (0 == strcmp(type, "udp")){
		ignorePort = (char *)nvram_safe_get( NVRAM_APS_IGNORE_UDP_PORTS);
	}

	if (!ignorePort || !strlen(ignorePort))
		return 0;

	websWrite(wp, T("%s"), ignorePort);
	return 0;
}
/* end append. radar */

static void ipportFilterDelete(webs_t wp, char_t *path, char_t *query)
{
	int i, j, rule_count;
	char_t name_buf[16];
	char_t *value;
	int *deleArray;
//	char *firewall_enable;

	char *new_rules;
    const char *rules = (char *)nvram_safe_get( "ip_filter_rules");
    if(!rules || !strlen(rules) )
        return;

	rule_count = getRuleNums((char *)rules);
	if(!rule_count)
		return;

	deleArray = (int *)malloc(rule_count * sizeof(int));
	/*start of modify by huangyikun at 20110818*/
	memset(deleArray,-1,rule_count * sizeof(int));
	/*end of modify by huangyikun at 20110818*/
	if(!deleArray)
		return;
	new_rules = strdup(rules);
	if(!new_rules){
		free(deleArray);
		return;
	}

	for(i=0, j=0; i< rule_count; i++){
		snprintf(name_buf, 16, "delRule%d", i);
		value = websGetVar(wp, name_buf, NULL);
		if(value){
			deleArray[j++] = i;
			doSystem("rm -rf /etc/cron/*");
		}
	}

	/* doSystem("rm -f /etc/crontab"); */
    doSystem("rm -rf /var/spool/cron/crontabs/admin");

	if(!j){
#ifdef	GOAHEAD_SHOWSUBMITINFO
	    websHeader(wp);
	    websWrite(wp, T("You didn't select any rules to delete.<br>\n"));
	    websFooter(wp);
	    websDone(wp, 200);
#else
		websRedirect(wp, "firewall/port_filtering.asp");
#endif
		return;
	}

	deleteNthValueMulti(deleArray, rule_count, new_rules, ';');
	free(deleArray);
/*start of delete the rule minpengwei 20120611*/
	/*start of modify by huangyikun at 20110818*/
	#if 0
	if(!strstr(new_rules,";"))
		nvram_set( "ip_filter_rules", "");
	else
	#endif
		nvram_set( "ip_filter_rules", new_rules);
	/*end of modify by huangyikun at 20110818*/
/*end of delete the rule minpengwei 20120611*/
	nvram_commit();
	free(new_rules);

	//iptablesIPPortFilterFlush();
	//iptablesIPPortFilterRun();
	//doSystem("rc firewall");
	notification_send_rc( OID_NETWORK_FIREWALL, NOTIFY_WAIT );

#ifdef	GOAHEAD_SHOWSUBMITINFO
    websHeader(wp);
    websWrite(wp, T("s<br>\n") );
    websWrite(wp, T("fromPort: <br>\n"));
    websWrite(wp, T("toPort: <br>\n"));
    websWrite(wp, T("protocol: <br>\n"));
    websWrite(wp, T("comment: <br>\n"));
    websFooter(wp);
    websDone(wp, 200);
#else
	websRedirect(wp, "port_filtering.asp");
#endif

	return;
}

static void portTriggerRulesAction(webs_t wp, char_t *path, char_t *query)
{
    char rec[280],temp_rules[32];
    char_t name_buf[16],oProto[8];
	char *Trigger_num = (char *)nvram_safe_get("port_trigger_num"),temp[4]={0};
	int i,j,k,rule_count,i_proto,Trigger_num_int = atoi(Trigger_num);
	char *actionType;
	int *actArray = NULL;
	char flgEnable;
	char *pRule = NULL;
	char_t *value;
	const char *rules = nvram_safe_get( "port_trigger_rules");
	char *new_rules = NULL;

	if(!rules || !strlen(rules) )
		goto end;
	rule_count = getRuleNums(rules);
	if(!rule_count)
		goto end;

	new_rules = strdup(rules);
	if (!new_rules)
	{
		goto end;
	}

	actionType = websGetVar(wp, T("actionType"), NULL);
	if (!actionType)
		goto end;
	printf("action:%s\n", actionType);
	/* unknown action */
	if ( (0 != strcmp(actionType, "en"))	//enable
		&& (0 != strcmp(actionType, "dis"))	//disable
		&& (0 != strcmp(actionType, "del")) )	//delete
		goto end;

	actArray = (int *)malloc(rule_count * sizeof(int));
	if (!actArray) {
		perror("malloc");
		goto end;
	}

	for(i=0, j=0; i< rule_count; i++){
		snprintf(name_buf, 16, "actRule%d", i);
		value = websGetVar(wp, T(name_buf), NULL);
		if(value){
			actArray[j++] = i;

			if(!strcmp(actionType, "del"))  //zengqingchu
			{
                if(getNthValueSafe(i, rules, ';', rec, sizeof(rec)) != -1)
                {

                    for(k = 0;k < 10; k++)
                    {
                        if(getNthValueSafe(k+2, rec, '|', temp_rules, sizeof(temp_rules)) == -1)
			                break;

						if(getNthValueSafe(5, temp_rules, ',', oProto, sizeof(oProto))== -1)
				            break;

						i_proto = atoi(oProto);

		            	switch(i_proto)
			           {
			                case PROTO_TCP:
			                case PROTO_UDP:Trigger_num_int -= 1 ;break;
			                case PROTO_TCP_UDP:Trigger_num_int -= 2 ;break;
			           }
					}

				}
			}
		}
	}
	if ( (0 == strcmp(actionType, "en")) || (0 == strcmp(actionType, "dis")) ){
		flgEnable = (actionType[0] == 'e') ? '1' : '0';
		for (i = 0; i < j; i++){
			pRule = getNthValueOffset(actArray[i], new_rules);
			printf("rule1:%s\n", pRule);
			if (pRule)
				pRule = strchr(pRule, '|');
			if (pRule)
				*(++pRule) = flgEnable;
			printf("rule2:%s\n", pRule);
		}
	}else if (0 == strcmp(actionType, "del") ){
	    sprintf(temp,"%d",Trigger_num_int);
		nvram_set( "port_trigger_num", temp);
		deleteNthValueMulti(actArray, rule_count, new_rules, ';');
	}else
		goto end;

	nvram_set( "port_trigger_rules", new_rules);
	nvram_commit();
	/* update rule */
	notification_send_rc( OID_NETWORK_FIREWALL, NOTIFY_WAIT );
end:
	if (actArray) free(actArray);
	websRedirect(wp, "port_trigger.asp");

	free(new_rules);

	return;
}

static void portForwardDelete(webs_t wp, char_t *path, char_t *query)
{
	int i, j, rule_count,proto,relus_num_int;
	char_t  rec[256],name_buf[16],protocol[8],ip_address[32],num[4] = {0};
	char *forwardingRule_num = (char *)nvram_safe_get("port_forward_num");
	char_t *value;
	int *deleArray;
	char *firewall_enable;

	char *new_rules;
    const char *rules = (char *)nvram_safe_get( "port_forward_rules");
    if(!rules || !strlen(rules) )
        return;

	rule_count = getRuleNums((char *)rules);
	if(!rule_count)
		return;

	deleArray = (int *)malloc(rule_count * sizeof(int));
	if(!deleArray)
		return;

	new_rules = strdup(rules);
	if(!new_rules){
		free(deleArray);
		return;
	}

	relus_num_int = atoi(forwardingRule_num);

	for(i=0, j=0; i< rule_count; i++){
		snprintf(name_buf, 16, "delRule%d", i);
		value = websGetVar(wp, T(name_buf), NULL);
		if(value){
			deleArray[j++] = i;
			if(getNthValueSafe(i,rules,';',rec,sizeof(rec))!= -1)
			{
				//Get ipAddress
			    #if 1 
			    if((getNthValueSafe(0, rec, ',', ip_address, sizeof(ip_address)) == -1))
				    continue;
			    else	
	        		doSystem("arp -d %s",ip_address);
			    #endif
                // get protocol
		      if((getNthValueSafe(5, rec, ',', protocol, sizeof(protocol)) == -1))
			     continue;

		      proto = atoi(protocol);
		      switch(proto){
			      case PROTO_TCP:
			      case PROTO_UDP:
					  relus_num_int -= 1;
					  break;
			      case PROTO_TCP_UDP:
					  relus_num_int -= 2;
			    	  break;
		                }
			}

		}
	}

    if(!j){
        websHeader(wp);
        websWrite(wp, T("You didn't select any rules to delete.<br>\n"));
        websFooter(wp);
        websDone(wp, 200);
        return;
    }

	sprintf(num,"%d",relus_num_int);  //zengqingchu
	nvram_set( "port_forward_num",num);

	deleteNthValueMulti(deleArray, rule_count, new_rules, ';');
	free(deleArray);

	nvram_set( "port_forward_rules", new_rules);
	nvram_commit();
	free(new_rules);

	// restart iptables if it is running
	firewall_enable = (char *)nvram_safe_get( "port_forward_en");
	if(firewall_enable){
		if(atoi(firewall_enable)){
			notification_send_rc( OID_NETWORK_FIREWALL, NOTIFY_WAIT );
		}
	}

#ifdef	GOAHEAD_SHOWSUBMITINFO
    websHeader(wp);
    websWrite(wp, T("s<br>\n") );
    websWrite(wp, T("fromPort: <br>\n"));
    websWrite(wp, T("toPort: <br>\n"));
    websWrite(wp, T("protocol: <br>\n"));
    websWrite(wp, T("comment: <br>\n"));
    websFooter(wp);
    websDone(wp, 200);
#else
	websRedirect(wp, "port_forward.asp");
#endif

	return;
}


typedef struct {
	int idx_in_web_sel;
	char *comment;
	unsigned int protocol;
	unsigned int port_from;
	unsigned int port_to;
}TfilterRules;
static const TfilterRules g_filter_rules[] = {
	{1, "HTTP", PROTO_TCP, 80, 0},
	{1, "HTTP", PROTO_TCP, 3128, 0},
	{1, "HTTP", PROTO_TCP, 8000, 0},
	{1, "HTTP", PROTO_TCP, 8001, 0},
	{1, "HTTP", PROTO_TCP, 8080, 0},
	{2, "SMTP", PROTO_TCP, 25, 0},
	{3, "NNTP", PROTO_TCP, 119, 0},
	{4, "POP3", PROTO_TCP, 110, 0},
	{5, "HTTPS", PROTO_TCP, 443, 0},
	{6, "FTP", PROTO_TCP, 21, 0},
	{7, "Telnet", PROTO_TCP, 23, 0},
	{8, "AIM", PROTO_TCP, 5190, 0},
	{9, "NetMeeting", PROTO_TCP, 1503, 0},
	{9, "NetMeeting", PROTO_TCP, 1720, 0},
	{10, "DNS", PROTO_UDP, 53, 0},
	{11, "ALL TCP", PROTO_TCP, 1, 65535},
	{12, "ALL UDP", PROTO_UDP, 1, 65535},
};

#ifdef HAVE_WEB_CNS 
static void ipportFilter(webs_t wp, char_t *path, char_t *query)
{
	char rule[8192] = {0};
	char *mac_address;
	char *sip_1, *sip_2, *sprf, *sprt, *protocol, *action_str, *comment;
	char *dip_1, *dip_2, *dprf, *dprt;
	char *day_all, *Mon_select, *Tue_select, *Wed_select, *Thu_select, *Fri_select;
	char *Sat_select, *Sun_select;
	char *all_hours, *start_time, *end_time;
	const char *IPPortFilterRules;
	/*start of modify by huangyikun at 20110818*/
	char *IPPortFilterDefaultRules;
	int default_action;
	/*end of modify by huangyikun at 20110818*/

	char *user_service;

	int sprf_int, sprt_int, dprf_int, dprt_int, proto, action;
	int index;
	char flg_needcommit = 0;

	user_service = websGetVar(wp, T("access_list"), T(""));
	nvram_set( "portselectService", user_service);
	//        nvram_commit(RT2860_NVRAM);

	day_all = websGetVar(wp, T("day_type_all"), T(""));
	Mon_select = websGetVar(wp, T("Mon_select"), T(""));
	Tue_select = websGetVar(wp, T("Tue_select"), T(""));
	Wed_select = websGetVar(wp, T("Wed_select"), T(""));
	Thu_select = websGetVar(wp, T("Thu_select"), T(""));
	Fri_select = websGetVar(wp, T("Fri_select"), T(""));
	Sat_select = websGetVar(wp, T("Sat_select"), T(""));
	Sun_select = websGetVar(wp, T("Sun_select"), T(""));
	all_hours = websGetVar(wp, T("radiobutton"), T(""));
	start_time = websGetVar(wp, T("minute"), T(""));
	end_time = websGetVar(wp, T("hour"), T(""));

	mac_address = websGetVar(wp, T("mac_address"), T(""));

	sip_1 = websGetVar(wp, T("sip_address"), T("any"));
	sip_2 = websGetVar(wp, T("sip_address2"), T(""));
	sprf = websGetVar(wp, T("sFromPort"), T("0"));
	sprt = websGetVar(wp, T("sToPort"), T(""));

	dip_1 = websGetVar(wp, T("dip_address"), T("any"));
	dip_2 = websGetVar(wp, T("dip_address2"), T(""));
	dprf = websGetVar(wp, T("dFromPort"), T("0"));
	dprt = websGetVar(wp, T("dToPort"), T(""));

	protocol = websGetVar(wp, T("protocol"), T(""));
	action_str = websGetVar(wp, T("action"), T(""));
	comment = websGetVar(wp, T("comment"), T(""));

	IPPortFilterRules = nvram_safe_get( "ip_filter_rules");
	/*start of modify by huangyikun at 20110818*/
	IPPortFilterDefaultRules = nvram_safe_get("ip_filter_default_policy");
	/*end of modify by huangyikun at 20110818*/
/*
    printf("%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s",day_all,Mon_select,Tue_select,Wed_select,
		Thu_select,Fri_select,Sat_select,Sun_select,all_hours,start_time,end_time);
*/
	if(! (strcmp(action_str, T("Accept"))))
		action = ACTION_ACCEPT;
	else if(! (strcmp(action_str, T("Drop"))))
		action = ACTION_DROP;
	else
		goto FILTER_END;

	/*start of modify by huangyikun at 20110818*/
	int cur_strlen = 0;
	int ret_strlen = 0;
	int buf_size = sizeof(rule) - 1;
	char *find;
	char IPPortFilterRulesTemp[8192] = {0};
	if (IPPortFilterRules && strlen(IPPortFilterRules)) 
	{
		strcpy(IPPortFilterRulesTemp,IPPortFilterRules);
		find=strstr(IPPortFilterRulesTemp,";,,0,0,,,0,0,0,");
		if(find)
		{
			*find = '\0';
		}
		
		ret_strlen = snprintf(rule, buf_size, "%s;", IPPortFilterRulesTemp);
		//strncpy(rule, IPPortFilterRules, ret_strlen);
		buf_size -= ret_strlen;
		cur_strlen += ret_strlen;
	}
	/*end of modify by huangyikun at 20110818*/

	if(strcmp(user_service, T("0")) == 0)
	{
		if(!mac_address || !sip_1 || !dip_1 || !sprf || !dprf)
			goto FILTER_END;

		if(!strlen(mac_address) && !strlen(sip_1) && !strlen(dip_1) && !strlen(sprf) && !strlen(dprf))
			goto FILTER_END;

		// we dont trust user input.....
		if(strlen(mac_address)){
			if(!isMacValid(mac_address))
				goto FILTER_END;
		}

		if(strlen(sip_1)){
			if(!isIpNetmaskValid(sip_1))
				goto FILTER_END;
		}else
			sip_1 = T("any/0");

		if(strlen(dip_1)){
			if(!isIpNetmaskValid(dip_1))
				goto FILTER_END;
		}else
	    		dip_1 = T("any/0");

		sip_2 = dip_2 = "0";

		if(! strcmp(protocol, T("TCP")))
			proto = PROTO_TCP;
		else if( !strcmp(protocol, T("UDP")))
			proto = PROTO_UDP;
		else if( !strcmp(protocol, T("None")))
			proto = PROTO_NONE;
		else if( !strcmp(protocol, T("ICMP")))
			proto = PROTO_ICMP;
		else
			goto FILTER_END;

		if(!strlen(sprf) || proto == PROTO_NONE || proto == PROTO_ICMP){
			sprf_int = 0;
		}else{
			sprf_int = atoi(sprf);
			if(sprf_int == 0 || sprf_int > 65535)
				goto FILTER_END;
		}

		if(!strlen(sprt) || proto == PROTO_NONE || proto == PROTO_ICMP){
			sprt_int = 0;
		}else{
			sprt_int = atoi(sprt);
			if(sprt_int ==0 || sprt_int > 65535)
				goto FILTER_END;
		}

		if (!sprf_int && sprt_int)
			sprf_int = 1;

		if(!strlen(dprf) || proto == PROTO_NONE || proto == PROTO_ICMP){
			dprf_int = 0;
		}else{
			dprf_int = atoi(dprf);
			if(dprf_int ==0 || dprf_int > 65535)
				goto FILTER_END;
		}

		if(!strlen(dprt) || proto == PROTO_NONE || proto == PROTO_ICMP){
			dprt_int = 0;
		}else{
			dprt_int = atoi(dprt);
			if(dprt_int ==0 || dprt_int > 65535)
				goto FILTER_END;
		}

		if (!dprf_int && dprt_int)
			dprf_int = 1;

		if(strlen(comment) > 32)
			goto FILTER_END;
		// i know you will try to break our box... ;)
		if(strchr(comment, ';') || strchr(comment, ','))
			goto FILTER_END;

		//snprintf(rule, sizeof(rule), "%s,%s,%d,%d,%s,%s,%d,%d,%d,%d,%s,%s", sip_1, sip_2, sprf_int, sprt_int, dip_1, dip_2, dprf_int, dprt_int, proto, action, comment, mac_address);
		ret_strlen = snprintf(rule + cur_strlen, buf_size, "%s,%s,%d,%d,%s,%s,%d,%d,%d,%d,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s;", sip_1, sip_2, sprf_int, sprt_int, dip_1, dip_2, dprf_int, dprt_int, proto, action, comment, mac_address,
		day_all,Mon_select,Tue_select,Wed_select,Thu_select,Fri_select,Sat_select,Sun_select,all_hours,start_time,end_time);
		cur_strlen += ret_strlen;
		buf_size -= ret_strlen;
	//	nvram_bufset(RT2860_NVRAM, "IPPortFilterRules", rule);
		flg_needcommit = 1;
	//	nvram_commit(RT2860_NVRAM);

	//	iptablesIPPortFilterFlush();
	//	iptablesIPPortFilterRun();

	}
	else
	{
        /* start for MAC/IP/Port filter settings by zym 2011-02-14 */
        mac_address = "";
        sprf = "0";
        sprt = "0";
        /* end for MAC/IP/Port filter settings by zym 2011-02-14 */

		const TfilterRules *pfilterRule = g_filter_rules;
		int service_idx = atoi(user_service);
		int loop_times = sizeof(g_filter_rules)/sizeof(g_filter_rules[0]);

		for (index = 0; (index < loop_times) && (service_idx != pfilterRule[index].idx_in_web_sel); index++)
			; // seek begin index.

		if (index >= loop_times) {
			printf("index not match!\n");
			goto FILTER_END;
		}

		sip_1 = T("any/0");
		dip_1 = T("any/0");
		sip_2 = dip_2 = "0";
		sprf_int = atoi(sprf);
		sprt_int = atoi(sprt);

		/* loop in g_filter_rules and set all preset rules */
		for (; (index < loop_times) && (service_idx == pfilterRule[index].idx_in_web_sel); index++) {
			dprf_int = pfilterRule[index].port_from;
			dprt_int = pfilterRule[index].port_to;
			comment = pfilterRule[index].comment;
			proto = pfilterRule[index].protocol;
			ret_strlen = snprintf(rule + cur_strlen, buf_size, "%s,%s,%d,%d,%s,%s,%d,%d,%d,%d,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s;",
					sip_1, sip_2, sprf_int, sprt_int, dip_1, dip_2, dprf_int, dprt_int, proto, action, comment, mac_address,
					day_all,Mon_select,Tue_select,Wed_select,Thu_select,Fri_select,Sat_select,Sun_select,all_hours,start_time,end_time);
			cur_strlen += ret_strlen;
			buf_size -= ret_strlen;

		}
		flg_needcommit = 1;
	}
	/*start of modify by huangyikun at 20110818*/
	if(!strcmp(IPPortFilterDefaultRules, T("0")))
		default_action = ACTION_ACCEPT;
	else if(!strcmp(IPPortFilterDefaultRules, T("1")))
		default_action = ACTION_DROP;
	else
		goto FILTER_END;
	
	ret_strlen = snprintf(rule + cur_strlen, buf_size, "%s,%s,%d,%d,%s,%s,%d,%d,%d,%d,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s;",
		"", "", 0, 0, "", "", 0, 0, 0, default_action, "", "", "", "", "","", "", "", "", "", "", "", "");
	cur_strlen += ret_strlen;
	buf_size -= ret_strlen;
	
	rule[cur_strlen-1] = '\0';
	printf("rules:%s\n", rule);
	/*end of modify by huangyikun at 20110818*/

FILTER_END:
	if (flg_needcommit) {
		nvram_set( "ip_filter_rules", rule);
		nvram_commit();
		//iptablesIPPortFilterFlush();
		//iptablesIPPortFilterRun();
		//doSystem("rc firewall");
		notification_send_rc( OID_NETWORK_FIREWALL, NOTIFY_WAIT );
	}

	websRedirect(wp, "port_filtering.asp");
    return;

}

#else
static void ipportFilter(webs_t wp, char_t *path, char_t *query)
{
	char rule[8192] = {0};
	char *mac_address;
	char *sip_1, *sip_2, *sprf, *sprt, *protocol, *action_str, *comment;
	char *dip_1, *dip_2, *dprf, *dprt;
	char *day_all, *Mon_select, *Tue_select, *Wed_select, *Thu_select, *Fri_select;
	char *Sat_select, *Sun_select;
	char *all_hours, *start_time, *end_time;
	const char *IPPortFilterRules;
	
	char *IPPortFilterDefaultRules;
	int default_action;

	char *user_service;

	int sprf_int, sprt_int, dprf_int, dprt_int, proto, action;
	int index;
	char flg_needcommit = 0;
	
	int cur_strlen = 0;
	int ret_strlen = 0;
	int buf_size = sizeof(rule) - 1;
	char *find;
	char IPPortFilterRulesTemp[8192] = {0};

	char *edit_index, *src, *dest, list[11][150]={0}, *comment_addr_changed, *comment_addr;
	int edit_index_int=-1, i, num;

	user_service = websGetVar(wp, T("access_list"), T(""));
	nvram_set( "portselectService", user_service);
	//        nvram_commit(RT2860_NVRAM);

	day_all = websGetVar(wp, T("day_type_all"), T(""));
	Mon_select = websGetVar(wp, T("Mon_select"), T(""));
	Tue_select = websGetVar(wp, T("Tue_select"), T(""));
	Wed_select = websGetVar(wp, T("Wed_select"), T(""));
	Thu_select = websGetVar(wp, T("Thu_select"), T(""));
	Fri_select = websGetVar(wp, T("Fri_select"), T(""));
	Sat_select = websGetVar(wp, T("Sat_select"), T(""));
	Sun_select = websGetVar(wp, T("Sun_select"), T(""));
	all_hours = websGetVar(wp, T("radiobutton"), T(""));
	start_time = websGetVar(wp, T("minute"), T(""));
	end_time = websGetVar(wp, T("hour"), T(""));

	mac_address = websGetVar(wp, T("mac_address"), T(""));

	sip_1 = websGetVar(wp, T("sip_address"), T("any"));
	sip_2 = websGetVar(wp, T("sip_address2"), T(""));
	sprf = websGetVar(wp, T("sFromPort"), T("0"));
	sprt = websGetVar(wp, T("sToPort"), T(""));

	dip_1 = websGetVar(wp, T("dip_address"), T("any"));
	dip_2 = websGetVar(wp, T("dip_address2"), T(""));
	dprf = websGetVar(wp, T("dFromPort"), T("0"));
	dprt = websGetVar(wp, T("dToPort"), T(""));

	protocol = websGetVar(wp, T("protocol"), T(""));
	action_str = websGetVar(wp, T("action"), T(""));
	comment = websGetVar(wp, T("comment"), T(""));

	IPPortFilterRules = nvram_safe_get( "ip_filter_rules");
	IPPortFilterDefaultRules = nvram_safe_get("ip_filter_default_policy");
/*
    printf("%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s",day_all,Mon_select,Tue_select,Wed_select,
		Thu_select,Fri_select,Sat_select,Sun_select,all_hours,start_time,end_time);
*/
	if(! (strcmp(action_str, T("Accept"))))
		action = ACTION_ACCEPT;
	else if(! (strcmp(action_str, T("Drop"))))
		action = ACTION_DROP;
	else
		goto FILTER_END;
	
	if(!strcmp(IPPortFilterDefaultRules, T("0")))
		default_action = ACTION_ACCEPT;
	else if(!strcmp(IPPortFilterDefaultRules, T("1")))
		default_action = ACTION_DROP;
	else
		goto FILTER_END;

	edit_index = websGetVar(wp, T("edit_index"), T(""));
	edit_index_int = atoi(edit_index); 
	if(edit_index_int == -1)// add
	{
		if (IPPortFilterRules && strlen(IPPortFilterRules)) 
		{
			strcpy(IPPortFilterRulesTemp,IPPortFilterRules);
			find=strstr(IPPortFilterRulesTemp,";,,0,0,,,0,0,0,");
			if(find)
			{
				*find = '\0';
			}
			
			ret_strlen = snprintf(rule, buf_size, "%s;", IPPortFilterRulesTemp);
			//strncpy(rule, IPPortFilterRules, ret_strlen);
			buf_size -= ret_strlen;
			cur_strlen += ret_strlen;
		}

		if(strcmp(user_service, T("0")) == 0)
		{
			if(!mac_address || !sip_1 || !dip_1 || !sprf || !dprf)
				goto FILTER_END;

			if(!strlen(mac_address) && !strlen(sip_1) && !strlen(dip_1) && !strlen(sprf) && !strlen(dprf))
				goto FILTER_END;

			// we dont trust user input.....
			if(strlen(mac_address)){
				if(!isMacValid(mac_address))
					goto FILTER_END;
			}

			if(strlen(sip_1)){
				if(!isIpNetmaskValid(sip_1))
					goto FILTER_END;
			}else
				sip_1 = T("any/0");

			if(strlen(dip_1)){
				if(!isIpNetmaskValid(dip_1))
					goto FILTER_END;
			}else
		    		dip_1 = T("any/0");

			sip_2 = dip_2 = "0";

			if(! strcmp(protocol, T("TCP")))
				proto = PROTO_TCP;
			else if( !strcmp(protocol, T("UDP")))
				proto = PROTO_UDP;
			else if( !strcmp(protocol, T("None")))
				proto = PROTO_NONE;
			else if( !strcmp(protocol, T("ICMP")))
				proto = PROTO_ICMP;
			else
				goto FILTER_END;

			if(!strlen(sprf) || proto == PROTO_NONE || proto == PROTO_ICMP){
				sprf_int = 0;
			}else{
				sprf_int = atoi(sprf);
				if(sprf_int == 0 || sprf_int > 65535)
					goto FILTER_END;
			}

			if(!strlen(sprt) || proto == PROTO_NONE || proto == PROTO_ICMP){
				sprt_int = 0;
			}else{
				sprt_int = atoi(sprt);
				if(sprt_int ==0 || sprt_int > 65535)
					goto FILTER_END;
			}

			if (!sprf_int && sprt_int)
				sprf_int = 1;

			if(!strlen(dprf) || proto == PROTO_NONE || proto == PROTO_ICMP){
				dprf_int = 0;
			}else{
				dprf_int = atoi(dprf);
				if(dprf_int ==0 || dprf_int > 65535)
					goto FILTER_END;
			}

			if(!strlen(dprt) || proto == PROTO_NONE || proto == PROTO_ICMP){
				dprt_int = 0;
			}else{
				dprt_int = atoi(dprt);
				if(dprt_int ==0 || dprt_int > 65535)
					goto FILTER_END;
			}

			if (!dprf_int && dprt_int)
				dprf_int = 1;

			if(strlen(comment) > 32)
				goto FILTER_END;
			// i know you will try to break our box... ;)
			if(strchr(comment, ';') || strchr(comment, ','))
				goto FILTER_END;

			//snprintf(rule, sizeof(rule), "%s,%s,%d,%d,%s,%s,%d,%d,%d,%d,%s,%s", sip_1, sip_2, sprf_int, sprt_int, dip_1, dip_2, dprf_int, dprt_int, proto, action, comment, mac_address);
			ret_strlen = snprintf(rule + cur_strlen, buf_size, "%s,%s,%d,%d,%s,%s,%d,%d,%d,%d,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s;", sip_1, sip_2, sprf_int, sprt_int, dip_1, dip_2, dprf_int, dprt_int, proto, action, comment, mac_address,
			day_all,Mon_select,Tue_select,Wed_select,Thu_select,Fri_select,Sat_select,Sun_select,all_hours,start_time,end_time);
			cur_strlen += ret_strlen;
			buf_size -= ret_strlen;
		//	nvram_bufset(RT2860_NVRAM, "IPPortFilterRules", rule);
			flg_needcommit = 1;
		//	nvram_commit(RT2860_NVRAM);

		//	iptablesIPPortFilterFlush();
		//	iptablesIPPortFilterRun();

		}
		else
		{
	        /* start for MAC/IP/Port filter settings by zym 2011-02-14 */
	        mac_address = "";
	        sprf = "0";
	        sprt = "0";
	        /* end for MAC/IP/Port filter settings by zym 2011-02-14 */

			const TfilterRules *pfilterRule = g_filter_rules;
			int service_idx = atoi(user_service);
			int loop_times = sizeof(g_filter_rules)/sizeof(g_filter_rules[0]);

			for (index = 0; (index < loop_times) && (service_idx != pfilterRule[index].idx_in_web_sel); index++)
				; // seek begin index.

			if (index >= loop_times) {
				printf("index not match!\n");
				goto FILTER_END;
			}

			sip_1 = T("any/0");
			dip_1 = T("any/0");
			sip_2 = dip_2 = "0";
			sprf_int = atoi(sprf);
			sprt_int = atoi(sprt);

			/* loop in g_filter_rules and set all preset rules */
			for (; (index < loop_times) && (service_idx == pfilterRule[index].idx_in_web_sel); index++) {
				dprf_int = pfilterRule[index].port_from;
				dprt_int = pfilterRule[index].port_to;
				comment = pfilterRule[index].comment;
				proto = pfilterRule[index].protocol;
				ret_strlen = snprintf(rule + cur_strlen, buf_size, "%s,%s,%d,%d,%s,%s,%d,%d,%d,%d,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s;",
						sip_1, sip_2, sprf_int, sprt_int, dip_1, dip_2, dprf_int, dprt_int, proto, action, comment, mac_address,
						day_all,Mon_select,Tue_select,Wed_select,Thu_select,Fri_select,Sat_select,Sun_select,all_hours,start_time,end_time);
				cur_strlen += ret_strlen;
				buf_size -= ret_strlen;

			}
			flg_needcommit = 1;
		}
	//	ret_strlen = snprintf(rule + cur_strlen, buf_size, "%s,%s,%d,%d,%s,%s,%d,%d,%d,%d,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s;",
	//		"", "", 0, 0, "", "", 0, 0, 0, default_action, "", "", "", "", "","", "", "", "", "", "", "", "");
	//	cur_strlen += ret_strlen;
	//	buf_size -= ret_strlen;
		
		rule[cur_strlen-1] = '\0';
		printf("rules:%s\n", rule);
	}	
	else //modify
	{
			if(!strlen(mac_address) && !strlen(sip_1) && !strlen(dip_1) && !strlen(sprf) && !strlen(dprf))
				goto FILTER_END;

			// we dont trust user input.....
			if(strlen(mac_address)){
				if(!isMacValid(mac_address))
					goto FILTER_END;
			}

			if(strlen(sip_1)){
				if(!isIpNetmaskValid(sip_1))
					goto FILTER_END;
			}else
				sip_1 = T("any/0");

			if(strlen(dip_1)){
				if(!isIpNetmaskValid(dip_1))
					goto FILTER_END;
			}else
		    		dip_1 = T("any/0");

			sip_2 = dip_2 = "0";

			if(! strcmp(protocol, T("TCP")))
				proto = PROTO_TCP;
			else if( !strcmp(protocol, T("UDP")))
				proto = PROTO_UDP;
			else if( !strcmp(protocol, T("None")))
				proto = PROTO_NONE;
			else if( !strcmp(protocol, T("ICMP")))
				proto = PROTO_ICMP;
			else
				goto FILTER_END;

			if(!strlen(sprf) || proto == PROTO_NONE || proto == PROTO_ICMP){
				sprf_int = 0;
			}else{
				sprf_int = atoi(sprf);
				if(sprf_int == 0 || sprf_int > 65535)
					goto FILTER_END;
			}

			if(!strlen(sprt) || proto == PROTO_NONE || proto == PROTO_ICMP){
				sprt_int = 0;
			}else{
				sprt_int = atoi(sprt);
				if(sprt_int ==0 || sprt_int > 65535)
					goto FILTER_END;
			}

			if (!sprf_int && sprt_int)
				sprf_int = 1;

			if(!strlen(dprf) || proto == PROTO_NONE || proto == PROTO_ICMP){
				dprf_int = 0;
			}else{
				dprf_int = atoi(dprf);
				if(dprf_int ==0 || dprf_int > 65535)
					goto FILTER_END;
			}

			if(!strlen(dprt) || proto == PROTO_NONE || proto == PROTO_ICMP){
				dprt_int = 0;
			}else{
				dprt_int = atoi(dprt);
				if(dprt_int ==0 || dprt_int > 65535)
					goto FILTER_END;
			}

			if (!dprf_int && dprt_int)
				dprf_int = 1;

			if(strlen(comment) > 32)
				goto FILTER_END;
			// i know you will try to break our box... ;)
			if(strchr(comment, ';') || strchr(comment, ','))
				goto FILTER_END;
			
			src = IPPortFilterRules;
			for(i=0; *src!='\0'; i++)
			{
				dest = list[i];
				while(*src!=';' && *src!='\0'){
					*dest++ = *src++;
				}
				*dest = *src;
				if(*src == ';')
					src++;
			}
			num = i;
			
			memset(list[edit_index_int],'0',sizeof(list[edit_index_int]));
			snprintf(list[edit_index_int], sizeof(list[edit_index_int]), "%s,%s,%d,%d,%s,%s,%d,%d,%d,%d,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s;", sip_1, sip_2, sprf_int, sprt_int, dip_1, dip_2, dprf_int, dprt_int, proto, action, comment, mac_address,
			day_all,Mon_select,Tue_select,Wed_select,Thu_select,Fri_select,Sat_select,Sun_select,all_hours,start_time,end_time);

			if(strlen(comment) && strstr("HTTP,NetMeeting",comment))
			{
				comment_addr_changed = strstr(list[edit_index_int],comment);
				for(i=0; i<num-1; i++)
				{
					if(i==edit_index_int)
						continue;
					if(comment_addr = strstr(list[i],comment))
					{
						*comment_addr = '\0';
						strcat(list[i],comment_addr_changed);
						comment_addr -= 2;
						if(action == 0)
							*comment_addr = '0';
						else
							*comment_addr = '1';
					}
				}
			}
				
			strcpy(rule,list[0]);
			for(i=1; i<num; i++)
			{
				strcat(rule,list[i]);
			}

			flg_needcommit = 1;
/*start of delete the last semicolon minpengwei 20120531*/
			cur_strlen = strlen(rule);
			rule[cur_strlen-1] = '\0';
			printf("rules:%s\n", rule);
/*end of delete the last semicolon minpengwei 20120531*/
	}

FILTER_END:
	if (flg_needcommit) {
		nvram_set( "ip_filter_rules", rule);
		nvram_commit();
		//iptablesIPPortFilterFlush();
		//iptablesIPPortFilterRun();
		//doSystem("rc firewall");
		notification_send_rc( OID_NETWORK_FIREWALL, NOTIFY_WAIT );
	}

	websRedirect(wp, "port_filtering.asp");
    return;

}

#endif

#ifdef HAVE_WEB_CNS 
static void portTrigger(webs_t wp, char_t *path, char_t *query)
{
    char rulesbuf[8192] = {0};
    char *Trigger_num = (char *)nvram_safe_get("port_trigger_num"),temp[4]={0};
	int i;
	char *radiosrv;
	char *srvname;
	char *cusAppName;
	char *tStart, *tEnd, *tProto, *oStart, *oEnd, *oProto;
	const char *portTriggerRules;
	char *pAppName = NULL;
	char *ptStart, *ptEnd, *ptProto, *poStart, *poEnd, *poProto;
	char *p_sub_num;
	int itStart, itEnd, itProto, ioStart, ioEnd, ioProto;
	int len = 0,Trigger_num_int;
	int size = 0;
	int nfree_size = 0;

	radiosrv = websGetVar(wp, T("radiosrv"), T("")); /* meaning: "preset App" or "custom rules" */
	srvname = websGetVar(wp, T("srvname"), T(""));
	cusAppName = websGetVar(wp, T("cusAppName"), T(""));
	tStart = websGetVar(wp, T("tStart"), T(""));
	tEnd = websGetVar(wp, T("tEnd"), T(""));
	tProto = websGetVar(wp, T("tProto"), T(""));
	oStart = websGetVar(wp, T("oStart"), T(""));
	oEnd = websGetVar(wp, T("oEnd"), T(""));
	oProto = websGetVar(wp, T("oProto"), T(""));
	p_sub_num = websGetVar(wp, T("subnum"), T(""));

	Trigger_num_int = atoi(Trigger_num);

	if (radiosrv[0] == '1') {	/* preset App */
		pAppName = srvname;
		printf("server name:%s\n", srvname);
	}else if (radiosrv[0] == '2') {
		pAppName = cusAppName;
		printf("customer app:%s\n", cusAppName);
	}else
		goto end;

	ptStart = tStart;
	ptEnd = tEnd;
	ptProto = tProto;
	poStart = oStart;
	poEnd = oEnd;
	poProto = oProto;
	size = snprintf(rulesbuf, sizeof(rulesbuf), "%s|1|", pAppName);
	for (i = 0; i < atoi(p_sub_num); i++) {
		itStart = atoi(ptStart);
		itEnd = atoi(ptEnd);
		itProto = atoi(ptProto);
		ioStart = atoi(poStart);
		ioEnd = atoi(poEnd);
		ioProto = atoi(poProto);

        switch(ioProto) //zengqingchu
        {
            case PROTO_TCP: Trigger_num_int += 1; break;
		   	case PROTO_UDP: Trigger_num_int += 1; break;
			case PROTO_TCP_UDP: Trigger_num_int += 2; break;
		}

		/* need check start & end port? and protocol again? */
		if (itStart && itEnd && ioStart && ioEnd){
			size += snprintf(rulesbuf+size, sizeof(rulesbuf) - size, "%d,%d,%d,%d,%d,%d|", itStart, itEnd, itProto,
				ioStart, ioEnd, ioProto);
		}else
			break;
		ptStart = strchr(ptStart, ' ');
		if (ptStart) ptStart++;
		ptEnd = strchr(ptEnd, ' ');
		if (ptEnd) ptEnd++;
		poStart = strchr(poStart, ' ');
		if (poStart) poStart++;
		poEnd = strchr(poEnd, ' ');
		if (poEnd) poEnd++;
		/* protocol ֻһλͨʽһֵ*/
		ptProto += 2;
		poProto += 2;
	}

	if (i > 0){
		printf("rules1:%s\n", rulesbuf);
		portTriggerRules = nvram_safe_get( "port_trigger_rules");
		printf("old rules:%s\n", portTriggerRules);
		len = strlen(portTriggerRules);
		nfree_size = sizeof(rulesbuf) - size - 1; /* including ";" */
		printf("size:%d, freesize:%d\n", size, nfree_size);
		if ( len && (len < nfree_size) ) {
			rulesbuf[size++] = ';';
			strncpy(rulesbuf+size, portTriggerRules, len);
		}
	}

	sprintf(temp,"%d",Trigger_num_int);
	nvram_set( "port_trigger_num", temp);

	printf("port_trigger_rules: %s\n", rulesbuf);
	nvram_set( "port_trigger_rules", rulesbuf);
	nvram_commit();

	/* update rule */
    notification_send_rc( OID_NETWORK_FIREWALL, NOTIFY_WAIT );

end:
	websRedirect(wp, "port_trigger.asp");
}

#else
static void portTrigger(webs_t wp, char_t *path, char_t *query)
{
    char rulesbuf[8192] = {0};
    char *Trigger_num = (char *)nvram_safe_get("port_trigger_num"),temp[4]={0};
	int i;
	char *radiosrv;
	char *srvname;
	char *cusAppName;
	char *tStart, *tEnd, *tProto, *oStart, *oEnd, *oProto;
	const char *portTriggerRules;
	char *pAppName = NULL;
	char *ptStart, *ptEnd, *ptProto, *poStart, *poEnd, *poProto;
	char *p_sub_num;
	int itStart, itEnd, itProto, ioStart, ioEnd, ioProto;
	int len = 0,Trigger_num_int;
	int size = 0;
	int nfree_size = 0;
	
	char *edit_index, *src, *dest, list[10][500]={0}, *protocol_num; 
	int index=-1, num=0, protocol_num_old, protocol_num_new=0; 

	radiosrv = websGetVar(wp, T("radiosrv"), T("")); /* meaning: "preset App" or "custom rules" */
	srvname = websGetVar(wp, T("srvname"), T(""));
	cusAppName = websGetVar(wp, T("cusAppName"), T(""));
	tStart = websGetVar(wp, T("tStart"), T(""));
	tEnd = websGetVar(wp, T("tEnd"), T(""));
	tProto = websGetVar(wp, T("tProto"), T(""));
	oStart = websGetVar(wp, T("oStart"), T(""));
	oEnd = websGetVar(wp, T("oEnd"), T(""));
	oProto = websGetVar(wp, T("oProto"), T(""));
	p_sub_num = websGetVar(wp, T("subnum"), T(""));

	Trigger_num_int = atoi(Trigger_num);


	if (radiosrv[0] == '1') {	/* preset App */
		pAppName = srvname;
		printf("server name:%s\n", srvname);
	}else if (radiosrv[0] == '2') {
		pAppName = cusAppName;
		printf("customer app:%s\n", cusAppName);
	}else
		goto end;

	ptStart = tStart;
	ptEnd = tEnd;
	ptProto = tProto;
	poStart = oStart;
	poEnd = oEnd;
	poProto = oProto;
	size = snprintf(rulesbuf, sizeof(rulesbuf), "%s|1|", pAppName);
	for (i = 0; i < atoi(p_sub_num); i++) {
		itStart = atoi(ptStart);
		itEnd = atoi(ptEnd);
		itProto = atoi(ptProto);
		ioStart = atoi(poStart);
		ioEnd = atoi(poEnd);
		ioProto = atoi(poProto);


		/* need check start & end port? and protocol again? */
		if (itStart && itEnd && ioStart && ioEnd){
			size += snprintf(rulesbuf+size, sizeof(rulesbuf) - size, "%d,%d,%d,%d,%d,%d|", itStart, itEnd, itProto,
				ioStart, ioEnd, ioProto);
		}else
			break;
		ptStart = strchr(ptStart, ' ');
		if (ptStart) ptStart++;
		ptEnd = strchr(ptEnd, ' ');
		if (ptEnd) ptEnd++;
		poStart = strchr(poStart, ' ');
		if (poStart) poStart++;
		poEnd = strchr(poEnd, ' ');
		if (poEnd) poEnd++;
		/* protocol ֻһλͨʽһֵ*/
		ptProto += 2;
		poProto += 2;
	}
	printf("rules1:%s\n", rulesbuf);
	portTriggerRules = nvram_safe_get( "port_trigger_rules");
	printf("old rules:%s\n", portTriggerRules);
	len = strlen(portTriggerRules);

	edit_index = websGetVar(wp, T("edit_index"), T(""));
	index = atoi(edit_index); 
	poProto = oProto;
	if(index == -1) //add 
	{
		for (i = 0; i < atoi(p_sub_num); i++) {
			ioProto = atoi(poProto);
	        switch(ioProto) //zengqingchu
	        {
	            case PROTO_TCP: Trigger_num_int += 1; break;
			   	case PROTO_UDP: Trigger_num_int += 1; break;
				case PROTO_TCP_UDP: Trigger_num_int += 2; break;
			}
			poProto += 2;
		}

		if (i > 0){
			nfree_size = sizeof(rulesbuf) - size - 1; /* including ";" */
			printf("size:%d, freesize:%d\n", size, nfree_size);
			if ( len && (len < nfree_size) ) {
				rulesbuf[size++] = ';';
				strncpy(rulesbuf+size, portTriggerRules, len);
			}
		}
	}
	else //modify
	{
		protocol_num = websGetVar(wp, T("protocol_num"), T(""));
		protocol_num_old = atoi(protocol_num); 
		for (i = 0; i < atoi(p_sub_num); i++) {
			ioProto = atoi(poProto);
			switch(ioProto) //zengqingchu
			{
				case PROTO_TCP: protocol_num_new += 1; break;
				case PROTO_UDP: protocol_num_new += 1; break;
				case PROTO_TCP_UDP: protocol_num_new += 2; break;
			}
			poProto += 2;
		}
		Trigger_num_int = Trigger_num_int + (protocol_num_new-protocol_num_old);
		
		src = portTriggerRules;
		for(i=0; *src!='\0'; i++)
		{
			dest = list[i];
			while(*src!=';' && *src!='\0'){
				*dest++ = *src++;
			}
			*dest = *src;
			if(*src == ';')
				src++;
		}
		num = i;
		
		memset(list[index],'0',sizeof(list[index]));
		strcpy(list[index],rulesbuf);
		if(index < num-1)
			strcat(list[index], ";");
		
		memset(rulesbuf,'0',sizeof(rulesbuf));
		strcpy(rulesbuf,list[0]);
		for(i=1; i<num; i++)
		{
			strcat(rulesbuf,list[i]);
		}	
	}

	sprintf(temp,"%d",Trigger_num_int);
	nvram_set( "port_trigger_num", temp);

	printf("port_trigger_rules: %s\n", rulesbuf);
	nvram_set( "port_trigger_rules", rulesbuf);
	nvram_commit();

	/* update rule */
    notification_send_rc( OID_NETWORK_FIREWALL, NOTIFY_WAIT );

end:
	websRedirect(wp, "port_trigger.asp");
}
#endif
static void portForward(webs_t wp, char_t *path, char_t *query)
{
	char rule[4096],macAddr[32]={0},num[4] = {0};
	char *relus_num = (char *)nvram_safe_get("port_forward_num");
	char *ip_address, *pfe, *prf, *prt,*Client_prf, *Client_prt,*protocol, *comment;
	char *PortForwardRules;

	int relus_num_int,prf_int, prt_int,Client_prf_int,Client_prt_int,proto,ret,macFlag;

	char *edit_index, *src, *dest, list[10][100]={0}, *protocol_save;
	int index, i, totalnum;
	
	macFlag = 0;

        /*start of Add portforward save button apply by zengxianzhu at 20110225*/
	//pfe = websGetVar(wp, T("portForwardEnabled"), T(""));
        pfe = (char *)nvram_safe_get( "port_forward_en");
        /*end of Add portforward save button apply by zengxianzhu at 20110225*/
	ip_address = websGetVar(wp, T("ip_address"), T(""));
	prf = websGetVar(wp, T("fromPort"), T(""));
	prt = websGetVar(wp, T("toPort"), T(""));
	protocol = websGetVar(wp, T("protocol"), T(""));
	comment = websGetVar(wp, T("comment"), T(""));

//    Client_ip_address = websGetVar(wp, T("Client_ip_address"), T(""));
	Client_prf = websGetVar(wp, T("Client_fromPort"), T(""));
	Client_prt = websGetVar(wp, T("Client_toPort"), T(""));

	relus_num_int = atoi(relus_num);


	if(!pfe && !strlen(pfe))
	    return;

//   printf("enter portForward!\n");

	if(!atoi(pfe)){
		nvram_set( "port_forward_en", "0");
		goto end;
	}

	if(!strlen(ip_address) && !strlen(prf) && !strlen(prt) && !strlen(comment)){	// user choose nothing but press "apply" only
		nvram_set( "port_forward_en", "1");
		goto end;
	}

	if(!ip_address && !strlen(ip_address))
	{
	    return;
	}
	if(!isIpValid(ip_address))
		return;

	// we dont trust user input.....
	if((!prf && !strlen(prf))||(!Client_prf && !strlen(Client_prf)))
		return;
	if((!(prf_int = atoi(prf)))||(!(Client_prf_int = atoi(Client_prf))) )
		return;
	if(prf_int > 65535||Client_prf_int > 65535)
		return;

	if(!prt || !Client_prt)
		return;
	if(strlen(prt)){
		if( !(prt_int = atoi(prt)) )
			return;
		if(prt_int < prf_int)
			return;
		if(prt_int > 65535)
			return;
	}else{
		prt_int = 0;
	}
	if(strlen(Client_prt)){
		if( !(Client_prt_int = atoi(Client_prt)) )
			return;
		if(Client_prt_int < Client_prf_int)
			return;
		if(Client_prt_int > 65535)
			return;
	}else{
		Client_prt_int = 0;
	}

	if(strlen(comment) > 32)
		return;
	/* i know you will try to break our box... ;) */
	if(strchr(comment, ';') || strchr(comment, ','))
		return;
	
	if(! strcmp(protocol, "TCP"))
	{
		proto = PROTO_TCP;
	}
	else if( !strcmp(protocol, "UDP"))
	{
		proto = PROTO_UDP;
	}
	else if( !strcmp(protocol, "TCP&UDP"))
	{
		proto = PROTO_TCP_UDP;
	}
	else
		return;

	/*Line added by srini to get MacAddress  
    If MacAddress found in arp Cache,It will be added with to portForward rules in nvram 
    */
    #if 1	
    ret = getMac_arpCache(ip_address,macAddr,&macFlag);
    #endif

	PortForwardRules = (char *)nvram_safe_get( "port_forward_rules");	
#ifdef HAVE_WEB_CNS
	if(! strcmp(protocol, "TCP"))
	{
		relus_num_int += 1;
	}
	else if( !strcmp(protocol, "UDP"))
	{
		relus_num_int += 1;
	}
	else if( !strcmp(protocol, "TCP&UDP"))
	{
        relus_num_int += 2;
	}
	else
		return;

	if( PortForwardRules && strlen( PortForwardRules) )
		snprintf(rule, sizeof(rule), "%s;%s,%d,%d,%d,%d,%d,%s",  PortForwardRules, ip_address, prf_int, prt_int, Client_prf_int, Client_prt_int, proto, comment);
	else
		snprintf(rule, sizeof(rule), "%s,%d,%d,%d,%d,%d,%s", ip_address, prf_int, prt_int, Client_prf_int, Client_prt_int, proto, comment);
#else
	edit_index = websGetVar(wp, T("edit_index"), T("")); 
	index = atoi(edit_index);
	if(index == -1) //add
	{
		if(! strcmp(protocol, "TCP"))
		{
			relus_num_int += 1;
		}
		else if( !strcmp(protocol, "UDP"))
		{
			relus_num_int += 1;
		}
		else if( !strcmp(protocol, "TCP&UDP"))
		{
	        relus_num_int += 2;
		}
		else
			return;
		
		if( PortForwardRules && strlen( PortForwardRules) )
			snprintf(rule, sizeof(rule), "%s;%s,%d,%d,%d,%d,%d,%s",  PortForwardRules, ip_address, prf_int, prt_int, Client_prf_int, Client_prt_int, proto, comment);
		else
			snprintf(rule, sizeof(rule), "%s,%d,%d,%d,%d,%d,%s", ip_address, prf_int, prt_int, Client_prf_int, Client_prt_int, proto, comment);
	}
	else //modify
	{
		protocol_save = websGetVar(wp, T("protocol_save"), T(""));
		if(strlen(protocol) == strlen(protocol_save))
			;//do nothing
		else if(strlen(protocol) > strlen(protocol_save))
			relus_num_int += 1;
		else if(strlen(protocol) < strlen(protocol_save))
			relus_num_int -= 1;
		
		src = PortForwardRules;
		for(i=0; *src!='\0'; i++)
		{
			dest = list[i];
			while(*src!=';' && *src!='\0'){
				*dest++ = *src++;
			}
			*dest = *src;
			if(*src == ';')
				src++;
		}
		totalnum = i;
		
		memset(list[index],'0',sizeof(list[index]));
		snprintf(list[index], sizeof(list[index]), "%s,%d,%d,%d,%d,%d,%s", ip_address, prf_int, prt_int, Client_prf_int, Client_prt_int, proto, comment);
		if(index < totalnum-1)
			strcat(list[index], ";");
		
		strcpy(rule,list[0]);
		for(i=1; i<totalnum; i++)
		{
			strcat(rule,list[i]);
		}
	}
#endif

	sprintf(num,"%d",relus_num_int);
	nvram_set("port_forward_num",num);

	nvram_set( "port_forward_en", "1");

	nvram_set( "port_forward_rules", rule);
	nvram_commit();
	/*Adding Static Entry to arp cache*/
    if(macFlag) {
    	doSystem("arp -s %s  %s ",ip_address,macAddr);
    }

end:
    notification_send_rc( OID_NETWORK_FIREWALL, NOTIFY_WAIT );

#ifdef	GOAHEAD_SHOWSUBMITINFO
	websHeader(wp);
	websWrite(wp, T("˿ת: %s<br>\n"), pfe);
	websWrite(wp, T("IPַ: %s<br>\n"), ip_address);
	websWrite(wp, T("Դ˿: %s<br>\n"), prf);
	websWrite(wp, T("ĿĶ˿: %s<br>\n"), prt);
	websWrite(wp, T("Э: %s<br>\n"), protocol);
	websWrite(wp, T("ע: %s<br>\n"), comment);

    websFooter(wp);
    websDone(wp, 200);
#else
	websRedirect(wp, "port_forward.asp");
#endif
}

/******Function to get MacAddress from Arp Cache for Chosen Ipaddress*****************/
   
static int getMac_arpCache(char *ipAddr,char *macAddr,int *macFlag)
{
   FILE *fp;
   char *p1,*p2,buf[128]={0},ip[16]={0};
   int i = 0;

   fp=fopen("/proc/net/arp","r");
   if(fp == NULL )
   {
       printf("Unable to open /proc/net/arp file\n");
       return -1;
   }
   if(NULL == fgets(buf,128,fp))
   {
       printf("/proc/net/arp cache may be Empty\n");
       fclose(fp);
       return -1;  	
   }
   while(fgets(buf,128,fp))
   {
	   p1 = strchr(buf,' ');  //ipAddr;
	   if(NULL == p1)
	   {   
	       fclose(fp);
	       printf("Get ip error!buf = %s\n",p1);
               return -1;
	   }
           strncpy(ip,buf,p1-buf);
           /*start of ipaddr strcmp brace*/
           if(!strcmp(ipAddr,ip))
           { 
	     	p1=strchr(buf,':');  //mac;
	   	if(NULL == p1)
	   	{
	      	  fclose(fp);
		  error(E_L, E_LOG, T("Get Mac -1 Error"));
	          return -1; 
	   	}
	   	p1 -= 2;
	   	p2 = strchr(p1,' ');
	   	
	   	if(NULL == p2)
		{
	    	fclose(fp);
		error(E_L, E_LOG, T("Get Mac-2 Error"));
	      	return -1; 
	   	}
	   	strncpy(macAddr,p1,p2-p1);
	   	*macFlag = 1;
           }//end of ipaddr,strcmp brace 

	   memset(buf,0,128);
	   memset(ip,0,16);
	   i++;
	   
   } /*end of while*/ 
   fclose(fp);
   if(*macFlag)
	return 0;
   return -1;
}
/* End of getMac_arpCache Function */

/*
 * This Function added by Srini to get values stored in portForwardRules of NVRAM.
 * If any values stored, those values will be added to arp table static entries.
 */
int getInitial_PortForwardValues() 
{
    int i,rule_count,ipFlag,macFlag;
    char_t  rec[256],ip_address[32],test_mac[32];

    if(!nvram_safe_get("port_forward_rules") || !strlen(nvram_safe_get("port_forward_rules")))
    {
      return -1;
    }

    rule_count = getRuleNums((char *)nvram_safe_get("port_forward_rules"));

    if(!rule_count) 
    {
	return -1;
    }
    for(i=0;i< rule_count; i++)
    {
        ipFlag = macFlag = 0;
 
	if(getNthValueSafe(i,nvram_safe_get("port_forward_rules"),';',rec,sizeof(rec))!= -1)
	{
	    //Get ipAddress
	    if((getNthValueSafe(0, rec, ',', ip_address, sizeof(ip_address)) == -1))
	        continue;
	    else	
		ipFlag = 1;
		
		
	    //Get Mac Address
	    if((getNthValueSafe(7, rec, ',', test_mac, sizeof(test_mac)) == -1))
	        continue;
	    else
		macFlag = 1;	

	   if(ipFlag & macFlag)
      	   doSystem("arp -s %s  %s ",ip_address,test_mac);
	}
   }
   return 0;
}
/*End of getPortForwardValues() Function*/

/*start of Add portforward save button apply by zengxianzhu at 20110225*/
static void portForwardEnable(webs_t wp, char_t *path, char_t *query)
{

	char *pfe;
	pfe = websGetVar(wp, T("portForwardEnabled"), T(""));
    if( atoi( pfe ) == 0 )
        nvram_set( "port_forward_en", "0");
    else
        nvram_set( "port_forward_en", "1");

	nvram_commit();

	//doSystem("rc firewall");
	notification_send_rc( OID_NETWORK_FIREWALL, NOTIFY_WAIT );

	websRedirect(wp, "port_forward.asp");

}
/*end of Add portforward save button apply by zengxianzhu at 20110225*/

static void BasicSettings(webs_t wp, char_t *path, char_t *query)
{
	char *default_policy, *firewall_enable;

	firewall_enable = websGetVar(wp, T("portFilterEnabled"), T(""));
	default_policy = websGetVar(wp, T("defaultFirewallPolicy"), T("0"));

	switch(atoi(firewall_enable)){
	case 0:
		nvram_set( "ip_filter_enable", "0");
		break;
	case 1:
		nvram_set( "ip_filter_enable", "1");
		break;
	}

	switch(atoi(default_policy)){
	case 1:
		nvram_set( "ip_filter_default_policy", "1");
		break;
	case 0:
	default:
		nvram_set( "ip_filter_default_policy", "0");
		break;
	}
	nvram_commit();

	notification_send_rc( OID_NETWORK_FIREWALL, NOTIFY_WAIT );

#ifdef	GOAHEAD_SHOWSUBMITINFO
	websHeader(wp);
	websWrite(wp, T("default_policy: %s<br>\n"), default_policy);
    websFooter(wp);
    websDone(wp, 200);
#else
	websRedirect(wp, "port_filtering.asp");
#endif
}

static void DMZ(webs_t wp, char_t *path, char_t *query)
{
    char *dmzE, *mac_address, *ip_address, *avoidtcp80;
   
    dmzE = websGetVar(wp, T("DMZEnabled"), T(""));
    ip_address = websGetVar(wp, T("DMZIpAddress"), T(""));
    mac_address = websGetVar(wp, T("DMZMacAddress"), T(""));

    /*start of judge whether super dmz is set before,if set need restart dhcpd by zengxianzhu at 20110810*/
    int i_Old_dmz_en = atoi( nvram_safe_get( "dmz_enable" ) );

    if(i_Old_dmz_en ==2)
    {
           nvram_set("restart_dhcpd_flag","1");
    }
    else
    {
           nvram_set("restart_dhcpd_flag","0");
    }
    /*end of judge whether super dmz is set before,if set need restart dhcpd by zengxianzhu at 20110810*/

    if(!dmzE && !strlen(dmzE))
        return;
    
    // we dont trust user input, check all things before doing changes
    if((atoi(dmzE)==1) && !isIpValid(ip_address))   // enable && invalid mac address
        return;
    
    if((atoi(dmzE)==2) && !isMacValid(mac_address)) // enable && invalid mac address
        return;
    
    if(atoi(dmzE)==1)
    {
        nvram_set( "dmz_enable", "1");
        if(strlen(ip_address))
            nvram_set( "dmz_host_addr", ip_address);
    }
    else if (atoi(dmzE)==2)
    {
        nvram_set( "dmz_enable", "2");
        if(strlen(mac_address))
        {
            nvram_set( "dmz_host_addr", mac_address);
        }
    }
    else
    {
        nvram_set( "dmz_enable", "0");
    }
    
    nvram_commit();

    notification_send_rc( OID_NETWORK_FIREWALL, NOTIFY_WAIT );
    
    websRedirect(wp, "port_dmz.asp");
}


//get MAC  and LAN IP

#define DEFAULT_LAN_IP "192.168.1.1"
char *getarpLanIP(void)
{
	static char buf[64];
	char *nl;
	FILE *fp;

	memset(buf, 0, sizeof(buf));
	if( (fp = popen("nvram get lan_ipaddr", "r")) == NULL )
		goto error;

	if(!fgets(buf, sizeof(buf), fp)){
		pclose(fp);
		goto error;
	}

	if(!strlen(buf)){
		pclose(fp);
		goto error;
	}
	pclose(fp);

	nl = strchr(buf, '\n');
	if( nl != NULL )
		*nl = '\0';

	return buf;

error:
	fprintf(stderr, "warning, cant find lan ip\n");
	return DEFAULT_LAN_IP;
}


#ifdef HAVE_WEB_FILTER
static void websURLFilterDelete(webs_t wp, char_t *path, char_t *query)
{
	int i, j, rule_count;
	char_t name_buf[16];
	char_t *value;
	int *deleArray;

	char *new_rules;
    const char *rules = (char *)nvram_safe_get( "url_filter_rules");
    if(!rules || !strlen(rules) )
        return;

	rule_count = getRuleNums((char *)rules);
	if(!rule_count)
		return;


	deleArray = (int *)malloc(rule_count * sizeof(int));
	if(!deleArray)
		return;

    new_rules = strdup(rules);
    if(!new_rules){
        free(deleArray);
        return;
    }

	for(i=0, j=0; i< rule_count; i++){
		snprintf(name_buf, 16, "DR%d", i);
		value = websGetVar(wp, name_buf, NULL);
		if(value){
			deleArray[j++] = i;
		}
	}
	if(!j){
	    websHeader(wp);
	    websWrite(wp, T("You didn't select any rules to delete.<br>\n"));
	    websFooter(wp);
	    websDone(wp, 200);
		return;
	}

	deleteNthValueMulti(deleArray, rule_count, new_rules, ';');
	free(deleArray);

	nvram_set( "url_filter_rules", new_rules);
	nvram_commit();

	free(new_rules);
    notification_send_rc( OID_NETWORK_FIREWALL, NOTIFY_WAIT );

	websRedirect(wp, "safe_urlfiltering.asp");


	return;
}

static void websURLFilter(webs_t wp, char_t *path, char_t *query)
{
	const char *urlfilters ;
	char *rule = websGetVar(wp, T("addURLFilter"), T(""));
	char ruletmp[4096];

	if(!rule)
		return;
	if(strchr(rule, ';'))
		return;


	if((urlfilters= nvram_safe_get( "url_filter_rules")) && strlen(urlfilters)){
			snprintf(ruletmp, sizeof(ruletmp), "%s;%s", urlfilters, rule);
	}else{
			snprintf(ruletmp, sizeof(ruletmp), "%s", rule);
	}
		nvram_set( "url_filter_rules", ruletmp);
		nvram_commit();

    notification_send_rc( OID_NETWORK_FIREWALL, NOTIFY_WAIT );

	websRedirect(wp, "safe_urlfiltering.asp");
}
#endif /* HAVE_WEB_FILTER */

static void DDos(webs_t wp, char_t *path, char_t *query)
{
	char *antiDosE, *bicmp_flood, *usicmp_flood,
	     *budp_flood, *usudp_flood, *btcp_flood, *ustcp_flood,
	     *ping_flood, *lanping_flood;

    antiDosE=bicmp_flood=usicmp_flood=budp_flood=usudp_flood=btcp_flood=ustcp_flood=
    ping_flood=lanping_flood=NULL;

	antiDosE = websGetVar(wp, T("ddos_en"), T(""));
	if (!antiDosE || !strlen(antiDosE))
	{
		return;
    }
	if (atoi(antiDosE) == 0){	/* disable */
		nvram_set( "anti_dos_attack_en", "0");

	}else{		/* enable */
		nvram_set( "anti_dos_attack_en", "1");
	}

    bicmp_flood = websGetVar(wp, T("icmp_flood_en"), T(""));
    if (atoi(antiDosE) == 1){
        usicmp_flood = websGetVar(wp, T("usIcmpFlood"), T(""));
        nvram_set( "icmp_flood_enable", bicmp_flood);
	    nvram_set( "icmp_flood_threshold", usicmp_flood);
    }
    budp_flood = websGetVar(wp, T("udp_flood_en"), T(""));
    if (atoi(antiDosE) == 1){
        usudp_flood = websGetVar(wp, T("usUdpFlood"), T(""));
        nvram_set( "udp_flood_enable", budp_flood);
	    nvram_set( "udp_flood_threshold", usudp_flood);
    }

    btcp_flood = websGetVar(wp, T("tcp_flood_en"), T(""));
    if (atoi(antiDosE) == 1){

        ustcp_flood = websGetVar(wp, T("usTcpSynFlood"), T(""));
		nvram_set( "tcp_flood_enable", btcp_flood);
		nvram_set( "tcp_flood_threshold", ustcp_flood);
		if (!strncmp(btcp_flood, "1", 1))
		{
		    nvram_set( "block_port_scan", "1");
		    nvram_set( "block_syn_flood", "1");
		}
		else
		{
		    nvram_set( "block_port_scan", "0");
		    nvram_set( "block_syn_flood", "0");
		}
    }
    else
    {
        nvram_set( "block_port_scan", "0");
		nvram_set( "block_syn_flood", "0");
    }

    ping_flood = websGetVar(wp, T("wan_ping_en"), T(""));
    lanping_flood = websGetVar(wp, T("lan_ping_en"), T(""));

	nvram_set( "wan_ping_deny", ping_flood);
	nvram_set( "lan_ping_deny", lanping_flood);

	nvram_commit();

	notification_send_rc( OID_NETWORK_FIREWALL, NOTIFY_WAIT );

	websRedirect(wp, "safe_advanceset.asp");
	return;
}

static void LanWebFilter(webs_t wp, char_t *path, char_t *query)
{
    char *lanweb_en, *lan_mac0, *lan_mac1, *lan_mac2, *lan_mac3; //*pc_mac;

    lanweb_en=lan_mac0=lan_mac1=lan_mac2=lan_mac3=NULL;

    lanweb_en = websGetVar(wp, T("enableWhitelist"), T(""));

    if (!strncmp(lanweb_en, "0", 2))
    {
        nvram_set( "lan_access_en", "0");
    }
    else
    {
        nvram_set( "lan_access_en", "1");
		lan_mac0 = websGetVar(wp, T("mac0"), T(""));
    	lan_mac1 = websGetVar(wp, T("mac1"), T(""));
   		lan_mac2 = websGetVar(wp, T("mac2"), T(""));
    	lan_mac3 = websGetVar(wp, T("mac3"), T(""));
    }

   
   // if( strlen( lan_mac0 )> 10 )
        nvram_set( "lan_access_mac0", lan_mac0);
    //if( strlen( lan_mac1 )> 10 )
        nvram_set( "lan_access_mac1", lan_mac1);
    //if( strlen( lan_mac2 )> 10 )
        nvram_set( "lan_access_mac2", lan_mac2);
   // if( strlen( lan_mac3 )> 10 )
        nvram_set( "lan_access_mac3", lan_mac3);

    nvram_commit();

    notification_send_rc( OID_NETWORK_FIREWALL, NOTIFY_WAIT );

	websRedirect(wp, "safe_lanweb.asp");
}


static void WanWebFilter(webs_t wp, char_t *path, char_t *query)
{
    char *wan_en, *wan_ip, *wan_port;

    wan_en=wan_ip=wan_port=NULL;

    wan_ip = websGetVar(wp, T("enableWhitelist"), T(""));

    wan_en = websGetVar(wp, T("rt_enable_h"), T(""));
    wan_ip = websGetVar(wp, T("rt_ipaddr"), T(""));
    wan_port = websGetVar(wp, T("rt_port"), T(""));


    nvram_set( "remote_access_en", wan_en);
    nvram_set( "remote_allow_ip", wan_ip);
    nvram_set( "remote_allow_port", wan_port);

    nvram_commit();

    notification_send_rc( OID_NETWORK_FIREWALL, NOTIFY_WAIT );

	websRedirect(wp, "safe_remoteweb.asp");
}

static int getSuperDMZBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef CONFIG_RALINKAPP_SUPERDMZ
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

void formDefineFirewall(void)
{
	websAspDefine(T("getDefaultFirewallPolicyASP"), getDefaultFirewallPolicyASP);
	websFormDefine(T("BasicSettings"), BasicSettings);

	websAspDefine(T("getIPPortFilterEnableASP"), getIPPortFilterEnableASP);
	websAspDefine(T("showIPPortFilterRulesASP"), showIPPortFilterRulesASP);
	websAspDefine(T("showARPRulesASP"), showARPRulesASP);
	websAspDefine(T("getIPPortRuleNumsASP"), getIPPortRuleNumsASP);
	websFormDefine(T("ipportFilter"), ipportFilter);
	websFormDefine(T("ipportFilterDelete"), ipportFilterDelete);
	websFormDefine(T("getRulesPacketCount"), getRulesPacketCount);

    websAspDefine(T("getSuperDMZBuilt"), getSuperDMZBuilt);
	websFormDefine(T("DMZ"), DMZ);
	websAspDefine(T("getDMZEnableASP"), getDMZEnableASP);
	websAspDefine(T("showDMZIPAddressASP"), showDMZIPAddressASP);
	websAspDefine(T("showSuperDMZMACAddressASP"), showSuperDMZMACAddressASP);

	websAspDefine(T("getPortForwardEnableASP"), getPortForwardEnableASP);
	websAspDefine(T("showPortForwardRulesASP"), showPortForwardRulesASP);
	websAspDefine(T("getPortForwardRuleNumsASP"), getPortForwardRuleNumsASP);
	websFormDefine(T("portForward"), portForward);
        /*start of Add portforward save button apply by zengxianzhu at 20110225*/
        websFormDefine(T("portForwardEnable"), portForwardEnable);
        /*end of Add portforward save button apply by zengxianzhu at 20110225*/
	websFormDefine(T("portForwardDelete"), portForwardDelete);

	websAspDefine(T("showPortTriggerRulesASP"), showPortTriggerRulesASP);
	websAspDefine(T("getPortTriggerRuleNumsASP"), getPortTriggerRuleNumsASP);
	websFormDefine(T("portTrigger"), portTrigger);
	websFormDefine(T("portTriggerRulesAction"), portTriggerRulesAction);

#ifdef HAVE_WEB_FILTER
	websFormDefine(T("websURLFilterDelete"), websURLFilterDelete);
	websFormDefine(T("websURLFilter"), websURLFilter);
#endif

	websAspDefine(T("checkIfUnderBridgeModeASP"), checkIfUnderBridgeModeASP);
/* add by radar@20081229. For web page Anti-attack. */
	websAspDefine(T("getAntiDoSEnableASP"), getAntiDoSEnableASP);
	/* add by zs@20100212 */
    websFormDefine(T("DDos"), DDos);
    websFormDefine(T("LanWebFilter"), LanWebFilter);
    websFormDefine(T("WanWebFilter"), WanWebFilter);

	websAspDefine(T("getAntiPortScanEnableASP"), getAntiPortScanEnableASP);
	websAspDefine(T("showFWSentryPortASP"), showFWSentryPortASP);
	websAspDefine(T("showFWIgnorePortASP"), showFWIgnorePortASP);
}

