/* vi: set sw=4 ts=4 sts=4 fdm=marker: */
/*
 *    internet.c -- Internet Settings
 *
 *    Copyright (c) Ralink Technology Corporation All Rights Reserved.
 *
 *    $Id: internet.c,v 1.195 2010-07-09 07:45:08 steven Exp $
 */

#include    <stdlib.h>
#include    <sys/ioctl.h>
#include    <net/if.h>
#include    <net/route.h>
#include    <string.h>
#include    <dirent.h>
#include    <notify.h>
#include    <oid.h>
#include    <net_conn.h>
#include    "bcmnvram.h"
#include    "webs.h"
#include    "utils.h"
#include     "firewall.h"
#include    "management.h"
#include    "station.h"
#include    "wireless.h"

#include    "linux/autoconf.h"  //kernel config
#include    "config/autoconf.h" //user config
#include    "user/busybox/include/autoconf.h" //busybox config

//PPPOE Dynamic head file
#include<stdio.h>
#include<inttypes.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <time.h>
#include <signal.h>
#ifdef CONFIG_RALINKAPP_SWQOS
#include      "qos.h"
#endif
#include    "ip_control.h" /* zym+ */
#include    "internet.h"

#ifdef CONFIG_USER_3G
//#include    <unistd.h>
//#include    <sys/types.h>
//#include    <sys/wait.h>
//#include    <dirent.h>
//#include    <fcntl.h>
#endif
#include "firewall.h"
#include "shared.h"

#ifdef DEFINE_RALINK_RT5350
typedef unsigned int __u32;
#endif

#include  <ra_ioctl.h>
/*end of add for wan status by zengxianzu at 20110414*/


enum {
    DT_EMPTY,
    DT_NORESPOND,
    DT_DHCP,
    DT_PPPOE,
    DT_ERROR,
    DT_UNPLUG,
};
static int gi_detect_type = DT_EMPTY;


static int getMeshBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getWDSBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getWSCBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getMBSSIDBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getUSBBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getStorageBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getFtpBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getSmbBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int Showentries(int eid, webs_t wp, int argc, char_t **argv);
static int getMediaBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getWebCamBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getPrinterSrvBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getUSBiNICBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getiTunesBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getIgmpProxyBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getIgmpSnoopingBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getVPNBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getSnmpBuilt(int eid, webs_t wp, int argc, char_t **argv);
/* add by radar@20081229 */
static int getAntiAttackBuilt(int eid, webs_t wp, int argc, char_t **argv);
/* end. radar */
static int getDnsmasqBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getGWBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getLltdBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getPppoeRelayBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getUpnpBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getRadvdBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getDynamicRoutingBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getSWQoSBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getDATEBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getDDNSBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getSysLogBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getETHTOOLBuilt(int eid, webs_t wp, int argc, char_t **argv);
static void advRouting(webs_t wp, char_t *path, char_t *query);
static int get3GBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int getDhcpCliListIp(int eid, webs_t wp, int argc, char_t **argv);
static int getDhcpCliListMac(int eid, webs_t wp, int argc, char_t **argv);
static int getDhcpCliList(int eid, webs_t wp, int argc, char_t **argv);
static int getDns(int eid, webs_t wp, int argc, char_t **argv);
static int getHostSupp(int eid, webs_t wp, int argc, char_t **argv);
static int getIfLiveWeb(int eid, webs_t wp, int argc, char_t **argv);
static int getIfIsUpWeb(int eid, webs_t wp, int argc, char_t **argv);

static int getWscModeOption(int eid, webs_t wp, int argc, char_t **argv);
static int getLanIp(int eid, webs_t wp, int argc, char_t **argv);
static int getLanMac(int eid, webs_t wp, int argc, char_t **argv);
static int getLanIfNameWeb(int eid, webs_t wp, int argc, char_t **argv);
static int getLanNetmask(int eid, webs_t wp, int argc, char_t **argv);
static int getWanIp(int eid, webs_t wp, int argc, char_t **argv);
static int getWanProto(int eid, webs_t wp, int argc, char_t **argv);
static int get_usbnet_signal(int eid, webs_t wp, int argc, char_t **argv);
static int getWanDetect(int eid, webs_t wp, int argc, char_t **argv);
static int getWanMac(int eid, webs_t wp, int argc, char_t **argv);
static int getWanIfNameWeb(int eid, webs_t wp, int argc, char_t **argv);
static int getWanNetmask(int eid, webs_t wp, int argc, char_t **argv);
static int getWanGateway(int eid, webs_t wp, int argc, char_t **argv);
//start of that control wan connect on status web by huangfeilan 20110927
static int getWanConnectStatus(int eid, webs_t wp, int argc, char_t **argv);
static int getWanConnectButton(int eid, webs_t wp, int argc, char_t **argv);
//end of that control wan connect on status web by huangfeilan 20110927
static int getWanLink(int eid, webs_t wp, int argc, char_t **argv);
static int getRoutingTable(int eid, webs_t wp, int argc, char_t **argv);
static void setLan(webs_t wp, char_t *path, char_t *query);
static int getWanRunTime(int eid, webs_t wp, int argc, char_t **argv);
static int getArpCliList(int eid, webs_t wp, int argc, char_t **argv);
/*start of get arp Ip list by zengxianzhu at 20110301*/
static int getArpIPList(int eid, webs_t wp, int argc, char_t **argv);
/*end of get arp Ip list by zengxianzhu at 20110301*/

static int getWolCliList(int eid, webs_t wp, int argc, char_t **argv);
static int getArpCliList_safelan(int eid, webs_t wp, int argc, char_t **argv);
#ifdef HAVE_WOL
static void WolWakeUp(webs_t wp, char_t *path, char_t *query);
static void WolAddPC(webs_t wp, char_t *path, char_t *query);
#endif
#ifdef HAVE_UPNPD
/* add by zs for upnp */
static void SetUpnp(webs_t wp, char_t *path, char_t *query);
#endif
#ifdef HAVE_IGMP_PROXY
/*start of add for mulitcast forwarding by zengxianzhu at 20110107*/
static void SetMcast(webs_t wp, char_t *path, char_t *query);
/*end of add for mulitcast forwarding by zengxianzhu at 20110107*/
#endif
static void addassignment(webs_t wp, char_t *path, char_t *query);
static void delassignment(webs_t wp, char_t *path, char_t *query);
static void setNetworkLan(webs_t wp, char_t *path, char_t *query);
#ifdef HAVE_UDHCPD
static void dhcpservice(webs_t wp, char_t *path, char_t *query);
#endif
static void setNetworkWan( webs_t wp, char_t *path, char_t *query );
static void setModemPin( webs_t wp, char_t *path, char_t *query );
static void setModemRoaming( webs_t wp, char_t *path, char_t *query );
static void wan_type_detect( webs_t wp, char_t *path, char_t *query );
static void clean3gFlow( webs_t wp, char_t *path, char_t *query );

//start of that control wan connect on status web by huangfeilan 20110927
static void controlWanConnect( webs_t wp, char_t *path, char_t *query );
static void controlWanDisconnect( webs_t wp, char_t *path, char_t *query );
//end of that control wan connect on status web by huangfeilan 20110927
static void setMAC(webs_t wp, char_t *path, char_t *query);
#ifdef HAVE_PPTPD
static void setPptpServer(webs_t wp, char_t *path, char_t *query);
static void setPptpServerClientAdd(webs_t wp, char_t *path, char_t *query);
static void setPptpServerClientDel(webs_t wp, char_t *path, char_t *query);
#endif
#ifdef CONFIG_USER_3G
//static void connectControl(webs_t wp, char_t *path, char_t *query);
#endif
static void getMyMAC(webs_t wp, char_t *path, char_t *query);
static void CheckWanConnection(webs_t wp, char_t *path, char_t *query);
static void addRouting(webs_t wp, char_t *path, char_t *query);
static void delRouting(webs_t wp, char_t *path, char_t *query);
static void dynamicRouting(webs_t wp, char_t *path, char_t *query);
static void alg(webs_t wp, char_t *path, char_t *query);
static void guideWanLnkCheck(webs_t wp, char_t *path, char_t *query);
#ifdef HAVE_SNMPD
static void snmpservice(webs_t wp, char_t *path, char_t *query);
#endif
#ifdef DEFINE_WIFIDOG
static void wifidogservice(webs_t wp, char_t *path, char_t *query);
#endif
inline void zebraRestart(void);
char* getwaninterface(void);

void ripdRestart(void);

void formDefineInternet(void) {
    websAspDefine(T("getDhcpCliListIp"), getDhcpCliListIp);
    websAspDefine(T("getDhcpCliListMac"), getDhcpCliListMac);
    websAspDefine(T("getDhcpCliList"), getDhcpCliList);
    websAspDefine(T("getDns"), getDns);
    websAspDefine(T("getHostSupp"), getHostSupp);
    websAspDefine(T("getIfLiveWeb"), getIfLiveWeb);
    websAspDefine(T("getIfIsUpWeb"), getIfIsUpWeb);
    websAspDefine(T("getIgmpProxyBuilt"), getIgmpProxyBuilt);
/* LiuBin,add */
    websAspDefine(T("getIgmpSnoopingBuilt"), getIgmpSnoopingBuilt);
/* LiuBin,add */
    websAspDefine(T("getVPNBuilt"), getVPNBuilt);
	
    websAspDefine(T("getWscModeOption"), getWscModeOption);
    
    websAspDefine(T("getLanIp"), getLanIp);
    websAspDefine(T("getLanMac"), getLanMac);
    websAspDefine(T("getLanIfNameWeb"), getLanIfNameWeb);
    websAspDefine(T("getLanNetmask"), getLanNetmask);
    websAspDefine(T("getDnsmasqBuilt"), getDnsmasqBuilt);
    websAspDefine(T("getGWBuilt"), getGWBuilt);
    websAspDefine(T("getLltdBuilt"), getLltdBuilt);
    websAspDefine(T("getPppoeRelayBuilt"), getPppoeRelayBuilt);
    websAspDefine(T("getUpnpBuilt"), getUpnpBuilt);
    websAspDefine(T("getRadvdBuilt"), getRadvdBuilt);
    websAspDefine(T("getWanIp"), getWanIp);
    websAspDefine(T("getWanProto"), getWanProto);
	websAspDefine(T("get_usbnet_signal"), get_usbnet_signal);
    websAspDefine(T("getWanDetect"), getWanDetect);
    websAspDefine(T("getWanMac"), getWanMac);
    websAspDefine(T("getWanIfNameWeb"), getWanIfNameWeb);
    websAspDefine(T("getWanNetmask"), getWanNetmask);
    websAspDefine(T("getWanGateway"), getWanGateway);
	//start of that control wan connect on status web by huangfeilan 20110927
	websAspDefine(T("getWanConnectStatus"), getWanConnectStatus);
	websAspDefine(T("getWanConnectButton"), getWanConnectButton);
	//end of that control wan connect on status web by huangfeilan 20110927
    websAspDefine(T("getWanLink"), getWanLink);
    websAspDefine(T("getRoutingTable"), getRoutingTable);
    websAspDefine(T("getMeshBuilt"), getMeshBuilt);
    websAspDefine(T("getWDSBuilt"), getWDSBuilt);
    websAspDefine(T("getWSCBuilt"), getWSCBuilt);
    websAspDefine(T("getMBSSIDBuilt"), getMBSSIDBuilt);
    websAspDefine(T("getUSBBuilt"), getUSBBuilt);
    websAspDefine(T("getStorageBuilt"), getStorageBuilt);
    websAspDefine(T("getFtpBuilt"), getFtpBuilt);
    websAspDefine(T("getSmbBuilt"), getSmbBuilt);
    websAspDefine(T("Showentries"), Showentries);
    websAspDefine(T("getMediaBuilt"), getMediaBuilt);
    websAspDefine(T("getWebCamBuilt"), getWebCamBuilt);
    websAspDefine(T("getPrinterSrvBuilt"), getPrinterSrvBuilt);
    websAspDefine(T("getUSBiNICBuilt"), getUSBiNICBuilt);
    websAspDefine(T("getiTunesBuilt"), getiTunesBuilt);
    websFormDefine(T("addassignment"), addassignment);
    websFormDefine(T("delassignment"), delassignment);
    websFormDefine(T("setNetworkLan"), setNetworkLan);
#ifdef HAVE_UDHCPD
    websFormDefine(T("dhcpservice"), dhcpservice);
#endif
    websFormDefine(T("setNetworkWan"), setNetworkWan);
	websFormDefine(T("setModemPin"), setModemPin);
	websFormDefine(T("setModemRoaming"), setModemRoaming);
	websFormDefine(T("wan_type_detect"), wan_type_detect);
	websFormDefine(T("clean3gFlow"), clean3gFlow);
	//start of that control wan connect on status web by huangfeilan 20110927
    websFormDefine(T("controlWanConnect"), controlWanConnect);
	websFormDefine(T("controlWanDisconnect"), controlWanDisconnect);
	//end of that control wan connect on status web by huangfeilan 20110927
#ifdef HAVE_PPTPD
    websFormDefine(T("setPptpServer"), setPptpServer);
    websFormDefine(T("setPptpServerClientAdd"), setPptpServerClientAdd);
    websFormDefine(T("setPptpServerClientDel"), setPptpServerClientDel);
#endif
    websFormDefine(T("setMAC"), setMAC);
#ifdef HAVE_UPNPD
    websFormDefine(T("SetUpnp"), SetUpnp);
#endif
#ifdef HAVE_IGMP_PROXY
    /*start of add for mulitcast forwarding by zengxianzhu at 20110107*/
    websFormDefine(T("SetMcast"), SetMcast);
    /*start of add for mulitcast forwarding by zengxianzhu at 20110107*/
#endif
    #ifdef HAVE_WOL
    websFormDefine(T("WolWakeUp"), WolWakeUp);
    websFormDefine(T("WolAddPC"), WolAddPC);
    #endif
    websFormDefine(T("guideWanLnkCheck"), guideWanLnkCheck);
#ifdef HAVE_SNMPD
    websFormDefine(T("snmpservice"), snmpservice);
#endif
#ifdef DEFINE_WIFIDOG
	websFormDefine(T("wifidogservice"), wifidogservice);
#endif
#ifdef CONFIG_USER_3G
//    websFormDefine(T("connectControl"), connectControl);
#endif
    websFormDefine(T("getMyMAC"), getMyMAC);
    websFormDefine(T("CheckWanConnection"), CheckWanConnection);
    websFormDefine(T("addRouting"), addRouting);
    websFormDefine(T("delRouting"), delRouting);
#ifdef HAVE_DYNAMIC_ROUTE
    websFormDefine(T("dynamicRouting"), dynamicRouting);
#endif
    websFormDefine(T("alg"), alg);
    websAspDefine(T("getDynamicRoutingBuilt"), getDynamicRoutingBuilt);
    websAspDefine(T("getSWQoSBuilt"), getSWQoSBuilt);
    websAspDefine(T("getDATEBuilt"), getDATEBuilt);
    websAspDefine(T("getDDNSBuilt"), getDDNSBuilt);
    websAspDefine(T("getSysLogBuilt"), getSysLogBuilt);
    websAspDefine(T("getETHTOOLBuilt"), getETHTOOLBuilt);
/* For SNMP build flag. append by radar@20081212. */
    websAspDefine(T("getSnmpBuilt"), getSnmpBuilt);
/* For Firewall Anti-attack. add by radar@20081229 */
    websAspDefine(T("getAntiAttackBuilt"), getAntiAttackBuilt);
    websAspDefine(T("get3GBuilt"), get3GBuilt);
   websAspDefine(T("getWanRunTime"), getWanRunTime);
   websAspDefine(T("getArpCliList"), getArpCliList);
   /*start of get arp Ip list by zengxianzhu at 20110301*/
   websAspDefine(T("getArpIPList"), getArpIPList);
   /*end of get arp Ip list by zengxianzhu at 20110301*/
   #ifdef HAVE_WOL
   websAspDefine(T("getWolCliList"), getWolCliList);
   #endif
   websAspDefine(T("getArpCliList_safelan"), getArpCliList_safelan);
}

static void mult(long op1, long op2, long *prod_hi, long *prod_lo)
{
    long op1_hi = (op1 >> 16) & 0xffff;
    long op1_lo = op1 & 0xffff;
    long op2_hi = (op2 >> 16) & 0xffff;
    long op2_lo = op2 & 0xffff;
    long cross_prod = op1_lo * op2_hi + op1_hi * op2_lo;
    *prod_hi = op1_hi * op2_hi + ((cross_prod >> 16) & 0xffff);
    *prod_lo = op1_lo * op2_lo + ((cross_prod << 16) & 0xffff0000);
}


/*
 * arguments: ifname  - interface name
 * description: test the existence of interface through /proc/net/dev
 * return: -1 = fopen error, 1 = not found, 0 = found
 */
int getIfLive(char *ifname)
{
    FILE *fp;
    char buf[256], *p;
    int i;

    if (NULL == (fp = fopen("/proc/net/dev", "r"))) {
        error(E_L, E_LOG, T("getIfLive: open /proc/net/dev error"));
        return -1;
    }

    fgets(buf, 256, fp);
    fgets(buf, 256, fp);
    while (NULL != fgets(buf, 256, fp)) {
        i = 0;
        while (isspace(buf[i++]))
            ;
        p = buf + i - 1;
        while (':' != buf[i++])
            ;
        buf[i-1] = '\0';
        if (!strcmp(p, ifname)) {
            fclose(fp);
            return 0;
        }
    }
    fclose(fp);
    error(E_L, E_LOG, T("getIfLive: device %s not found"), ifname);
    return 1;
}

/*
 * arguments: ifname  - interface name
 *            if_addr - a 18-byte buffer to store mac address
 * description: fetch mac address according to given interface name
 */
int getIfMac(char *ifname, char *if_hw)
{
    struct ifreq ifr;
    char *ptr;
    int skfd;

    if((skfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
        error(E_L, E_LOG, T("getIfMac: open socket error"));
        return -1;
    }

    strncpy(ifr.ifr_name, ifname, IF_NAMESIZE);
    if(ioctl(skfd, SIOCGIFHWADDR, &ifr) < 0) {
        close(skfd);
        //error(E_L, E_LOG, T("getIfMac: ioctl SIOCGIFHWADDR error for %s"), ifname);
        return -1;
    }

    ptr = (char *)&ifr.ifr_addr.sa_data;
    sprintf(if_hw, "%02X:%02X:%02X:%02X:%02X:%02X",
            (ptr[0] & 0377), (ptr[1] & 0377), (ptr[2] & 0377),
            (ptr[3] & 0377), (ptr[4] & 0377), (ptr[5] & 0377));

    close(skfd);
    return 0;
}

/*
 * arguments: ifname  - interface name
 *            if_addr - a 16-byte buffer to store ip address
 * description: fetch ip address, netmask associated to given interface name
 */
int getIfIp(char *ifname, char *if_addr)
{
    struct ifreq ifr;
    int skfd = 0;

    if((skfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
        error(E_L, E_LOG, T("getIfIp: open socket error"));
        return -1;
    }

    strncpy(ifr.ifr_name, ifname, IF_NAMESIZE);
    if (ioctl(skfd, SIOCGIFADDR, &ifr) < 0) {
        close(skfd);
        //error(E_L, E_LOG, T("getIfIp: ioctl SIOCGIFADDR error for %s"), ifname);
        return -1;
    }
    strcpy(if_addr, inet_ntoa(((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr));

    close(skfd);
    return 0;
}

/*
 * arguments: ifname - interface name
 * description: return 1 if interface is up
 *              return 0 if interface is down
 */
int getIfIsUp(char *ifname)
{
    struct ifreq ifr;
    int skfd;

    skfd = socket(AF_INET, SOCK_DGRAM, 0);
    if (skfd == -1) {
        perror("socket");
        return -1;
    }
    strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
    if (ioctl(skfd, SIOCGIFFLAGS, &ifr) < 0) {
        perror("ioctl");
        close(skfd);
        return -1;
    }
    close(skfd);
    if (ifr.ifr_flags & IFF_UP)
        return 1;
    else
        return 0;
}

/*
 * arguments: ifname - interface name
 *            if_net - a 16-byte buffer to store subnet mask
 * description: fetch subnet mask associated to given interface name
 *              0 = bridge, 1 = gateway, 2 = wirelss isp
 */
int getIfNetmask(char *ifname, char *if_net)
{
    struct ifreq ifr;
    int skfd = 0;

    if((skfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
        error(E_L, E_LOG, T("getIfNetmask: open socket error"));
        return -1;
    }

    strncpy(ifr.ifr_name, ifname, IF_NAMESIZE);
    if (ioctl(skfd, SIOCGIFNETMASK, &ifr) < 0) {
        close(skfd);
        //error(E_L, E_LOG, T("getIfNetmask: ioctl SIOCGIFNETMASK error for %s\n"), ifname);
        return -1;
    }
    strcpy(if_net, inet_ntoa(((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr));
    close(skfd);
    return 0;
}

/*
 * description: return WAN interface name
 *              0 = bridge, 1 = gateway, 2 = wirelss isp
 */
char* getWanIfName(void)
{
    const char *mode = nvram_safe_get( "OperationMode");
    static char *if_name = "br0";

    if (NULL == mode)
        return if_name;
    if (!strncmp(mode, "0", 2))
        if_name = "br0";
    else if (!strncmp(mode, "1", 2)) {
#if defined CONFIG_RAETH_ROUTER || defined CONFIG_MAC_TO_MAC_MODE || defined CONFIG_RT_3052_ESW
#if defined (CONFIG_RAETH_SPECIAL_TAG)
#if defined (CONFIG_WAN_AT_P0)
        if_name = "eth2.1";
#else
        if_name = "eth2.5";
#endif
#else
        if_name = "eth2.2";
#endif
#else /* MARVELL & CONFIG_ICPLUS_PHY */
        if_name = "eth2";
#endif
    }
    else if (!strncmp(mode, "2", 2))
        if_name = "ra0";
    else if (!strncmp(mode, "3", 2))
        if_name = "apcli0";
    return if_name;
}

char* getWanIfNamePPP(void)
{
    const char *cm = nvram_safe_get( "wan_proto");
    if (!strncmp(cm, "PPPOE", 6) || !strncmp(cm, "L2TP", 5) || !strncmp(cm, "PPTP", 5)
#ifdef CONFIG_USER_3G
        || !strncmp(cm, "3G", 3)
#endif
    ){
        return "ppp0";
    }

    return getWanIfName();
}


/*
 * description: return LAN interface name
 */
char* getLanIfName(void)
{
    const char *mode = nvram_safe_get( "OperationMode");
    static char *if_name = "br0";

    if (NULL == mode)
        return if_name;
    if (!strncmp(mode, "0", 2))
        if_name = "br0";
    else if (!strncmp(mode, "1", 2)) {
#if defined CONFIG_RAETH_ROUTER || defined CONFIG_MAC_TO_MAC_MODE || defined CONFIG_RT_3052_ESW
        if_name = "br0";
#elif defined  CONFIG_ICPLUS_PHY && CONFIG_RT2860V2_AP_MBSS
        char *num_s = nvram_safe_get( "BssidNum");
        if(atoi(num_s) > 1)    // multiple ssid
            if_name = "br0";
        else
            if_name = "ra0";
#else
        if_name = "ra0";
#endif
    }
    else if (!strncmp(mode, "2", 2)) {
        if_name = "eth2";
    }
    else if (!strncmp(mode, "3", 2)) {
        if_name = "br0";
    }
    return if_name;
}

/*
 * description: get the value "WAN" or "LAN" the interface is belong to.
 */
char *getLanWanNamebyIf(char *ifname)
{
    const char *mode = nvram_safe_get( "OperationMode");

    if (NULL == mode)
        return "Unknown";

    if (!strcmp(mode, "0")){    // bridge mode
        if(!strcmp(ifname, "br0"))
            return "LAN";
        return ifname;
    }

    if (!strcmp(mode, "1")) {    // gateway mode
#if defined CONFIG_RAETH_ROUTER || defined CONFIG_MAC_TO_MAC_MODE || defined CONFIG_RT_3052_ESW
        if(!strcmp(ifname, "br0"))
            return "LAN";
#if defined (CONFIG_RAETH_SPECIAL_TAG)
#if defined (CONFIG_WAN_AT_P0)
        if(!strcmp(ifname, "eth2.1") || !strcmp(ifname, "ppp0"))
#else
        if(!strcmp(ifname, "eth2.5") || !strcmp(ifname, "ppp0"))
#endif
#else
        if(!strcmp(ifname, "eth2.2") || !strcmp(ifname, "ppp0"))
#endif
            return "WAN";
        return ifname;
#elif defined  CONFIG_ICPLUS_PHY && CONFIG_RT2860V2_AP_MBSS
        char *num_s = nvram_safe_get( "BssidNum");
        if(atoi(num_s) > 1 && !strcmp(ifname, "br0") )    // multiple ssid
            return "LAN";
        if(atoi(num_s) == 1 && !strcmp(ifname, "ra0"))
            return "LAN";
        if (!strcmp(ifname, "eth2") || !strcmp(ifname, "ppp0"))
            return "WAN";
        return ifname;
#else
        if(!strcmp(ifname, "ra0"))
            return "LAN";
        return ifname;
#endif
    }else if (!strncmp(mode, "2", 2)) {    // ethernet convertor
        if(!strcmp("eth2", ifname))
            return "LAN";
        if(!strcmp("ra0", ifname))
            return "WAN";
        return ifname;
    }else if (!strncmp(mode, "3", 2)) {    // apcli mode
        if(!strcmp("br0", ifname))
            return "LAN";
        if(!strcmp("apcli0", ifname))
            return "WAN";
        return ifname;
    }
    return ifname;
}


/*
 * description: write DHCP client list
 */
static int getDhcpCliListIp(int eid, webs_t wp, int argc, char_t **argv)
{
    FILE *fp;
    struct dhcpOfferedAddr {
        unsigned char hostname[16];
        unsigned char mac[16];
        unsigned long ip;
        unsigned long expires;
    } lease;
    int i;
    struct in_addr addr;
    fp = fopen("/tmp/udhcpd.leases", "r");
    if (NULL == fp)
        return websWrite(wp, T(""));
    while (fread(&lease, 1, sizeof(lease), fp) == sizeof(lease)) {
        /* skip zero mac */
        if ( (0 == lease.mac[0]) && (0 == lease.mac[1]) && (0 == lease.mac[2])
            && (0 == lease.mac[3]) && (0 == lease.mac[4]) && (0 == lease.mac[5]) )
            continue;
        addr.s_addr = lease.ip;
        websWrite(wp, T("<option value=\"%02X"), lease.mac[0]);
        for (i = 1; i < 6; i++)
            websWrite(wp, T(":%02X"), lease.mac[i]);
        websWrite(wp, T(";%s;%s\">%-16s</option>"), lease.hostname,inet_ntoa(addr), lease.hostname);
    }
    fclose(fp);
    return 0;
}

static int getDhcpCliListMac(int eid, webs_t wp, int argc, char_t **argv)
{
    FILE *fp;
    struct dhcpOfferedAddr {
        unsigned char hostname[16];
        unsigned char mac[16];
        unsigned long ip;
        unsigned long expires;
    } lease;
    int i;

    fp = fopen("/tmp/udhcpd.leases", "r");
    if (NULL == fp)
        return websWrite(wp, T(""));
    while (fread(&lease, 1, sizeof(lease), fp) == sizeof(lease)) {
        /* skip zero mac */
        if ( (0 == lease.mac[0]) && (0 == lease.mac[1]) && (0 == lease.mac[2])
            && (0 == lease.mac[3]) && (0 == lease.mac[4]) && (0 == lease.mac[5]) )
            continue;
        /* end add. radar */
        websWrite(wp, T("<option value=\"%02X"), lease.mac[0]);
        for (i = 1; i < 6; i++)
            websWrite(wp, T(":%02X"), lease.mac[i]);
        websWrite(wp, T("\">%-16s</option>"), lease.hostname);
    }

    fclose(fp);
    return 0;
}

/*
 * description: write DHCP client list
 */
static int getDhcpCliList(int eid, webs_t wp, int argc, char_t **argv)
{
    FILE *fp;
	FILE *tmpfp;
    struct dhcpOfferedAddr {
		unsigned char hostname[16];
        unsigned char mac[16];
        unsigned long ip;
        unsigned long expires;
    } lease;

    int i;
    struct in_addr addr;
    unsigned long expires;
    unsigned d, h, m;
    doSystem("killall -q -USR1 udhcpd");
	unsigned char macaddr[1500] = {0};
	unsigned char  *temp;
	const char *str = "static_lease";
	unsigned long filelen;
	unsigned char strg[6][8] = {0};
	bool staticiP = 0;

    sleep(1);
	
    fp = fopen("/tmp/udhcpd.leases", "r");
    if (NULL == fp){
        return websWrite(wp, T(""));
        }
	
    while (fread(&lease, 1, sizeof(lease), fp) == sizeof(lease)) {
        /* skip zero mac */
        if ( (0 == lease.mac[0]) && (0 == lease.mac[1]) && (0 == lease.mac[2])
            && (0 == lease.mac[3]) && (0 == lease.mac[4]) && (0 == lease.mac[5]) )
                continue;
        /* end add. radar */

        if( 0 == lease.expires)
             continue;
        websWrite(wp, T("<tr><td class=\"leftnone\">%-16s</td>"), lease.hostname);

        websWrite(wp, T("<td>%02X"), lease.mac[0]);
        for (i = 1; i < 6; i++)
            websWrite(wp, T(":%02X"), lease.mac[i]);

        addr.s_addr = lease.ip;

		if (!(tmpfp = fopen("/tmp/udhcpd.conf", "r"))) {
			perror("/tmp/udhcpd.conf");
			return errno;
		}
		fseek(tmpfp, 0, SEEK_END);
		filelen = ftell(tmpfp);
		fseek(tmpfp, 0, SEEK_SET);
		
		fread(macaddr, 1, filelen, tmpfp);
		temp = strstr(macaddr, str);
		while (temp) {
			sprintf(strg[0], "%02X", lease.mac[0]);
			sprintf(strg[1], "%02X", lease.mac[1]);
			sprintf(strg[2], "%02X", lease.mac[2]);
			sprintf(strg[3], "%02X", lease.mac[3]);
			sprintf(strg[4], "%02X", lease.mac[4]);
			sprintf(strg[5], "%02X", lease.mac[5]);
			
			if ( (*(temp+13) == strg[0][0]) && (*(temp+14) == strg[0][1])
				 && (*(temp+16) == strg[1][0]) && (*(temp+17) == strg[1][1]) 
				 && (*(temp+19) == strg[2][0]) && (*(temp+20) == strg[2][1])
				 && (*(temp+22) == strg[3][0])	&& (*(temp+23) == strg[3][1])
				 && (*(temp+25) == strg[4][0])	&& (*(temp+26) == strg[4][1])
				 && (*(temp+28) == strg[5][0]) && (*(temp+29) == strg[5][1])) {
				staticiP = 1;
				websWrite(wp, T("</td><td>%s</td><td class=\"rightnone\">"), inet_ntoa(addr));
				websWrite(wp, T("Static IP</td></tr>\n"));
				break;
			} 
		   temp = strstr((temp + 13), str);
		}
		fclose(tmpfp);
		if (staticiP == 0){
	        expires = ntohl(lease.expires);
			printf("in static lease time is %d",expires);
	        websWrite(wp, T("</td><td>%s</td><td class=\"rightnone\">"), inet_ntoa(addr));

	        d = expires / (24*60*60); expires %= (24*60*60);
	        h = expires / (60*60); expires %= (60*60);
	        m = expires / 60; expires %= 60;
	        if (d) websWrite(wp, T("%u days "), d);
	        websWrite(wp, T("%02u:%02u:%02u</td></tr>\n"), h, m, (unsigned)expires);
		}
		staticiP = 0;
    }
    fclose(fp);
    return 0;
}

char* getwaninterface(void)
{
	char *ifname;
	if(nvram_match("sw_mode_ex", "7")||nvram_match("sw_mode_ex", "8"))
	{
		if(nvram_match("wan_ifname",""))
			ifname = "apcli0";
		else
			ifname = nvram_safe_get("wan_ifname");
	}
	else
	{
		if(nvram_match("wan_ifname",""))
			ifname = "eth2.2";
		else
			ifname = nvram_safe_get("wan_ifname");
	}
	return ifname;
	
}

/*
* description: write Arp client list
*/
static int getArpCliList(int eid, webs_t wp, int argc, char_t **argv)
{
   FILE *fp;
   int i=0;
   char *p1,*p2,*clienttype,buf[128]={0},ip[16]={0},mac[18]={0};
   char mac1[3]={0},mac2[3]={0},mac3[3]={0},mac4[3]={0},mac5[3]={0},mac6[3]={0};

   /*start of ignore wan interface arp table by huangzhangping at 2011-07-07*/
   char *wan_interface;
   /*end of ignore wan interface arp table by huangzhangping at 2011-07-07*/

   fp=fopen("/proc/net/arp","r");
   if(NULL == fp)
   {
       return websWrite(wp,T(""));
   }

   if(NULL == fgets(buf,128,fp))
   {
       fclose(fp);
       return websWrite(wp,T(""));
   }

   while(fgets(buf,128,fp))
   {
       p1 = strchr(buf,' ');  //ip;
       if(NULL == p1)
       {
           fclose(fp);
           printf("[getArpcliList]get ip error!buf = %s\n",p1);
           return websWrite(wp,T(""));
	   }
           /*start of ignore wan interface arp table by zengxianzhu at 20110429*/
           wan_interface = getwaninterface();

		   //wan_interface=nvram_safe_get( "wan_ifname" );
            printf("***********wan_interface is %s ************\n",wan_interface);
            if(strstr(buf,wan_interface) != NULL)
            {
                    continue;
            }
            /*end of ignore wan interface arp table by zengxianzhu at 20110429*/ 
       strncpy(ip,buf,p1-buf);

       websWrite(wp,T("<tr><td>%s</td>"),ip);

       p1=strchr(buf,':');  //mac;
       if(NULL == p1)
       {
          fclose(fp);
          printf("[getArpCliList]get mac error!buf = %s\n",p1);
          return websWrite(wp,T(""));
       }
#if 0	   
		/*start of ignore wan interface arp table by huangzhangping at 2011-07-07*/
        wan_interface = getWanIfNamePPP();
        if(strstr(buf,wan_interface) != NULL)
        {
                continue;
        }
        /*end of ignore wan interface arp table by huangzhangping at 2011-07-07*/
#endif	   
       p1 -= 2;
       p2 = strchr(p1,' ');
       strncpy(mac,p1,p2-p1);
       if(NULL == p2)
       {
          fclose(fp);
          printf("[getArpCliList]get mac error!\n");
          return websWrite(wp,T(""));
       }
	   
       strncpy(mac1,p1,2);
       p1+=3;
       strncpy(mac2,p1,2);
       p1+=3;
       strncpy(mac3,p1,2);
       p1+=3;
       strncpy(mac4,p1,2);
       p1+=3;
       strncpy(mac5,p1,2);
       p1+=3;
       strncpy(mac6,p1,2);


       websWrite(wp,T("<td><script>function SelectMacFromPopup_%d(){\n"),i);
       websWrite(wp,T("if(0 == document.arp_list.tableType.value){"));
       websWrite(wp,T("parent.opener.self.document.wol_form.Mac1.value=\"%s\";\n"),mac1);
       websWrite(wp,T("parent.opener.self.document.wol_form.Mac2.value=\"%s\";\n"),mac2);
       websWrite(wp,T("parent.opener.self.document.wol_form.Mac3.value=\"%s\";\n"),mac3);
       websWrite(wp,T("parent.opener.self.document.wol_form.Mac4.value=\"%s\";\n"),mac4);
       websWrite(wp,T("parent.opener.self.document.wol_form.Mac5.value=\"%s\";\n"),mac5);
       websWrite(wp,T("parent.opener.self.document.wol_form.Mac6.value=\"%s\";\n}"),mac6);
       websWrite(wp,T("if(1 == document.arp_list.tableType.value){"));
       websWrite(wp,T("parent.opener.self.document.addassignment.Mac1.value=\"%s\";\n"),mac1);
       websWrite(wp,T("parent.opener.self.document.addassignment.Mac2.value=\"%s\";\n"),mac2);
       websWrite(wp,T("parent.opener.self.document.addassignment.Mac3.value=\"%s\";\n"),mac3);
       websWrite(wp,T("parent.opener.self.document.addassignment.Mac4.value=\"%s\";\n"),mac4);
       websWrite(wp,T("parent.opener.self.document.addassignment.Mac5.value=\"%s\";\n"),mac5);
       websWrite(wp,T("parent.opener.self.document.addassignment.Mac6.value=\"%s\";\n}"),mac6);
       websWrite(wp,T("if(2 == document.arp_list.tableType.value){"));
       websWrite(wp,T("parent.opener.self.document.security_form.Mac1.value=\"%s\";\n"),mac1);
       websWrite(wp,T("parent.opener.self.document.security_form.Mac2.value=\"%s\";\n"),mac2);
       websWrite(wp,T("parent.opener.self.document.security_form.Mac3.value=\"%s\";\n"),mac3);
       websWrite(wp,T("parent.opener.self.document.security_form.Mac4.value=\"%s\";\n"),mac4);
       websWrite(wp,T("parent.opener.self.document.security_form.Mac5.value=\"%s\";\n"),mac5);
       websWrite(wp,T("parent.opener.self.document.security_form.Mac6.value=\"%s\";\n}"),mac6);
       websWrite(wp,T("if(3 == document.arp_list.tableType.value){"));
       websWrite(wp,T("parent.opener.self.document.DMZ.superDMZMac1.value=\"%s\";\n"),mac1);
       websWrite(wp,T("parent.opener.self.document.DMZ.superDMZMac2.value=\"%s\";\n"),mac2);
       websWrite(wp,T("parent.opener.self.document.DMZ.superDMZMac3.value=\"%s\";\n"),mac3);
       websWrite(wp,T("parent.opener.self.document.DMZ.superDMZMac4.value=\"%s\";\n"),mac4);
       websWrite(wp,T("parent.opener.self.document.DMZ.superDMZMac5.value=\"%s\";\n"),mac5);
       websWrite(wp,T("parent.opener.self.document.DMZ.superDMZMac6.value=\"%s\";\n}"),mac6);
       /* start for adding search mac page by zym 2011-02-25 */
       websWrite(wp,T("if(4 == document.arp_list.tableType.value){"));
       websWrite(wp,T("parent.opener.self.document.ipportFilter.mac1.value=\"%s\";\n"),mac1);
       websWrite(wp,T("parent.opener.self.document.ipportFilter.mac2.value=\"%s\";\n"),mac2);
       websWrite(wp,T("parent.opener.self.document.ipportFilter.mac3.value=\"%s\";\n"),mac3);
       websWrite(wp,T("parent.opener.self.document.ipportFilter.mac4.value=\"%s\";\n"),mac4);
       websWrite(wp,T("parent.opener.self.document.ipportFilter.mac5.value=\"%s\";\n"),mac5);
       websWrite(wp,T("parent.opener.self.document.ipportFilter.mac6.value=\"%s\";\n}"),mac6);
       /* end for adding search mac page by zym 2011-02-25 */
       websWrite(wp,T("self.close();}</script>"));


       websWriteWithTranslation(wp, T("<a href=\"#\" id=\"underline\" onclick=\"SelectMacFromPopup_%d();\">%s</a></td></tr>"), i,mac);

       memset(buf,0,128);
       memset(ip,0,16);
       i++;

   }

       fclose(fp);
       return 0;

}


/*start of get arp Ip list by zengxianzhu at 20110301*/
static int getArpIPList(int eid, webs_t wp, int argc, char_t **argv)
{
   FILE *fp;
   int i=0;
   char *p1,*p2,*clienttype,buf[128]={0},ip[16]={0},mac[18]={0};
   char mac1[3]={0},mac2[3]={0},mac3[3]={0},mac4[3]={0},mac5[3]={0},mac6[3]={0};
   char ip1[4]={0},ip2[4]={0},ip3[4]={0},ip4[4]={0};
   char *pstr;
   char iptmp[16] = {0};

   /*start of ignore wan interface arp table by huangzhangping at 2011-07-07*/
   char *wan_interface;
   /*end of ignore wan interface arp table by huangzhangping at 2011-07-07*/


   fp=fopen("/proc/net/arp","r");
   if(NULL == fp)
   {
       return websWrite(wp,T(""));
   }

   if(NULL == fgets(buf,128,fp))
   {
       fclose(fp);
       return websWrite(wp,T(""));
   }

   while(fgets(buf,128,fp))
   {
       p1 = strchr(buf,' ');  //ip;
       if(NULL == p1)
       {
           fclose(fp);
           printf("[getArpcliList]get ip error!buf = %s\n",p1);
               return websWrite(wp,T(""));
       }

   		/*start of ignore wan interface arp table by huangzhangping at 2011-07-07*/
        wan_interface = getwaninterface();
        //printf("***********wan_interface is %s ************\n",wan_interface);
        if(strstr(buf,wan_interface) != NULL)
        {
                continue;
        }
        /*end of ignore wan interface arp table by huangzhangping at 2011-07-07*/
            strncpy(ip,buf,p1-buf);
            strncpy(iptmp,ip,p1-buf);


            strcpy(ip1,strtok(iptmp, "."));
            strcpy(ip2,strtok(NULL, "."));
            strcpy(ip3,strtok(NULL, "."));
            strcpy(ip4,strtok(NULL, "."));
            websWrite(wp,T("<tr><td><script>function SelectMacFromPopup_%d(){\n"),i);
       websWrite(wp,T("if(0 == document.arp_list_ip.tableType.value){"));
       websWrite(wp,T("parent.opener.self.document.portForward.lanIp1.value=\"%s\";\n"),ip1);
       websWrite(wp,T("parent.opener.self.document.portForward.lanIp2.value=\"%s\";\n"),ip2);
       websWrite(wp,T("parent.opener.self.document.portForward.lanIp3.value=\"%s\";\n"),ip3);
       websWrite(wp,T("parent.opener.self.document.portForward.lanIp4.value=\"%s\";\n}"),ip4);
          websWrite(wp,T("if(1 == document.arp_list_ip.tableType.value){"));
       websWrite(wp,T("parent.opener.self.document.DMZ.DMZAddress1.value=\"%s\";\n"),ip1);
       websWrite(wp,T("parent.opener.self.document.DMZ.DMZAddress2.value=\"%s\";\n"),ip2);
       websWrite(wp,T("parent.opener.self.document.DMZ.DMZAddress3.value=\"%s\";\n"),ip3);
       websWrite(wp,T("parent.opener.self.document.DMZ.DMZAddress4.value=\"%s\";\n}"),ip4);
            websWrite(wp,T("if(2 == document.arp_list_ip.tableType.value){"));
       websWrite(wp,T("parent.opener.self.document.ipportFilter.sIp1.value=\"%s\";\n"),ip1);
       websWrite(wp,T("parent.opener.self.document.ipportFilter.sIp2.value=\"%s\";\n"),ip2);
       websWrite(wp,T("parent.opener.self.document.ipportFilter.sIp3.value=\"%s\";\n"),ip3);
       websWrite(wp,T("parent.opener.self.document.ipportFilter.sIp4.value=\"%s\";\n}"),ip4);
           websWrite(wp,T("self.close();}</script>"));

           websWriteWithTranslation(wp, T("<a href=\"#\" id=\"underline\" onclick=\"SelectMacFromPopup_%d();\">%s</a></td>"), i,ip);

       p1=strchr(buf,':');  //mac;
       if(NULL == p1)
       {
          fclose(fp);
          printf("[getArpCliList]get mac error!buf = %s\n",p1);
          return websWrite(wp,T(""));
       }
       p1 -= 2;
       p2 = strchr(p1,' ');
       strncpy(mac,p1,p2-p1);
       if(NULL == p2)
       {
          fclose(fp);
          printf("[getArpCliList]get mac error!\n");
          return websWrite(wp,T(""));
       }

            websWrite(wp,T("<td>%s</td></tr>"),mac);
       memset(buf,0,128);
       memset(ip,0,16);
            memset(iptmp,0,16);
       i++;

   }

       fclose(fp);
       return 0;

}
/*end of get arp Ip list by zengxianzhu at 20110301*/



/*
* description: write Arp client list
*/
static int getArpCliList_safelan(int eid, webs_t wp, int argc, char_t **argv)
{
   FILE *fp;
   int i=0;
   char buf[128]={0},ip[16]={0},mac[18]={0},*p1,*p2;
   char mac1[3]={0},mac2[3]={0},mac3[3]={0},mac4[3]={0},mac5[3]={0},mac6[3]={0};

   /*start of ignore wan interface arp table by zengxianzhu at 20110429*/
   char *wan_interface;
   /*end of ignore wan interface arp table by zengxianzhu at 20110429*/
   fp=fopen("/proc/net/arp","r");
   if(NULL == fp)
   {
       return websWrite(wp,T(""));
   }

   if(NULL == fgets(buf,128,fp))
   {
       fclose(fp);
       return websWrite(wp,T(""));
   }

   while(fgets(buf,128,fp))
   {
       p1 = strchr(buf,' ');  //ip;
       if(NULL == p1)
       {
           fclose(fp);
           printf("[getArpcliList]get ip error!buf = %s\n",p1);
           return websWrite(wp,T(""));
	   }
       /*start of ignore wan interface arp table by zengxianzhu at 20110429*/
       wan_interface = getwaninterface();
       //printf("***********wan_interface is %s ************\n",wan_interface);
       if(strstr(buf,wan_interface) != NULL)
       {
            continue;
       }
       /*end of ignore wan interface arp table by zengxianzhu at 20110429*/
       strncpy(ip,buf,p1-buf);


       websWrite(wp,T("<tr><td>%s</td>"),ip);

       p1=strchr(buf,':');  //mac;
       if(NULL == p1)
       {
          fclose(fp);
          printf("[getArpCliList]get mac error!buf = %s\n",p1);
          return websWrite(wp,T(""));
       }
       p1 -= 2;
       p2 = strchr(p1,' ');
       strncpy(mac,p1,p2-p1);
       if(NULL == p2)
       {
          fclose(fp);
          printf("[getArpCliList]get mac error!\n");
          return websWrite(wp,T(""));
       }

       strncpy(mac1,p1,2);
       p1+=3;
       strncpy(mac2,p1,2);
       p1+=3;
       strncpy(mac3,p1,2);
       p1+=3;
       strncpy(mac4,p1,2);
       p1+=3;
       strncpy(mac5,p1,2);
       p1+=3;
       strncpy(mac6,p1,2);

       websWrite(wp,T("<td><script>function SelectMacFromPopup_%d(){\n"),i);
       websWrite(wp,T("if(1 == parent.opener.self.document.lanweb.mac_0.value){"));
       websWrite(wp,T("parent.opener.self.document.lanweb.mac01.value=\"%s\";\n"),mac1);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac02.value=\"%s\";\n"),mac2);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac03.value=\"%s\";\n"),mac3);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac04.value=\"%s\";\n"),mac4);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac05.value=\"%s\";\n"),mac5);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac06.value=\"%s\";\n"),mac6);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac_0.value = 0;\n}"));
       websWrite(wp,T("else if(1 == parent.opener.self.document.lanweb.mac_1.value){"));
       websWrite(wp,T("parent.opener.self.document.lanweb.mac11.value=\"%s\";\n"),mac1);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac12.value=\"%s\";\n"),mac2);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac13.value=\"%s\";\n"),mac3);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac14.value=\"%s\";\n"),mac4);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac15.value=\"%s\";\n"),mac5);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac16.value=\"%s\";\n"),mac6);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac_1.value = 0;\n}"));
       websWrite(wp,T("else if(1 == parent.opener.self.document.lanweb.mac_2.value){"));
       websWrite(wp,T("parent.opener.self.document.lanweb.mac21.value=\"%s\";\n"),mac1);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac22.value=\"%s\";\n"),mac2);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac23.value=\"%s\";\n"),mac3);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac24.value=\"%s\";\n"),mac4);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac25.value=\"%s\";\n"),mac5);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac26.value=\"%s\";\n"),mac6);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac_2.value = 0;\n}"));
       websWrite(wp,T("else if(1 == parent.opener.self.document.lanweb.mac_3.value){"));
       websWrite(wp,T("parent.opener.self.document.lanweb.mac31.value=\"%s\";\n"),mac1);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac32.value=\"%s\";\n"),mac2);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac33.value=\"%s\";\n"),mac3);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac34.value=\"%s\";\n"),mac4);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac35.value=\"%s\";\n"),mac5);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac36.value=\"%s\";\n"),mac6);
       websWrite(wp,T("parent.opener.self.document.lanweb.mac_3.value = 0;\n}"));

       websWrite(wp,T("self.close();}</script>"));

       websWriteWithTranslation(wp, T("<a href=\"#\" id=\"underline\" onclick=\"SelectMacFromPopup_%d();\">%s</a></td></tr>"), i,mac);

       memset(buf,0,128);
       memset(ip,0,16);
       i++;

   }

       fclose(fp);
       return 0;

}

/*
* description: write Wol client list
*/
static int getWolCliList(int eid, webs_t wp, int argc, char_t **argv)
{
   int i=0;
   char_t rec[128],comment[128],mac[18];
   char_t *rules = (char *)nvram_safe_get( "WolRules");

   while(getNthValueSafe(i++, rules, ';', rec, sizeof(rec)) != -1 )
   {
      if((getNthValueSafe(0, rec, ',', mac, sizeof(mac)) == -1)){
            continue;
        }
      if((getNthValueSafe(1, rec, ',', comment, sizeof(comment)) == -1)){
            continue;
        }
#ifdef HAVE_WEB_CNS
		websWrite(wp, T("<tr>\n"));
		websWrite(wp, T("<td> %d&nbsp; <input type=\"checkbox\" name=\"WolRule%d\"> </td>"), i, i-1 );

		websWrite(wp, T("<td align=center>%s</td>"), mac);

		if(strlen(comment))
			  websWrite(wp, T("<td align=center>%s</td>"), comment);
		else
			  websWrite(wp, T("<td align=center> &nbsp; </td>"));

		websWrite(wp, T("</tr>\n"));
#else
      websWrite(wp, T("<tr>\n"));
      websWrite(wp, T("<td class=\"leftnone\">%d&nbsp;</td>"), i);

      websWrite(wp, T("<td id=\"mac%d\">%s</td>"), i-1, mac);

      if(strlen(comment))
            websWrite(wp, T("<td id=\"cmt%d\">%s</td>"), i-1, comment);
      else                                                                                  
            websWrite(wp, T("<td id=\"cmt%d\">&nbsp;</td>"), i-1);
	  
	  websWrite(wp, T("<td><a href=\"#edit\" onClick=\"wol_edit_action(%d); return false;\" ><img src=\"/images/edit.jpg\" width=\"16\" /></a></td>"), i-1 );
	  websWrite(wp, T("<td class=\"rightnone\"><input type=\"checkbox\" name=\"WolRule%d\" onClick=\"disableDelete(1);\"> </td>"), i-1 );

      websWrite(wp, T("</tr>\n"));
#endif
   }



   return 0;

}


/*
* description: wake on Lan
*/
#ifdef HAVE_WOL
static void WolWakeUp(webs_t wp, char_t *path, char_t *query)
{
    int i,j,rule_count,i_num;
    char_t mac[256]={0},*rules = (char *)nvram_safe_get( "WolRules");
    char_t name_buf[10],*wolflag,*pRule = NULL;
    char_t *p,*value,*wol_count;
    char *new_rules = strdup(rules);
    int *deleArray;

    wolflag = websGetVar(wp,T("wolflag"),T(""));

    rule_count= getRuleNums((char *)rules);

    deleArray = (int *)malloc(rule_count * sizeof(int));
    if(!strcmp(wolflag,"1"))
    {
        printf("wake up!\n");
        for(i=0,j=0;i<rule_count;i++)
        {
            snprintf(name_buf, 16, "WolRule%d", i);
            value = websGetVar(wp, T(name_buf), NULL);

            if(value)
            {
               pRule=getNthValueOffset(i,rules);
               p = strchr(pRule,',');
               if(!p)
               {
                   printf("[WolWakeUp]wake up error!!!\n");
                   return  websRedirect(wp, "wol.asp");
               }
               strncpy(mac+j,pRule,p - pRule);
               strcat(mac,";");

               j += p-pRule+1;
               printf("mac = %s,j=%d\n",mac,j);
            }
        }

            mac[j-1] = '\0';
            nvram_set("wol_mac", mac);
            nvram_commit();
            notification_send_rc( OID_SYSTEM_WOL, NOTIFY_WAIT );
            websRedirect(wp, "wol.asp");

    }
    else
    {
            wol_count = (char *)nvram_safe_get( "wol_count");
            i_num = atoi(wol_count);
            for(i=0,j=0;i<rule_count;i++)
            {
                snprintf(name_buf, 10, "WolRule%d", i);
                value = websGetVar(wp, T(name_buf), NULL);

                if(value)
                {
                    deleArray[j++] = i;
                    i_num--;
                }
            }

            i_num += 48;

            deleteNthValueMulti(deleArray, rule_count, new_rules, ';');
            free(deleArray);

            nvram_set( "WolRules", new_rules);
            nvram_set( "wol_count", (char *)&i_num);
            nvram_commit();
            free(new_rules);
            return websRedirect(wp, "wol.asp");
    }


    return ;
}


/*
* description: wake on Lan add client pc information
*/
#ifdef HAVE_WEB_CNS 
static void WolAddPC(webs_t wp, char_t *path, char_t *query)
{
    char_t rules[2048]={0};
    int count;
    char_t *mac,*comment,*wol_count;
    const char *WolRules;

    wol_count = (char *)nvram_safe_get( "wol_count");
    WolRules = nvram_safe_get( "WolRules");

    mac = websGetVar(wp,T("macadd_address"),T(""));
    comment = websGetVar(wp,T("comment"),T(""));

    if(WolRules && strlen(WolRules))
    {
       snprintf(rules,sizeof(rules),"%s;%s,%s",WolRules,mac,comment);
    }
    else
    {
       snprintf(rules,sizeof(rules),"%s,%s",mac,comment);
    }

    count = atoi(wol_count);
    count++;
    count += 48;

    nvram_set( "WolRules", rules);
    if(count <= 57)
        nvram_set( "wol_count", (char *)&count);
    else
        nvram_set( "wol_count", "10");
    nvram_commit();
    websRedirect(wp, "wol.asp");
    return ;
}

#else
static void WolAddPC(webs_t wp, char_t *path, char_t *query)
{
    char_t rules[2048]={0};
    int count, index=-1, i, num;
    char_t *mac,*comment,*wol_count, *edit_index, *src, *dest, list[10][52]={0};
    const char *WolRules;

    wol_count = (char *)nvram_safe_get( "wol_count");
    WolRules = nvram_safe_get( "WolRules");

    mac = websGetVar(wp,T("macadd_address"),T(""));
    comment = websGetVar(wp,T("comment"),T(""));

	edit_index = websGetVar(wp, T("edit_index"), T(""));  
	index = atoi(edit_index); 
	if(index == -1) //add
	{
	    count = atoi(wol_count);
	    count++;
    	count += 48;
	    if(count <= 57)
	        nvram_set( "wol_count", (char *)&count);
	    else
	        nvram_set( "wol_count", "10");
		
		if(WolRules && strlen(WolRules))
		{
		   snprintf(rules,sizeof(rules),"%s;%s,%s",WolRules,mac,comment);
		}
		else
		{
		   snprintf(rules,sizeof(rules),"%s,%s",mac,comment);
		}
	}
	else //modify
	{
		src = WolRules;
		for(i=0; *src!='\0'; i++)
		{
			dest = list[i];
			while(*src!=';' && *src!='\0'){
				*dest++ = *src++;
			}
			*dest = *src;
			if(*src == ';')
				src++;
		}
		num = i;
		
		memset(list[index],'0',sizeof(list[index]));
		snprintf(list[index],sizeof(list[index]),"%s,%s",mac,comment);
		if(index < num-1)
			strcat(list[index], ";");
		
		strcpy(rules,list[0]);
		for(i=1; i<num; i++)
		{
			strcat(rules,list[i]);
		}
	}

    nvram_set( "WolRules", rules);
    nvram_commit();
    websRedirect(wp, "wol.asp");
    return ;
}
#endif

#endif


/*
 * arguments: type - 1 = write Primary DNS
 *                   2 = write Secondary DNS
 * description: write DNS ip address accordingly
 */
 static int get_ppp_demand_status()
{
    FILE *fp;
    char buff[10];

    fp = fopen("/etc/ppp_demand_status", "r");
    if (fp == NULL) return 0;

    fgets(buff, 10, fp);

    fclose(fp);

    return (buff[0] - '0');
}

/*
 * arguments: type - 1 = write Primary DNS
 *                   2 = write Secondary DNS
 * description: write DNS ip address accordingly
 */
static int getDns(int eid, webs_t wp, int argc, char_t **argv)
{
    FILE *fp;
    char buf[80] = {0}, ns_str[11], dns[16] = {0};
    int type, idx = 0, req = 0;
    char if_addr[16];

    const char *value;
    const char *pppoe_opmode;

    value = nvram_safe_get( "wan_proto");
    if (strcmp(value, WAN_PROTO_PPPOE) == 0)
    {
        pppoe_opmode = nvram_safe_get( "wan_pppoe_opmode");
        if (strcmp(pppoe_opmode, "on_demand") == 0)
        {
            if (get_ppp_demand_status() == 0)
                return websWrite(wp, T("0.0.0.0"));
        }
    }

    if (ejArgs(argc, argv, T("%d"), &type) == 1) {
        if (1 == type)
            req = 1;
        else if (2 == type)
            req = 2;
        else
            return websWrite(wp, T("0.0.0.0"));
    }

    fp = fopen("/etc/resolv.conf", "r");
    if (NULL == fp)
        return websWrite(wp, T("0.0.0.0"));
	
    while (fgets(buf, sizeof(buf), fp) != NULL) {
        if (sscanf(buf, "%s%s", ns_str, dns) != 2 || NULL == strchr(buf,'.'))
            continue;
        idx++;
        if (idx == req) {
            if (strcmp("", nvram_safe_get("wan_ipaddr")) == 0 || !strcmp("", dns))
			{
                websWrite(wp, T("0.0.0.0"));
            }
            else
                websWrite(wp, T("%s"), dns);
            memset(if_addr, 0, sizeof(if_addr));
            break;
        }
    }
    fclose(fp);
    return 0;
}

/*
 * arguments:
 * description: return 1 if hostname is supported
 */
static int getHostSupp(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef GA_HOSTNAME_SUPPORT
    ejSetResult(eid, "1");
#else
    ejSetResult(eid, "0");
#endif
    return 0;
}

/*
 * arguments: name - interface name (ex. eth0, rax ..etc)
 * description: write the existence of given interface,
 *              0 = ifc dosen't exist, 1 = ifc exists
 */
static int getIfLiveWeb(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t *name;
    char exist[2] = "0";

    if (ejArgs(argc, argv, T("%s"), &name) < 1) {
        websError(wp, 400, T("Insufficient args\n"));
        return -1;
    }
    exist[0] = (getIfLive(name) == 0)? '1' : '0';
    return websWrite(wp, T("%s"), exist);
}

/*
 * arguments: name - interface name (ex. eth0, rax ..etc)
 * description: write the existence of given interface,
 *              0 = ifc is down, 1 = ifc is up
 */
static int getIfIsUpWeb(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t *name;
    char up[2] = "1";

    if (ejArgs(argc, argv, T("%s"), &name) < 1) {
        websError(wp, 400, T("Insufficient args\n"));
        return -1;
    }
    up[0] = (getIfIsUp(name) == 1)? '1' : '0';
    return websWrite(wp, T("%s"), up);
}

static int getIgmpProxyBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef CONFIG_USER_IGMP_PROXY
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getIgmpSnoopingBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef CONFIG_USER_IGMP_SNOOPING
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}
/* LiuBin,add for igmp snooping */

static int getVPNBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_NF_CONNTRACK_PPTP || defined CONFIG_NF_CONNTRACK_PPTP_MODULE || \
    defined CONFIG_IP_NF_PPTP        || defined CONFIG_IP_NF_PPTP_MODULE
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getMeshBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_RT2860V2_AP_MESH || defined CONFIG_RT2860V2_STA_MESH
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getWDSBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_RT2860V2_AP_WDS
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getWSCBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_RT2860V2_AP_WSC || defined CONFIG_RT2860V2_STA_WSC
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getMBSSIDBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_RT2860V2_AP_MBSS
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getUSBBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_USB
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getStorageBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_USB_STORAGE && defined CONFIG_USER_STORAGE
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getFtpBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_USER_PROFTPD
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getSmbBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_USER_SAMBA
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int Showentries(int eid, webs_t wp, int argc, char_t **argv)
{
    int ret = 0;
    const char *dhcpStaticno = nvram_safe_get( "dhcpStaticno");
    const char *dhcpStatic1 = nvram_safe_get( "dhcpStatic1");
    const char *dhcpStatic2 = nvram_safe_get( "dhcpStatic2");
    const char *dhcpStatic3 = nvram_safe_get( "dhcpStatic3");
    const char *dhcpStatic4 = nvram_safe_get( "dhcpStatic4");
    char *p, mac[18], ip[16];
    char s_buf[256] = { 0 };
    char str1[]=";";
    int i = 0, len = 0, no = 0;
    memset(mac, '\0', 18);
    memset(ip, '\0', 16);
    sprintf(s_buf, "%s;%s;%s;%s",dhcpStatic1, dhcpStatic2, dhcpStatic3, dhcpStatic4);
    p = s_buf;
    len = strlen(dhcpStatic1);
    no = atoi(dhcpStaticno);
    for(i=1; i<=no; i++)
    {
        ret = websWrite(wp, T("<TR align=middle>\n"));
        ret = websWrite(wp, T("<TD>%d&nbsp;<input type=\"checkbox\" value=\"0\" name=\"delassignment%d\" onclick=\"checkbox%d()\"></TD>\n"),i, i, i);
        ret = websWrite(wp, T("<TD>%s</TD>\n"), strsep(&p, str1));
        ret = websWrite(wp, T("<TD>%s</TD>\n"), strsep(&p, str1));
        ret = websWrite(wp, T("</TR>"));
    }
    return ret;
}
static int getMediaBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_USB && defined CONFIG_USER_USHARE
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getWebCamBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_USB && defined CONFIG_USER_UVC_STREAM
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getPrinterSrvBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_USB && defined CONFIG_USER_P910ND
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getUSBiNICBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_RTDEV_USB
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getiTunesBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_USER_MTDAAPD
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getDynamicRoutingBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_USER_ZEBRA
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getSWQoSBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_RALINKAPP_SWQOS
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getSnmpBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_USER_UCDSNMP
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getDATEBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_DATE
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getAntiAttackBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_USER_ANTIATTACK
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getDDNSBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef HAVE_DDNS
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getSysLogBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_LOGREAD && defined CONFIG_KLOGD
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getETHTOOLBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_USER_ETHTOOL
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int get3GBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_USER_3G
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static inline char * strcat_r(const char *s1, const char *s2, char *buf)
{
	strcpy(buf, s1);
	strcat(buf, s2);
	return buf;
}



static int getWscModeOption(int eid, webs_t wp, int argc, char_t **argv)
{
    return websWrite(wp, T("%s"), nvram_safe_get("WscModeOption"));
}

/*
 * description: write LAN ip address accordingly
 */
static int getLanIp(int eid, webs_t wp, int argc, char_t **argv)
{
    char if_addr[16] = {0};
    strncpy(if_addr, nvram_safe_get("lan_ipaddr"),sizeof(if_addr)-1);
    if ( strcmp("",if_addr) == 0) {
        return websWrite(wp, T(""));
    }
    return websWrite(wp, T("%s"), if_addr);
}

/*
 * description: write LAN MAC address accordingly
 */
static int getLanMac(int eid, webs_t wp, int argc, char_t **argv)
{
    char if_mac[18] = {0};
    strncpy(if_mac,nvram_safe_get("lan_hwaddr"),sizeof(if_mac)-1);
    if (strcmp("", if_mac) == 0) {
        return websWrite(wp, T(""));
    }
    return websWrite(wp, T("%s"), if_mac);
}

/*
 * arguments: type - 0 = return LAN interface name (default)
 *                   1 = write LAN interface name
 * description: return or write LAN interface name accordingly
 */
static int getLanIfNameWeb(int eid, webs_t wp, int argc, char_t **argv)
{
    int type;
    char *name = getLanIfName();

    if (ejArgs(argc, argv, T("%d"), &type) == 1) {
        if (1 == type) {
            return websWrite(wp, T("%s"), name);
        }
    }
    ejSetResult(eid, name);
    return 0;
}

/*
 * description: write LAN subnet mask accordingly
 */
static int getLanNetmask(int eid, webs_t wp, int argc, char_t **argv)
{
    char if_net[16];
    strncpy(if_net, nvram_safe_get("lan_netmask"), sizeof(if_net)-1);
    if ( strcmp("",if_net) == 0 ) {
        return websWrite(wp, T(""));
    }
    return websWrite(wp, T("%s"), if_net);
}

static int getGWBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_LAN_WAN_SUPPORT || defined CONFIG_MAC_TO_MAC_MODE
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getDnsmasqBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef CONFIG_USER_DNSMASQ
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getLltdBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_USER_LLTD && defined CONFIG_RT2860V2_AP_LLTD
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getPppoeRelayBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef CONFIG_USER_RPPPPOE_RELAY
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getUpnpBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
//#ifdef CONFIG_USER_UPNP_IGD
#ifdef HAVE_UPNPD
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

static int getRadvdBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef CONFIG_USER_RADVD
    return websWrite(wp, T("1"));
#else
    return websWrite(wp, T("0"));
#endif
}

/*
 * description: write WAN ip address accordingly
 */
static int getWanIp(int eid, webs_t wp, int argc, char_t **argv)
{
    char if_addr[16] = {0};
    strncpy(if_addr, nvram_safe_get("wan_ipaddr"), sizeof(if_addr)-1);
    if(strcmp("", if_addr) == 0)
    {
        return websWrite(wp, T("0.0.0.0"));
    }

    return websWrite(wp, T("%s"), if_addr);
}

static int getWanProto(int eid, webs_t wp, int argc, char_t **argv)
{
    char proto[8] = {0};
    strncpy(proto, nvram_safe_get("wan_proto"), sizeof(proto)-1);
    if(strcmp("", proto) == 0)
    {
        return websWriteWithTranslation(wp, T("<#Wan_Body_noplug#>"));
    }
    else if(strcmp("dhcp", proto) == 0)
    {
            return websWrite(wp, T("DHCP"));
    }
    else if(strcmp("static", proto) == 0)
    {
            return websWrite(wp, T("STATIC"));
    }
    else if(strcmp("pppoe", proto) == 0)
    {
            return websWrite(wp, T("PPPOE"));
    }
    else if(strcmp("pptp", proto) == 0)
    {
            return websWrite(wp, T("PPTP"));
    }
    else if(strcmp("l2tp", proto) == 0)
    {
            return websWrite(wp, T("L2TP"));
    }
	else if(strcmp("3g", proto) == 0)
    {
#ifdef DEFINE_G4
		return websWrite(wp, T("3G/4G"));
#else
		return websWrite(wp, T("3G"));
#endif
	}
    else
    {
            return websWrite(wp, T("%s"), proto);
    }
}

static int get_usbnet_signal(int eid, webs_t wp, int argc, char_t **argv)
{
	int ret = 0;
	int net3g_st = 0;

	char proto[8] = {0};
	strncpy(proto, nvram_safe_get("wan_proto"), sizeof(proto)-1);
	if( nvram_match("sw_mode_ex", "1") && strcmp("3g", proto) == 0  )
	{
		net3g_st = check_net3g_status();
		
		if ( net3g_st == USB3G_CONN || net3g_st == USB3G_SEARCH || net3g_st == USB3G_DIAL )
		{
			//return before
			ret = websWrite(wp, T(nvram_safe_get("net3g_signal")));
		
			//Get 4G/3G signal
			system("killall -SIGUSR1 umodem");
		}
		else
		{
			ret = websWrite(wp, T("0"));
		}
	}
	else
	{
		ret = websWrite(wp, T("0"));
	}

	return ret;
}

static int getWanDetect(int eid, webs_t wp, int argc, char_t **argv)
{
    int i_ret = 0;
	i_ret = websWrite( wp, "%d", gi_detect_type );
	gi_detect_type = DT_EMPTY;
    return i_ret;
}


#include <sys/sysinfo.h>    //for sysinfo()
/*
 * description: get wan run time from /etc/wan_run_time
 */
static int getWanRunTime(int eid, webs_t wp, int argc, char_t **argv)
{
//    char buf[21] = {0}; /* support to 64bit long */
//    char temp[12],*ptr1;
    char if_addr[16];
    long totaltime;
    int day,hours,min,sec;
    FILE *fp;

	strncpy(if_addr, nvram_safe_get("wan_ipaddr"), 16);

	if (!strncmp(if_addr, "", 1))
    {
        return websWrite(wp, T(""));
    }
    else
	{

        fp = fopen("/etc/wan_run_time","r");

        if(NULL == fp)
        {
             return websWrite(wp,T(""));
        }
        else
        {
            struct sysinfo info;
            long curSysUpTime = 0;
            if (0 != sysinfo(&info) )
                perror("sysinfo");
            else
                curSysUpTime = info.uptime;
            fscanf(fp, "%ld,", &totaltime);
            fclose(fp);

            totaltime = curSysUpTime - totaltime;

            if(totaltime <= 0)
            {
                return websWrite(wp,T(""));
            }

            day = (int)totaltime / (3600*24);
            totaltime = (int)totaltime % (3600*24);
            hours =(int) totaltime / 3600;
            totaltime = (int)totaltime % 3600;
            min = (int)totaltime / 60;
            totaltime = (int)totaltime % 60;
            sec = (int)totaltime;

            if(day > 0)
            {
                return websWrite(wp, T("%d day%s, %d hour%s, %d min%s, %d sec%s"),\
                        day, (day <= 1)? "" : "s",hours, (hours <= 1)? "" : "s",\
                        min, (min <= 1)? "" : "s",sec, (sec <= 1)? "" : "s");
            }
            else if(hours > 0)
            {
                return websWrite(wp, T("%d hour%s, %d min%s, %d sec%s"),\
                        hours, (hours <= 1)? "" : "s",\
                        min, (min <= 1)? "" : "s",sec, (sec <= 1)? "" : "s");

            }
            else if(min > 0)
            {
                return websWrite(wp, T("%d min%s, %d sec%s"),\
                        min, (min <= 1)? "" : "s",sec, (sec <= 1)? "" : "s");

            }
            else
            {
                 return websWrite(wp, T("%d sec%s"),sec, (sec <= 1)? "" : "s");
            }
        }
    }

}

/*
 * description: write WAN MAC address accordingly
 */
static int getWanMac(int eid, webs_t wp, int argc, char_t **argv)
{
        char if_mac[18] = {0};
	if(nvram_match("sw_mode_ex", "7")||nvram_match("sw_mode_ex", "8"))
	{
		/*strncpy(if_mac, nvram_safe_get("apcli0_hwaddr"), sizeof(if_mac)-1);
		if (strcmp("", if_mac) == 0) 
		{
			if(-1 == getIfMac("apcli0", if_mac)) 
			{
				return websWrite(wp, T(""));
			}
			else
			{
				nvram_set("apcli0_hwaddr",if_mac);
				nvram_commit();
			}
		}*/
		
		if(-1 == getIfMac("apcli0", if_mac)) 
		{
				return websWrite(wp, T(""));
		}
		
		
	}
	else
	{
	    strncpy(if_mac, nvram_safe_get("wan_hwaddr"), sizeof(if_mac)-1);
	    if (strcmp("", if_mac) == 0) {
	        return websWrite(wp, T(""));
	    }
	}
    return websWrite(wp, T("%s"), if_mac);
}

/*
 * arguments: type - 0 = return WAN interface name (default)
 *                   1 = write WAN interface name
 * description: return or write WAN interface name accordingly
 */
static int getWanIfNameWeb(int eid, webs_t wp, int argc, char_t **argv)
{
    int type;
    char *name = getWanIfName();

    if (ejArgs(argc, argv, T("%d"), &type) == 1) {
        if (1 == type) {
            return websWrite(wp, T("%s"), name);
        }
    }
    ejSetResult(eid, name);
    return 0;
}

/*
 * description: write WAN subnet mask accordingly
 */
static int getWanNetmask(int eid, webs_t wp, int argc, char_t **argv)
{
    char if_net[16] = {0};
    strncpy(if_net, nvram_safe_get("wan_netmask"),sizeof(if_net)-1);
    if ( strcmp("",if_net) == 0) {
        return websWrite(wp, T("0.0.0.0"));
    }
    return websWrite(wp, T("%s"), if_net);

}

//start of that control wan connect on status web by huangfeilan 20110927

static int getWanConnectStatus(int eid, webs_t wp, int argc, char_t **argv)
{
	if(is_phyconnected() >0)
	{
		if(nvram_match("wan_ipaddr",""))
			return websWrite(wp, T("Disconnect"));
		else
			return websWrite(wp, T("Connected"));
	}
	else
	{
		return websWrite(wp, T("Disconnect"));
	}
}

static int getWanConnectButton(int eid, webs_t wp, int argc, char_t **argv)
{
	if(is_phyconnected() >0)
	{
		if(nvram_match("wan_ipaddr",""))
			return websWrite(wp, T("Connect"));
		else
			return websWrite(wp, T("Disconnect"));
	}
	else
	{
		return websWrite(wp, T("Connect"));
	}
}
//end of that control wan connect on status web by huangfeilan 20110927

/*
 * description: write WAN default gateway accordingly
 */
static int getWanGateway(int eid, webs_t wp, int argc, char_t **argv)
{
    char sgw[16] = {0};
    strncpy(sgw, nvram_safe_get("wan_gateway"),sizeof(sgw)-1);
    if ( strcmp("",sgw) == 0) {
        return websWrite(wp, T("0.0.0.0"));
    }
    return websWrite(wp, T("%s"), sgw);
}

int check_apcli_connect()
{
        char long_buf[4096];
	 FILE *fp;
	 memset(long_buf, 0, 4096);
	if(!(fp = popen("iwconfig apcli0", "r")))
		return 0;
	 fread(long_buf, 1, 4096, fp);
	 //dprintf("check apcli0 interface : %s \n\n",long_buf);
	 pclose(fp);

	if(strstr(long_buf,"Not-Associated") != NULL)
	{
              return 0;
	}
	else
	{
              return 1;
	}

	
}

static int getWanLink(int eid, webs_t wp, int argc, char_t **argv)
{
	/*start of add by huangyikun at 20110810*/
	char proto[8] = {0};
	char wan_ifname[8] = {0};
	strncpy(proto, nvram_safe_get("wan_proto"), sizeof(proto)-1);
	strncpy(wan_ifname, nvram_safe_get("wan_ifname"), sizeof(wan_ifname)-1);
	/*end of add by huangyikun at 20110810*/
	int opmode = atoi( nvram_safe_get( "sw_mode_ex" ) );
	if(opmode==1)
	{
	    if(is_phyconnected() >0)
	    {
			#if 0 //disabled by zhadaijin :Solve the problem of status.asp refresh slow slow slow ...
			/*start of add by huangyikun at 20110810*/
		    if((strcmp("l2tp", proto) == 0) || (strcmp("pptp", proto) == 0) || (strcmp("pppoe", proto) == 0))
		    {
				if(!strcmp("pppoe", proto) && nvram_invmatch("wan_auto","1") && discover_pppoe() != 2)
				{
					return websWrite(wp, T("0"));				
				}
				if(strcmp("ppp0", wan_ifname) == 0)
				{

					return websWrite(wp, T("1"));
				}
				else
				{
					return websWrite(wp, T("0"));
				}
		    }
			/*end of add by huangyikun at 20110810*/
			#endif
		
	        return websWrite(wp, T("1"));
	    }
        else if ( (!strcmp("3g", proto)) && !(strcmp("ppp0", wan_ifname) ) )
    	{
    		return websWrite(wp, T("1"));
    	}
	}
	else if (opmode==7 || opmode==8)
	{
		/*start of add by huangyikun at 20110810*/
			if(check_apcli_connect())
			{
		    	if((strcmp("l2tp", proto) == 0) || (strcmp("pptp", proto) == 0) || (strcmp("pppoe", proto) == 0))
		    	{
					if(strcmp("ppp0", wan_ifname) == 0)
					{
						return websWrite(wp, T("1"));
					}
				else
				{
					return websWrite(wp, T("0"));
				}
		    	}
			
			 return websWrite(wp, T("1"));
			}
			/*end of add by huangyikun at 20110810*/
	}
    return websWrite(wp, T("0"));
}


#define DD printf("%d\n", __LINE__);fflush(stdout);

/*
 *
 */
int getIndexOfRoutingRule(char *dest, char *netmask, char *interface)
{
    int index=0;
    char *rrs, one_rule[256];
    char dest_f[32], netmask_f[32], interface_f[32];

    rrs = (char *) nvram_safe_get( "static_route_rules");
    if(!rrs || !strlen(rrs))
        return -1;

    while( getNthValueSafe(index, rrs, ';', one_rule, 256) != -1 ){
        if((getNthValueSafe(0, one_rule, ',', dest_f, sizeof(dest_f)) == -1)){
            index++;
            continue;
        }
        if((getNthValueSafe(1, one_rule, ',', netmask_f, sizeof(netmask_f)) == -1)){
            index++;
            continue;
        }
        if((getNthValueSafe(4, one_rule, ',', interface_f, sizeof(interface_f)) == -1)){
            index++;
            continue;
        }

        if( (!strcmp(dest, dest_f)) && (!strcmp(netmask, netmask_f)) && (!strcmp(interface, interface_f))){
            return index;
        }
        index++;
    }

    return -1;
}

static void removeRoutingRule(char *dest, char *netmask, char *ifname)
{
    char cmd[1024];
    strcpy(cmd, "route del ");

    // host or net?
    if(!strcmp(netmask, "255.255.255.255") )
        strcat(cmd, "-host ");
    else
        strcat(cmd, "-net ");

    // destination
    strcat(cmd, dest);
    strcat(cmd, " ");

    // netmask
    if(strcmp(netmask, "255.255.255.255"))
        sprintf(cmd, "%s netmask %s", cmd, netmask);

    //interface
    sprintf(cmd, "%s dev %s ", cmd, ifname);
    doSystem(cmd);
}

void staticRoutingInit(void)
{
    int index=0;
    char one_rule[256];
    char *rrs;
    struct in_addr dest_s, gw_s, netmask_s;
    char dest[32], netmask[32], gw[32], interface[32], true_interface[32], custom_interface[32], comment[32];
    int    flgs, ref, use, metric, nl=0;
    unsigned long int d,g,m;
    int isGatewayMode = (!strcmp("1", nvram_safe_get( "sw_mode_ex"))) ? 1 : 0 ;

    // delete old user rules
    FILE *fp = fopen("/proc/net/route", "r");
    if(!fp)
        return;

    while (fgets(one_rule, sizeof(one_rule), fp) != NULL) {
        if (nl) {
            if (sscanf(one_rule, "%s%lx%lx%X%d%d%d%lx",
                    interface, &d, &g, &flgs, &ref, &use, &metric, &m) != 8) {
                printf("format error\n");
                fclose(fp);
                return;
            }
            dest_s.s_addr = d;
            gw_s.s_addr = g;
            netmask_s.s_addr = m;

            strncpy(dest, inet_ntoa(dest_s), sizeof(dest));
            strncpy(gw, inet_ntoa(gw_s), sizeof(gw));
            strncpy(netmask, inet_ntoa(netmask_s), sizeof(netmask));

            // check if internal routing rules
            if( (index=getIndexOfRoutingRule(dest, netmask, interface)) != -1){
                removeRoutingRule(dest, netmask, interface);
            }
        }
        nl++;
    }
    fclose(fp);

    index = 0;
    rrs = (char *) nvram_safe_get( "static_route_rules");
    if(!rrs|| !strlen(rrs))
        return;

    while( getNthValueSafe(index, rrs, ';', one_rule, 256) != -1 ){
        char cmd[1024];

        if((getNthValueSafe(0, one_rule, ',', dest, sizeof(dest)) == -1)){
            index++;
            continue;
        }
        if((getNthValueSafe(1, one_rule, ',', netmask, sizeof(netmask)) == -1)){
            index++;
            continue;
        }
        if((getNthValueSafe(2, one_rule, ',', gw, sizeof(gw)) == -1)){
            index++;
            continue;
        }
        if((getNthValueSafe(3, one_rule, ',', interface, sizeof(interface)) == -1)){
            index++;
            continue;
        }
        if((getNthValueSafe(4, one_rule, ',', true_interface, sizeof(true_interface)) == -1)){
            index++;
            continue;
        }
        if((getNthValueSafe(5, one_rule, ',', custom_interface, sizeof(custom_interface)) == -1)){
            index++;
            continue;
        }
        if((getNthValueSafe(6, one_rule, ',', comment, sizeof(comment)) == -1)){
            index++;
            continue;
        }

        strcpy(cmd, "route add ");

        // host or net?
        if(!strcmp(netmask, "255.255.255.255") )
            strcat(cmd, "-host ");
        else
            strcat(cmd, "-net ");

        // destination
        strcat(cmd, dest);
        strcat(cmd, " ");

        // netmask
        if(strcmp(netmask, "255.255.255.255") )
            sprintf(cmd, "%s netmask %s", cmd, netmask);

        // gateway
        if(strcmp(gw, "0.0.0.0"))
            sprintf(cmd, "%s gw %s", cmd, gw);

        sprintf(cmd, "%s dev %s ", cmd, true_interface);

        strcat(cmd, "2>&1 ");

        if(strcmp(interface, "WAN") || (!strcmp(interface, "WAN") && isGatewayMode)  ){
            doSystem(cmd);
        }else{
            printf("Skip WAN routing rule in the non-Gateway mode: %s\n", cmd);
        }

        index++;
    }
    return;
}

static inline int getNums(char *value, char delimit)
{
    char *pos = value;
    int count=1;

    if(!pos || !strlen(pos))
        return 0;
    while( (pos = strchr(pos, delimit))){
        pos = pos+1;
        count++;
    }
    return count;
}

/*
 * description: get routing table
 */
static int getRoutingTable(int eid, webs_t wp, int argc, char_t **argv)
{
    char   result[4096] = {0};
    char   buff[512];
    int    nl = 0, index;
    char   ifname[32], interface[128];
    struct in_addr dest, gw, netmask;
    char   dest_str[32], gw_str[32], netmask_str[32], comment[32];
    int    flgs, ref, use, metric;
    int       *running_rules = NULL;
    unsigned long int d,g,m;
    char *rrs = (char *) nvram_safe_get( "static_route_rules");
    int  rule_count;
    FILE *fp = fopen("/proc/net/route", "r");
    if(!fp)
        return -1;

    rule_count = getNums(rrs, ';');

    if(rule_count){
        running_rules = calloc(1, sizeof(int) * rule_count);
        if(!running_rules)
            return -1;
    }

    strncat(result, "\"", sizeof(result));
    while (fgets(buff, sizeof(buff), fp) != NULL) {
        if (nl) {
            if (sscanf(buff, "%s%lx%lx%X%d%d%d%lx",
                    ifname, &d, &g, &flgs, &ref, &use, &metric, &m) != 8) {
                printf("format error\n");
                fclose(fp);
                return websWrite(wp, T(""));
            }
            dest.s_addr = d;
            gw.s_addr = g;
            netmask.s_addr = m;

            if(! (flgs & 0x1) )    // skip not usable
                continue;

            strncpy(dest_str, inet_ntoa(dest), sizeof(dest_str));
            strncpy(gw_str, inet_ntoa(gw), sizeof(gw_str));
            strncpy(netmask_str, inet_ntoa(netmask), sizeof(netmask_str));

            if(nl > 1)
                strncat(result, ";", sizeof(result));
            strncat(result, ifname, sizeof(result));        strncat(result, ",", sizeof(result));
            strncat(result, dest_str, sizeof(result));        strncat(result, ",", sizeof(result));
            strncat(result, gw_str, sizeof(result));            strncat(result, ",", sizeof(result));
            strncat(result, netmask_str, sizeof(result) );    strncat(result, ",", sizeof(result));
            snprintf(result, sizeof(result), "%s%d,%d,%d,%d,", result, flgs, ref, use, metric);

            // check if internal routing rules
            strcpy(comment, " ");
            if( (index=getIndexOfRoutingRule(dest_str, netmask_str, ifname)) != -1){
                char one_rule[256];

                if(index < rule_count)
                    running_rules[index] = 1;
                else
                    printf("fatal error in %s\n", __FUNCTION__);

                snprintf(result, sizeof(result), "%s%d,", result, index);
                if(rrs && strlen(rrs)){
                    if( getNthValueSafe(index, rrs, ';', one_rule, sizeof(one_rule)) != -1){

                        if( getNthValueSafe(3, one_rule, ',', interface, sizeof(interface)) != -1){
                            strncat(result, interface, sizeof(result));
                            strncat(result, ",", sizeof(result));
                        }
                        if( getNthValueSafe(5, one_rule, ',', comment, sizeof(comment)) != -1){
                            // do nothing;
                        }
                    }
                }
            }else{
                strncat(result, "-1,", sizeof(result));
                strncat(result, getLanWanNamebyIf(ifname), sizeof(result));
                strncat(result, ",", sizeof(result));
            }
            strncat(result, "0,", sizeof(result));    // used rule
            strncat(result, comment, sizeof(result));
        }
        nl++;
    }

    for(index=0; index < rule_count; index++){
        char one_rule[256];

        if(running_rules[index])
            continue;

        if(getNthValueSafe(index, rrs, ';', one_rule, sizeof(one_rule)) == -1)
            continue;

        if(getNthValueSafe(0, one_rule, ',', dest_str, sizeof(dest_str)) == -1)
            continue;

        if(getNthValueSafe(1, one_rule, ',', netmask_str, sizeof(netmask_str)) == -1)
            continue;

        if(getNthValueSafe(2, one_rule, ',', gw_str, sizeof(gw_str)) == -1)
            continue;

        if(getNthValueSafe(3, one_rule, ',', interface, sizeof(interface)) == -1)
            continue;

        if(getNthValueSafe(4, one_rule, ',', ifname, sizeof(ifname)) == -1)
            continue;

        if(getNthValueSafe(5, one_rule, ',', comment, sizeof(comment)) == -1)
            continue;

        if(strlen(result))
            strncat(result, ";", sizeof(result));

        snprintf(result, sizeof(result), "%s%s,%s,%s,%s,0,0,0,0,%d,%s,1,%s", result, ifname, dest_str, gw_str, netmask_str, index, interface, comment);
    }

    strcat(result, "\"");
    websLongWrite(wp, result);
    fclose(fp);
    if(running_rules)
        free(running_rules);

    return 0;
}

static void addRouting(webs_t wp, char_t *path, char_t *query)
{
    char_t *dest, *hostnet, *netmask, *gateway, *itf, *true_interface, *custom_interface, *comment;
    char s_itf[ 32 ] = { 0 };
    char s_buf[ 1024 ] = { 0 };

    FILE *fp = NULL;

    dest = websGetVar(wp, T("dest"), T(""));
    hostnet = websGetVar(wp, T("hostnet"), T(""));
    netmask = websGetVar(wp, T("netmask"), T(""));
    gateway = websGetVar(wp, T("gateway"), T(""));
    itf = websGetVar(wp, T("interface"), T(""));
    custom_interface = websGetVar(wp, T("custom_interface"), T(""));
    comment = websGetVar(wp, T("comment"), T(""));

    if( !dest)
        goto ADD_RULE_END;

    printf( "gw: %s\n", gateway );

    if( !strncasecmp( itf, "LAN", strlen( "LAN" ) ) )
        strncpy( s_itf, nvram_safe_get( "lan_ifname" ), sizeof( s_itf ) - 1 );
    else if( !strncasecmp( itf, "WAN", strlen( "WAN" ) ) )
        strncpy( s_itf, nvram_safe_get( "wan_ifname" ), sizeof( s_itf ) - 1 );
    else
        strncpy( s_itf, custom_interface, sizeof( s_itf ) - 1 );

    strncpy( s_buf, nvram_safe_get( "static_route_rules"), sizeof( s_buf ) - 1 );
    if( strlen( s_buf ) > 2 )
        strcat( s_buf, ";" );
    if( strlen( netmask ) < 2 )
        netmask = "255.255.255.255";
    snprintf( s_buf, sizeof( s_buf ) - 1, "%s%s,%s,%s,%s,%s,%s",
        s_buf, dest, netmask, gateway, itf, s_itf, comment );
    nvram_set( "static_route_rules", s_buf );
    nvram_commit();

    notification_send_rc( OID_STATIC_ROUTE, NOTIFY_WAIT );

ADD_RULE_END:
    websRedirect(wp, "router_list.asp");

    return ;
}

static void delRouting(webs_t wp, char_t *path, char_t *query)
{
    int index, rule_count;
    char_t *value, dest[256], netmask[256], true_interface[256];
    char name_buf[16] = {0};
    char *rrs;
    char *new_rrs;
    int *deleArray, j=0;

    rrs = (char *) nvram_safe_get( "static_route_rules");
    if(!rrs || !strlen(rrs))
        return;

    rule_count = getNums(rrs, ';');
    if(!rule_count)
        return;

    if(!(deleArray = malloc(sizeof(int) * rule_count) ) )
        return;

    if(! (new_rrs = strdup(rrs))){
        free(deleArray);
        return;
    }

#ifdef    GOAHEAD_SHOWSUBMITINFO
    websHeader(wp);
#endif

    for(index=0; index< rule_count; index++){
        snprintf(name_buf, sizeof(name_buf), "DR%d", index);
        value = websGetVar(wp, name_buf, NULL);
        if(value){
            deleArray[j++] = index;
            if(strlen(value) > 256)
                continue;
            sscanf(value, "%s%s%s", dest, netmask, true_interface);
            removeRoutingRule(dest, netmask, true_interface);
#ifdef    GOAHEAD_SHOWSUBMITINFO
            websWrite(wp, T("Delete entry: %s,%s,%s<br>\n"), dest, netmask, true_interface);
#endif
        }
    }

    if(j>0){
        deleteNthValueMulti(deleArray, j, new_rrs, ';');
        nvram_set( "static_route_rules", new_rrs);
        nvram_commit();
    }

#ifdef    GOAHEAD_SHOWSUBMITINFO
    websFooter(wp);
    websDone(wp, 200);
#else
    websRedirect(wp, "router_list.asp");
#endif

    free(deleArray);
    free(new_rrs);
}


#ifdef HAVE_DYNAMIC_ROUTE
static void dynamicRouting(webs_t wp, char_t *path, char_t *query)
{
    char_t *rip,*versionselect,*AuthenticationTypeSelect,*AuthenticationCode;
    const char *RIPEnable;

    versionselect=websGetVar(wp,T("RipVersionSelect"),T(""));
    AuthenticationTypeSelect=websGetVar(wp,T("AuthenticationTypeSelect"),T(""));
    AuthenticationCode=websGetVar(wp,T("KeyCode"),T(""));
    rip = websGetVar(wp, T("RIPSelect"), T(""));

    nvram_set( "rip_enable", rip);
    nvram_set( "rip_version",versionselect);
    nvram_set( "rip_auth_type",AuthenticationTypeSelect);
    nvram_set( "rip_key_code",AuthenticationCode);

    nvram_commit();
    notification_send_rc( OID_DYNAMIC_ROUTE, NOTIFY_WAIT );

    websRedirect(wp, "router_setting.asp");
}
#endif /* HAVE_DYNAMIC_ROUTE */

static void alg(webs_t wp, char_t *path, char_t *query)
{
    char_t *ftp_en, *tftp_en, *h323_en, *sip_en, *pptp_en, *spi_en, *l2tp_en,
           *ipsec_en;

    spi_en = websGetVar(wp, T("spi"), T(""));
    ftp_en = websGetVar(wp, T("ftp_alg"), T(""));
    tftp_en = websGetVar(wp, T("tftp_alg"), T(""));
    h323_en = websGetVar(wp, T("h323_alg"), T(""));
    sip_en = websGetVar(wp, T("sip_alg"), T(""));
    pptp_en = websGetVar(wp, T("pptp_passthrough"), T(""));
    l2tp_en = websGetVar(wp, T("l2tp_passthrough"), T(""));
    ipsec_en = websGetVar(wp, T("ipsec_passthrough"), T(""));


    /* ALG */
    if( atoi( ftp_en ) )
        nvram_set( "alg_ftp_enable", "1");
    else
        nvram_set( "alg_ftp_enable", "0");
    if( atoi( tftp_en ) )
        nvram_set( "alg_tftp_enable", "1");
    else
        nvram_set( "alg_tftp_enable", "0");
    if( atoi( h323_en ) )
        nvram_set( "alg_h323_enable", "1");
    else
        nvram_set( "alg_h323_enable", "0");
    if( atoi( sip_en ) )
        nvram_set( "alg_sip_enable", "1");
    else
        nvram_set( "alg_sip_enable", "0");

    /* VPN passthrough */
    if( atoi( pptp_en ) )
        nvram_set( "alg_pptp_enable", "1");
    else
        nvram_set( "alg_pptp_enable", "0");
    if( atoi( l2tp_en ) )
        nvram_set( "alg_l2tp_enable", "1");
    else
        nvram_set( "alg_l2tp_enable", "0");
    if( atoi( ipsec_en ) )
        nvram_set( "alg_ipsec_enable", "1");
    else
        nvram_set( "alg_ipsec_enable", "0");

    /* SPI */
    if ( atoi( spi_en ))
        nvram_set( "spi_fw_enable", "1");
    else
        nvram_set( "spi_fw_enable", "0");

    nvram_commit();
    notification_send_rc( OID_NETWORK_FIREWALL, NOTIFY_WAIT );
    websRedirect(wp, "safe_basicset.asp");
    return;
}


/*start of add to make sure the wan port is active by zengxianzhu at 20110411*/
int esw_fd;

int
switch_init(void)
{
        esw_fd = socket(AF_INET, SOCK_DGRAM, 0);
        if (esw_fd < 0) {
                perror("socket");
                return -1;
        }
        return 0;
}

void
switch_fini(void)
{
        close(esw_fd);
}

int ra3052_reg_read(int offset, int *value)
{
        struct ifreq ifr;
        esw_reg reg;

        if (value == NULL)
                return -1;
        reg.off = offset;
        strncpy(ifr.ifr_name, "eth2", 5);
        ifr.ifr_data = &reg;
        if (-1 == ioctl(esw_fd, RAETH_ESW_REG_READ, &ifr)) {
                perror("ioctl");
                close(esw_fd);
                return -1;
        }
        *value = reg.val;
        return 0;
}

int is_phyconnected(void)
{
        int val = 0, idx = 1, ret;

        if(switch_init() < 0)
                return 0;

        if(ra3052_reg_read(0x80, &val) != 0)
          {
            switch_fini();
            return 0;
              
          }
        #ifdef CONFIG_WAN_AT_P0
        ret = ((val & (idx << 25))) >> 25;
        #endif

        #ifdef CONFIG_WAN_AT_P4
        ret = ((val & (idx << 29))) >> 29;
        #endif
        
        switch_fini();

        return ret;
}
/*end of add to make sure the wan port is active by zengxianzhu at 20110411*/


static void CheckWanConnection(webs_t wp, char_t *path, char_t *query)
{
/*start of use read reg method to replace ethtool method to check wan status by zengxianzhu at 20110412*/
    if(is_phyconnected() >0)
    {
    	websRedirect(wp, "quick_setup2.asp");
        nvram_set("WanConnection", "1");
    }
    else
    {
        websRedirect(wp, "quick_setup3.asp");
        nvram_set("WanConnection", "0");
    }
  /*end of use read reg method to replace ethtool method to check wan status by zengxianzhu at 20110412*/

            return;
}
static void getMyMAC(webs_t wp, char_t *path, char_t *query)
{
    char myMAC[32];

    arplookup(wp->ipaddr, myMAC);
    websWrite(wp, T("HTTP/1.1 200 OK\nContent-type: text/plain\nPragma: no-cache\nCache-Control: no-cache\n\n"));
    websWrite(wp, T("%s"), myMAC);
    websDone(wp, 200);
}

#ifdef HAVE_UDHCPD
/* goform/dhcpservice */
static void dhcpservice(webs_t wp, char_t *path, char_t *query)
{
    char_t *StartIPAdress, *EndIpAddress, *AddressLease, *GateWay;
    char_t *defaultDomain, *PriDns, *SecondDns, *dchponoff;

    dchponoff = websGetVar(wp, T("dhcpserver"), T("2"));
    if (!strncmp(dchponoff, "0", 2))
        nvram_set( "dhcpd_enable", "0");
    else if (!strncmp(dchponoff, "1", 2)) {
        nvram_set( "dhcpd_enable", "1");
    StartIPAdress= websGetVar(wp, T("ip1"), T(""));
    EndIpAddress = websGetVar(wp, T("ip2"), T(""));
    AddressLease = websGetVar(wp, T("Lease"), T("86400"));
    GateWay = websGetVar(wp, T("gatewaytext"), T(""));
    defaultDomain = websGetVar(wp, T("domain"), T(""));
    PriDns = websGetVar(wp, T("dnsserver1"), T(""));
    SecondDns = websGetVar(wp, T("dnsserver2"), T(""));
        if (-1 == inet_addr(StartIPAdress)) {
            nvram_commit();
            websError(wp, 200, "invalid DHCP Start IP");
            return;
        }
        nvram_set( "dhcp_start", StartIPAdress);

        if (-1 == inet_addr(EndIpAddress)) {
            nvram_commit();
            websError(wp, 200, "invalid DHCP End IP");
            return;
        }
        nvram_set( "dhcp_end", EndIpAddress);
        nvram_set( "dhcp_prim_dns", PriDns);
        nvram_set( "dhcp_sec_dns", SecondDns);
        nvram_set( "dhcp_gateway", GateWay);
        nvram_set( "dhcp_lease", AddressLease);
        nvram_set( "dhcp_domain", defaultDomain);
    }
	
	nvram_set("wan_set", "1");
    nvram_commit();

    notification_send_rc( OID_SERVICE_DHCPD, NOTIFY_WAIT );

    websRedirect(wp, "dhcp_service.asp");
}
#endif

/* goform/setNetworkLan */
static void setNetworkLan(webs_t wp, char_t *path, char_t *query)
{
    char_t *ip, *dhcp_m, *m_mask, *p, *pmode,*mpmode;
    char_t *dhcp_s, *dhcp_e;
	char_t *temp;
    int iplen, lastmode,mlastmode,startip,endip;
    ip = websGetVar(wp, T("lanIp"), T("192.168.1.1"));

    dhcp_m = websGetVar(wp, T("lanmask"), T("2"));

    if (!strncmp(dhcp_m, "0", 2))
    {
        nvram_set( "lan_netmask", "255.0.0.0");
        nvram_set( "lanmask_other", "0");
        nvram_set( "dhcp_netmask", "255.0.0.0");
    }
    else if (!strncmp(dhcp_m, "1", 2))
    {
        nvram_set( "lan_netmask", "255.255.0.0");
        nvram_set( "lanmask_other", "0");
        nvram_set( "dhcp_netmask", "255.255.0.0");
    }
    else if (!strncmp(dhcp_m, "2", 2))
    {
        nvram_set( "lan_netmask", "255.255.255.0");
        nvram_set( "lanmask_other", "0");
        nvram_set( "dhcp_netmask", "255.255.255.0");
    }
    else if (!strncmp(dhcp_m, "3", 2))
    {
        m_mask = websGetVar(wp, T("inputmask"), T("255.255.255.0"));
        nvram_set( "lan_netmask", m_mask);
        nvram_set( "lanmask_other", "1");
        nvram_set( "dhcp_netmask", m_mask);
    }

    nvram_set( "lan_ipaddr", ip);
    nvram_set( "dhcp_prim_dns", ip);
    nvram_set( "dhcp_sec_dns", ip);
    nvram_set( "dhcp_gateway", ip);
    p = (char *)malloc(16);
    dhcp_s = (char *)malloc(16);
    dhcp_e = (char *)malloc(16);
    memset(p, '\0', 16);
    memset(dhcp_s, '\0', 16);
    memset(dhcp_e, '\0', 16);
    iplen = strlen(ip);
    pmode = strrchr(ip, '.');

    pmode ++;
    lastmode = atoi(pmode);
    //lastmode ++;
    while(*(ip+iplen) != '.')
    {
        iplen--;

    }
    iplen++;
    *(ip+iplen) = '\0';
    strncpy(p, ip, iplen);

    //sprintf(pmode, "%d",lastmode);
    strcpy(dhcp_s, p);
    strcpy(dhcp_e, p);

	//start of that perfect LAN setting,set DHCP pool according to lan ip and netmask by huangfeilan 20111227
    if(!strncmp(dhcp_m, "3", 2))
	{		
		mpmode=strrchr(m_mask, '.');
		mpmode ++;
		mlastmode = atoi(mpmode);
		if(mlastmode!=0)
		{
			printf("lastmode=%d,mlastmode=%d\n",lastmode,mlastmode);
			startip=(lastmode & mlastmode)+1;
			printf("startip=%d\n",startip);
			if(startip==255)
				startip=254;
			sprintf(pmode,"%d",startip);
			strncat(dhcp_s, pmode, 3);
			endip=startip+(254-mlastmode-1);
			printf("endip=%d\n",endip);
			sprintf(pmode, "%d",endip);
   			strncat(dhcp_e, pmode, 3);
		}
		else
		{
			strncat(dhcp_s, "100", 3);
   			strncat(dhcp_e, "200", 3);
		}
	}
	else
	{

   		strncat(dhcp_s, "100", 3);
   		strncat(dhcp_e, "200", 3);
	}
	//end of that perfect LAN setting,set DHCP pool according to lan ip and netmask by huangfeilan 20111227


	//end of set  DHCP pool from 100 to 200 by huangfeilan at 20111103

    nvram_set( "dhcp_start", dhcp_s);
    nvram_set( "dhcp_end", dhcp_e);

    free(p);
    free(dhcp_s);
    free(dhcp_e);

    nvram_commit();

    //notification_send_rc( OID_NETWORK_LAN, NOTIFY_WAIT );
    printf("ip has changed. rc restart...\n");
	kill(1, SIGHUP);

    //websWrite(wp, T("HTTP/1.0 200 OK\n"));

}

/*goform/addassignment */
#define MAX_DHCP_STATIC_NUM 25
#ifdef HAVE_WEB_CNS
static void addassignment(webs_t wp, char_t *path, char_t *query)
{
    char_t *dhcp_static_ip, *dhcp_static_mac;
    char_t dhcp_static_RulesStr[1024]={0};
    int i;

    dhcp_static_ip = websGetVar(wp, T("s_ip"), T(""));
    dhcp_static_mac = websGetVar(wp, T("s_mac"), T(""));

    i=0;
    while(i<strlen(dhcp_static_mac))
    {
        if(dhcp_static_mac[i]=='-')
        {
            dhcp_static_mac[i]=':';
        }
        i++;
    }



    if((0 !=strcmp(dhcp_static_ip, "")) && (0 !=strcmp(dhcp_static_mac, ""))){
        strcpy(dhcp_static_RulesStr, nvram_safe_get( "dhcp_static_rules_str"));

        strcat(dhcp_static_RulesStr, dhcp_static_mac);
        strcat(dhcp_static_RulesStr, " ");
        strcat(dhcp_static_RulesStr, dhcp_static_ip);
        strcat(dhcp_static_RulesStr, "|");

        nvram_set( "dhcp_static_rules_str", dhcp_static_RulesStr);
        nvram_commit();

        notification_send_rc( OID_SERVICE_DHCPD, NOTIFY_WAIT );
    }

    websRedirect(wp, "dhcp_assignment.asp");

}

#else
static void addassignment(webs_t wp, char_t *path, char_t *query)
{
    char_t *dhcp_static_ip, *dhcp_static_mac;
    char_t dhcp_static_RulesStr[1024]={0};
    int i;
	/*start of add for edit list bu huangyikun at 20111017 */
	char_t *edit_index, *src, *dest,  list[10][35]={0};
	int index, num;
	/*end of add for edit list bu huangyikun at 20111017 */

    dhcp_static_ip = websGetVar(wp, T("s_ip"), T(""));
    dhcp_static_mac = websGetVar(wp, T("s_mac"), T(""));
	/*start of add for edit list bu huangyikun at 20111017 */
    edit_index = websGetVar(wp, T("edit_index"), T(""));
	index = atoi(edit_index);
	/*end of add for edit list bu huangyikun at 20111017 */
    i=0;
    while(i<strlen(dhcp_static_mac))
    {
        if(dhcp_static_mac[i]=='-')
        {
            dhcp_static_mac[i]=':';
        }
        i++;
    }

    if((0 !=strcmp(dhcp_static_ip, "")) && (0 !=strcmp(dhcp_static_mac, ""))){
        strcpy(dhcp_static_RulesStr, nvram_safe_get( "dhcp_static_rules_str"));
		/*start of add for edit list bu huangyikun at 20111017 */
		if(index == -1) //add
		{
			if(strlen(dhcp_static_RulesStr)==0)
	        	strcpy(dhcp_static_RulesStr, dhcp_static_mac);
			else
	        	strcat(dhcp_static_RulesStr, dhcp_static_mac);
	        strcat(dhcp_static_RulesStr, " ");
	        strcat(dhcp_static_RulesStr, dhcp_static_ip);
	        strcat(dhcp_static_RulesStr, "|");
		}
		else //modify
		{
			src = dhcp_static_RulesStr;
			for(i=0; *src!='\0'; i++)
			{
				dest = list[i];
				while(*src!='|'){
					*dest++ = *src++;
				}
				*dest = *src;
				src++;
			}
			num = i;
			
			memset(list[index],'0',sizeof(list[index]));
			strcpy(list[index], dhcp_static_mac);
			strcat(list[index], " ");
			strcat(list[index], dhcp_static_ip);
			strcat(list[index], "|");
			
			memset(dhcp_static_RulesStr,'0',sizeof(dhcp_static_RulesStr));
			strcpy(dhcp_static_RulesStr, list[0]);
			for(i=1; i<num; i++){
				strcat(dhcp_static_RulesStr, list[i]);
			}
		}
		/*end of add for edit list bu huangyikun at 20111017 */
        nvram_set( "dhcp_static_rules_str", dhcp_static_RulesStr);
        nvram_commit();

        notification_send_rc( OID_SERVICE_DHCPD, NOTIFY_WAIT );
    }

    websRedirect(wp, "dhcp_assignment.asp");

}
#endif

static void delassignment(webs_t wp, char_t *path, char_t *query)
{
    char_t temp_name[16] = {0};
    char_t *temp_delid = NULL;
    int i = 0;

    int *deleArray = NULL;
    int del_rule_count = 0;
    const char_t *dhcp_static_RulesStr = nvram_safe_get( "dhcp_static_rules_str");
    char *new_dhcp_static = NULL;


    /* get delete_item number.*/
    i = 0;
    do{
        memset(temp_name, 0, sizeof(temp_name));
        sprintf(temp_name, "delid%d", i);
        temp_delid = websGetVar(wp, temp_name, T(""));
        if(0 == strcmp(temp_delid, ""))
            break;
        else
            del_rule_count++;

        i++;

    }while(1);

    if(del_rule_count != 0) {
        deleArray = (int *)malloc(del_rule_count * sizeof(int));
        memset(deleArray,0,sizeof(deleArray));

        /* get delete_item index array.*/
        i = 0;
        do{
            memset(temp_name, 0, sizeof(temp_name));
            sprintf(temp_name, "delid%d", i);
            temp_delid = websGetVar(wp, temp_name, T(""));
            if(0 == strcmp(temp_delid, ""))
                break;
            else
                deleArray[i] = atoi(temp_delid);

            i++;

        }while(1);

        if(! (new_dhcp_static = strdup(dhcp_static_RulesStr))){
            free(deleArray);
            return;
        }
        /* delete items */
        deleteNthValueMulti(deleArray, del_rule_count, new_dhcp_static, '|');

        free(deleArray);

        nvram_set( "dhcp_static_rules_str", new_dhcp_static);

        nvram_commit();
        notification_send_rc( OID_SERVICE_DHCPD, NOTIFY_WAIT );
    }

    websRedirect(wp, "dhcp_assignment.asp");

    free(new_dhcp_static);
}

#ifdef HAVE_UPNPD
/* goform/setUpnp */
static void SetUpnp(webs_t wp, char_t *path, char_t *query)
{
    char_t    *upnp_stat;

    upnp_stat = websGetVar(wp, T("upnpstat"), T(""));
    nvram_set( "upnp_enabled", upnp_stat);

    nvram_commit();

    notification_send_rc( OID_SERVICE_UPNP, NOTIFY_WAIT );

	websRedirect(wp, "port_upnp.asp");

    if (upnp_stat && ('1' != upnp_stat[0]))
        unlink("/tmp/pmlist");
}
#endif

#ifdef HAVE_IGMP_PROXY
/*start of add for mulitcast forwarding by zengxianzhu at 20110107*/
static void SetMcast(webs_t wp, char_t *path, char_t *query)
{
    char_t    *mcast_stat;

    mcast_stat = websGetVar(wp, T("mcaststat"), T(""));
    nvram_set( "igmp_enabled", mcast_stat);

    nvram_commit();

    notification_send_rc(OID_SERVICE_IGMP_PROXY, NOTIFY_WAIT);

	websRedirect(wp, "port_multicast.asp");
}
/*end of add for mulitcast forwarding by zengxianzhu at 20110107*/
#endif

static void setMAC(webs_t wp, char_t *path, char_t *query)
{
    char *clone_mac, *clone_en;

    // mac clone
    clone_en = websGetVar(wp, T("macCloneEnbl"), T("0"));
    clone_mac = websGetVar(wp, T("macCloneMac"), T(""));
    nvram_set( "mac_clone_en", clone_en);
    if (!strncmp(clone_en, "1", 2))
        nvram_set( "cl0macaddr", clone_mac);
    else {
		/* start of modify by zhengshan 2011-7-4*/
        nvram_set( "cl0macaddr", "");
		/* end of modify by zhengshan 2011-7-4 */
    }

    nvram_commit();

    notification_send_rc( OID_NETWORK_WAN, NOTIFY_WAIT );

    websRedirect(wp, "mac_clone.asp");

}



//start of that control wan connect on status web by huangfeilan 20110927
static void controlWanConnect( webs_t wp, char_t *path, char_t *query )
{

	nvram_set("set_wan_stop","0");

	nvram_commit();
	notification_send_rc( OID_NETWORK_WAN, NOTIFY_WAIT );
#ifdef HAVE_WEB_CNS
	websRedirect(wp, "status.asp");
#else
    websRedirect(wp, "index.asp");
#endif

}

static void controlWanDisconnect( webs_t wp, char_t *path, char_t *query )
{

	nvram_set("set_wan_stop","1");

	nvram_commit();
	notification_send_rc( OID_NETWORK_WAN, NOTIFY_WAIT );
#ifdef HAVE_WEB_CNS
	websRedirect(wp, "status.asp");
#else
    websRedirect(wp, "index.asp");
#endif

}
//end of that control wan connect on status web by huangfeilan 20110927

/* gofrom/setNetworkWan */
static void setNetworkWan( webs_t wp, char_t *path, char_t *query )
{
    char *p_wn_type = NULL;
    char *p_ip = NULL, *p_mask = NULL, *p_gw = NULL, *p_dns1 = NULL,
        *p_dns2 = NULL, *p_mtu = NULL;
    char *p_user = NULL, *p_pwd = NULL;
	char *p_mtu_type = NULL;
	char *pc_action_mode = NULL;
	char *nPhone = NULL;
	char *wan_auto_mode = NULL;
    const char *opmode = nvram_safe_get( "sw_mode_ex");
    const char *lan_ip = nvram_safe_get( "lan_ipaddr");

	wan_auto_mode = websGetVar(wp,T("AutoDetectMode"),T(""));
	pc_action_mode = websGetVar(wp, "action_mode", "");
	printf("pc_action_mode=%s\n",pc_action_mode);
	nPhone= websGetVar(wp, "nPhone", "");;

	if(!strncmp(wan_auto_mode,"1",2))
		nvram_set("wan_auto","1");
	else 
		nvram_set("wan_auto","0");
		
	if( !strcmp( pc_action_mode, "detect" ) )
	{
#if 1
		int i_type = 0;
		printf( "Let's get start to detect wan connection type ...\n" );
		i_type = is_phyconnected();
		if( i_type == 0 )
		{
			printf( "wan connection type: Unplugged\n" );
			gi_detect_type = DT_UNPLUG;
		}
		else
		{
			i_type = discover_pppoe();
			if( i_type == 2 )
			{
				printf( "wan connection type: pppoe\n" );
				gi_detect_type = DT_PPPOE;
			}
			else
			{
				i_type = discover_all();
				if( i_type == 0 )
				{
					printf( "wan connection type: no-respond\n" );
					gi_detect_type = DT_NORESPOND;
				}
				else if( i_type == 1 )
				{
					printf( "wan connection type: dhcp\n" );
					gi_detect_type = DT_DHCP;
				}
				else if( i_type == 2 )
				{
					printf( "wan connection type: pppoe\n" );
					gi_detect_type = DT_PPPOE;
				}
				else
				{
					printf( "wan connection type: error\n" );
					gi_detect_type = DT_ERROR;
				}
			}
		}
#else
		printf( "Didn't enable this funcation yet.\n" );
#endif

#ifdef DEFINE_PHONE_WEB
		if(!strncmp(nPhone,"1",sizeof(nPhone)-1))
			websRedirect( wp, "phone_wan_dhcp.asp" );
		else
			websRedirect( wp, "wan_dhcp.asp" );
#else
		websRedirect( wp, "wan_dhcp.asp" );	
#endif
		return;
	}
	
    p_wn_type = websGetVar(wp, T("connectionType"), T(""));
    nvram_set( "wan_ipaddr", "");
    nvram_set( "wan_netmask", "");
    nvram_set( "wan_gateway", "");    
    /* ========== DHCP ========== */
    if( !strncmp( p_wn_type, WAN_PROTO_DHCP, strlen( WAN_PROTO_DHCP ) ) )
    {
#ifdef HAVE_UDHCPC
        const char *flag_mode = websGetVar(wp, T("flagMode"), T("0"));
        const char *dns_manual =  websGetVar(wp, T("manual"), T(""));
        p_mtu = websGetVar(wp, T("mtu_dhcp"), T(""));

		p_mtu_type = websGetVar(wp, T("wan_dhcp_mtu_type"), T("0"));
		switch(atoi(p_mtu_type)){
		case 1:
			nvram_set( "mtu_enable", "1");
			break;
		case 0:
		default:
			nvram_set( "mtu_enable", "0");
			break;
		}

        nvram_set( "wan_dhcp_ipaddr", "");
        nvram_set( "wan_dhcp_netmask", "");
        nvram_set( "wan_dhcp_gateway", "");    
        if( NULL != dns_manual && !strncmp( dns_manual, "2", 2 ))
        {
            p_dns1 = websGetVar(wp, T("dnsserver"), T(""));
            p_dns2 = websGetVar(wp, T("dnsserver2"), T(""));
            nvram_set( "wan_dhcp_primary_dns", p_dns1);
            nvram_set( "wan_dhcp_secondary_dns", p_dns2);
            nvram_set( "wan_primary_dns", p_dns1);
            nvram_set( "wan_secondary_dns", p_dns2);
            nvram_set( "wan_dnsmanual", "1" );
#ifdef DEFINE_G4
			nvram_set( "wan_dhcp_dnsmanual","1");
#endif
        }
        else
        {
            nvram_set( "wan_dhcp_primary_dns", "");
            nvram_set( "wan_dhcp_secondary_dns", "");
            nvram_set( "wan_primary_dns", "");
            nvram_set( "wan_secondary_dns", "");
            nvram_set( "wan_dnsmanual", "0" );
#ifdef DEFINE_G4 
			nvram_set( "wan_dhcp_dnsmanual","0");
#endif

		}

        nvram_set( "wan_dhcp_mtu", p_mtu );
        nvram_set( "wan_mtu", p_mtu );
        nvram_set( "wan_dhcp_flag_mode", flag_mode );

        nvram_set( "wan_proto",  p_wn_type );
        nvram_commit();
#endif
    }
    /* ========== STATIC ========== */
    else if( !strncmp( p_wn_type, WAN_PROTO_STATIC, strlen( WAN_PROTO_STATIC ) ) )
    {
        p_ip = websGetVar(wp, T("staticIp"), T(""));
        p_mask = websGetVar(wp, T("staticNetmask"), T("0"));
        p_gw = websGetVar(wp, T("staticGateway"), T(""));
        p_dns1 = websGetVar(wp, T("staticPriDns"), T(""));
        p_dns2 = websGetVar(wp, T("staticSecDns"), T(""));
        p_mtu = websGetVar(wp, T("mtu_static"), T(""));

		p_mtu_type = websGetVar(wp, T("wan_static_mtu_type"), T("0"));
		switch(atoi(p_mtu_type)){
		case 1:
			nvram_set( "mtu_enable", "1");
			break;
		case 0:
		default:
			nvram_set( "mtu_enable", "0");
			break;
		}


        if (-1 == inet_addr( p_ip ) )
        {
            nvram_commit();
            websError(wp, 200, "invalid IP Address");
            return;
        }
        /*
         * lan and wan ip should not be the same except in bridge mode
         */
        if (NULL != opmode && (strcmp(opmode, "2")||strcmp(opmode, "3")||strcmp(opmode, "4")||strcmp(opmode, "5")||strcmp(opmode, "6")) && !strncmp(p_ip, lan_ip, 15))
        {
            nvram_commit();
            websError(wp, 200, "IP address is identical to LAN");
            return;
        }
        nvram_set( "wan_static_ipaddr",  p_ip );
        nvram_set( "wan_ipaddr",  p_ip );
        if( -1 == inet_addr( p_mask ) )
        {
            nvram_commit();
            websError(wp, 200, "invalid Subnet Mask");
            return;
        }
        nvram_set( "wan_static_netmask",  p_mask );
        nvram_set( "wan_netmask",  p_mask );
        /*
         * in Bridge Mode, lan and wan are bridged together and associated with
         * the same ip address
         */
        if (NULL != opmode && (!strcmp(opmode, "2")||!strcmp(opmode, "3")||!strcmp(opmode, "4")||!strcmp(opmode, "5")||!strcmp(opmode, "6")))
        {
            nvram_set( "wan_static_ipaddr",  p_ip );
            nvram_set( "wan_static_netmask",  p_mask );
            nvram_set( "wan_ipaddr",  p_ip );
            nvram_set( "wan_netmask",  p_mask );
        }
        nvram_set( "wan_static_mtu",  p_mtu );
        nvram_set( "wan_static_gateway",  p_gw);
        nvram_set( "wan_static_primary_dns",  p_dns1);
        nvram_set( "wan_static_secondary_dns",  p_dns2);
        nvram_set( "wan_mtu",  p_mtu );
        nvram_set( "wan_gateway",  p_gw);
        nvram_set( "wan_primary_dns",  p_dns1);
        nvram_set( "wan_secondary_dns",  p_dns2);
        nvram_set( "wan_dnsmanual", "1" );

        nvram_set( "wan_proto",  p_wn_type );
        nvram_commit();
    }
    /* ========== PPPOE ========== */
#ifdef HAVE_PPPD
    else if( !strncmp( p_wn_type, WAN_PROTO_PPPOE, strlen( WAN_PROTO_PPPOE ) ) )
    {
        char *pppoe_opmode, *pppoe_optime;
        char *pppoe_starthour,*pppoe_startmin,*pppoe_endthour,*pppoe_endmin;
        char *netsniper_en, *pppoe_srv_name;
        char *autodns;
        p_user = websGetVar(wp, T("pppoeUser"), T(""));
        p_pwd = websGetVar(wp, T("pppoePass"), T(""));
        pppoe_opmode = websGetVar(wp, T("pppoeOPMode"), T(""));
        netsniper_en = websGetVar(wp, T("netsniper_enable"), T(""));
        pppoe_srv_name = websGetVar(wp, T("pppoeSrv_name"), T(""));
        autodns = websGetVar(wp, T("dnsmode"), T("0"));

        if( !strcmp(pppoe_opmode, "on_demand"))
        {
            pppoe_optime = websGetVar(wp, T("pppoeIdleTime"), T(""));
            nvram_set( "wan_pppoe_optime", pppoe_optime);
            nvram_set( "wan_pppoe_opmode", "on_demand" );
        }
        else if( !strcmp(pppoe_opmode, "schedule"))
        {
            pppoe_starthour = websGetVar(wp, T("pppoestarttimehour"), T(""));
            pppoe_startmin = websGetVar(wp, T("pppoestarttimemin"), T(""));
            pppoe_endthour = websGetVar(wp, T("pppoeendtimehour"), T(""));
            pppoe_endmin = websGetVar(wp, T("pppoeendtimemin"), T(""));

            nvram_set( "wan_pppoe_starthour", pppoe_starthour);
            nvram_set( "wan_pppoe_startmin", pppoe_startmin);
            nvram_set( "wan_pppoe_endhour", pppoe_endthour);
            nvram_set( "wan_pppoe_endmin", pppoe_endmin);
            nvram_set( "wan_pppoe_opmode",  "schedule" );
        }
        else if( !strcmp(pppoe_opmode, "keep_alive"))
        {
            nvram_set( "wan_pppoe_opmode",  "keep_alive" );
        }
        if( NULL != pppoe_srv_name )
            nvram_set( "wan_pppoe_srvname", pppoe_srv_name);
        else
            nvram_set( "wan_pppoe_srvname", "");

        nvram_set( "wan_pppoe_user", p_user);
        nvram_set( "wan_pppoe_pwd", p_pwd);
        nvram_set("wan_pppoe_autodns", autodns);

        /* new setting */
        if (0 == strcmp(autodns, "0"))
        {
            p_dns1 = websGetVar(wp, T("dns1"), T(""));
            p_dns2 = websGetVar(wp, T("dns2"), T(""));
            nvram_set( "wan_pppoe_primary_dns", p_dns1);
            nvram_set( "wan_pppoe_secondary_dns", p_dns2);
			nvram_set( "wan_primary_dns", p_dns1);
            nvram_set( "wan_secondary_dns", p_dns2);
            nvram_set( "wan_dnsmanual", "1" );
#ifdef DEFINE_G4
			nvram_set(	"wan_pppoe_dnsmanual","1");
#endif
        }
        else
        {
            nvram_set( "wan_pppoe_primary_dns", "");
            nvram_set( "wan_pppoe_secondary_dns", "");
			nvram_set( "wan_primary_dns", "");
            nvram_set( "wan_secondary_dns", "");
            nvram_set( "wan_dnsmanual", "0" );
#ifdef DEFINE_G4
			nvram_set(	"wan_pppoe_dnsmanual","1");
#endif
        }

        p_mtu = websGetVar(wp, T("mtu_pppoe"), T(""));

		p_mtu_type = websGetVar(wp, T("wan_pppoe_mtu_type"), T("0"));
		switch(atoi(p_mtu_type)){
		case 1:
			nvram_set( "mtu_enable", "1");
			break;
		case 0:
		default:
			nvram_set( "mtu_enable", "0");
			break;
		}

        nvram_set( "wan_pppoe_mtu",  p_mtu );
        nvram_set( "wan_mtu", p_mtu);

        if (!strncmp(netsniper_en, "0", 2))
           nvram_set( "wan_pppoe_netsniper", "1");
        else
           nvram_set( "wan_pppoe_netsniper", "0");

        nvram_set( "wan_proto",  p_wn_type );
        nvram_commit();
    }
#endif
#ifdef HAVE_PPTP_CLIENT
    /* ========== PPTP ========== */
    else if( !strncmp( p_wn_type, WAN_PROTO_PPTP, strlen( WAN_PROTO_PPTP ) ) )
    {
    	char_t *pptp_opmode, *pptp_optime, *pptp_sername,
			   *pptp_dhcp_enable;

        pptp_sername = websGetVar(wp, T("pptpServer"), T(""));
        p_user = websGetVar(wp, T("pptpUser"), T(""));
        p_pwd = websGetVar(wp, T("pptpPass"), T(""));
        pptp_dhcp_enable = websGetVar(wp, T("pptpMode"), T("0"));
        p_ip = websGetVar(wp, T("pptpIp"), T(""));
        p_mask = websGetVar(wp, T("pptpNetmask"), T(""));
        p_gw = websGetVar(wp, T("pptpGateway"), T(""));
        pptp_opmode = websGetVar(wp, T("linktype_pptp"), T(""));
        p_dns1 = websGetVar(wp, T("sta_dns_pptp"), T(""));
        pptp_optime = websGetVar(wp, T("pptp_idletime"), T("15"));

        nvram_set( "wan_proto", p_wn_type);
        nvram_set( "wan_pptp_server", pptp_sername);
        nvram_set( "wan_pptp_user", p_user);
        nvram_set( "wan_pptp_pass", p_pwd);
        nvram_set( "wan_pptp_dhcp", pptp_dhcp_enable);
        nvram_set( "wan_pptp_opmode", pptp_opmode);
        nvram_set( "wan_pptp_optime", pptp_optime);
        if (!strncmp(pptp_dhcp_enable, "1", 2))
		{
            nvram_set( "wan_pptp_ip", p_ip);
            nvram_set( "wan_pptp_netmask", p_mask);
            nvram_set( "wan_pptp_gateway", p_gw);
            nvram_set( "wan_pptp_dns", p_dns1);
			nvram_set( "wan_primary_dns", p_dns1);
            nvram_set( "wan_dnsmanual", "0" );
        }
        else
        {
            nvram_set( "wan_dhcp_primary_dns", "");
            nvram_set( "wan_dhcp_secondary_dns", "");
			nvram_set( "wan_primary_dns", "");
            nvram_set( "wan_secondary_dns", "");
			nvram_set( "wan_dnsmanual", "0" );
        }


        p_mtu = websGetVar(wp, T("mtu_pptp"), T(""));

		p_mtu_type = websGetVar(wp, T("wan_pptp_mtu_type"), T("0"));
		switch(atoi(p_mtu_type)){
		case 1:
			nvram_set( "mtu_enable", "1");
			break;
		case 0:
		default:
			nvram_set( "mtu_enable", "0");
			break;
		}

		
        nvram_set( "wan_mtu", p_mtu);
        /*start of modify wan pptp conflict with pptp pass-throught by zengxianzhu at 20110817*/
        nvram_set( "alg_pptp_enable", "0" );
        nvram_commit();
        /*end of modify wan pptp conflict with pptp pass-throught by zengxianzhu at 20110817*/
    }
#endif
#ifdef HAVE_L2TP
    /* ========== L2TP ========== */
    else if( !strncmp( p_wn_type, WAN_PROTO_L2TP, strlen( WAN_PROTO_L2TP ) ) )
    {
    	char_t *l2tp_opmode, *l2tp_optime, *l2tp_dhcp_enable, *l2tp_srvname;

        l2tp_srvname = websGetVar(wp, T("l2tpServer"), T(""));
        p_user = websGetVar(wp, T("l2tpUser"), T(""));
        p_pwd = websGetVar(wp, T("l2tpPass"), T(""));
        l2tp_dhcp_enable = websGetVar(wp, T("l2tpMode"), T("0"));
        p_ip = websGetVar(wp, T("l2tpIp"), T(""));
        p_mask = websGetVar(wp, T("l2tpNetmask"), T(""));
        p_gw = websGetVar(wp, T("l2tpGateway"), T(""));
        l2tp_opmode = websGetVar(wp, T("linktype_l2tp"), T(""));
        p_dns1 = websGetVar(wp, T("sta_dns_l2tp"), T(""));
        l2tp_optime = websGetVar(wp, T("l2tp_idletime"), T("15"));

        nvram_set( "wan_proto", p_wn_type);
        nvram_set( "wan_l2tp_server", l2tp_srvname);
        nvram_set( "wan_l2tp_user", p_user);
        nvram_set( "wan_l2tp_pass", p_pwd);
        nvram_set( "wan_l2tp_dhcp", l2tp_dhcp_enable);
        nvram_set( "wan_l2tp_opmode", l2tp_opmode);
        nvram_set( "wan_l2tp_optime", l2tp_optime);
        if (!strncmp(l2tp_dhcp_enable, "1", 2)) {
            nvram_set( "wan_l2tp_ip", p_ip);
            nvram_set( "wan_l2tp_netmask", p_mask);
            nvram_set( "wan_l2tp_gateway", p_gw);
            nvram_set( "wan_l2tp_dns", p_dns1);

			nvram_set( "wan_ipaddr", p_ip);
			nvram_set( "wan_netmask", p_mask);
			nvram_set( "wan_gateway", p_gw);
			nvram_set( "wan_primary_dns", p_dns1);
            nvram_set( "wan_secondary_dns", p_dns2);
            nvram_set( "wan_dnsmanual", "0" );
        }
        else
        {
            nvram_set( "wan_dhcp_primary_dns", "");
            nvram_set( "wan_dhcp_secondary_dns", "");
			nvram_set( "wan_primary_dns", "");
            nvram_set( "wan_secondary_dns", "");
			nvram_set( "wan_dnsmanual", "0" );
        }

        p_mtu = websGetVar(wp, T("mtu_l2tp"), T(""));

		p_mtu_type = websGetVar(wp, T("wan_l2tp_mtu_type"), T("0"));
		switch(atoi(p_mtu_type)){
		case 1:
			nvram_set( "mtu_enable", "0");
			break;
		case 0:
		default:
			nvram_set( "mtu_enable", "0");
			break;
		}

		
        nvram_set( "wan_mtu", p_mtu);
        /*start of modify wan l2tp packet will be drop if l2tp passthrought is disable by zengxianzhu at 20110817*/
        nvram_set( "alg_l2tp_enable", "1" );
        nvram_commit();
       /*end of modify wan l2tp packet will be drop if l2tp passthrought is disable by zengxianzhu at 20110817*/
    }
#endif
#ifdef HAVE_BIGPOND
    /* ========== Big Pond ========== */
    else if( !strncmp( p_wn_type, WAN_PROTO_BIGPOND, strlen( WAN_PROTO_BIGPOND ) ) )
    {
    }
#endif
#ifdef HAVE_USB_3G
    /* ========== 3G ========== */
    else if( !strncmp( p_wn_type, WAN_PROTO_3G, strlen( WAN_PROTO_3G ) ) )
    {
		char *OperatorsName3g = NULL, *pinCodeSt = NULL, *pinCode3g = NULL, *dialNum3g = NULL, 
			*APN3g = NULL, *userName3g = NULL, *passWord3g = NULL, *connectMode3g = NULL, *mtu3g = NULL;

		char *idleTime3g = NULL, *state3g = NULL, *net3gMannual = NULL;
		
		OperatorsName3g = websGetVar(wp, T("net3gOperatorsName"), T(""));
		pinCodeSt = websGetVar(wp, T("net3gPinSt"), T(""));
        pinCode3g = websGetVar(wp, T("net3gPinCode"), T(""));
        dialNum3g = websGetVar(wp, T("net3gDialNum"), T(""));
        APN3g = websGetVar(wp, T("net3gAPN"), T(""));
        userName3g = websGetVar(wp, T("net3gUsername"), T(""));
        passWord3g = websGetVar(wp, T("net3gPassword"), T(""));
        connectMode3g = websGetVar(wp, T("net3gConnectMode"), T(""));
		mtu3g = websGetVar(wp, T("net3gMtu"), T(""));
		state3g = websGetVar(wp, T("net3gState"), T(""));
		net3gMannual = websGetVar(wp, T("net3gMannual"), T(""));
		
		if( !strcmp(connectMode3g, "ConnectOnDemand"))
		{
			idleTime3g = websGetVar(wp, T("net3gIdleTime"), T(""));
			nvram_set( "net3gIdleTime", idleTime3g);
			nvram_set( "net3gConnectMode", "ConnectOnDemand" );
		}
		else if( !strcmp(connectMode3g, "KeepAlive"))
		{
			nvram_set( "net3gIdleTime", "0");
			nvram_set( "net3gConnectMode",	"KeepAlive" );
		}
		
		nvram_set( "net3gOperatorsName", OperatorsName3g);
		nvram_set( "net3gUsername", userName3g);
		nvram_set( "net3gPassword", passWord3g);
		nvram_set( "net3gPinSt", pinCodeSt);
		nvram_set( "net3gPinCode", pinCode3g);
		nvram_set( "net3gState", state3g);
		
		if( NULL != net3gMannual && !strncmp( net3gMannual, "1", 2 ))
			nvram_set( "net3gMannual", "1");
		else
			nvram_set( "net3gMannual", "0");
		
        nvram_set( "net3gDialNum", dialNum3g);
		nvram_set( "net3gAPN", APN3g);
        nvram_set( "net3gMtu", mtu3g );
		nvram_set( "wan_proto", p_wn_type);
    }
#endif
    /* ========== Russian PPTP ========== */
    else if( !strncmp( p_wn_type, WAN_PROTO_RU_PPTP, strlen( WAN_PROTO_RU_PPTP ) ) )
    {
    }
    /* ========== Russian L2TP ========== */
    else if( !strncmp( p_wn_type, WAN_PROTO_RU_L2TP, strlen( WAN_PROTO_RU_L2TP ) ) )
    {
    }
    /* ========== PPPOE Plus ========== */
    else if( !strncmp( p_wn_type, WAN_PROTO_PPPOEPLUS, strlen( WAN_PROTO_PPPOEPLUS ) ) )
    {
    }
    /* ========== DHCP Plus ========== */
    else if( !strncmp( p_wn_type, WAN_PROTO_DHCPPLUS, strlen( WAN_PROTO_DHCPPLUS ) ) )
    {
    }
    else
    {
        websHeader(wp);
        websWrite(wp, T("<h2>Unknown Connection Type: %s</h2><br>\n"), p_wn_type);
        websFooter(wp);
        websDone(wp, 200);

        return;
    }
	
	nvram_set("wan_set", "1");
    nvram_commit();

    notification_send_rc( OID_NETWORK_WAN, NOTIFY_WAIT );
	
	if(!strncmp(nPhone,"1",sizeof(nPhone)-1))		
	{
#ifdef HAVE_USB_3G
		if( !strncmp( p_wn_type, WAN_PROTO_3G, strlen( WAN_PROTO_3G ) ) )
			websRedirect( wp, "phone_net3g.asp" );
		else
#endif
		{
			sleep(3);  //more time for dhcp IP return 
			websRedirect( wp, "phone_wan_dhcp.asp" );
		}
	}
	else
	{	
#ifdef HAVE_USB_3G
		if( !strncmp( p_wn_type, WAN_PROTO_3G, strlen( WAN_PROTO_3G ) ) )
			websRedirect( wp, "net3g.asp" );
		else
#endif
		{
			sleep(3);  //more time for dhcp IP return 
			websRedirect( wp, "wan_dhcp.asp" );
		}
	}

	return;
}


static void setModemRoaming( webs_t wp, char_t *path, char_t *query )
{
	char *net3g_roaming_en = NULL;
	
	net3g_roaming_en= websGetVar(wp, "net3g_roaming_en", "1");
	nvram_set( "net3g_roaming_en", net3g_roaming_en);
	nvram_commit();
	
    notification_send_rc( OID_NETWORK_WAN, NOTIFY_WAIT );

	websRedirect( wp, "sys_3G_roaming.asp" );
	return;
}

/* gofrom/setModemPin */
static void setModemPin( webs_t wp, char_t *path, char_t *query )
{
	char *nPhone = NULL;
	
	nPhone= websGetVar(wp, "nPhone", "");
	printf("--nPhone=%s\n",nPhone);
		
#ifdef HAVE_USB_3G
	char *pinCodeSt = NULL, *pinCode3g = NULL;

	pinCodeSt = websGetVar(wp, T("net3gPinSt"), T(""));
    pinCode3g = websGetVar(wp, T("net3gPinCode"), T(""));

	nvram_set( "net3gPinSt", pinCodeSt);
	nvram_set( "net3gPinCode", pinCode3g);

    nvram_commit();

    notification_send_rc( OID_NETWORK_WAN, NOTIFY_WAIT );
	
	if(!strncmp(nPhone,"1",sizeof(nPhone)-1))		
	{
		websRedirect( wp, "phone_PinSet.asp" );
	}
	else
	{	
		websRedirect( wp, "sys_PinSet.asp" );
	}
#endif

	return;
}


static void wan_type_detect( webs_t wp, char_t *path, char_t *query )
{
	if( nvram_match("sw_mode_ex","1")
		&& nvram_invmatch("wan_proto","3g")  //·ģʽ+ APģʽ + 3Gģʽ
		)
	{
		int gi_detect_type_old = atoi(nvram_get("wan_type"));
		printf( "gi_detect_type_old = %d \n" ,gi_detect_type_old);
		int i_type = 0;
		printf( "Let's get start to detect wan connection type ...\n" );
		
		i_type = is_phyconnected();
		if( i_type == 0 )
		{
			printf( "wan connection type: Unplugged\n" );
			nvram_set("wan_proto","");
			gi_detect_type = DT_UNPLUG;
		}
		else
		{
			i_type = discover_pppoe();
			if( i_type == 2 )
			{
				printf( "wan connection type: pppoe\n" );
				gi_detect_type = DT_PPPOE;
				nvram_set("wan_proto",WAN_PROTO_PPPOE);
			}
			else
			{
				i_type = discover_all();
				if( i_type == 0 )
				{
					printf( "wan connection type: no-respond\n" );
					nvram_set("wan_proto",WAN_PROTO_STATIC);
					gi_detect_type = DT_NORESPOND;
				}
				else if( i_type == 1 )
				{
					printf( "wan connection type: dhcp\n" );
					nvram_set("wan_proto",WAN_PROTO_DHCP);
					gi_detect_type = DT_DHCP;
				}
				else if( i_type == 2 )
				{
					printf( "wan connection type: pppoe\n" );
					nvram_set("wan_proto",WAN_PROTO_PPPOE);
					gi_detect_type = DT_PPPOE;
				}
				else
				{
					printf( "wan connection type: error\n" );
					nvram_set("wan_proto",WAN_PROTO_STATIC);
					gi_detect_type = DT_ERROR;
				}
			}
			if(gi_detect_type != gi_detect_type_old)
			{
				notification_send_rc( OID_NETWORK_WAN, NOTIFY_WAIT );
				nvram_set_int("wan_type",gi_detect_type);
				sleep(2);
			}
		}
	}

	
	websWrite(wp, T("HTTP/1.1 200 OK\nContent-type: text/plain\nPragma: no-cache\nCache-Control: no-cache\n\n"));
    websDone(wp, 200);
	return;
}

static void clean3gFlow( webs_t wp, char_t *path, char_t *query )
{
	char_t *Connect3G = NULL;
	Connect3G = websGetVar(wp, T("Connect3G"), T(""));
	if(!strncmp(Connect3G, "cleanflow",sizeof(Connect3G)-1))
	{
		nvram_set_int("rx_month_data", 0);
		nvram_set_int("tx_month_data", 0);
		nvram_set_int("rx_day_data", 0);
		nvram_set_int("tx_day_data", 0);
	}

}

#ifdef HAVE_PPTPD
//goform/setPptpServer
static void setPptpServer(webs_t wp, char_t *path, char_t *query)
{
    char_t *ps_pptpd_en = NULL;
    char_t *ps_pptpd_mppe_en = NULL;

    ps_pptpd_en = websGetVar(wp, T("pptpServerEn"), T("0"));
    ps_pptpd_mppe_en = websGetVar(wp, T("pptpServerEnMppe"), T("0"));

    if( atoi(ps_pptpd_en) == 0 )
    {
        nvram_set( "pptpd_enable", ps_pptpd_en);
    }
    else
    {
        nvram_set( "pptpd_enable", ps_pptpd_en);
        nvram_set( "pptpd_mppe_en", ps_pptpd_mppe_en);
    }
    nvram_commit();

    notification_send_rc( OID_SERVICE_PPTPD, NOTIFY_WAIT );

    websRedirect(wp, "pptp_server.asp");
}

//goform/setPptpServerClientAdd
static void setPptpServerClientAdd(webs_t wp, char_t *path, char_t *query)
{
    char_t *pst_pptpd_user = NULL;
    char_t *pst_pptpd_pass = NULL;
    char_t *pst_pptpd_remote_ip = NULL;
    char *ps_pptpd_user = NULL;
    char *ps_pptpd_pass = NULL;
    char *ps_pptpd_remote_ip = NULL;
    char ca_temp[128] = { 0 };

    pst_pptpd_user = websGetVar(wp, T("pptpServerUser"), T(""));
    pst_pptpd_pass = websGetVar(wp, T("pptpServerPass"), T(""));
    pst_pptpd_remote_ip = websGetVar(wp, T("pptpServerClientIp"), T(""));

    ps_pptpd_user = (char *)nvram_safe_get( "pptpd_user");
    ps_pptpd_pass = (char *)nvram_safe_get( "pptpd_pass");
    ps_pptpd_remote_ip = (char *)nvram_safe_get( "pptpd_remote");

    if( strcmp(ps_pptpd_user, "") == 0)
    {
        nvram_set( "pptpd_user", pst_pptpd_user);
        nvram_set( "pptpd_pass", pst_pptpd_pass);
        nvram_set( "pptpd_remote", pst_pptpd_remote_ip);
    }
    else
    {
        sprintf(ca_temp, "%s:%s", ps_pptpd_user, pst_pptpd_user);
        nvram_set( "pptpd_user", ca_temp);
        sprintf(ca_temp, "%s:%s", ps_pptpd_pass, pst_pptpd_pass);
        nvram_set( "pptpd_pass", ca_temp);
        sprintf(ca_temp, "%s:%s", ps_pptpd_remote_ip, pst_pptpd_remote_ip);
        nvram_set( "pptpd_remote", ca_temp);
    }
    nvram_commit();
    notification_send_rc( OID_SERVICE_PPTPD, NOTIFY_WAIT );
    websRedirect(wp, "pptp_server.asp");
}

//goform/setPptpServerClientDel
static void setPptpServerClientDel(webs_t wp, char_t *path, char_t *query)
{
    char_t *ps_disc_index = NULL;
    char_t *ps_del_user = NULL;
    char_t *ps_rule_num = NULL;
    int i_rule_num = 0, i_list;
    char pa_temp_name[64] = { 0 };
    char *ps_pptpd_user = NULL;
    char *ps_pptpd_pass = NULL;
    char *ps_pptpd_remote_ip = NULL;
    int *deleArray = NULL;
    char *new_user_rules = NULL, *new_pass_rules = NULL, *new_ip_rules = NULL;
    int j = 0;
    char reUserValue[64] = { 0 };
    char reIpValue[64] = { 0 };
    char rePidValue[16] = { 0 };
    char *del_action = websGetVar(wp, T("delAction"), T(""));

    ps_pptpd_user = (char *)nvram_safe_get( "pptpd_user");
    ps_pptpd_pass = (char *)nvram_safe_get( "pptpd_pass");
    ps_pptpd_remote_ip = (char *)nvram_safe_get( "pptpd_remote");
    ps_rule_num = websGetVar(wp, T("ruleNum"), T("0"));
    i_rule_num = atoi(ps_rule_num);

    if( (strcmp(ps_pptpd_user, "") == 0) || (!i_rule_num) )
        return;

    if( !strcmp(del_action, "1") )
    {
        for( i_list = 0, j = 0; i_list < i_rule_num; i_list++ )
        {
            sprintf(pa_temp_name, "disc_%d", i_list);
            ps_del_user = websGetVar(wp, pa_temp_name, T(""));
            if( atoi(ps_del_user) )
            {
                FILE *fp;
                char ca_value[256] = { 0 };

                getNthValueSafe(i_list, ps_pptpd_user, ':', reUserValue, sizeof(reUserValue));
                getNthValueSafe(i_list, ps_pptpd_remote_ip, ':', reIpValue, sizeof(reIpValue));
                if( access("/tmp/pptp_connected", F_OK) == 0 )
                {
                    fp = fopen("/tmp/pptp_connected", "r");
                    while( fgets(ca_value, 256, fp) )
                    {
                        printf("client message:%s\n", ca_value);
                        if( strstr(ca_value, reUserValue)
                            && strstr(ca_value, reIpValue) )
                        {
                            getNthValueSafe(0, ca_value, ' ', rePidValue, sizeof(rePidValue));
                            kill(atoi(rePidValue), SIGTERM);
                            printf("kill pid:%s\n", rePidValue);
                        }
                    }
                }
            }
        }
    }
    else if( !strcmp(del_action, "2") )
    {
        deleArray = (int *)malloc(i_rule_num * sizeof(int));
        if(!deleArray)
            return;

        new_user_rules = strdup(ps_pptpd_user);
        new_pass_rules = strdup(ps_pptpd_pass);
        new_ip_rules = strdup(ps_pptpd_remote_ip);
        if( (!new_user_rules)
            || (!new_pass_rules)
            || (!new_ip_rules) )
        {
            free(deleArray);
            return;
        }

        for( i_list = 0, j = 0; i_list < i_rule_num; i_list++ )
        {
            sprintf(pa_temp_name, "del_%d", i_list);
            ps_del_user = websGetVar(wp, pa_temp_name, T(""));
            if( atoi(ps_del_user) )
            {
                deleArray[j++] = i_list;
                FILE *fp;
                char ca_value[256] = { 0 };

                getNthValueSafe(i_list, ps_pptpd_user, ':', reUserValue, sizeof(reUserValue));
                getNthValueSafe(i_list, ps_pptpd_remote_ip, ':', reIpValue, sizeof(reIpValue));
                if( access("/tmp/pptp_connected", F_OK) == 0 )
                {
                    fp = fopen("/tmp/pptp_connected", "r");
                    while( fgets(ca_value, 256, fp) )
                    {
                        printf("client message:%s\n", ca_value);
                        if( strstr(ca_value, reUserValue)
                            && strstr(ca_value, reIpValue) )
                        {
                            getNthValueSafe(0, ca_value, ' ', rePidValue, sizeof(rePidValue));
                            kill(atoi(rePidValue), SIGTERM);
                            printf("kill pid:%s\n", rePidValue);
                        }
                    }
                }
            }
        }

        deleteNthValueMulti(deleArray, i_rule_num, new_user_rules, ':');
        deleteNthValueMulti(deleArray, i_rule_num, new_pass_rules, ':');
        deleteNthValueMulti(deleArray, i_rule_num, new_ip_rules, ':');
        free(deleArray);

        nvram_set( "pptpd_user", new_user_rules);
        nvram_set( "pptpd_pass", new_pass_rules);
        nvram_set( "pptpd_remote", new_ip_rules);
        nvram_commit();
        free(new_user_rules);
        free(new_pass_rules);
        free(new_ip_rules);

        notification_send_rc( OID_SERVICE_PPTPD, NOTIFY_WAIT );
    }
    websRedirect(wp, "pptp_server.asp");
}
#endif
int killProcByName(char*proc)
{
    FILE *f;
    int ret =0;
    char line[512] = {0};
    char command[128] = {0};

    system("ps > /tmp/ps");
    f = fopen("/tmp/ps","r");
    if(f == NULL)
        return 0;

    while(fgets(line,sizeof(line),f)){
        if(strstr(line,proc)){
         strcat(command, "kill -9 ");
         strcat(command, strtok(line, " "));
         system(command);
            ret=1;
            break;
        }
    }
    fclose(f);
    unlink("/tmp/ps");
    return ret;

}


#ifdef    CONFIG_WAN_AT_P0
#define    WAN_PORT    0
#elif defined(CONFIG_WAN_AT_P4)
#define    WAN_PORT    4
#endif

enum{emWLS_off, emWLS_on}; /* wan link state */

static int getWanLnkState(void)
{
    int link_stat = emWLS_off;

/*start of use read reg method to replace ethtool method to check wan status by zengxianzhu at 20110412*/
    if(is_phyconnected() >0)
    {
        link_stat = emWLS_on;
    }
/*end of use read reg method to replace ethtool method to check wan status by zengxianzhu at 20110412*/
    return link_stat;
}

static void guideWanLnkCheck(webs_t wp, char_t *path, char_t *query)
{
    FILE *fppppoe, *fpudhcpc;
    char spppoe[80];
    char sudhcpc[80];
    websWrite(wp, T("HTTP/1.1 200 OK\nContent-type: text/plain\nPragma: no-cache\nCache-Control: no-cache\n\n"));
    //printf("--------guideWanLnkCheck------");

        doSystem("rm /etc/*detect");

        doSystem("pppoe-discovery -i eth2.2 ");
        doSystem("udhcpc-discovery -i eth2.2 ");

        do{
        fppppoe = fopen("/etc/pppoedetect", "r");
        fpudhcpc = fopen("/etc/udhcpcdetect", "r");
        }
        while((fppppoe==NULL)||(fpudhcpc==NULL));

        fscanf(fppppoe, "%s", spppoe);
        fscanf(fpudhcpc, "%s", sudhcpc);

        if(strstr(spppoe, "pppoe-discovery=success")!=NULL)
            {
            websWrite(wp, T("%s"), "1");
            }

        else if(strstr(sudhcpc, "udhcpc-discovery=success")!=NULL)
            {
            websWrite(wp, T("%s"), "2");
            }

        else if(emWLS_on == getWanLnkState())
            {
            websWrite(wp, T("%s"), "3");
            }
        else {
            websWrite(wp, T("%s"), "0");    /* link off */
            }
        fclose(fppppoe);
        fclose(fpudhcpc);

        websDone(wp, 200);
        return;
}

#ifdef HAVE_SNMPD
/* goform/snmpservice */
static void snmpservice(webs_t wp, char_t *path, char_t *query)
{
    char_t *snmpd_enable, *snmpd_syslocation, *snmpd_syscontact, *snmpd_sysname, *snmpd_rocommunity, *snmpd_rwcommunity, *snmpd_trapsink;

    snmpd_enable = websGetVar(wp, T("snmpserver"), T("1"));
	
    if (!strncmp(snmpd_enable, "0", 2))
    {
        nvram_set( "snmpd_enable", "0");
    }
    else if (!strncmp(snmpd_enable, "1", 2)) 
	{
        nvram_set( "snmpd_enable", "1");
    	snmpd_syslocation= websGetVar(wp, T("location"), T(""));
    	snmpd_syscontact= websGetVar(wp, T("contact"), T(""));
    	snmpd_sysname= websGetVar(wp, T("name"), T(""));
    	snmpd_rocommunity= websGetVar(wp, T("ro_community"), T(""));
    	snmpd_rwcommunity= websGetVar(wp, T("rw_community"), T(""));
    	snmpd_trapsink= websGetVar(wp, T("trapsink"), T(""));
        nvram_set( "snmpd_syslocation", snmpd_syslocation);
        nvram_set( "snmpd_syscontact", snmpd_syscontact);
        nvram_set( "snmpd_sysname", snmpd_sysname);
        nvram_set( "snmpd_rocommunity", snmpd_rocommunity);
        nvram_set( "snmpd_rwcommunity", snmpd_rwcommunity);
        nvram_set( "snmpd_trapsink", snmpd_trapsink);
    }

    nvram_commit();

    notification_send_rc( OID_SERVICE_SNMP, NOTIFY_WAIT );

    websRedirect(wp, "sys_snmp.asp");
}
#endif


#ifdef DEFINE_WIFIDOG
static void wifidogservice(webs_t wp, char_t *path, char_t *query)
{
    char_t *wifidog_enable;

    wifidog_enable = websGetVar(wp, T("wifidogserver"), T("1"));
	
    if (!strncmp(wifidog_enable, "0", 2))
    {
        nvram_set( "wifidog_enable", "0");
    }
    else if (!strncmp(wifidog_enable, "1", 2)) 
	{
		char *wd_mac_white_list, *wd_gatewayid, *wd_webservername, *wd_port, *wd_usermaxinum, *wd_interval, *wd_timeout;
		char *wd_AuthServerHost, *wd_AuthServerPort, *wd_AuthServerSSL, *wd_AuthServerPath;
    	wd_mac_white_list= websGetVar(wp, T("mac_white_list"), T(""));
    	wd_gatewayid= websGetVar(wp, T("gatewayid"), T(""));
    	wd_webservername= websGetVar(wp, T("webservername"), T(""));
    	wd_port= websGetVar(wp, T("port"), T(""));
    	wd_usermaxinum= websGetVar(wp, T("usermaxinum"), T(""));
    	wd_interval= websGetVar(wp, T("interval"), T(""));
		wd_timeout= websGetVar(wp, T("timeout"), T(""));

		wd_AuthServerHost= websGetVar(wp, T("AuthServerHost"), T(""));
		wd_AuthServerPort= websGetVar(wp, T("AuthServerPort"), T(""));
		wd_AuthServerSSL= websGetVar(wp, T("AuthServerSSL"), T("no"));
		wd_AuthServerPath= websGetVar(wp, T("AuthServerPath"), T(""));
		
        nvram_set( "wifidog_enable", "1");
		
        nvram_set( "wd_mac_white_list", wd_mac_white_list);
        nvram_set( "wd_gatewayid", wd_gatewayid);
        nvram_set( "wd_webservername", wd_webservername);
        nvram_set( "wd_port", wd_port);
        nvram_set( "wd_usermaxinum", wd_usermaxinum);
        nvram_set( "wd_interval", wd_interval);
		nvram_set( "wd_timeout", wd_timeout);

		nvram_set( "wd_AuthServerHost", wd_AuthServerHost);
		nvram_set( "wd_AuthServerPort", wd_AuthServerPort);
		nvram_set( "wd_AuthServerSSL", wd_AuthServerSSL);
		nvram_set( "wd_AuthServerPath", wd_AuthServerPath);
    }

    nvram_commit();

    notification_send_rc( OID_SERVICE_WIFIDOG, NOTIFY_WAIT );

    websRedirect(wp, "sys_wifidog.asp");
}
#endif

