#include    <stdlib.h>
#include    <sys/ioctl.h>
#include    <net/if.h>
#include    <net/route.h>
#include    <string.h>
#include    <dirent.h>
#include	<sys/socket.h>
#include <netinet/in.h>
#include	<arpa/inet.h>
#include    <net_conn.h>

#include    "internet.h"
#include    "bcmnvram.h"
#include    "webs.h"
#include    "config/autoconf.h" //user config
#include	"ip_control.h"

//#define _M_DEBUG
#ifdef _M_DEBUG
#define mdbg(format, args...) printf("[%d:%s()]=>"format, __LINE__, __FUNCTION__, ##args)
#else
#define mdbg(format, args...)
#endif

#define	IPV4_MASK_SIZE		4
#define	IP_MASK_SIZE	IPV4_MASK_SIZE

#define	CMD_BUF_SIZE	256

#define doSystem(fmt ...)	do \
{	\
	char cmd[CMD_BUF_SIZE] = {0};	\
	snprintf(cmd, sizeof(cmd)-1, fmt);	\
	system(cmd);	\
}while(0)

typedef enum {
	RF_UNKNOWN = 0,
	RF_UPLOAD,
	RF_DOWNLOAD,
	RF_MAX
} enRuleFor;
#define	QOS_MANGLE_UPLOAD_CHAIN	"qos_mangle_upload"
#define	QOS_MANGLE_DOWNLOAD_CHAIN	"qos_mangle_download"

static void QoS_ClearRules(void)
{
//	doSystem("tc qdisc del dev ppp0   root 1>/dev/null 2>&1");
//	doSystem("tc qdisc del dev eth2   root 1>/dev/null 2>&1");
//	doSystem("tc qdisc del dev ra0    root 1>/dev/null 2>&1");
//	doSystem("tc qdisc del dev apcli0 root 1>/dev/null 2>&1");

	/* clear uplink rules */
	doSystem("tc qdisc del dev eth2.2 root 1>/dev/null 2>&1");
	/* clear downlink rules */
	doSystem("tc qdisc del dev br0  root 1>/dev/null 2>&1");

	doSystem("iptables -t mangle -D PREROUTING -j %s 1>/dev/null 2>&1", QOS_MANGLE_UPLOAD_CHAIN);
	doSystem("iptables -t mangle -F %s 1>/dev/null 2>&1", QOS_MANGLE_UPLOAD_CHAIN);
	doSystem("iptables -t mangle -X %s 1>/dev/null 2>&1", QOS_MANGLE_UPLOAD_CHAIN);
//	doSystem("iptables -t mangle -N %s 1>/dev/null 2>&1", QOS_MANGLE_UPLOAD_CHAIN);
//	doSystem("iptables -t mangle -A PREROUTING -j %s 1>/dev/null 2>&1", QOS_MANGLE_UPLOAD_CHAIN);

	doSystem("iptables -t mangle -D POSTROUTING -j %s 1>/dev/null 2>&1", QOS_MANGLE_DOWNLOAD_CHAIN);
	doSystem("iptables -t mangle -F %s 1>/dev/null 2>&1", QOS_MANGLE_DOWNLOAD_CHAIN);
	doSystem("iptables -t mangle -X %s 1>/dev/null 2>&1", QOS_MANGLE_DOWNLOAD_CHAIN);
//	doSystem("iptables -t mangle -N %s 1>/dev/null 2>&1", QOS_MANGLE_DOWNLOAD_CHAIN);
//	doSystem("iptables -t mangle -A POSTROUTING -j %s 1>/dev/null 2>&1", QOS_MANGLE_DOWNLOAD_CHAIN);
}

static void QoS_AddChain(void)
{
	doSystem("iptables -t mangle -N %s 1>/dev/null 2>&1", QOS_MANGLE_UPLOAD_CHAIN);
	doSystem("iptables -t mangle -A PREROUTING -j %s 1>/dev/null 2>&1", QOS_MANGLE_UPLOAD_CHAIN);

	doSystem("iptables -t mangle -N %s 1>/dev/null 2>&1", QOS_MANGLE_DOWNLOAD_CHAIN);
	doSystem("iptables -t mangle -A POSTROUTING -j %s 1>/dev/null 2>&1", QOS_MANGLE_DOWNLOAD_CHAIN);
}

/* return different mask bit count */
static int countDiffMaskBits(char *ip1, char *ip2)
{
	struct in_addr addr1, addr2;
	uint32_t net_addr1, net_addr2;
	int i;

	uint32_t totalbits = sizeof(addr1) * 8;
	uint32_t mask = 0x1 << ( totalbits -1 );

	if (-1 == inet_aton(ip1, &addr1) )
		return totalbits;
	if (-1 == inet_aton(ip2, &addr2) )
		return totalbits;

	net_addr1 = htonl((uint32_t)addr1.s_addr);
	net_addr2 = htonl((uint32_t)addr2.s_addr);

	mdbg("addr1:%#x, addr2:%#x\n", net_addr1 , net_addr2);

	for ( i=0; i < totalbits; i++) {
		if ( (net_addr1 & mask ) != (net_addr2 & mask ))
			break;
		mask >>= 1;
	}
	return i;
}

/* rule format:
rule_en, ip_begin_str, ip_end_str, from_port, to_port, protol_str, up_bw_min, up_bw_max, down_bw_min, down_bw_max
like: on 192.168.1.1 192.168.1.2 10 20 all 100 200 300 400
*/
static void QoS_Set_SingleIpMangle(enRuleFor rf,
	char *str_ip_addr_begin, char *str_ip_addr_end,
	int nform_port, int nto_port, char *str_protocol,
	int rule_idx)
{
	char *pIptablesChain = NULL;	/* which chain: QOS_MANGLE_UPLOAD_CHAIN or QOS_MANGLE_DOWNLOAD_CHAIN */
	char *pIpSrcDst = NULL;	/* -s or -d */
	char *pPortSrcDst = NULL;	/* --sport or --dport */
	char *pProtocol = NULL;
	char bufPortRange[32] = {0};
	char bufTemp[32] = {0};
	char bufIpSetCmd[32] = {0};
	char flgHavePortSet = 0;
	struct in_addr addr1, addr2;
	int ip_rang = 0;
	char *pIp = NULL;
	char bufIpAddr[16] = {0};
	struct in_addr addr = {0};

	if (rf >= RF_MAX)
		return;
	if (!str_ip_addr_begin || !str_ip_addr_end || !str_protocol)
		return;

	switch(rf) {
		case RF_UPLOAD:
			pIptablesChain = QOS_MANGLE_UPLOAD_CHAIN;
			pIpSrcDst = "-s";
			pPortSrcDst = "--sport";
			break;
		case RF_DOWNLOAD:
			pIptablesChain = QOS_MANGLE_DOWNLOAD_CHAIN;
			pIpSrcDst = "-d";
			pPortSrcDst = "--dport";
			break;
		default:
			return;
	}

	/* protocol */
	if (str_protocol){
		if ( 0 == strcasecmp(str_protocol, "tcp") ) {
			flgHavePortSet = 1;
			pProtocol = "-p tcp";
		}else if ( 0 == strcasecmp(str_protocol, "udp") ) {
			flgHavePortSet = 1;
			pProtocol = "-p udp";
		}else if ( 0 == strcasecmp(str_protocol, "icmp") )
			pProtocol = "-p icmp";
		else if ( 0 == strcasecmp(str_protocol, "all") )
			pProtocol = "-p all";
		else
			pProtocol = "";
		//strcat(bufCmd, pProtocol);
	}

	/* [--sport/--dport] port1[:/,]port2 */
	if (nform_port && flgHavePortSet) {
		snprintf(bufPortRange, sizeof(bufPortRange)-1, "%s %d", pPortSrcDst, nform_port);
		//strcat(bufCmd, bufChildCmd);
		if (nto_port) {
			snprintf(bufTemp, sizeof(bufTemp)-1, ":%d", nto_port);
			strcat(bufPortRange, bufTemp);
		}
	}

	if (0 < inet_aton(str_ip_addr_begin, &addr1) ) {
		addr = addr1;
		if (0 < inet_aton(str_ip_addr_end, &addr2) )
			ip_rang = ntohl(addr2.s_addr - addr1.s_addr);
	}

	mdbg("ip_range:%d\n", ip_rang);
	/* ڿIPܳȫȫҼδ֪*/
	do {
		if (0 < addr.s_addr) {
			bufIpSetCmd[0] = '\0';
			bufIpAddr[0] = '\0';
			pIp = NULL;

			pIp = (char *)inet_ntoa_r(addr, bufIpAddr);
			//mdbg("ipaddr:%s, ip:%s\n", bufIpAddr, pIp);
			if (pIp && (strlen(pIp) > 7) )
				snprintf(bufIpSetCmd, sizeof(bufIpSetCmd)-1, "%s %s", pIpSrcDst, pIp);
			addr.s_addr = htonl(ntohl(addr.s_addr) + 1);
		}
//iptables -t mangle -A <qos_chain> [-s/-d IP] [-p tcp/udp/icmp/all ] [--sport/--dport prt1:prt2] -j MARK --set-mark 1

		doSystem("iptables -t mangle -A %s %s %s %s -j MARK --set-mark %d",
			pIptablesChain, bufIpSetCmd, pProtocol, bufPortRange, rule_idx);
		doSystem("iptables -t mangle -A %s %s %s %s -j RETURN",
			pIptablesChain, bufIpSetCmd, pProtocol, bufPortRange);

	}while ( ip_rang-- > 0 );

	return;
}

static void QoS_Set_SingleIpBw(
	char *bw_min, char *bw_max,
	int tc_root_idx, int tc_father_idx, char *dev_name,
	int rule_idx)
{
	int cur_idx = (tc_root_idx*100 + tc_father_idx*10 + rule_idx);

/*
tc class add dev eth2.2 parent 1:1 classid 1:111 htb rate 20Mbit ceil 20Mbit burst 15k
tc qdisc add dev eth2.2 parent 1:111 handle 111: sfq perturb 10
tc filter add dev eth2.2 parent 1:0 protocol all prio 1 handle 1 fw classid 1:111
*/
	/* set class */
	doSystem("tc class add dev %s parent %d:%d classid %d:%d htb rate %skbit ceil %skbit burst 15k",
		dev_name,
		tc_root_idx, tc_father_idx,
		tc_root_idx, cur_idx,
		bw_min, bw_max);

	/* qdisc */
	doSystem("tc qdisc add dev %s parent %d:%d handle %d: sfq perturb 10",
		dev_name,
		tc_root_idx, cur_idx,
		cur_idx);

	/* set filter */
	doSystem("tc filter add dev %s parent %d:0 protocol all prio 1 handle %d fw classid %d:%d",
		dev_name, tc_root_idx, rule_idx,
		tc_root_idx, cur_idx);

	return;
}

static int position( const char *ps_src, const char c_mark,
    const int i_idx, char *ps_out, const int i_out_len )
{
    int i = 0;
    char *pc_end = NULL, *pc_begin = NULL, *pc_curr = NULL;

    if( ps_src == NULL || i_idx < 1 || ps_out == NULL || i_out_len < 1 )
        return -1;

    pc_begin = (char *)ps_src;
    pc_end = (char *)ps_src;
    for( i = 0; i < i_idx; ++i)
    {
        pc_begin = pc_end;
        pc_end = strchr( pc_begin + 1, c_mark );
        if( pc_end == NULL )
            break;
    }

    if( pc_end == NULL )
        pc_end = (char *)ps_src + strlen( ps_src );

    i = 0;
	if( pc_begin != ps_src )
		++pc_begin;
    pc_curr = ps_out;
    while( pc_begin != pc_end )
    {
        *pc_curr = *pc_begin;
        if( ( *pc_begin == '\0' ) || ( i >= i_out_len ) )
            break;

        ++i;
        ++pc_curr;
        ++pc_begin;
    }

    if( pc_curr < ( ps_out + i_out_len - 1 ) )
        *pc_curr = '\0';
    else
        *( ps_out + i_out_len - 1 ) = '\0';

    return 0;
}

/*
 tc_parent: is the parent node name which  tc qdisc/class has been created where the sub-ip-rules will hook on;
 dev_name: is tc command needs.
*/
static void QoS_Set_IpRules(int tc_root_idx, int tc_father_idx, char *up_dev_name, char *down_dev_name)
{
	const char *ps_rules = NULL;
	char as_rule_en[ 8 ] = { 0 };
	char as_ip_addr_begin[ 32 ] = { 0 };
	char as_ip_addr_end[ 32 ] = { 0 };
	char as_protocol[ 16 ] = { 0 };
	char as_up_bw_min[ 32 ] = { 0 };
	char as_up_bw_max[ 32 ] = { 0 };
	char as_down_bw_min[ 32 ] = { 0 };
	char as_down_bw_max[ 32 ] = { 0 };
	char as_rule[ 128 ] = { 0 }, *p_ch = NULL;
//	char *pc_itf = NULL, *pc_speed = NULL, s_cmd[ 256 ] = { 0 };
	int i_form_port = 0, i_to_port = 0;
	int i = 0, i_ipctl_count = 0;
	int i_ret = 0, i_rule_en = 0;

	mdbg( "enter.\n" );

	i_ipctl_count = atoi( nvram_safe_get(  NVRAM_IPCTL_COUNT ) );
	mdbg( "i_ipctl_count=%d\n", i_ipctl_count );
	if( i_ipctl_count == 0 || i_ipctl_count < 0 )
		return;

	ps_rules = nvram_safe_get(  NVRAM_IPCTL_RULES );
	mdbg( "ps_rules=%s\n", ps_rules );
	if( !ps_rules || !strlen(ps_rules))
		return;

	mdbg( "explor rules.\n" );
	for( i = 0; i < i_ipctl_count; ++i )
	{
		i_ret = position( (char *)ps_rules, ';', i + 1, as_rule, sizeof( as_rule ) );
		if( i_ret < 0 )
			continue;

		p_ch = as_rule;
		while( *p_ch )
		{
			if( *p_ch == ',' )
				*p_ch = ' ';
			++p_ch;
		}

		/* rule format:
		rule_en, ip_begin, ip_end, from_port, to_port, protol_str, up_bw_min, up_bw_max, down_bw_min, down_bw_max
		*/
		mdbg( "%d: %s\n", i + 1, as_rule );
		i_ret = sscanf( as_rule, "%s %s %s %d %d %s %s %s %s %s",
			as_rule_en, as_ip_addr_begin, as_ip_addr_end, &i_form_port,
			&i_to_port, as_protocol, as_up_bw_min, as_up_bw_max, as_down_bw_min,
			as_down_bw_max );
		mdbg( "i_ret=%d\n", i_ret );
		/* ʽ£i_retڵĲȡͬʱ*/
		if( i_ret != 10 )
		{
			printf( "sscanf fail\n" );
			continue;
		}
		else {
			mdbg( "explor:%s,%s,%s,%d,%d,%s,%s,%s,%s,%s\n",
				as_rule_en, as_ip_addr_begin, as_ip_addr_end, i_form_port,
				i_to_port, as_protocol, as_up_bw_min, as_up_bw_max, as_down_bw_min,
				as_down_bw_max );

			i_rule_en = (0 == strcasecmp(as_rule_en, "on")) ? 1 : 0;
			mdbg("i_rule_en:%d\n", i_rule_en);
			if (i_rule_en) {
				mdbg("(as_up_bw_min):%d, (as_up_bw_max):%d\n",
						atoi(as_up_bw_min), atoi(as_up_bw_max) );
				/* upload */
				if ( (atoi(as_up_bw_min) > 0) && (atoi(as_up_bw_max) > 0) ) {
					QoS_Set_SingleIpBw(
						as_up_bw_min, as_up_bw_max,
						tc_root_idx, tc_father_idx, up_dev_name, i+1);
				mdbg("QoS_Set_SingleIpMangle\n");
					/* set iptables mangle */
					QoS_Set_SingleIpMangle(RF_UPLOAD,
						as_ip_addr_begin, as_ip_addr_end,
						i_form_port, i_to_port, as_protocol, i+1);
				}
				mdbg("(as_down_bw_min):%d, (as_down_bw_max):%d\n",
					atoi(as_down_bw_min), atoi(as_down_bw_max) );
				/* download */
				if ( (atoi(as_down_bw_min) > 0) && (atoi(as_down_bw_max) > 0) ) {
					QoS_Set_SingleIpBw(
						as_down_bw_min, as_down_bw_max,
						tc_root_idx, tc_father_idx, down_dev_name, i+1);
					mdbg("QoS_Set_SingleIpMangle\n");
					/* set iptables mangle */
					QoS_Set_SingleIpMangle(RF_DOWNLOAD,
						as_ip_addr_begin, as_ip_addr_end,
						i_form_port, i_to_port, as_protocol, i+1);
				}
			}
		}
	}

	return;
}

/* set some device Root BW and its ip rules. */
static void QoS_Set_BwRules(int bandwidth, char *dev_name)
{
	/* upload */
	/* set root bandwidth */
	doSystem("tc qdisc add dev %s root handle 1: htb default 2", dev_name);
	/* father: first child */
	doSystem("tc class add dev %s parent 1:0 classid 1:1 htb rate %dkbit ceil %dkbit",
		dev_name, bandwidth, bandwidth);

	/* default bandwidth, un-mark data will go this way. */
	doSystem("tc class add dev %s parent 1:1 classid 1:2 htb rate %dkbit ceil %dkbit",
		dev_name, bandwidth, bandwidth);
	doSystem("tc qdisc add dev %s parent 1:2 handle 102: sfq perturb 10", dev_name);

	return;
}

/*
set management bandwidth rules.  for management, can use full bandwidth.
notice: QoS_Set_ManagementBw must call after root bandwidth setting (here names: QoS_Set_BwRules() )
*/
static void QoS_Set_ManagementBw(char *down_link_dev, const char *pManageIp)
{
	doSystem("tc class add dev %s parent 1:0 classid 1:3 htb rate 100mbit", down_link_dev);
	doSystem("tc filter add dev %s parent 1:0 protocol all prio 1 handle 254 fw classid 1:3", down_link_dev);
	doSystem("iptables -t mangle -A %s -s %s -j MARK --set-mark 254",
		QOS_MANGLE_DOWNLOAD_CHAIN, pManageIp);
	doSystem("iptables -t mangle -A %s -s %s -j RETURN", QOS_MANGLE_DOWNLOAD_CHAIN, pManageIp);
}

int main(void)
{
	const char *qos_en = NULL;
	int iUp_bw, iDown_bw;
	char *up_dev_name = "eth2.2";
	char *down_dev_name = "br0";
	const char *management_ip = NULL;

	QoS_ClearRules();

	qos_en = nvram_safe_get( NVRAM_IPCTL_EN);
         /*start of modify Qos conflict with hw_nat by zengxianzhu at 20110228*/
         #if	defined(CONFIG_RALINK_RT3352_2T2R)  ||defined(CONFIG_RALINK_RT5350_1T1R)
         if (!qos_en || ('1' != qos_en[0]) )
         {
                doSystem("insmod hw_nat.ko");
                return 0;
         }
         else
         {
                doSystem("rmmod hw_nat");
         }
         #endif
        /*end of modify Qos conflict with hw_nat by zengxianzhu at 20110228*/


	QoS_AddChain();

	iUp_bw = atoi( nvram_safe_get( NVRAM_IPCTL_UPBW) );
	if (!iUp_bw)
		iUp_bw = 100 * 1024;	/* 100Mb */

	iDown_bw = atoi( nvram_safe_get( NVRAM_IPCTL_DOWNBW) );
	if (!iDown_bw)
		iDown_bw = 100 * 1024;	/* 100Mb */

	management_ip = nvram_safe_get( "lan_ipaddr");
	if (strlen(management_ip) <= 0) {
		mdbg("Error: management ip can't get from nvram.\n");
		exit(-1);
	}

	mdbg("Upload Bandwidth\n");
	QoS_Set_BwRules(iUp_bw, up_dev_name);

	mdbg("Download Bandwidth\n");
	QoS_Set_BwRules(iDown_bw, down_dev_name);

	mdbg("Management Bandwidth\n");
	QoS_Set_ManagementBw(down_dev_name, management_ip);

	mdbg("IP Rules \n");
	/* root: 1, father: 1 */
	QoS_Set_IpRules(1, 1, up_dev_name, down_dev_name);

	return 0;
}
