#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <arpa/inet.h>
#include "internet.h"
#include	"bcmnvram.h"
#include	"utils.h"
#include	"webs.h"
#include "wireless.h"
#include "product.h"

struct wan_parameter wan_para;
struct wireless_parameter wireless_para;

/*
  *  return -1:  error when parse
  *   return action: parse ok, include no parse
  */
int wan_parse(const char *buf, enum product_action action, struct product_module *me)
{


	unsigned int ip, mask, gateway;

	char *pos = (char *)buf;
	char *pre_pos;

	if ((action == GET_ACTION && me->get_action == NULL) ||
		(action == SET_ACTION && me->set_action == NULL)) {
		return action;
	}
	memset(&wan_para, 0,  sizeof(wan_para));

	//parse ip
	if (memcmp(pos, "ip", 2) != 0)
		goto error;
	pos += 3;
	pre_pos = pos;
	if ((pos = strchr(pos, ':')) != NULL) {
		*pos++ = 0;
		strcpy(wan_para.ip, pre_pos);
	}
	else
		goto error;

	//parse mask
	if (memcmp(pos, "mask", 4) != 0)
		goto error;
	pos += 5;
	pre_pos = pos;
	if ((pos = strchr(pos, ':')) != NULL) {
		*pos++ = 0;
		strcpy(wan_para.mask, pre_pos);
	}
	else
		goto error;



	//parse gateway
	if (memcmp(pos, "gateway", 7) != 0)
		goto error;
	pos += 8;
	pre_pos = pos;
	strcpy(wan_para.gateway, pre_pos);


	DEBUGP("server: parse %s parament, the ip is %s, the mask is %s, the gateway is %s\n",
			me->name, wan_para.ip, wan_para.mask, wan_para.gateway);

	ip = inet_addr(wan_para.ip);
	mask = inet_addr(wan_para.mask);
	gateway = inet_addr(wan_para.gateway);
	if (ip == INADDR_NONE || mask == INADDR_NONE ||
				gateway == INADDR_NONE ) {
		memset(&wan_para, 0, sizeof(wan_para));
		goto error;
	}
	return action;

error:
	printf("server: %s parse parameter error\n", "wan");
	return -1;
}


int wan_set(struct product_module *me)
{
	printf("do actual setting for %s\n", me->name);
	struct wan_parameter *para= (struct wan_parameter*)me->parameter;
	char *ctype = "STATIC";

	//we have check the parameter valid
	nvram_set( "wanConnectionMode", ctype);
	nvram_set( "wan_ipaddr", para->ip);
	nvram_set( "wan_netmask", para->mask);
	nvram_set( "wan_gateway", para->gateway);

	nvram_commit();
	return 1;
}

int wireless_parse(const char *buf, enum product_action action, struct product_module *me)
{
	char *pos = (char *)buf;
	if ((action == GET_ACTION && me->get_action == NULL) ||
		(action == SET_ACTION && me->set_action == NULL)) {
		return action;
	}
	memset(&wireless_para, 0,  sizeof(wireless_para));

	if (memcmp(pos, "wmm_enable", 10) == 0) {
		strcpy(wireless_para.sub_module, "wmm_enable");
		pos += 11;
		strcpy(wireless_para.sub_para, pos);
	}else if (memcmp(pos, "security_enable", 15) == 0) {
		strcpy(wireless_para.sub_module, "security_enable");
		pos += 16;
		strcpy(wireless_para.sub_para, pos);
		//set the security enable 1 is not allowed now!
		if (pos[0] == '1') {
			printf("not support security enable setting yet\n");
			goto error;
		}
	}else {
		goto error;
	}
	printf("%s: parse parameter: the sub module is %s, the para is %s\n",
			me->name, wireless_para.sub_module, wireless_para.sub_para);
	if (wireless_para.sub_para[0] == '1' || wireless_para.sub_para[0] == '0')
		return action;
	else
		goto error;
error:
	return -1;
}

int product_do_wmm_setting(char *buf)
{
	int ssid_num, wlan_mode;
	printf("setting wmm %s\n", (buf[0] == '1' ? "enable": "disable"));
	if (NULL != nvram_safe_get( "BssidNum"))
		ssid_num = atoi(nvram_safe_get( "BssidNum"));
	else
		ssid_num = 1;
	wlan_mode = atoi(nvram_safe_get( "WirelessMode"));
	nvram_set( "WmmCapable", buf);
	if (!strncmp(buf, "1", 2)) {
		if (wlan_mode < 5)
			nvram_set( "TxBurst", "0");
	}
	nvram_commit();
	return 1;
}


int wireless_set(struct product_module *me)
{
	struct wireless_parameter *para = me->parameter;
	if (memcmp(para->sub_module, "wmm_enable", 10) == 0) {
		product_do_wmm_setting(para->sub_para);
	} else if (memcmp(para->sub_module, "security_enable", 15) == 0) {
		product_do_security_setting(para->sub_para);
	} else {
		printf("should never get here, unrecognize sub module\n");
		return -1;
	}
	return 1;
}

typedef int (*T_NULL_FUNC)(struct product_module *me, char *buf);
typedef int (*Tset_action)(struct product_module *me);

struct product_module product_modules[] = {
	{"wan", (void*)&wan_para, wan_parse, NULL, &wan_set, &product_modules[0]},
	{"wireless", (void*)&wireless_para, wireless_parse, NULL, wireless_set, &product_modules[1]}
};


