#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include "translate.h"

extern kw_t dictionary;

void release_dictionary (pkw_t pkw)
{
        if (pkw == NULL)  {
                return;
        }

        pkw->len = pkw->tlen = 0;

        if (pkw->idx != NULL)   {
                free (pkw->idx);
                pkw->idx = NULL;
        }

        if (pkw->buf != NULL)   {
                free (pkw->buf);
                pkw->buf = NULL;
        }
}

int load_dictionary (char *lang, pkw_t pkw)
{
        char dfn[64];
        char *p, *q;
        FILE *dfp = NULL;
        int dict_size = 0;
        struct timeval tv1, tv2;
        const char *eng_dict = "/etc_ro/web/lang/English.dict";

        static char loaded_dict[12] = {0};


        gettimeofday (&tv1, NULL);
        if (lang == NULL || (lang != NULL && strlen (lang) == 0)) {
                snprintf (dfn, sizeof (dfn), eng_dict);
        } else {
                snprintf (dfn, sizeof (dfn), "/etc_ro/web/lang/%s.dict", lang);
        }

        if (strcmp (dfn, loaded_dict) == 0)     {
                return 1;
        }
        release_dictionary (pkw);

        do{
                dfp = fopen (dfn, "r");
                if (dfp != NULL)        {
                        snprintf (loaded_dict, sizeof (loaded_dict), "%s", dfn);
                        break;
                }

                if (dfp == NULL && strcmp (dfn, eng_dict) == 0) {
                        return 0;
                } else {
                        snprintf (dfn, sizeof (dfn), eng_dict);
                }
        } while (1);

        memset (pkw, 0, sizeof (kw_t));
        fseek (dfp, 0L, SEEK_END);
        dict_size = ftell (dfp) + 128;

        REALLOC_VECTOR (pkw->idx, pkw->len, pkw->tlen, sizeof (unsigned char*));
        pkw->buf = q = malloc (dict_size);

        fseek (dfp, 0L, SEEK_SET);

        while (!feof (dfp))     {
                // if pkw->idx is not enough, add 32 item to pkw->idx
                REALLOC_VECTOR (pkw->idx, pkw->len, pkw->tlen, sizeof (unsigned char*));

                fscanf (dfp, "%[^\n]%*c", q);
                if ((p = strchr (q, '=')) != NULL)      {
                        pkw->idx[pkw->len] = q;
                        pkw->len++;
                        q = p + strlen (p);
                        *q = '\0';
                        q++;
                }
        }

        fclose (dfp);
        gettimeofday (&tv2, NULL);

        return 1;
}


char* search_desc (pkw_t pkw, char *name)
{
        int i;
        char *p, *ret = NULL;
	char *temp = NULL;

        if (pkw == NULL || (pkw != NULL && pkw->len <= 0))      {
                return NULL;
        }
        for (i = 0; i < pkw->len; ++i)  {
                p = pkw->idx[i];
                if (strncmp (name, p, strlen (name)) == 0)      {
                        ret = p + strlen (name);
                        break;
                }
        }

	/*ɾĿַеĻַ
	   ֹҳjavascriptֳ﷨*/
	if(ret){
		temp = strchr(ret, '\r');
		if(temp) {
			*temp = '\0';
		}
	}

        return ret;
}


static const char *kw_mark1 = "<#", *kw_mark2 = "#>";

// Call this function if and only if we can read whole <#....#> pattern.
static char *translate_lang (char *s, char *e, webs_t wp, kw_t *pkw)
{
        char *end = NULL, *name = NULL, *desc = NULL;

        if (s == NULL || e == NULL || wp == NULL || pkw == NULL || s >= e)       {
                return NULL;
        }

        for (name = s; name < e; name = end) {
                /* Skip initial whitespace */
                for (; isspace((int)*name); name++);
                if (!(end = strstr(name, kw_mark2)))
                        break;
                *end++ = '=';           // '#' --> '=', search_desc() need '='
                *end++ = '\0';          // '>' --> '\0'

                desc = search_desc (pkw, name);
                if (desc != NULL) {
		websWriteBlock(wp, desc, strlen(desc));
                }

                // skip kw_mark2
                end = e + strlen (kw_mark2);
                break;
        }

        return end;
}


int websWriteBlockWithTranslation(webs_t wp, char_t *buf, int nChars)
{

	char *key = NULL, *key_end = NULL;
	char *start_pat, *end_pat;

	char *pBuf = ballocUniToAsc(buf, nChars);

	start_pat = pBuf;
	end_pat = start_pat + nChars;

	key = strstr (start_pat, kw_mark1);

	while (start_pat < end_pat) {
	        int postproc = 0;
	        char *s, *e, *p;

	        s = start_pat;
	        e = end_pat;

	        if (key != NULL ) {
	                e = key;
	                key_end = strstr (key, kw_mark2);
	                if (key_end != NULL) {
	                        postproc = 1;
	                }
	        } else {
	               postproc = 0;
	        }

		if (e > s) {
			websWriteBlock(wp, s, (size_t) (e - s));
			start_pat = e;
		}


	        // post process
	        p = NULL;
	        if (postproc == 1) {                            // translate
	                p = translate_lang (key + strlen (kw_mark1), key_end, wp, &dictionary);
	                if (p != NULL) {
	                        key = strstr (p, kw_mark1);
	                }

		} else if (postproc == 0) {
	                break;
		}

	        if (p != NULL)  {
	                start_pat = p;
	        }
	}
	bfree(B_L, pBuf);
	return 1;
}


/******************************************************************************/
/*
 *	Do formatted output to the browser. This is the public ASP and form
 *	write procedure when translation is needed.
 */

int websWriteWithTranslation(webs_t wp, char_t *fmt, ...)
{
	va_list		 vargs;
	char_t		*buf;
	int			 rc;

	a_assert(websValid(wp));

	va_start(vargs, fmt);

	buf = NULL;
	rc = 0;

	if (fmtValloc(&buf, WEBS_BUFSIZE, fmt, vargs) >= WEBS_BUFSIZE) {
		trace(0, T("webs: websWrite lost data, buffer overflow\n"));
	}

	va_end(vargs);
	a_assert(buf);
	if (buf) {
		rc = websWriteBlockWithTranslation(wp, buf, gstrlen(buf));
		bfree(B_L, buf);
	}
	return rc;
}

