/* vi: set sw=4 ts=4 sts=4: */
/*
 *	usb.c -- USB Application Settings
 *
 *	Copyright (c) Ralink Technology Corporation All Rights Reserved.
 *
 *	$Id: usb.c,v 1.23 2010-07-23 09:12:59 chhung Exp $
 */

#include	<stdlib.h>
#include	<dirent.h>
#include	<arpa/inet.h>

#include <oid.h>
#include <notify.h>
#include	"webs.h"
#include	"bcmnvram.h"
#include	"usb.h"
#include 	"utils.h"
#include 	"internet.h"
#include	"linux/autoconf.h"  //kernel config
#include	"config/autoconf.h" //user config

#if defined CONFIG_RALINKAPP_MPLAYER
#include	"media.h"
#endif
#if defined HAVE_STORAGE
static void storageAdm(webs_t wp, char_t *path, char_t *query);
static void StorageAddUser(webs_t wp, char_t *path, char_t *query);
static void StorageEditUser(webs_t wp, char_t *path, char_t *query);
static void storageDiskAdm(webs_t wp, char_t *path, char_t *query);
static void storageDiskPart(webs_t wp, char_t *path, char_t *query);
static void setFirstPart(void);
static void storageGetFirmwarePath(webs_t wp, char_t *path, char_t *query);
#endif

#if defined HAVE_FTPD
static void storageFtpSrv(webs_t wp, char_t *path, char_t *query);
#endif

static int GetNthFree(char *entry);
static int ShowPartition(int eid, webs_t wp, int argc, char_t **argv);
static int ShowAllDir(int eid, webs_t wp, int argc, char_t **argv);
static int getCount(int eid, webs_t wp, int argc, char_t **argv);
static int getMaxVol(int eid, webs_t wp, int argc, char_t **argv);
static int getupnpPort(int eid, webs_t wp, int argc, char_t **argv);

#if defined HAVE_SAMBA
static int ShowSmbDir(int eid, webs_t wp, int argc, char_t **argv);
static void storageSmbSrv(webs_t wp, char_t *path, char_t *query);
static void SmbDirAdd(webs_t wp, char_t *path, char_t *query);
static void SmbDirEdit(webs_t wp, char_t *path, char_t *query);
static void SetSambaSrv();
static int fetchSmbConfig(void);
#endif

#if defined HAVE_DOWNLOAD
static void storageDownload(webs_t wp, char_t *path, char_t *query);
#endif

#if defined CONFIG_USB && defined CONFIG_USER_USHARE
static void storageMediaSrv(webs_t wp, char_t *path, char_t *query);
static void MediaDirAdd(webs_t wp, char_t *path, char_t *query);
static int ShowMediaDir(int eid, webs_t wp, int argc, char_t **argv);
static void fetchMediaConfig(void);
static void RunMediaSrv();
#endif

#if defined CONFIG_USB && defined CONFIG_USER_UVC_STREAM
static void webcamra(webs_t wp, char_t *path, char_t *query);
#endif

#if defined CONFIG_USB && defined CONFIG_USER_P910ND
static void printersrv(webs_t wp, char_t *path, char_t *query);
#endif

#if defined CONFIG_USB && CONFIG_RTDEV_USB
static void USBiNIC(webs_t wp, char_t *path, char_t *query);
#endif

#if defined CONFIG_USER_MTDAAPD
static void iTunesSrv(webs_t wp, char_t *path, char_t *query);
#endif

#if defined HAVE_DLNA
	static void dlna(webs_t wp, char_t *path, char_t *query);
#endif


#define	LSDIR_INFO		"/tmp/lsdir"
#define	MOUNT_INFO		"/proc/mounts"

#define USB_STORAGE_PATH    "/media"
#define USB_STORAGE_SIGN    "/media/sd"
#define MIN_SPACE_FOR_FIRMWARE			(1024 * 1024 * 8)// minimum space for firmware upload
#define MIN_FIRMWARE_SIZE				(1048576)		// minium firmware size(1MB)
#define IH_MAGIC			0x27051956					// firmware magic number in header

#define DEBUG(x) do{fprintf(stderr, #x); \
					fprintf(stderr, ": %s\n", x); \
				 }while(0)

char firmware_path[1024] = {0};

void formDefineUSB(void) {
	websAspDefine(T("ShowPartition"), ShowPartition);
	websAspDefine(T("ShowAllDir"), ShowAllDir);
	websAspDefine(T("getCount"), getCount);
	websAspDefine(T("getMaxVol"), getMaxVol);
    websAspDefine(T("getupnpPort"), getupnpPort);

#if defined HAVE_STORAGE
	websFormDefine(T("storageAdm"), storageAdm);
	websFormDefine(T("StorageAddUser"), StorageAddUser);
	websFormDefine(T("StorageEditUser"), StorageEditUser);
	websFormDefine(T("storageDiskAdm"), storageDiskAdm);
	websFormDefine(T("storageDiskPart"), storageDiskPart);
	websFormDefine(T("storageGetFirmwarePath"), storageGetFirmwarePath);
#endif

#if defined HAVE_DOWNLOAD
	websFormDefine(T("storageDownload"), storageDownload);
#endif

#if defined HAVE_FTPD
	websFormDefine(T("storageFtpSrv"), storageFtpSrv);
#endif

#if defined HAVE_SAMBA
       websAspDefine(T("ShowSmbDir"), ShowSmbDir);
	websFormDefine(T("storageSmbSrv"), storageSmbSrv);
	websFormDefine(T("SmbDirAdd"), SmbDirAdd);
	websFormDefine(T("SmbDirEdit"), SmbDirEdit);
#endif

#if defined CONFIG_USB && defined CONFIG_USER_USHARE
       websAspDefine(T("ShowMediaDir"), ShowMediaDir);
	websFormDefine(T("storageMediaSrv"), storageMediaSrv);
	websFormDefine(T("MediaDirAdd"), MediaDirAdd);
#endif

#if defined CONFIG_USB && defined CONFIG_USER_UVC_STREAM
	websFormDefine(T("webcamra"), webcamra);
#endif

#if defined CONFIG_USB && defined CONFIG_USER_P910ND
	websFormDefine(T("printersrv"), printersrv);
#endif

#if defined CONFIG_RTDEV_USB
	websFormDefine(T("USBiNIC"), USBiNIC);
#endif

#if defined CONFIG_USER_MTDAAPD
	websFormDefine(T("iTunesSrv"), iTunesSrv);
#endif

#if defined HAVE_DLNA
	websFormDefine(T("DLNA"), dlna);
#endif


}

#if defined HAVE_DLNA
static void dlna(webs_t wp, char_t *path, char_t *query)
{
	char_t *enable, *dir_path, *name;

	enable = websGetVar(wp, T("DLNAEnabled"), T("0"));
	dir_path = websGetVar(wp, T("DLNAPort"), T(""));

	nvram_set( "dlna_enable", enable);
	nvram_set( "dlna_port", dir_path);
	nvram_commit();
       notification_send_rc( OID_SERVICE_DLNA, NOTIFY_WAIT );

       websRedirect(wp, "dlna.asp");
}
#endif


static int dir_count;
static int part_count;
static int media_dir_count;
static char first_part[13]="/media/sda1";

#if defined HAVE_SAMBA
#define		DIREXIST(bitmap,index)		bitmap[index/32] & 1<<(index%32)?1:0

struct smb_dir {
	char path[40];
	char name[20];
	char permit[1024];
};

struct smb_dir_config {
	int count;
	int bitmap[3];
	struct smb_dir dir_list[100];
};
static struct smb_dir_config smb_conf;
#endif

#if defined CONFIG_USB && defined CONFIG_USER_USHARE
struct media_config {
	char path[40];
};
#endif



#if defined HAVE_STORAGE
static void FormatPart(char *part, char *path)
{
		FILE *fp = NULL;
		char temp[30];

		if (0 == strcmp(part, "") && 0 < strcmp(path, "")) {
			if (NULL == (fp = fopen("/proc/mounts", "r"))) {
				perror(__FUNCTION__);
				return;
			}
			while(EOF != fscanf(fp, "%s %s %*s %*s %*s %*s\n", part, temp))
			{
				if (0 == strcmp(temp, path)) {
					break;
				}
				strcpy(part, "");
			}
			fclose(fp);
		}
		if (0 == strcmp(part, "")) {
				perror(__FUNCTION__);
				return;
		} else {
			char blk_size[30];
			if (NULL == (fp = fopen("/proc/partitions", "r"))) {
				perror(__FUNCTION__);
				return;
			}
			while(EOF != fscanf(fp, "%*s %*s %s %s\n", blk_size, temp)) {
				char temp_part[30];
				sprintf(temp_part, "/dev/%s", temp);
				if (0 == strcmp(part, temp_part)) {
					doSystem("storage.sh format %s %s", part, blk_size);
					break;
				}
			}
			fclose(fp);
		}
#if defined HAVE_SAMBA
		if (0 == strcmp(path, nvram_safe_get("usb_first_part")))
			memset(&smb_conf, 0, sizeof(struct smb_dir_config));
#endif
		return;
}

/* goform/storageAdm */
static void storageAdm(webs_t wp, char_t *path, char_t *query)
{
	char_t *submit;

	submit = websGetVar(wp, T("hiddenButton"), T(""));

	if (0 == strcmp(submit, "delete"))
	{
		char_t *user_select;
		char feild[20];

		feild[0] = '\0';
		user_select = websGetVar(wp, T("storage_user_select"), T(""));
		sprintf(feild, "user%s", user_select);
		nvram_set( feild, "");
		doSystem("rm -rf \"%s/home/%s\"", nvram_safe_get("usb_first_part"), nvram_safe_get( feild));
		sprintf(feild, "user%s_passwd", user_select);
		nvram_set( feild, "");
		sprintf(feild, "ftp_user%s", user_select);
		nvram_set( feild, "");
		sprintf(feild, "smb_user%s", user_select);
		nvram_set( feild, "");
		nvram_commit();
		notification_send_rc( OID_SERVICE_USB_USER, NOTIFY_WAIT );
		//doSystem("storage.sh admin");
	}
	else if (0 == strcmp(submit, "apply"))
	{
		initUSB();
	}
	websRedirect(wp, "STORAGEuser_admin.asp");
}

/* goform/StorageAddUser */
static void StorageAddUser(webs_t wp, char_t *path, char_t *query)
{
	char_t *name, *password;
	char_t *user_ftp_enable, *user_smb_enable;
	char mode[6], feild[20];
	int index;

	mode[0] = '\0';
	feild[0] = '\0';

	name = websGetVar(wp, T("adduser_name"), T(""));
	password = websGetVar(wp, T("adduser_pw"), T(""));
	user_ftp_enable = websGetVar(wp, T("adduser_ftp"), T(""));
	user_smb_enable = websGetVar(wp, T("adduser_smb"), T(""));

	index = GetNthFree("user");

	if (0 != index) {
		sprintf(feild, "user%d", index);
		nvram_set( feild, name);
		sprintf(feild, "user%d_passwd", index);
		nvram_set( feild, password);
		sprintf(feild, "ftp_user%d", index);
		nvram_set( feild, user_ftp_enable);
		sprintf(feild, "smb_user%d", index);
		nvram_set( feild, user_smb_enable);
		nvram_commit();
		//doSystem("storage.sh admin");
		notification_send_rc( OID_SERVICE_USB_USER, NOTIFY_WAIT );

		websRedirect(wp, "STORAGEuser_admin.asp");
	} else {
		websHeader(wp);
		websWrite(wp, T("<h2>User Quota is Full!</h2><br>\n"));
		websFooter(wp);
		websDone(wp, 200);
	}
}

/* goform/StorageEditUser */
static void StorageEditUser(webs_t wp, char_t *path, char_t *query)
{
	char_t *index, *password;
	char_t *user_ftp_enable, *user_smb_enable;
	char feild[20];

	feild[0] = '\0';
	index = websGetVar(wp, T("hiddenIndex"), T(""));
	password = websGetVar(wp, T("edituser_pw"), T(""));
	user_ftp_enable = websGetVar(wp, T("edituser_ftp"), T(""));
	user_smb_enable = websGetVar(wp, T("edituser_smb"), T(""));

	sprintf(feild, "user%s_passwd", index);
	nvram_set( feild, password);
	sprintf(feild, "ftp_user%s", index);
	nvram_set( feild, user_ftp_enable);
	sprintf(feild, "smb_user%s", index);
	nvram_set( feild, user_smb_enable);
	nvram_commit();
	//doSystem("storage.sh admin");
	notification_send_rc( OID_SERVICE_USB_USER, NOTIFY_WAIT );

	websRedirect(wp, "STORAGEuser_admin.asp");
}

static void storageDiskAdm(webs_t wp, char_t *path, char_t *query)
{
	char_t *submit;

	submit = websGetVar(wp, T("hiddenButton"), T(""));
	if (0 == strcmp(submit, "delete"))
	{
		char_t *dir_path = websGetVar(wp, T("dir_path"), T(""));
#if defined HAVE_SAMBA
		FILE *fp = NULL;
		char smb_config_file[25];
		int exist, index = 0;

		sprintf(smb_config_file, "%s/.smb_config", nvram_safe_get("usb_first_part"));
		if (NULL == (fp = fopen(smb_config_file, "w")))
		{
			perror(__FUNCTION__);
			return;
		}
		while(96 > index)
		{
			exist = DIREXIST(smb_conf.bitmap, index);
			if ((exist) &&
				(0 == strcmp(dir_path, smb_conf.dir_list[index].path)))
			{
				smb_conf.bitmap[index/32] &= ~(1<<(index%32));
				smb_conf.count--;
				break;
			}
			index++;
		}
		fwrite(&smb_conf, sizeof(struct smb_dir_config), 1, fp);
		fclose(fp);
#endif
		doSystem("rm -rf \"%s\"", dir_path);
		websRedirect(wp, "STORAGEdisk_admin.asp");
	}
	else if (0 == strcmp(submit, "add"))
	{
		char_t *dir_name, *disk_part;

		dir_name = websGetVar(wp, T("adddir_name"), T(""));
		disk_part = websGetVar(wp, T("disk_part"), T(""));
		doSystem("mkdir -p \"%s/%s\"", disk_part, dir_name);
		doSystem("chmod 777 \"%s/%s\"", disk_part, dir_name);
	}
	else if (0 == strcmp(submit, "format"))
	{
		char_t *disk_part = websGetVar(wp, T("disk_part"), T(""));
		char part[30] = "";
		FormatPart(part, disk_part);
		sleep(5);
		doSystem("storage.sh restart");
		sleep(5);

		websRedirect(wp, "STORAGEdisk_admin.asp");
	}
	else if (0 == strcmp(submit, "remove"))
	{
		FILE *fp_mount = NULL;
		char part[30];

		if (NULL == (fp_mount = fopen("/proc/mounts", "r")))
		{
			perror(__FUNCTION__);
			return;
		}
		while(EOF != fscanf(fp_mount, "%s %*s %*s %*s %*s %*s\n", part))
		{
			if (NULL != strstr(part, "/dev/sd"))
				doSystem("umount -l %s", part);
		}
		fclose(fp_mount);
		websRedirect(wp, "STORAGEdisk_admin.asp");
	}
}

static void storageDiskPart(webs_t wp, char_t *path, char_t *query)
{
	char_t *part1_vol, *part2_vol, *part3_vol;
	int max_part = atoi(websGetVar(wp, T("max_part"), T("")));
	FILE *fp = NULL;
	char part[30];
	if (NULL == (fp = fopen("/proc/mounts", "r")))
	{
		perror(__FUNCTION__);
		return;
	}
	while(EOF != fscanf(fp, "%s %*s %*s %*s %*s %*s\n", part))
	{
		if (NULL != strstr(part, "/dev/sd"))
			doSystem("umount -l %s", part);
	}
	fclose(fp);
	part1_vol = websGetVar(wp, T("part1_vol"), T(""));
	part2_vol = websGetVar(wp, T("part2_vol"), T(""));
	part3_vol = websGetVar(wp, T("part3_vol"), T(""));
	doSystem("storage.sh reparted %s %s %s %d",
			  part1_vol, part2_vol, part3_vol, max_part);
	sleep(50);
	memset(part, 0, 30);
	do {
		sprintf(part, "/dev/sda%d", max_part+4);
		FormatPart(part, "");
		sleep(5);
	} while (max_part--);
	doSystem("storage.sh restart");
	sleep(5);
	websRedirect(wp, "STORAGEdisk_admin.asp");
}

static void storageGetFirmwarePath(webs_t wp, char_t *path, char_t *query)
{
	websWrite(wp, T("HTTP/1.1 200 OK\nContent-type: text/plain\nPragma: no-cache\nCache-Control: no-cache\n\n"));

	websWrite(wp, "%s", firmware_path);
	websDone(wp, 200);
}

static void setFirstPart(void)
{
	FILE *pp = popen("cat /proc/mounts | grep media", "r");
	char dir[13];

	memset(first_part, 0, sizeof(first_part));
	while(EOF != fscanf(pp, "%*s %s %*s %*s %*s %*s\n", dir))
	{
		if (!memcmp(dir, "/media/", 7))
		{
			fprintf(stderr, "get the first partition: %s\n", dir);
			strcpy(first_part, dir);
			nvram_set("usb_first_part",dir);
			break;
		}

	}
	pclose(pp);
}


#endif





#if defined HAVE_FTPD
/* goform/storageFtpSrv */
static void storageFtpSrv(webs_t wp, char_t *path, char_t *query)
{
	char_t *ftp, *name, *anonymous, *port, *max_sessions;
	char_t *adddir, *rename, *remove, *read, *write, *download, *upload;

	ftp = websGetVar(wp, T("ftp_enabled"), T("0"));

	if ( strcmp(ftp, "1") == 0 )
	{
		name = websGetVar(wp, T("ftp_name"), T("0"));
		anonymous = websGetVar(wp, T("ftp_anonymous"), T("0"));
		port = websGetVar(wp, T("ftp_port"), T(""));
		max_sessions = websGetVar(wp, T("ftp_max_sessions"), T(""));
		adddir = websGetVar(wp, T("ftp_adddir"), T("0"));
		rename = websGetVar(wp, T("ftp_rename"), T("0"));
		remove = websGetVar(wp, T("ftp_remove"), T("0"));
		read = websGetVar(wp, T("ftp_read"), T("0"));
		write = websGetVar(wp, T("ftp_write"), T("0"));
		download = websGetVar(wp, T("ftp_download"), T("0"));
		upload = websGetVar(wp, T("ftp_upload"), T("0"));
		
		nvram_set( "ftp_name", name);
		nvram_set( "ftp_anonymous", anonymous);
		nvram_set( "ftp_port", port);
		nvram_set( "ftp_max_sessions", max_sessions);
		nvram_set( "ftp_add_dir", adddir);
		nvram_set( "ftp_rename", rename);
		nvram_set( "ftp_remove", remove);
		nvram_set( "ftp_read", read);
		nvram_set( "ftp_write", write);
		nvram_set( "ftp_download", download);
		nvram_set( "ftp_upload", upload);
	}
	
	nvram_set( "ftp_enabled", ftp);

	nvram_commit();

	
       notification_send_rc( OID_SERVICE_FTPD, NOTIFY_WAIT );
	//doSystem("storage.sh ftp");

	sleep(5);
	websRedirect(wp, "FTPftpsrv.asp");
}
#endif

#if defined HAVE_DOWNLOAD
/* goform/storageDownload */
static void storageDownload(webs_t wp, char_t *path, char_t *query)
{
	char_t *submit;

	submit = websGetVar(wp, T("hiddenButton"), T(""));
	if (0 == strcmp(submit, "apply"))
	{	
		char_t *enable;
		enable = websGetVar(wp, T("dwld_enabled"), T(""));
		nvram_set("dwld_enabled", enable);	
		char_t  *dir, *cmd;
		FILE *fp_mount = fopen(MOUNT_INFO, "r");
		char part[30]; 
		char path[50];
		char *firstpart = NULL;
		if (NULL == fp_mount) {
			nvram_set( "dwld_dir", "Maybe You forgot to plug in storage devices,Please check it.");
			nvram_set( "dwld_cmd", "");
			goto loop;
		}
		while (EOF != fscanf(fp_mount, "%s %s %*s %*s %*s %*s\n", part, path))
		{
			DIR *dp;
			if (0 != strncmp(path, "/media/sd", 9)) {
				continue;
			}
			if (NULL == (dp = opendir(path))) {
				nvram_set( "dwld_dir", "Maybe You forgot to plug in storage devices,Please check it.");
				nvram_set( "dwld_cmd", "");
				goto loop;
			}
			nvram_set("usb_first_part", path);
		}
		if (NULL == (firstpart = nvram_get("usb_first_part"))) {
			nvram_set( "dwld_dir", "Maybe You forgot to plug in storage devices,Please check it.");
			nvram_set( "dwld_cmd", "");
			goto loop;
		}
		dir = websGetVar(wp, T("dwld_dir"), T(""));
		cmd = websGetVar(wp, T("dwld_cmd"), T(""));
	
		nvram_set( "dwld_dir", dir);
		nvram_set( "dwld_cmd", cmd);
loop:	
		nvram_commit();
		notification_send_rc( OID_SERVICE_DOWNLOAD, NOTIFY_WAIT );
		websRedirect(wp, "DOWNload.asp");
	}
}
#endif

#if defined HAVE_SAMBA
/* goform/storageSmbSrv */
static void storageSmbSrv(webs_t wp, char_t *path, char_t *query)
{
	char_t *submit;

	submit =  websGetVar(wp, T("hiddenButton"), T(""));
	if (0 == strcmp(submit, "delete"))
	{
		int index;
		FILE *fp;
		char smb_config_file[25];

		sprintf(smb_config_file, "%s/.smb_config", nvram_safe_get("usb_first_part"));
		fp = fopen(smb_config_file, "w");
		if (NULL == fp) {
			perror(__FUNCTION__);
			return;
		}
		index = atoi(websGetVar(wp, T("selectIndex"), T("")));
		smb_conf.bitmap[index/32] &= ~(1<<(index%32));
		smb_conf.count--;
		fwrite(&smb_conf, sizeof(struct smb_dir_config), 1, fp);
		fclose(fp);
		websRedirect(wp, "SAMBAsmbsrv.asp");
	}
	else if (0 == strcmp(submit, "apply"))
	{
		char_t *smb_enable, *wg, *netbios;

		smb_enable = websGetVar(wp, T("smb_enabled"), T(""));
		wg = websGetVar(wp, T("smb_workgroup"), T(""));
		netbios = websGetVar(wp, T("smb_netbios"), T(""));

		nvram_set( "smb_enabled", smb_enable);
		nvram_set( "smb_hostname", wg);
		nvram_set( "smb_netbios", netbios);
		nvram_commit();
		//SetSambaSrv();
		notification_send_rc( OID_SERVICE_SAMBAD, NOTIFY_WAIT );
		websRedirect(wp, "SAMBAsmbsrv.asp");
	}
}

/* goform/SmbDirAdd */
static void SmbDirAdd(webs_t wp, char_t *path, char_t *query)
{
	char_t *dir_name, *dir_path, *allow_users;
	FILE *fp;
	int exist, index = 0;
	char smb_config_file[25];

	sprintf(smb_config_file, "%s/.smb_config", nvram_safe_get("usb_first_part"));
	fp = fopen(smb_config_file, "w");

	if (NULL == fp) {
		perror(__FUNCTION__);
		return;
	}

	dir_name = websGetVar(wp, T("dir_name"), T(""));
	dir_path = websGetVar(wp, T("dir_path"), T(""));
	allow_users = websGetVar(wp, T("allow_user"), T(""));
	while(96 > index)
	{
		exist = DIREXIST(smb_conf.bitmap, index);
		if (exist && 0 == strcmp(dir_name, smb_conf.dir_list[index].name))
		{
				fprintf(stderr, "Existed Samba Shared Dir: %s\n", dir_name);
				fwrite(&smb_conf, sizeof(struct smb_dir_config), 1, fp);
				fclose(fp);
				return;
		}
		index++;
	}
	index = 0;

	while(96 > index)
	{
		exist = DIREXIST(smb_conf.bitmap, index);
		if (!exist)
		{
			strcpy(smb_conf.dir_list[index].path, dir_path);
			strcpy(smb_conf.dir_list[index].name, dir_name);
			strcpy(smb_conf.dir_list[index].permit, allow_users);
			smb_conf.bitmap[index/32] |= 1<<(index%32);
			smb_conf.count++;
			break;
		}
		index++;
	}

	fwrite(&smb_conf, sizeof(struct smb_dir_config), 1, fp);
	fclose(fp);
}

/* goform/SmbDirEdit */
static void SmbDirEdit(webs_t wp, char_t *path, char_t *query)
{
	char_t *allow_user;
	int index = 0;
	FILE *fp;
	char smb_config_file[25];

	sprintf(smb_config_file, "%s/.smb_config", nvram_safe_get("usb_first_part"));
	fp = fopen(smb_config_file, "w");
	if (NULL == fp) {
		perror(__FUNCTION__);
		return;
	}
	index = atoi(websGetVar(wp, T("hidden_index"), T("")));
	allow_user = websGetVar(wp, T("allow_user"), T(""));
	strcpy(smb_conf.dir_list[index].permit, allow_user);
	fwrite(&smb_conf, sizeof(struct smb_dir_config), 1, fp);
	fclose(fp);
}

static int fetchSmbConfig(void)
{
	FILE *fp = NULL;
	char smb_config_file[25];

	memset(&smb_conf, 0, sizeof(struct smb_dir_config));
	sprintf(smb_config_file, "%s/.smb_config", nvram_safe_get("usb_first_part"));
	if (NULL == (fp = fopen(smb_config_file, "r")))
	{
		return -1;
	}
	fread(&smb_conf, sizeof(struct smb_dir_config), 1, fp);
	fclose(fp);

	return 0;
}

static int ShowSmbDir(int eid, webs_t wp, int argc, char_t **argv)
{
	int exist, i = 0, count = 0;

	if (-1 == fetchSmbConfig())
	{
		return 0;
	}

	websWrite(wp, T("<tr align=\"center\">"));
	websWrite(wp, T("<td>--</td>"));
	websWrite(wp, T("<td>public</td>"));
	websWrite(wp, T("<td>%s/%s</td>"), nvram_safe_get("usb_first_part"), "public");
	websWrite(wp, T("<td>All Users</td>"));
	websWrite(wp, T("</tr>"));
	while(smb_conf.count > 0 && i < 96)
	{
		exist = DIREXIST(smb_conf.bitmap, i);
		if (exist)
		{
			websWrite(wp, T("<tr align=\"center\">"));
			websWrite(wp, T("<td><input type=\"radio\" name=\"smb_dir\" value=\"%s\"></td>"),
					  smb_conf.dir_list[i].name);
			websWrite(wp, T("<td>%s</td>"), smb_conf.dir_list[i].name);
			websWrite(wp, T("<input type=\"hidden\" name=\"smb_dir_path\" value=\"%s\">"),
					  smb_conf.dir_list[i].path);
			websWrite(wp, T("<td>%s</td>"), smb_conf.dir_list[i].path);
			websWrite(wp, T("<input type=\"hidden\" name=\"smb_dir_permit\" value=\"%s\">"),
					  smb_conf.dir_list[i].permit);
			if (0 == strlen(smb_conf.dir_list[i].permit))
				websWrite(wp, T("<td>%s</td>"), "<br />");
			else
				websWrite(wp, T("<td>%s</td>"), smb_conf.dir_list[i].permit);
			websWrite(wp, T("</tr>"));
			count++;
		}
		i++;
	}
	smb_conf.count = count;
	return 0;
}

static void SetSambaSrv()
{
	int i;
    char user_name[64] = {0};
    strncpy(user_name, nvram_safe_get("http_username"), sizeof(user_name)-1);
	const char *admin_id = user_name;
	doSystem("storage.sh samba");
	if (1 == atoi(nvram_safe_get( "smb_enabled")) &&
		strlen(nvram_safe_get("usb_first_part")) > 7) {
		if (fetchSmbConfig() == 0) {
			for(i=0;i<smb_conf.count;i++)
			{
				doSystem("samba_add_dir.sh \"%s\" \"%s\" \"%s\" \"%s\"",
						smb_conf.dir_list[i].name,
						smb_conf.dir_list[i].path,
						admin_id,
						smb_conf.dir_list[i].permit);
			}
		}
		doSystem("nmbd");
		doSystem("smbd");
	}
}
#endif
/*
static void runcurlcmd()
{
	char user_dir[48] = {0};
	char user_cmd[96] = {0};
	strncpy(user_dir, nvram_safe_get("dwld_dir"), sizeof(user_dir)-1);
	strncpy(user_cmd, nvram_safe_get("dwld_cmd"), sizeof(user_cmd)-1);
	const char *dir = user_dir;
	const char *cmd = user_cmd;
	doSystem("curl -o \"%s\" \"%s\"", dir, cmd);
}
*/
#if defined CONFIG_USB && defined CONFIG_USER_USHARE
static struct media_config media_conf[4];
static void storageMediaSrv(webs_t wp, char_t *path, char_t *query)
{
	char_t *submit;

	submit =  websGetVar(wp, T("hiddenButton"), T(""));

	if (0 == strcmp(submit, "delete"))
	{
		int index;
		FILE *fp;
		char media_config_file[25];

		sprintf(media_config_file, "%s/.media_config", nvram_safe_get("usb_first_part"));
		fp = fopen(media_config_file, "w");

		if (NULL == fp) {
			perror(__FUNCTION__);
			return;
		}

		index = atoi(websGetVar(wp, T("media_dir"), T("")));
		memset(&media_conf[index].path, 0, sizeof(media_conf[index].path));
		fwrite(media_conf, sizeof(media_conf), 1, fp);
		fclose(fp);
		websRedirect(wp, "USHAREmediasrv.asp");
	}
	else if (0 == strcmp(submit, "apply"))
	{
		char_t *media_enabled, *media_name;
		int i;

		media_enabled = websGetVar(wp, T("media_enabled"), T(""));
		media_name = websGetVar(wp, T("media_name"), T(""));

		nvram_set( "media_srv_enabled", media_enabled);
		nvram_set( "media_srv_name", media_name);
		nvram_commit();
		if (0 == strcmp(media_enabled, "0"))
			memset(media_conf, 0, sizeof(media_conf));
		RunMediaSrv();

	}
}

static void MediaDirAdd(webs_t wp, char_t *path, char_t *query)
{
	char_t *dir_path;
	FILE *fp;
	char media_config_file[25];
	int index = 0;

	sprintf(media_config_file, "%s/.media_config", nvram_safe_get("usb_first_part"));
	fp = fopen(media_config_file, "w");

	if (NULL == fp) {
		perror(__FUNCTION__);
		return;
	}
	dir_path = websGetVar(wp, T("dir_path"), T(""));
	while (4 > index)
	{
		if (0 == strcmp(dir_path, media_conf[index].path))
		{
			fprintf(stderr, "Existed Media Shared Dir: %s\n", dir_path);
			fwrite(media_conf, sizeof(media_conf), 1, fp);
			fclose(fp);
			return;
		}
		index++;
	}
	index = 0;
	while (4 > index)
	{
		if (0 == strlen(media_conf[index].path))
		{
			strcpy(media_conf[index].path, dir_path);
			break;
		}
		index++;
	}
	if (index == 5)
	{
		perror("Media Server Shared Dirs exceed 4");
		fclose(fp);
		return;
	}
	fwrite(media_conf, sizeof(media_conf), 1, fp);
	fclose(fp);
}

static int ShowMediaDir(int eid, webs_t wp, int argc, char_t **argv)
{
	int index;

	fetchMediaConfig();
	media_dir_count = 0;
	for(index=0;index<4;index++)
	{
		if (0 != strlen(media_conf[index].path))
		{
			websWrite(wp, T("<tr align=\"center\">"));
			websWrite(wp, T("<td><input type=\"radio\" name=\"media_dir\" value=\"%d\"></td>"),
					  index);
			websWrite(wp, T("<td>%s</td>"), media_conf[index].path);
			websWrite(wp, T("</tr>"));
			media_dir_count++;
		}
	}
	return 0;
}

static void fetchMediaConfig(void)
{
	FILE *fp = NULL;
	char media_config_file[25];

	memset(media_conf, 0, sizeof(media_conf));
	sprintf(media_config_file, "%s/.media_config", nvram_safe_get("usb_first_part"));
	if (NULL == (fp = fopen(media_config_file, "r")))
	{
		perror(__FUNCTION__);
		return;
	}
	fread(media_conf, sizeof(media_conf), 1, fp);
	fclose(fp);
}

static void RunMediaSrv()
{
	char mediasrv_dir[160];
	int i;

	memset(mediasrv_dir, 0, sizeof(mediasrv_dir));
	for(i=0;i<4;i++)
		if (0 != strlen(media_conf[i].path))
			sprintf(mediasrv_dir, "%s %s", mediasrv_dir, media_conf[i].path);

	doSystem("storage.sh media \"%s\"", mediasrv_dir);
}
#endif

#if defined CONFIG_USB && defined CONFIG_USER_UVC_STREAM
/* goform/webcamra */
static void webcamra(webs_t wp, char_t *path, char_t *query)
{
	char_t *enable, *resolution, *fps, *port;

	enable = websGetVar(wp, T("enabled"), T(""));
	resolution = websGetVar(wp, T("resolution"), T(""));
	fps = websGetVar(wp, T("fps"), T(""));
	port = websGetVar(wp, T("port"), T(""));

	nvram_set( "webcam_enabled", enable);
	nvram_set( "webcam_resolution", resolution);
	nvram_set( "webcam_fps", fps);
	nvram_set( "webcam_port", port);
	nvram_commit();

	doSystem("killall uvc_stream");
	if (0 == strcmp(enable, "1"))
	{
		doSystem("uvc_stream -r %s -f %s -p %s -b",
				  resolution, fps, port);
	}

}
#endif

#if defined CONFIG_USB && defined CONFIG_USER_P910ND
static void printersrv(webs_t wp, char_t *path, char_t *query)
{
	char_t *enable;

	//start of that insert printer server module by huangfeilan at 20110915

	enable = websGetVar(wp, T("printersrvEnabled"), T(""));

	nvram_set( "printersrv_enabled", enable);
	//end of that insert printer server module by huangfeilan at 20110915
	nvram_commit();
	
	notification_send_rc( OID_SERVICE_PRINTER, NOTIFY_WAIT );

	websRedirect(wp, "P910NDprintersrv.asp");

}
#endif

#if defined CONFIG_RTDEV_USB
static void USBiNIC(webs_t wp, char_t *path, char_t *query)
{
	char_t *enable;

	enable = websGetVar(wp, T("inic_enable"), T(""));
	nvram_set("inic_usb_enable", enable);
	nvram_commit();
}
#endif

int initUSB(void)
{
	printf("\n##### USB init #####\n");
	sleep(1);			// wait for sub-storage initiation
#if defined HAVE_STORAGE
	setFirstPart();
#ifdef HAVE_MINI_HTTPD
    if(access(nvram_safe_get("usb_first_part"), F_OK)==0)
    {
        doSystem("mini_httpd -dd %s &",nvram_safe_get("usb_first_part"));
    }
    else
    {
        doSystem("killall mini_httpd");
    }
#endif
	doSystem("storage.sh admin");
#endif
#if defined HAVE_FTPD
	doSystem("storage.sh ftp");
#endif
#if defined HAVE_SAMBA
	SetSambaSrv();
#endif

#if defined CONFIG_USB && defined CONFIG_USER_USHARE
	fetchMediaConfig();
	RunMediaSrv();
#endif

#if defined CONFIG_USB && defined CONFIG_USER_UVC_STREAM
	printf("UVC init\n");
	const char *webcamebl = nvram_safe_get( "webcam_enabled");
	doSystem("killall uvc_stream");
	if (0 == strcmp(webcamebl, "1"))
	{
	printf("UVC start\n");
		doSystem("uvc_stream -r %s -f %s -p %s -b",
				  nvram_safe_get( "webcam_resolution"),
				  nvram_safe_get( "webcam_fps"),
				  nvram_safe_get( "webcam_port"));
	}
#endif

#if defined CONFIG_USB && defined CONFIG_USER_P910ND
	printf("P910ND init\n");
	const char *printersrvebl = nvram_safe_get( "printersrv_enabled");
	doSystem("killall p910nd");
	if (0 == strcmp(printersrvebl, "1"))
	{
	        printf("P910ND start\n");
		doSystem("p910nd -b -f /dev/lp0");
	}
#endif

#if defined CONFIG_USB && defined CONFIG_USER_MTDAAPD
	doSystem("killall mt-daapd; killall mDNSResponder");
	if (strcmp(nvram_safe_get( "itunes_enable"), "1") == 0)
		doSystem("config-iTunes.sh \"%s\" \"%s\" \"%s\"",
				 nvram_safe_get( "itunes_srvname"),
				 nvram_safe_get( "password"),
				 nvram_safe_get( "itunes_dir"));
#endif

	return 0;
}

static int getCount(int eid, webs_t wp, int argc, char_t **argv)
{
	int type;
	char_t *field;
	char count[3];

	if (2 > ejArgs(argc, argv, T("%d %s"), &type, &field))
	{
		return websWrite(wp, T("Insufficient args\n"));
	}

	if (0 == strcmp(field, "AllDir"))
	{
		sprintf(count, "%d", dir_count);
	}
	else if (0 == strcmp(field, "AllPart"))
	{
		sprintf(count, "%d", part_count);
	}
#if defined HAVE_SAMBA
	else if (0 == strcmp(field, "AllSmbDir"))
	{
		sprintf(count, "%d", smb_conf.count);
	}
#endif
	else if (0 == strcmp(field, "AllMediaDir"))
	{
		sprintf(count, "%d", media_dir_count);
	}

	if (1 == type) {
		if (!strcmp(count, ""))
			return websWrite(wp, T("0"));
		return websWrite(wp, T("%s"), count);
	}
	if (!strcmp(count, ""))
		ejSetResult(eid, "0");
	ejSetResult(eid, count);

	return 0;
}

static int GetNthFree(char *entry)
{
	int index = 1;
	char feild[7];
	const char *user_name;

	feild[0] = '\0';
	do
	{
		sprintf(feild, "%s%d", entry, index);
		user_name = (const char *) nvram_safe_get( feild);
		if (strlen(user_name) == 0)
			return index;
		index++;
	} while (index <= 10);

	return 0;
}

static int ShowAllDir(int eid, webs_t wp, int argc, char_t **argv)
{
	FILE *fp_mount = fopen(MOUNT_INFO, "r");
	char part[30], path[50];
	char dir_name[30];
	int dir_len = 0;

	if (NULL == fp_mount) {
        perror(__FUNCTION__);
		return -1;
	}

	dir_count = 0;

	while(EOF != fscanf(fp_mount, "%s %s %*s %*s %*s %*s\n", part, path))
	{
		DIR *dp;
		struct dirent *dirp;
		struct stat statbuf;

		if (0 != strncmp(path, "/media/sd", 9))
		{
			continue;
		}
		if (NULL == (dp = opendir(path)))
		{
			fprintf(stderr, "open %s error\n", path);
			return -1;
		}
		chdir(path);
		while(NULL != (dirp = readdir(dp)))
		{
			lstat(dirp->d_name, &statbuf);
			if(S_ISDIR(statbuf.st_mode))
			{
				if (0 == strncmp(dirp->d_name, ".", 1) ||
					0 == strcmp(dirp->d_name, "home"))
					continue;
				strcpy(dir_name, dirp->d_name);
				dir_len = strlen(dir_name);
				if (dir_len < 30 && dir_len > 0)
				{
					websWrite(wp, T("<tr><td><input type=\"radio\" name=\"dir_path\" value=\"%s/%s\"></td>"),
							  path, dir_name);
					websWrite(wp, T("<td>%s/%s</td>"), path, dir_name);
					websWrite(wp, T("<input type=\"hidden\" name=\"dir_part\" value=\"%s\">"),
							  part);
					websWrite(wp, T("<td>%s</td>"), part);
					websWrite(wp, T("</tr>"));
					dir_count++;
				}
			}
		}
		chdir("/");
		closedir(dp);
	}
	fclose(fp_mount);

	return 0;
}

static int ShowPartition(int eid, webs_t wp, int argc, char_t **argv)
{
	FILE *fp = fopen(MOUNT_INFO, "r");
	char part[30], path[50];
	if (NULL == fp) {
        perror(__FUNCTION__);
		return -1;
	}
	part_count = 0;

	while(EOF != fscanf(fp, "%s %s %*s %*s %*s %*s\n", part, path))
	{
		if (0 != strncmp(path, "/media/sd", 9))
		{
			continue;
		}
		websWrite(wp, T("<tr align=center>"));
		websWrite(wp, T("<td><input type=\"radio\" name=\"disk_part\" value=\"%s\"></td>"),
				  path);
		websWrite(wp, T("<td>%s</td>"), part);
		websWrite(wp, T("<td>%s</td>"), path);
		websWrite(wp, T("</tr>"));
		part_count++;
	}
	fclose(fp);
	return 0;
}

static int getMaxVol(int eid, webs_t wp, int argc, char_t **argv)
{
	FILE *pp = popen("fdisk -l /dev/sda", "r");
	char maxvol[5], unit[5];
	unsigned int result;

	fscanf(pp, "%*s %*s ");
	fscanf(pp, "%s %s %*s %*s\n", maxvol, unit);

	result = atoi(maxvol);
	if (NULL != strstr(unit, "GB,"))
		websWrite(wp, T("%d"), result*1000);
	else if (NULL != strstr(unit, "MB,"))
		websWrite(wp, T("%d"), result);
	pclose(pp);

	return 0;
}

static int getupnpPort(int eid, webs_t wp, int argc, char_t **argv)
{
	FILE *fp;
	char buff[15] = { 0 };
	int result = EOF;

	fp = fopen("/etc/ushare_port", "r");
	if (fp == NULL) return 0;

	while(fgets(buff,sizeof(buff),fp))
	{
            result = atoi(buff);
       }
       websWrite(wp, T("%d"), result);
	fclose(fp);

	return 0;
}

static int isStorageExist(void)
{
	char buf[256];
	FILE *fp = fopen("/proc/mounts", "r");
	if(!fp){
		perror(__FUNCTION__);
		return 0;
	}

	while(fgets(buf, sizeof(buf), fp)){
		if(strstr(buf, USB_STORAGE_SIGN)){
			fclose(fp);
			return 1;
		}
	}

	fclose(fp);
	dprintf("no usb disk found\n");
	return 0;
}

/*
 *  taken from "mkimage -l" with few modified....
 */
int checkIfFirmware(char *imagefile)
{
	unsigned int magic;
	FILE *fp = fopen(imagefile, "r");
	if(!fp){
		perror(imagefile);
		return 0;
	}
	if( fread(&magic, sizeof(magic), 1, fp) != 1){
		perror(imagefile);
		fclose(fp);
		return 0;
	}

	fclose(fp);

	if(magic == ntohl(IH_MAGIC))
		return 1;
	else
		return 0;
}


/*
 *  find the firmware recursively in usb disk.
 */
static void lookupFirmware(char *dir_path)
{
	struct dirent *dir;
	struct stat stat_buf;
	char whole_path[1024];

	DIR *d = opendir(dir_path);
	if(!d){
		perror(dir_path);
		return;			// give up to open the directory
	}

	while((dir = readdir(d))){
		if(!strcmp(dir->d_name, ".") || !strcmp(dir->d_name, ".."))
			continue;

		snprintf(whole_path, 1024, "%s/%s", dir_path, dir->d_name);
		if(stat( whole_path, &stat_buf) == -1){
			perror(dir_path);
			continue;
		}

		if( S_ISDIR( stat_buf.st_mode)){
			lookupFirmware(whole_path);	//recursive
			continue;
		}

		if(stat_buf.st_size < MIN_FIRMWARE_SIZE)
			continue;

		if(checkIfFirmware(whole_path)){
			if(!strlen(firmware_path))
				snprintf(firmware_path, sizeof(firmware_path), "%s", whole_path);
			else
				snprintf(firmware_path, sizeof(firmware_path), "%s\n%s", firmware_path, whole_path);
		}
	}
	closedir(d);
	return;
}

static int isSpaceEnough(char *part)
{
	int count;
	char path[1024];
	snprintf(path, 1024, "%s/.delete_me", part);
	FILE *fp = fopen(path, "w");

	if(!fp){
        perror(__FUNCTION__);
		return 0;
	}
	for(count =0; count <MIN_SPACE_FOR_FIRMWARE; count += sizeof(path)){
		if( fwrite(path, sizeof(path), 1, fp) != 1){
			fclose(fp);
			unlink(path);
			sync();
			fprintf(stderr, __FILE__":no enough space left in %s.\n", part);
			return 0;
		}
	}
	fclose(fp);
	unlink(path);
	sync();
	return 1;
}

/*
 *  choose a suitable usb disk partition for the firmware upload.
 *  return NULL if not found.
 */
char *isStorageOK(void)
{
    char buf[256];
	char device_path[256];
	static char mount_path[256];
    FILE *fp = fopen(MOUNT_INFO, "r");
    if(!fp){
        perror(__FUNCTION__);
        return 0;
    }

    while(fgets(buf, sizeof(buf), fp)){
		sscanf(buf, "%s %s", device_path, mount_path);
		if(strstr(mount_path, USB_STORAGE_SIGN)){
			if(!isSpaceEnough(mount_path))
				continue;
			fclose(fp);
			fprintf(stderr, __FILE__":choose USB %s for firmrware space.\n", mount_path);
			return mount_path;
		}
    }

    fclose(fp);
	fprintf(stderr, "No suitable usb disk partition found \n.");
    return NULL;
}

void setFirmwarePath(void)
{
	firmware_path[0] = '\0';
	if(!isStorageExist())
		return;

	lookupFirmware(USB_STORAGE_PATH);
	sync();
	printf("firmware found: %s\n", firmware_path);
}



#if defined CONFIG_USER_MTDAAPD
static void iTunesSrv(webs_t wp, char_t *path, char_t *query)
{
	char_t *enable, *dir_path, *name;

	enable = websGetVar(wp, T("enabled"), T("0"));
	dir_path = websGetVar(wp, T("dir_path"), T(""));
	name = websGetVar(wp, T("srv_name"), T(""));

	nvram_set( "itunes_enable", enable);
	nvram_set( "itunes_dir", dir_path);
	nvram_set( "itunes_srvname", name);
	nvram_commit();

	doSystem("killall mt-daapd; killall mDNSResponder");
	if (strcmp(enable, "1") == 0)
		doSystem("config-iTunes.sh \"%s\" \"%s\" \"%s\"",
				 name,
				 nvram_safe_get( "Password"),
				 dir_path);
}
#endif

/*
 *  Hotpluger signal handler
 */
void hotPluglerHandler(int signo)
{
#if defined CONFIG_RALINKAPP_MPLAYER
	cleanup_musiclist();
#endif
	initUSB();
}

