/* vi: set sw=4 ts=4 sts=4: */
/*
 *	utils.c -- System Utilities
 *
 *	Copyright (c) Ralink Technology Corporation All Rights Reserved.
 *
 *	$Id: utils.c,v 1.112 2010-07-21 04:55:27 chhung Exp $
 */
#include	<time.h>
#include	<signal.h>
#include	<sys/ioctl.h>
#include	<sys/time.h>

#include <sys/types.h>
#include <unistd.h>
#include	<sys/socket.h>
#include	<sys/types.h>
#include	<sys/wait.h>
#include	<netinet/in.h>
#include	<arpa/inet.h>
#include <stdio.h>
#include <stdlib.h>

#include    <sched.h>
#include	<bcmnvram.h>
#include    <shutils.h>

#include	"ralink_gpio.h"
#include	"linux/autoconf.h"  //kernel config
#include	"config/autoconf.h" //user config

#include	"webs.h"
#include	"internet.h"
#include	"utils.h"
#include	"wireless.h"
#include    "internet.h"
#include    "net_conn.h"


#if defined CONFIG_USB_STORAGE && defined CONFIG_USER_STORAGE
extern void setFirmwarePath(void);
#endif

static int  getModIns(int eid, webs_t wp, int argc, char_t **argv);
static int  getCfgGeneral(int eid, webs_t wp, int argc, char_t **argv);
static int  getCfgToHTML(int eid, webs_t wp, int argc, char_t **argv);
static int  getCfgNthGeneral(int eid, webs_t wp, int argc, char_t **argv);
static int  getCfgZero(int eid, webs_t wp, int argc, char_t **argv);
static int  getCfgNthZero(int eid, webs_t wp, int argc, char_t **argv);
static int  getCfg2General(int eid, webs_t wp, int argc, char_t **argv);
static int  getCfg2NthGeneral(int eid, webs_t wp, int argc, char_t **argv);
static int  getCfg2Zero(int eid, webs_t wp, int argc, char_t **argv);
static int  getCfg2NthZero(int eid, webs_t wp, int argc, char_t **argv);
static int  getCfg3General(int eid, webs_t wp, int argc, char_t **argv);
static int  getCfg3Zero(int eid, webs_t wp, int argc, char_t **argv);
static int  getDpbSta(int eid, webs_t wp, int argc, char_t **argv);
static int  getLangBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int  getMiiInicBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int  getPlatform(int eid, webs_t wp, int argc, char_t **argv);
static int  getStationBuilt(int eid, webs_t wp, int argc, char_t **argv);
static int  getSysBuildTime(int eid, webs_t wp, int argc, char_t **argv);
static int  getSdkVersion(int eid, webs_t wp, int argc, char_t **argv);
static int  getSysUptime(int eid, webs_t wp, int argc, char_t **argv);
static int  get3GLinktime(int eid, webs_t wp, int argc, char_t **argv);


static int  getPortStatus(int eid, webs_t wp, int argc, char_t **argv);
static int  isOnePortOnly(int eid, webs_t wp, int argc, char_t **argv);
static void forceMemUpgrade(webs_t wp, char_t *path, char_t *query);
static void setOpMode(webs_t wp, char_t *path, char_t *query);
static int  getSwVersion(int eid, webs_t wp, int argc, char_t **argv);
static int  have_dynamic_route(int eid, webs_t wp, int argc, char_t **argv);
static int  have_pptpd(int eid, webs_t wp, int argc, char_t **argv);
static int  have_qos(int eid, webs_t wp, int argc, char_t **argv);
static int  have_dlna(int eid, webs_t wp, int argc, char_t **argv);
//start of that insert printer server module by huangfeilan at 20110915
static int  have_printersrv(int eid, webs_t wp, int argc, char_t **argv);
//end of that insert printer server module by huangfeilan at 20110915
static int  have_samba(int eid, webs_t wp, int argc, char_t **argv);
static int  have_download(int eid, webs_t wp, int argc, char_t **argv);
static int  have_ftpd(int eid, webs_t wp, int argc, char_t **argv);
static int  have_email(int eid, webs_t wp, int argc, char_t **argv);
static int  have_storage(int eid, webs_t wp, int argc, char_t **argv);
static int  have_wol(int eid, webs_t wp, int argc, char_t **argv);
static int  have_fw(int eid, webs_t wp, int argc, char_t **argv);
static int  have_upnpd(int eid, webs_t wp, int argc, char_t **argv);
static int  have_guest(int eid, webs_t wp, int argc, char_t **argv);
static int  have_wps(int eid, webs_t wp, int argc, char_t **argv);
static int  have_wds(int eid, webs_t wp, int argc, char_t **argv);
static int  have_wlsecu(int eid, webs_t wp, int argc, char_t **argv);
static int  have_wlmac(int eid, webs_t wp, int argc, char_t **argv);
static int  have_wladv(int eid, webs_t wp, int argc, char_t **argv);
static int  have_wlhostset(int eid, webs_t wp, int argc, char_t **argv);
static int	have_macclone(int eid, webs_t wp, int argc, char_t **argv);
static int  have_ntpclient(int eid, webs_t wp, int argc, char_t **argv);
static int  have_igmp_proxy(int eid, webs_t wp, int argc, char_t **argv);
static int  have_ddns(int eid, webs_t wp, int argc, char_t **argv);
static int  have_3g_display(int eid, webs_t wp, int argc, char_t **argv);
static int  have_3g_hidden(int eid, webs_t wp, int argc, char_t **argv);
static int  have_routerMode(int eid, webs_t wp, int argc, char_t **argv);
static int  have_ireader(int eid, webs_t wp, int argc, char_t **argv);
static int  have_web_filter(int eid, webs_t wp, int argc, char_t **argv);
static int  have_udhcpd(int eid, webs_t wp, int argc, char_t **argv);
static int  have_snmp(int eid, webs_t wp, int argc, char_t **argv);
static int  have_wifidog(int eid, webs_t wp, int argc, char_t **argv);
static int  have_pwd(int eid, webs_t wp, int argc, char_t **argv);
static int  have_neutral(int eid, webs_t wp, int argc, char_t **argv);
static int  have_airmobi_display(int eid, webs_t wp, int argc, char_t **argv);
static int  copyright_company_display(int eid, webs_t wp, int argc, char_t **argv);
static int  isHideModel(int eid, webs_t wp, int argc, char_t **argv);
static int  have_hxwt_display(int eid, webs_t wp, int argc, char_t **argv);
static int  have_harvilon_display(int eid, webs_t wp, int argc, char_t **argv);
static int  have_allnet_display(int eid, webs_t wp, int argc, char_t **argv);
static int  have_language_en(int eid, webs_t wp, int argc, char_t **argv);
static int  language_options(int eid, webs_t wp, int argc, char_t **argv);
static int  wan_type_options(int eid, webs_t wp, int argc, char_t **argv);
static int  wan_type_options_quicksetup(int eid, webs_t wp, int argc, char_t **argv);
#if 0
static int  have_sys_statistics(int eid, webs_t wp, int argc, char_t **argv);
#endif

#if defined CONFIG_USB_STORAGE && defined CONFIG_USER_STORAGE
static void ScanUSBFirmware(webs_t wp, char_t *path, char_t *query);
#endif
static void doReboot(webs_t wp, char_t *path, char_t *query);
static void portspeed(webs_t wp, char_t *path, char_t *query);

/*********************************************************************
 * System Utilities
 */

void arplookup(char *ip, char *arp)
{
    char buf[256];
    FILE *fp = fopen("/proc/net/arp", "r");
    if(!fp){
        trace(0, T("no proc fs mounted!\n"));
        return;
    }
    strcpy(arp, "00:00:00:00:00:00");
    while(fgets(buf, 256, fp)){
        char ip_entry[32], hw_type[8],flags[8], hw_address[32];
        sscanf(buf, "%s %s %s %s", ip_entry, hw_type, flags, hw_address);
        if(!strcmp(ip, ip_entry)){
            strcpy(arp, hw_address);
            break;
        }
    }
    nvram_set( "PcMac", arp);
    fclose(fp);
    return;
}


/*
 * description: kill process whose pid was recorded in file
 *              (va is supported)
 */
int doKillPid(char_t *fmt, ...)
{
	va_list		vargs;
	char_t		*pid_fname = NULL;
	struct stat	st;

	va_start(vargs, fmt);
	if (fmtValloc(&pid_fname, WEBS_BUFSIZE, fmt, vargs) >= WEBS_BUFSIZE) {
		trace(0, T("doKillPid: lost data, buffer overflow\n"));
	}
	va_end(vargs);

	if (pid_fname) {
		if (-1 == stat(pid_fname, &st)) //check the file existence
			return 0;
		doSystem("kill `cat %s`", pid_fname);
		doSystem("rm -f %s", pid_fname);
		bfree(B_L, pid_fname);
	}
	return 0;
}

/*
 * description: parse va and do system
 */
int doSystem(char_t *fmt, ...)
{
	va_list		vargs;
	char_t		*cmd = NULL;
	int			rc = 0;

	va_start(vargs, fmt);
	if (fmtValloc(&cmd, WEBS_BUFSIZE, fmt, vargs) >= WEBS_BUFSIZE) {
		trace(0, T("doSystem: lost data, buffer overflow\n"));
	}
	va_end(vargs);

	if (cmd) {
		trace(0, T("%s\n"), cmd);
        dprintf( "%s\n", cmd );
		rc = system(cmd);
		bfree(B_L, cmd);
	}
	return rc;
}

int doSystembk(char_t *fmt, ...)
{
	va_list		vargs;
	char_t		*cmd = NULL;
	int		rc = 0;
	pid_t		pid;

	pid = vfork();
	if (0 == pid)
	{	/* if pid == 0, then we are in the child process */
		va_start(vargs, fmt);
		if (fmtValloc(&cmd, WEBS_BUFSIZE, fmt, vargs) >= WEBS_BUFSIZE)
		{
			trace(0, T("doSystem: lost data, buffer overflow\n"));
		}
		va_end(vargs);

		if (cmd)
		{
			pid = vfork(); /* son of son will be born. */
			if (pid > 0)
				exit(0); /* first son exit */

			if (0 == pid) { /* son of son run */
				execl("/bin/sh", "sh", "-c", cmd, (char *)0);
			}
			bfree(B_L, cmd);
		}
		exit(0);
	}
	else
	{
		if ( waitpid(pid, NULL, 0) != pid )
		{
			printf("waitpid error\n");
			return -1;
		}
	}

	return rc;
}

/*
 * Function Name: do_system().
 * Description  : launch shell command in the child process.
 * Parameters   : command - shell command to launch.
 * Returns      : status 0 - OK, -1 - ERROR.
*/
int do_system( char *command, int i_debug )
{
   int pid = 0, status = 0;
   extern char **environ;

   if ( command == 0 )
       return 1;

   pid = fork();
   if ( pid == -1 )
      return -1;

   setenv("PATH", "/sbin:/bin:/usr/sbin:/usr/bin", 1);
   if ( pid == 0 ) {
      char *argv[4];
      argv[0] = "sh";
      argv[1] = "-c";
      argv[2] = command;
      argv[3] = 0;

      if( i_debug )
        printf("%s\r\n", command);
      execve("/bin/sh", argv, environ);
      exit(127);
   }  // if ( pid == 0 )

   /* wait for child process return */
   do {
      if ( waitpid(pid, &status, 0) == -1 ) {
         if ( errno != EINTR )
            return -1;
      } else
         return status;
   } while ( 1 );

   return status;
}


/*
 * arguments: index - index of the Nth value
 *            values - un-parsed values
 * description: parse values delimited by semicolon, and return the value "offset of memory"
 *              according to given index (starts from 0)
 * MARK: the original values will not be destroyed by strtok
 * WARNING: offset will over the values's buffer range in some case
 */
char *getNthValueOffset(int index, char *values)
{
	int i;
	char *offset;
	char *preOffset = NULL;

	if (NULL == values)
		return NULL;
	preOffset = values;
	for (i = 0, offset = strstr(values, ";");
			(i < index) && offset;
			i++, offset = strstr(offset, ";")) {
		preOffset = ++offset;
	}
	return preOffset;
}


/*
 * arguments: index - index of the Nth value
 *            values - un-parsed values
 * description: parse values delimited by semicolon, and return the value
 *              according to given index (starts from 0)
 * WARNING: the original values will be destroyed by strtok
 */
char *getNthValue(int index, char *values)
{
	int i;
	static char *tok;

	if (NULL == values)
		return NULL;
	for (i = 0, tok = strtok(values, ";");
			(i < index) && tok;
			i++, tok = strtok(NULL, ";")) {
		;
	}
	if (NULL == tok)
		return "";
	return tok;
}

/*
 * arguments: index - index of the Nth value (starts from 0)
 *            old_values - un-parsed values
 *            new_value - new value to be replaced
 * description: parse values delimited by semicolon,
 *              replace the Nth value with new_value,
 *              and return the result
 * WARNING: return the internal static string -> use it carefully
 */
char *setNthValue(int index, char *old_values, char *new_value)
{
	int i;
	char *p, *q;
	static char ret[2048];
	char buf[8][256];

	memset(ret, 0, 2048);
	for (i = 0; i < 8; i++)
		memset(buf[i], 0, 256);

	//copy original values
	for ( i = 0, p = old_values, q = strchr(p, ';')  ;
	      i < 8 && q != NULL                         ;
	      i++, p = q + 1, q = strchr(p, ';')         )
	{
		strncpy(buf[i], p, q - p);
	}
	strcpy(buf[i], p); //the last one

	//replace buf[index] with new_value
	strncpy(buf[index], new_value, 256);

	//calculate maximum index
	index = (i > index)? i : index;

	//concatenate into a single string delimited by semicolons
	strcat(ret, buf[0]);
	for (i = 1; i <= index; i++) {
		strncat(ret, ";", 2);
		strncat(ret, buf[i], 256);
	}

	p = ret;
	return p;
}

/*
 * arguments: values - values delimited by semicolon
 * description: parse values delimited by semicolon, and return the number of
 *              values
 */
int getValueCount(char *values)
{
	int cnt = 0;

	if (NULL == values)
		return 0;
	while (*values++ != '\0') {
		if (*values == ';')
			++cnt;
	}
	return (cnt + 1);
}

/*
 * check the existence of semicolon in str
 */
int checkSemicolon(char *str)
{
	char *c = strchr(str, ';');
	if (c)
		return 1;
	return 0;
}

/*
 * argument: ip address
 * return: 1 = the given ip address is within LAN's scope
 *         0 = otherwise
 */
int isInLan(char *radius_ip_addr)
{
    char lan_if_addr[16];
    char lan_if_netmask[16];

    struct in_addr lan_ip;
    struct in_addr lan_netmask;
    struct in_addr radius_ip;

    if ((getIfIp(getLanIfName(), lan_if_addr)) == -1) {
        printf("getLanIP error\n");
        return 0;
    }
    if ((getIfNetmask(getLanIfName(), lan_if_netmask)) == -1) {
        printf("getLanNetmask error\n");
        return 0;
    }

    inet_aton(lan_if_addr, &lan_ip);
    inet_aton(lan_if_netmask, &lan_netmask);
    inet_aton(radius_ip_addr, &radius_ip);

    printf("lan_ip:%08x\n", lan_ip.s_addr);
    printf("lan_netmask:%08x\n", lan_netmask.s_addr);
    printf("radius_ip:%08x\n", radius_ip.s_addr);

    if ((lan_ip.s_addr & lan_netmask.s_addr) == (radius_ip.s_addr & lan_netmask.s_addr) ){
        printf("in Lan\n");
        return 1;
    } else {
        printf("not in lan\n");
        return 0;
    }
}

/*
 * substitution of getNthValue which dosen't destroy the original value
 */
int getNthValueSafe(int index, const char *value, char delimit, char *result, int len)
{
    int i=0, result_len=0;
    char *begin, *end;

    if(!value || !result || !len)
        return -1;

    begin = (char *)value;
    end = strchr(begin, delimit);

    while(i<index && end){
        begin = end+1;
        end = strchr(begin, delimit);
        i++;
    }

    //no delimit
    if(!end){
		if(i == index){
			end = begin + strlen(begin);
			result_len = (len-1) < (end-begin) ? (len-1) : (end-begin);
		}else
			return -1;
	}else
		result_len = (len-1) < (end-begin)? (len-1) : (end-begin);

	memcpy(result, begin, result_len );
	*(result+ result_len ) = '\0';

	return 0;
}

/*
 *  argument:  [IN]     index -- the index array of deleted items(begin from 0)
 *             [IN]     count -- deleted itmes count.
 *             [IN/OUT] value -- original string/return string
 *             [IN]     delimit -- delimitor
 */
int deleteNthValueMulti(int index[], int count, char *value, char delimit)
{
	char *begin, *end;
	int i=0,j=0;
	int need_check_flag=0;
	char *buf = strdup(value);

	/* add by radar. 2010.7.23 */
	if (NULL == buf){
		perror("strdup");
		return 0;
	}
	/* end add. radar */
	begin = buf;

	end = strchr(begin, delimit);
	while(end){
		if(i == index[j]){
			memset(begin, 0, end - begin );
			if(index[j] == 0)
				need_check_flag = 1;
			j++;
			if(j >=count)
				break;
		}
		begin = end;

		end = strchr(begin+1, delimit);
		i++;
	}

	if(!end && index[j] == i)
		memset(begin, 0, strlen(begin));

	if(need_check_flag){
		for(i=0; i<strlen(value); i++){
			if(buf[i] == '\0')
				continue;
			if(buf[i] == delimit)
				buf[i] = '\0';
			break;
		}
	}

	for(i=0, j=0; i<strlen(value); i++){
		if(buf[i] != '\0'){
			value[j++] = buf[i];
		}
	}
	value[j] = '\0';

	free(buf);
	return 0;
}



/*
 * nanosleep(2) don't depend on signal SIGALRM and could cooperate with
 * other SIGALRM-related functions(ex. setitimer(2))
 */
unsigned int Sleep(unsigned int secs)
{
	int rc;
	struct timespec ts, remain;
	ts.tv_sec  = secs;
	ts.tv_nsec = 0;

sleep_again:
	rc = nanosleep(&ts, &remain);
	if(rc == -1 && errno == EINTR){
		ts.tv_sec = remain.tv_sec;
		ts.tv_nsec = remain.tv_nsec;
		goto sleep_again;
	}
	return 0;
}

/*
 * The setitimer() is Linux-specified.
 */
int setTimer(int microsec, void ((*sigroutine)(int)))
{
	struct itimerval value, ovalue;

	signal(SIGALRM, sigroutine);
	value.it_value.tv_sec = 0;
	value.it_value.tv_usec = microsec;
	value.it_interval.tv_sec = 0;
	value.it_interval.tv_usec = microsec;
	return setitimer(ITIMER_REAL, &value, &ovalue);
}

void stopTimer(void)
{
	struct itimerval value, ovalue;

	value.it_value.tv_sec = 0;
	value.it_value.tv_usec = 0;
	value.it_interval.tv_sec = 0;
	value.it_interval.tv_usec = 0;
	setitimer(ITIMER_REAL, &value, &ovalue);
}

/*
 * configure LED blinking with proper frequency (privatly use only)
 *   on: number of ticks that LED is on
 *   off: number of ticks that LED is off
 *   blinks: number of on/off cycles that LED blinks
 *   rests: number of on/off cycles that LED resting
 *   times: stop blinking after <times> times of blinking
 * where 1 tick == 100 ms
 */
static int gpioLedSet(int gpio, unsigned int on, unsigned int off,
		unsigned int blinks, unsigned int rests, unsigned int times)
{
	int fd;
	ralink_gpio_led_info led;

	//parameters range check
	if (gpio < 0 || gpio >= RALINK_GPIO_NUMBER ||
			on > RALINK_GPIO_LED_INFINITY ||
			off > RALINK_GPIO_LED_INFINITY ||
			blinks > RALINK_GPIO_LED_INFINITY ||
			rests > RALINK_GPIO_LED_INFINITY ||
			times > RALINK_GPIO_LED_INFINITY) {
		return -1;
	}
	led.gpio = gpio;
	led.on = on;
	led.off = off;
	led.blinks = blinks;
	led.rests = rests;
	led.times = times;

	fd = open("/dev/gpio", O_RDONLY);
	if (fd < 0) {
		perror("/dev/gpio");
		return -1;
	}
	if (ioctl(fd, RALINK_GPIO_LED_SET, &led) < 0) {
		perror("ioctl");
		close(fd);
		return -1;
	}
	close(fd);
	return 0;
}

int ledAlways(int gpio, int on)
{
	if (on)
		return gpioLedSet(gpio, RALINK_GPIO_LED_INFINITY, 0, 1, 1, RALINK_GPIO_LED_INFINITY);
	else
		return gpioLedSet(gpio, 0, RALINK_GPIO_LED_INFINITY, 1, 1, RALINK_GPIO_LED_INFINITY);
}

int ledWps(int gpio, int mode)
{
#if 0 //Use the system LED,should always lights except progress
	switch (mode) {
		case WPS_LED_RESET:
			//printf("********* LED Reset\n");
			return gpioLedSet(gpio, 0, RALINK_GPIO_LED_INFINITY, 1, 1, RALINK_GPIO_LED_INFINITY);
			break;
		case WPS_LED_PROGRESS:
			//printf("********* LED In Progress\n");
			return gpioLedSet(gpio, 2, 1, RALINK_GPIO_LED_INFINITY, 1, RALINK_GPIO_LED_INFINITY);
			break;
		case WPS_LED_ERROR:
			//printf("********* LED Error\n");
			return gpioLedSet(gpio, 1, 1, RALINK_GPIO_LED_INFINITY, 1, RALINK_GPIO_LED_INFINITY);
			break;
		case WPS_LED_SESSION_OVERLAP:
			//printf("********* LED Overlap\n");
			return gpioLedSet(gpio, 1, 1, 10, 5, RALINK_GPIO_LED_INFINITY);
			break;
		case WPS_LED_SUCCESS:
			//printf("********* LED Success\n");
			gpioLedSet(gpio, 3000, 1, 1, 1, 1);
			break;
	}
#endif
	switch (mode) {
		case WPS_LED_PROGRESS:
			//printf("********* LED In Progress\n");
			return gpioLedSet(gpio, 2, 1, RALINK_GPIO_LED_INFINITY, 1, RALINK_GPIO_LED_INFINITY);
			break;
		case WPS_LED_ERROR:
		case WPS_LED_RESET:
		case WPS_LED_SESSION_OVERLAP:
		case WPS_LED_SUCCESS:
			gpioLedSet(gpio, RALINK_GPIO_LED_INFINITY, 1, 1, 1, RALINK_GPIO_LED_INFINITY);
			break;
	}
	return 0;
}

/*
 * concatenate a string with an integer
 * ex: racat("SSID", 1) will return "SSID1"
 */
char *racat(char *s, int i)
{
	static char str[32];
	snprintf(str, 32, "%s%1d", s, i);
	return str;
}

void websLongWrite(webs_t wp, char *longstr)
{
    char tmp[513] = {0};
    int len = strlen(longstr);
    char *end = longstr + len;

    while(longstr < end){
        strncpy(tmp, longstr, 512);
        websWrite(wp, T("%s"), tmp);
        longstr += 512;
    }
    return;
}

/*********************************************************************
 * Web Related Utilities
 */

void formDefineUtilities(void)
{
	websAspDefine(T("getModIns"), getModIns);
	websAspDefine(T("getCfgGeneral"), getCfgGeneral);
	websAspDefine(T("getCfgToHTML"), getCfgToHTML);
	websAspDefine(T("getCfgNthGeneral"), getCfgNthGeneral);
	websAspDefine(T("getCfgZero"), getCfgZero);
	websAspDefine(T("getCfgNthZero"), getCfgNthZero);
	websAspDefine(T("getCfg2General"), getCfg2General);
	websAspDefine(T("getCfg2NthGeneral"), getCfg2NthGeneral);
	websAspDefine(T("getCfg2Zero"), getCfg2Zero);
	websAspDefine(T("getCfg2NthZero"), getCfg2NthZero);
	websAspDefine(T("getCfg3General"), getCfg3General);
	websAspDefine(T("getCfg3Zero"), getCfg3Zero);
	websAspDefine(T("getDpbSta"), getDpbSta);
	websAspDefine(T("getLangBuilt"), getLangBuilt);
	websAspDefine(T("getMiiInicBuilt"), getMiiInicBuilt);
	websAspDefine(T("getPlatform"), getPlatform);
	websAspDefine(T("getStationBuilt"), getStationBuilt);
	websAspDefine(T("getSysBuildTime"), getSysBuildTime);
	websAspDefine(T("getSdkVersion"), getSdkVersion);
	websAspDefine(T("getSysUptime"), getSysUptime);
	websAspDefine(T("get3GLinktime"), get3GLinktime);
	
	websAspDefine(T("getPortStatus"), getPortStatus);
	websAspDefine(T("getSwVersion"), getSwVersion);
    websAspDefine(T("have_dynamic_route"), have_dynamic_route);
    websAspDefine(T("have_pptpd"), have_pptpd);
    websAspDefine(T("have_qos"), have_qos);
    websAspDefine(T("have_dlna"), have_dlna);
	//start of that insert printer server module by huangfeilan at 20110915
	websAspDefine(T("have_printersrv"), have_printersrv);
	//end of that insert printer server module by huangfeilan at 20110915
    websAspDefine(T("have_samba"), have_samba);
	websAspDefine(T("have_download"), have_download);
     websAspDefine(T("have_ftpd"), have_ftpd);
	 websAspDefine(T("have_email"), have_email);
     websAspDefine(T("have_storage"), have_storage);
     websAspDefine(T("have_wol"), have_wol);
	 websAspDefine(T("have_upnpd"), have_upnpd);
	 websAspDefine(T("have_fw"), have_fw);
	websAspDefine(T("have_guest"), have_guest);
	 websAspDefine(T("have_wps"), have_wps);
	 websAspDefine(T("have_wds"), have_wds);
	 websAspDefine(T("have_wlsecu"), have_wlsecu);
	 websAspDefine(T("have_wlmac"), have_wlmac);
	 websAspDefine(T("have_wladv"), have_wladv);
	 websAspDefine(T("have_wlhostset"), have_wlhostset);
	 websAspDefine(T("have_macclone"), have_macclone);
	 websAspDefine(T("have_udhcpd"), have_udhcpd);
	 websAspDefine(T("have_snmp"), have_snmp);
	 websAspDefine(T("have_wifidog"), have_wifidog);
	 websAspDefine(T("have_pwd"), have_pwd);
	 websAspDefine(T("have_neutral"), have_neutral);
	 websAspDefine(T("have_airmobi_display"), have_airmobi_display);
	 websAspDefine(T("copyright_company_display"), copyright_company_display);	 
	 websAspDefine(T("isHideModel"), isHideModel);
	 websAspDefine(T("have_hxwt_display"), have_hxwt_display);
	 websAspDefine(T("have_harvilon_display"), have_harvilon_display);	 
	 websAspDefine(T("have_allnet_display"), have_allnet_display);
	 websAspDefine(T("have_language_en"), have_language_en);
	 websAspDefine(T("have_igmp_proxy"), have_igmp_proxy);
	 websAspDefine(T("have_ddns"), have_ddns);
	 websAspDefine(T("have_3g_display"), have_3g_display);
	 websAspDefine(T("have_3g_hidden"), have_3g_hidden);
	 websAspDefine(T("have_routerMode"), have_routerMode);
	 websAspDefine(T("have_ireader"), have_ireader);
	 websAspDefine(T("have_ntpclient"), have_ntpclient);
    websAspDefine(T("have_web_filter"), have_web_filter);
    websAspDefine(T("language_options"), language_options);
    websAspDefine(T("wan_type_options"), wan_type_options);
    websAspDefine(T("wan_type_options_quicksetup"), wan_type_options_quicksetup);
	websAspDefine(T("isOnePortOnly"), isOnePortOnly);
	websFormDefine(T("forceMemUpgrade"), forceMemUpgrade);
	websFormDefine(T("setOpMode"), setOpMode);
#if defined CONFIG_USB_STORAGE && defined CONFIG_USER_STORAGE
	websFormDefine(T("ScanUSBFirmware"), ScanUSBFirmware);
#endif
	websFormDefine(T("doReboot"), doReboot);
	websFormDefine(T("portspeed"), portspeed);
#if 0	
	websFormDefine(T("have_sys_statistics"), have_sys_statistics);
#endif
}


/*
 * arguments: type - 0 = return the status of module insertion (default)
 *                   1 = write the status of module insertion
 *            modname - module name
 * description: test the insertion of module through /proc/modules
 * result: -1 = fopen error, 1 = module inserted, 0 = not inserted yet
 */
static int getModIns(int eid, webs_t wp, int argc, char_t **argv)
{
	int i, type, result = 0;
	char_t *modname;
	FILE *fp;
	char buf[128];

	if (ejArgs(argc, argv, T("%d %s"), &type, &modname) < 2) {
		return websWrite(wp, T("Insufficient args\n"));
	}

	if (NULL == (fp = fopen("/proc/modules", "r"))) {
		//error(E_L, E_LOG, T("getModIns: open /proc/modules error"));
		if (1 == type)
			return websWrite(wp, T("-1"));
		else {
			ejSetResult(eid, "-1");
			return 0;
		}
	}

	while (NULL != fgets(buf, 128, fp)) {
		i = 0;
		while (!isspace(buf[i++]))
			;
		buf[i - 1] = '\0';
		if (!strcmp(buf, modname)) {
			result = 1;
			break;
		}
	}
	fclose(fp);

	if (1 == type)
		return websWrite(wp, T("%d"), result);
	ejSetResult(eid, (char *)result);
	return 0;
}

/*
 * arguments: type - 0 = return the configuration of 'field' (default)
 *                   1 = write the configuration of 'field'
 *            field - parameter name in nvram
 * description: read general configurations from nvram
 *              if value is NULL -> ""
 */
static int getCfgGeneral(int eid, webs_t wp, int argc, char_t **argv)
{
	int type;
	char_t *field;
	char *value;

	if (ejArgs(argc, argv, T("%d %s"), &type, &field) < 2) {
		return websWrite(wp, T("Insufficient args\n"));
	}
	value = (char *) nvram_get( field);
	if (1 == type) {
		if (NULL == value)
			return websWrite(wp, T(""));
		return websWrite(wp, T("%s"), value);
	}
	if (NULL == value)
		ejSetResult(eid, "");
	ejSetResult(eid, value);
	return 0;
}

static int getCfgToHTML(int eid, webs_t wp, int argc, char_t **argv)
{
	int type;
	unsigned int offset, len;
	char_t *field;
	char *value, *head, *tok;
	char ret[32+1];

	if (ejArgs(argc, argv, T("%d %s"), &type, &field) < 2) {
		return websWrite(wp, T("Insufficient args\n"));
	}
	value = (char *) nvram_safe_get( field);
	memset(ret, 0, 33);
	tok = value;
	head = tok;
	do {
		if (*tok == 34) {
			offset = (unsigned int) head - (unsigned int) value;
			len = (unsigned int) tok - (unsigned int) head;
			strncat(ret, value+offset, len);
			strcat(ret, "&#34;");
			head = tok+1;
		}
		else if (*tok == 39) {
			offset = (unsigned int)head - (unsigned int) value;
			len = (unsigned int) tok - (unsigned int) head;
			strncat(ret, value+offset, len);
			strcat(ret, "&#39;");
			head = tok+1;
		}
		tok++;
	} while (*tok != '\0');
	offset = (unsigned int) head - (unsigned int) value;
	strcat(ret, value+offset);
	if (1 == type) {
		if (NULL == ret)
			return websWrite(wp, T(""));
		return websWrite(wp, T("%s"), ret);
	}
	if (NULL == ret)
		ejSetResult(eid, "");
	ejSetResult(eid, ret);
	return 0;
}
/*
 * arguments: type - 0 = return the configuration of 'field' (default)
 *                   1 = write the configuration of 'field'
 *            field - parameter name in nvram
 *            idx - index of nth
 * description: read general configurations from nvram (if value is NULL -> "")
 *              parse it and return the Nth value delimited by semicolon
 */
/*start of patch up super dmz function by zym 2011-01-13 */
static int getCfgNthGeneral(int eid, webs_t wp, int argc, char_t **argv)
{
	int type, idx;
	char_t *field;
	char *value;
	char nth[1024];

	if (ejArgs(argc, argv, T("%d %s %d"), &type, &field, &idx) < 3) {
		return websWrite(wp, T("Insufficient args\n"));
	}
	value = (char *) nvram_get( field);
	if (1 == type) {
		if (NULL == value){
			return websWrite(wp, T(""));
		}
		getNthValueSafe(idx, value, ';', nth, sizeof(nth));
		if (NULL == nth){
			return websWrite(wp, T(""));
		}
		return websWrite(wp, T("%s"), nth);
	}
	if (NULL == value)
		ejSetResult(eid, "");
	getNthValueSafe(idx, value, ';', nth, sizeof(nth));
	if (NULL == nth)
		ejSetResult(eid, "");
	ejSetResult(eid, value);
	return 0;
}
/* end of patch up super dmz function by zym 2011-01-13 */

/*
 * arguments: type - 0 = return the configuration of 'field' (default)
 *                   1 = write the configuration of 'field'
 *            field - parameter name in nvram
 * description: read general configurations from nvram
 *              if value is NULL -> "0"
 */
static int getCfgZero(int eid, webs_t wp, int argc, char_t **argv)
{
	int type;
	char_t *field;
	char *value;

	if (ejArgs(argc, argv, T("%d %s"), &type, &field) < 2) {
		return websWrite(wp, T("Insufficient args\n"));
	}
	value = (char *) nvram_safe_get( field);
	if (1 == type) {
		if (!strcmp(value, ""))
			return websWrite(wp, T("0"));
		return websWrite(wp, T("%s"), value);
	}
	if (!strcmp(value, ""))
		ejSetResult(eid, "0");
	ejSetResult(eid, value);
	return 0;
}

/*
 * arguments: type - 0 = return the configuration of 'field' (default)
 *                   1 = write the configuration of 'field'
 *            field - parameter name in nvram
 *            idx - index of nth
 * description: read general configurations from nvram (if value is NULL -> "0")
 *              parse it and return the Nth value delimited by semicolon
 */
static int getCfgNthZero(int eid, webs_t wp, int argc, char_t **argv)
{
	int type, idx;
	char_t *field;
	char *value;
	char *nth;

	if (ejArgs(argc, argv, T("%d %s %d"), &type, &field, &idx) < 3) {
		return websWrite(wp, T("Insufficient args\n"));
	}
	value = (char *) nvram_safe_get( field);
	if (1 == type) {
		if (!strcmp(value, ""))
			return websWrite(wp, T("0"));
		nth = getNthValue(idx, value);
		if (NULL == nth)
			return websWrite(wp, T("0"));
		return websWrite(wp, T("%s"), nth);
	}
	if (!strcmp(value, ""))
		ejSetResult(eid, "0");
	nth = getNthValue(idx, value);
	if (NULL == nth)
		ejSetResult(eid, "0");
	ejSetResult(eid, value);
	return 0;
}

/*
 * arguments: type - 0 = return the configuration of 'field' (default)
 *                   1 = write the configuration of 'field'
 *            field - parameter name in nvram
 * description: read general configurations from nvram
 *              if value is NULL -> ""
 */
#if defined (RTDEV_SUPPORT)
static int getCfg2General(int eid, webs_t wp, int argc, char_t **argv)
{
	int type;
	char_t *field;
	char *value;

	if (ejArgs(argc, argv, T("%d %s"), &type, &field) < 2) {
		return websWrite(wp, T("Insufficient args\n"));
	}
	value = (char *) nvram_get(field);
	if (1 == type) {
		if (NULL == value)
			return websWrite(wp, T(""));
		return websWrite(wp, T("%s"), value);
	}
	if (NULL == value)
		ejSetResult(eid, "");
	ejSetResult(eid, value);
	return 0;
}

/*
 * arguments: type - 0 = return the configuration of 'field' (default)
 *                   1 = write the configuration of 'field'
 *            field - parameter name in nvram
 *            idx - index of nth
 * description: read general configurations from nvram (if value is NULL -> "")
 *              parse it and return the Nth value delimited by semicolon
 */
static int getCfg2NthGeneral(int eid, webs_t wp, int argc, char_t **argv)
{
	int type, idx;
	char_t *field;
	char *value;
	char *nth;

	if (ejArgs(argc, argv, T("%d %s %d"), &type, &field, &idx) < 3) {
		return websWrite(wp, T("Insufficient args\n"));
	}
	value = (char *) nvram_get(field);
	if (1 == type) {
		if (NULL == value)
			return websWrite(wp, T(""));
		nth = getNthValue(idx, value);
		if (NULL == nth)
			return websWrite(wp, T(""));
		return websWrite(wp, T("%s"), nth);
	}
	if (NULL == value)
		ejSetResult(eid, "");
	nth = getNthValue(idx, value);
	if (NULL == nth)
		ejSetResult(eid, "");
	ejSetResult(eid, value);
	return 0;
}

/*
 * arguments: type - 0 = return the configuration of 'field' (default)
 *                   1 = write the configuration of 'field'
 *            field - parameter name in nvram
 * description: read general configurations from nvram
 *              if value is NULL -> "0"
 */
static int getCfg2Zero(int eid, webs_t wp, int argc, char_t **argv)
{
	int type;
	char_t *field;
	char *value;

	if (ejArgs(argc, argv, T("%d %s"), &type, &field) < 2) {
		return websWrite(wp, T("Insufficient args\n"));
	}
	value = (char *) nvram_safe_get(field);
	if (1 == type) {
		if (!strcmp(value, ""))
			return websWrite(wp, T("0"));
		return websWrite(wp, T("%s"), value);
	}
	if (!strcmp(value, ""))
		ejSetResult(eid, "0");
	ejSetResult(eid, value);
	return 0;
}

/*
 * arguments: type - 0 = return the configuration of 'field' (default)
 *                   1 = write the configuration of 'field'
 *            field - parameter name in nvram
 *            idx - index of nth
 * description: read general configurations from nvram (if value is NULL -> "0")
 *              parse it and return the Nth value delimited by semicolon
 */
static int getCfg2NthZero(int eid, webs_t wp, int argc, char_t **argv)
{
	int type, idx;
	char_t *field;
	char *value;
	char *nth;

	if (ejArgs(argc, argv, T("%d %s %d"), &type, &field, &idx) < 3) {
		return websWrite(wp, T("Insufficient args\n"));
	}
	value = (char *) nvram_safe_get(field);
	if (1 == type) {
		if (!strcmp(value, ""))
			return websWrite(wp, T("0"));
		nth = getNthValue(idx, value);
		if (NULL == nth)
			return websWrite(wp, T("0"));
		return websWrite(wp, T("%s"), nth);
	}
	if (!strcmp(value, ""))
		ejSetResult(eid, "0");
	nth = getNthValue(idx, value);
	if (NULL == nth)
		ejSetResult(eid, "0");
	ejSetResult(eid, value);
	return 0;
}
#else
static int  getCfg2General(int eid, webs_t wp, int argc, char_t **argv)
{
	return 0;
}

static int  getCfg2NthGeneral(int eid, webs_t wp, int argc, char_t **argv)
{
	return 0;
}

static int  getCfg2Zero(int eid, webs_t wp, int argc, char_t **argv)
{
	return 0;
}

static int  getCfg2NthZero(int eid, webs_t wp, int argc, char_t **argv)
{
	return 0;
}
#endif

#if defined (CONFIG_RT2561_AP) || defined (CONFIG_RT2561_AP_MODULE)
/*
 * arguments: type - 0 = return the configuration of 'field' (default)
 *                   1 = write the configuration of 'field'
 *            field - parameter name in nvram
 * description: read general configurations from nvram
 *              if value is NULL -> ""
 */
static int getCfg3General(int eid, webs_t wp, int argc, char_t **argv)
{
	int type;
	char_t *field;
	char *value;

	if (ejArgs(argc, argv, T("%d %s"), &type, &field) < 2) {
		return websWrite(wp, T("Insufficient args\n"));
	}
	value = (char *) nvram_get(field);
	if (1 == type) {
		if (NULL == value)
			return websWrite(wp, T(""));
		return websWrite(wp, T("%s"), value);
	}
	if (NULL == value)
		ejSetResult(eid, "");
	ejSetResult(eid, value);
	return 0;
}

/*
 * arguments: type - 0 = return the configuration of 'field' (default)
 *                   1 = write the configuration of 'field'
 *            field - parameter name in nvram
 * description: read general configurations from nvram
 *              if value is NULL -> "0"
 */
static int getCfg3Zero(int eid, webs_t wp, int argc, char_t **argv)
{
	int type;
	char_t *field;
	char *value;

	if (ejArgs(argc, argv, T("%d %s"), &type, &field) < 2) {
		return websWrite(wp, T("Insufficient args"));
	}
	value = (char *) nvram_safe_get(field);
	if (1 == type) {
		if (!strcmp(value, ""))
			return websWrite(wp, T("0"));
		return websWrite(wp, T("%s"), value);
	}
	if (!strcmp(value, ""))
		ejSetResult(eid, "0");
	ejSetResult(eid, value);
	return 0;
}
#else
static int  getCfg3General(int eid, webs_t wp, int argc, char_t **argv)
{
	return 0;
}
static int  getCfg3Zero(int eid, webs_t wp, int argc, char_t **argv)
{
	return 0;
}
#endif

static int getDpbSta(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined (CONFIG_RT2860V2_STA_DPB) || defined (CONFIG_RT2860V2_STA_ETH_CONVERT)
	return websWrite(wp, T("1"));
#else
	return websWrite(wp, T("0"));
#endif
}

static int getLangBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
	char_t *lang;

	if (ejArgs(argc, argv, T("%s"), &lang) < 1) {
		return websWrite(wp, T("0"));
	}
	if (!strncmp(lang, "en", 3))
#ifdef CONFIG_USER_GOAHEAD_LANG_EN
		return websWrite(wp, T("1"));
#else
		return websWrite(wp, T("0"));
#endif
	else if (!strncmp(lang, "zhtw", 5))
#ifdef CONFIG_USER_GOAHEAD_LANG_ZHTW
		return websWrite(wp, T("1"));
#else
		return websWrite(wp, T("0"));
#endif
	else if (!strncmp(lang, "zhcn", 5))
#ifdef CONFIG_USER_GOAHEAD_LANG_ZHCN
		return websWrite(wp, T("1"));
#else
		return websWrite(wp, T("0"));
#endif

	return websWrite(wp, T("0"));
}

static int getMiiInicBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined (CONFIG_RTDEV_MII)
	return websWrite(wp, T("1"));
#else
	return websWrite(wp, T("0"));
#endif
}

/*
 * description: write the current system platform accordingly
 */
static int getPlatform(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef CONFIG_RAETH_ROUTER
	return websWrite(wp, T("RT2880 with IC+ MACPHY"));
#endif
#ifdef CONFIG_ICPLUS_PHY
    return websWrite(wp, T("RT2880 with IC+ PHY"));
#endif
#ifdef CONFIG_RT_MARVELL
	return websWrite(wp, T("RT2880 with MARVELL"));
#endif
#ifdef CONFIG_MAC_TO_MAC_MODE
	return websWrite(wp, T("RT2880 with Vitesse"));
#endif
#ifdef CONFIG_RT_3052_ESW
	return websWrite(wp, T("RT3052 embedded switch"));
#endif

	return 0;
}

static int getStationBuilt(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined CONFIG_RT2860V2_STA || defined CONFIG_RT2860V2_STA_MODULE
	return websWrite(wp, T("1"));
#else
	return websWrite(wp, T("0"));
#endif
}

/*
 * description: write System build time
 */
static int getSysBuildTime(int eid, webs_t wp, int argc, char_t **argv)
{
	return websWrite(wp, T("%s"), __DATE__);
}

/*
 * description: write RT288x SDK version
 */
static int getSdkVersion(int eid, webs_t wp, int argc, char_t **argv)
{
	FILE *fp = fopen("/etc_ro/web/cgi-bin/History", "r");
	char version[16] = "";

	if (fp != NULL)
	{
		char buf[30];
		while (fgets(buf, sizeof(buf), fp) != NULL) {
			if (strncasecmp(buf, "Version", 7) != 0)
				continue;
			sscanf(buf, "%*s%s", version);
			break;
		}
		fclose(fp);
	}

	return websWrite(wp, T("%s"), version);
}

/*
 * description: write System Uptime
 */
#include <sys/sysinfo.h>	//for sysinfo()
static int getSysUptime(int eid, webs_t wp, int argc, char_t **argv)
{
	struct sysinfo info;
	int updays, uphours, upminutes, upsecs;

	if (0 != sysinfo(&info) ){
		struct tm *utime;
		time_t usecs;

		time(&usecs);
		utime = localtime(&usecs);
		updays = 0;
		uphours = utime->tm_hour;
		upminutes = utime->tm_min;
		upsecs = utime->tm_sec;
	}else{
		updays = (int) info.uptime / (60*60*24);
		upminutes = (int) info.uptime / 60;
		uphours = (upminutes / 60) % 24;
		upminutes %= 60;
		upsecs = (int) (info.uptime % 60);
	}
	//web show
	if (updays > 0)
		return websWrite(wp, T("%d day%s, %d hour%s, %d min%s, %d sec%s"),
				updays, (updays <= 1)? "" : "s",
				uphours, (uphours <= 1)? "" : "s",
				upminutes, (upminutes <= 1)? "" : "s",
				upsecs, (upsecs <= 1)? "" : "s");
	else if (uphours > 0)
		return websWrite(wp, T("%d hour%s, %d min%s, %d sec%s"),
				uphours, (uphours <= 1)? "" : "s",
				upminutes, (upminutes <= 1)? "" : "s",
				upsecs, (upsecs <= 1)? "" : "s");
	else if (upminutes > 0)
		return websWrite(wp, T("%d min%s, %d sec%s"),
				upminutes, (upminutes <= 1)? "" : "s",
				upsecs, (upsecs <= 1)? "" : "s");
	else
		return websWrite(wp, T("%d sec%s"),
				upsecs, (upsecs <= 1)? "" : "s");
	return 0;
}


static int get3GLinktime(int eid, webs_t wp, int argc, char_t **argv)
{
	struct sysinfo info;
	FILE *fp = NULL;
	char buf[128] = { 0 };
	unsigned long updays, uphours, upminutes, upsecs; 
	unsigned long sec_sum = 0;
	unsigned long linktime = 0;
	
	fp = fopen( "/tmp/3Glinkdate", "r" );
	if( fp == NULL )
		return websWrite(wp, T("--"));

	while( fgets( buf, sizeof(buf), fp ) != NULL )
	{  
		sscanf(buf, "%ul", &sec_sum);
		//printf("sec_sum = %ul \n",sec_sum);
	}		
	fclose(fp);	
	
	if ( 0 == sysinfo(&info) )
	{
		linktime = info.uptime - sec_sum;
		updays = linktime / (60*60*24);
		upminutes = linktime / 60;
		uphours = (upminutes / 60) % 24;
		upminutes %= 60;
		upsecs = linktime% 60;
	}
	
	if (updays > 0)
		return websWrite(wp, T("%d day%s, %d hour%s, %d min%s, %d sec%s"),
				updays, (updays <= 1)? "" : "s",
				uphours, (uphours <= 1)? "" : "s",
				upminutes, (upminutes <= 1)? "" : "s",
				upsecs, (upsecs <= 1)? "" : "s");
	else if (uphours > 0)
		return websWrite(wp, T("%d hour%s, %d min%s, %d sec%s"),
				uphours, (uphours <= 1)? "" : "s",
				upminutes, (upminutes <= 1)? "" : "s",
				upsecs, (upsecs <= 1)? "" : "s");
	else if (upminutes > 0)
		return websWrite(wp, T("%d min%s, %d sec%s"),
				upminutes, (upminutes <= 1)? "" : "s",
				upsecs, (upsecs <= 1)? "" : "s");
	else
		return websWrite(wp, T("%d sec%s"),
				upsecs, (upsecs <= 1)? "" : "s");
	return 0;
	
}

static int getPortStatus(int eid, webs_t wp, int argc, char_t **argv)
{
#if (defined (CONFIG_RAETH_ROUTER) || defined CONFIG_RT_3052_ESW) && defined (CONFIG_USER_ETHTOOL)
	int port, rc;
	FILE *fp;
	char buf[1024];
    /*start of remove rt3052 port layout by zengxianzhu at 20110228*/
    for(port=0; port<5; port++){
     /*end of remove rt3052 port layout by zengxianzhu at 20110228*/
		char *pos;
		char link = '0';
		int speed = 100;
		char duplex = 'F';
	#ifdef CONFIG_RALINK_RT3052
		FILE *proc_file = fopen("/proc/rt3052/gmac", "w"); /*for 3.4 raeth*/
	#endif
	#ifdef CONFIG_RALINK_RT2880
		FILE *proc_file = fopen("/proc/rt2880/gmac", "w");
	#endif
	#ifdef CONFIG_RALINK_RT3352
		FILE *proc_file = fopen("/proc/rt3352/gmac", "w");
	#endif
	#ifdef CONFIG_RALINK_RT3383
		FILE *proc_file = fopen("/proc/rt3383/gmac", "w");
	#endif
		if(!proc_file){
			websWrite(wp, T("-1"));		// indicate error
			return 0;
		}
		fprintf(proc_file, "%d", port);
		fclose(proc_file);

		if((fp = popen("ethtool eth2", "r")) == NULL){
			websWrite(wp, T("-1"));		// indicate error
			return 0;
		}
		rc = fread(buf, 1, 1024, fp);
		pclose(fp);
		if(rc == -1){
			websWrite(wp, T("-1"));		// indicate error
			return 0;
		}else{
			// get Link status
			if((pos = strstr(buf, "Link detected: ")) != NULL){
				pos += strlen("Link detected: ");
				if(*pos == 'y')
					link = '1';
			}
			// get speed
			if((pos = strstr(buf, "Speed: ")) != NULL){
				pos += strlen("Speed: ");
				if(*pos == '1' && *(pos+1) == '0' && *(pos+2) == 'M')
					speed = 10;
				if(*pos == '1' && *(pos+1) == '0' && *(pos+2) == '0' && *(pos+3) == '0' && *(pos+4) == 'M')
					speed = 1000;
			}
			// get duplex
			if((pos = strstr(buf, "Duplex: ")) != NULL){
				pos += strlen("Duplex: ");
				if(*pos == 'H')
					duplex = 'H';
			}

			websWrite(wp, T("%c,%d,%c,"), link, speed, duplex);
		}
	}
	return 0;
#else
	//websWrite(wp, T("-1"));
    if(is_phyconnected() >0)
    {
        websWrite(wp, T("1,1"));
    }
    else
    {
        websWrite(wp, T("0,0"));
    }
	return 0;
#endif

}
/* append by radar@20081219. for getting software version */
static int getSwVersion(int eid, webs_t wp, int argc, char_t **argv)
{
	FILE *fp = fopen("/home/software_version", "r");
	if (fp){
		char version[64] = {0};
		fgets(version, sizeof(version)-1, fp);
		fclose(fp);
		return websWrite(wp, T("%s"), version);
	}else
		return websWrite(wp, T("Unknown"));
	return 0;
}

static int have_dynamic_route(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef HAVE_DYNAMIC_ROUTE
    return websWrite(wp, T(" "));
#else
    return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int  have_pptpd(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef HAVE_PPTPD
    if(nvram_match("sw_mode_ex", "0"))
    {
            return websWrite(wp, T("style=\"display: none\""));
    }
    else
    {
            return websWrite(wp, T(" "));
    }
#else
    return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int have_qos(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef HAVE_QOS
    return websWrite(wp, T(" "));
#else
    return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int  have_dlna(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef HAVE_DLNA
    return websWrite(wp, T(" "));
#else
    return websWrite(wp, T("style=\"display: none\""));
#endif
}

//start of that insert printer server module by huangfeilan at 20110915
static int  have_printersrv(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef CONFIG_USER_P910ND
    return websWrite(wp, T(" "));
#else
    return websWrite(wp, T("style=\"display: none\""));
#endif
}
//end of that insert printer server module by huangfeilan at 20110915

static int  have_samba(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef HAVE_SAMBA
    return websWrite(wp, T(" "));
#else
    return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int have_download(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef HAVE_DOWNLOAD
		return websWrite(wp, T(" "));
#else
		return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int  have_ftpd(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef HAVE_FTPD
#if 0 
	return websWrite(wp, T(" "));
#else
	return websWrite(wp, T("style=\"display: none\"")); // hidden ftpserver website
#endif
#else
    return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int  have_email(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef HAVE_EMAIL
    return websWrite(wp, T(" "));
#else
    return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int  have_storage(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef HAVE_STORAGE
    return websWrite(wp, T(" "));
#else
    return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int  have_wol(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef HAVE_WOL
    if(nvram_match("sw_mode_ex", "0"))
    {
            return websWrite(wp, T("style=\"display: none\""));
    }
    else
    {
            return websWrite(wp, T(" "));
    }
    
#else
    return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int have_web_filter(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef HAVE_WEB_FILTER
    return websWrite(wp, T(" "));
#else
    return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int  have_fw(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef DEFINE_FW
    return websWrite(wp, T(" "));
#else
    return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int  have_upnpd(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef HAVE_UPNPD
    return websWrite(wp, T(" "));
#else
    return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int  have_guest(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef HAVE_GUEST
    return websWrite(wp, T(" "));
#else
    return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int  have_wps(int eid, webs_t wp, int argc, char_t **argv)
{
	int opmode = atoi( nvram_safe_get( "sw_mode_ex" ) );
	if(opmode==3 || opmode==4 || opmode==7)
		return websWrite(wp, T("style=\"display: none\""));
	else
    return websWrite(wp, T(" "));

}

static int  have_wds(int eid, webs_t wp, int argc, char_t **argv)
{

    return websWrite(wp, T("style=\"display: none\""));

}

static int	have_wlsecu(int eid, webs_t wp, int argc, char_t **argv)
{
	int opmode = atoi( nvram_safe_get( "sw_mode_ex" ) );
	if(opmode==3 || opmode==4 || opmode==7)
		return websWrite(wp, T("style=\"display: none\""));
	else
		return websWrite(wp, T(" "));
}

static int	have_wlmac(int eid, webs_t wp, int argc, char_t **argv)
{
	int opmode = atoi( nvram_safe_get( "sw_mode_ex" ) );
	if(opmode==3 || opmode==4 || opmode==7)
		return websWrite(wp, T("style=\"display: none\""));
	else
		return websWrite(wp, T(" "));
}

static int	have_wladv(int eid, webs_t wp, int argc, char_t **argv)
{
	int opmode = atoi( nvram_safe_get( "sw_mode_ex" ) );
	if(opmode==3 || opmode==4 || opmode==7)
		return websWrite(wp, T("style=\"display: none\""));
	else
		return websWrite(wp, T(" "));
}

static int	have_wlhostset(int eid, webs_t wp, int argc, char_t **argv)
{
	int opmode = atoi( nvram_safe_get( "sw_mode_ex" ) );
	if(opmode==3 || opmode==4 || opmode==7)
		return websWrite(wp, T("style=\"display: none\""));
	else
		return websWrite(wp, T(" "));
}

static int have_macclone(int eid, webs_t wp, int argc, char_t **argv)
{
	int opmode = atoi( nvram_safe_get( "sw_mode_ex" ) );

	if(opmode!=1 || nvram_match("wan_proto", WAN_PROTO_3G))
		return websWrite(wp, T("style=\"display: none\""));
	else
		return websWrite(wp, T(" "));
}



static int  have_igmp_proxy(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef HAVE_IGMP_PROXY
    return websWrite(wp, T(" "));
#else
    return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int	have_ddns(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef HAVE_DDNS
	return websWrite(wp, T(" "));
#else
	return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int	have_3g_display(int eid, webs_t wp, int argc, char_t **argv)
{
	if( nvram_match("wan_proto", WAN_PROTO_3G) )
		return websWrite(wp, T(" "));
	else
		return websWrite(wp, T("style=\"display: none\""));
}

static int	have_3g_hidden(int eid, webs_t wp, int argc, char_t **argv)
{
	if( nvram_invmatch("wan_proto", WAN_PROTO_3G) )
		return websWrite(wp, T(" "));
	else
		return websWrite(wp, T("style=\"display: none\""));
}

static int	have_routerMode(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef NO_ROUTER_MODE
	return websWrite(wp, T("style=\"display: none\""));
#else
	return websWrite(wp, T(" "));
#endif
}

static int	have_ireader(int eid, webs_t wp, int argc, char_t **argv)
{
	if( nvram_match("productid", "iReader") )
		return websWrite(wp, T("style=\"display: none\""));
	else
		return websWrite(wp, T(" "));
}

static int	have_udhcpd(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef HAVE_UDHCPD
    if(nvram_match("sw_mode_ex", "0"))
    {
            return websWrite(wp, T("style=\"display: none\""));
    }
    else
    {
            return websWrite(wp, T(" "));
    }
#else
	return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int	have_snmp(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef HAVE_SNMPD
    return websWrite(wp, T(" "));
#else
    return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int	have_wifidog(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef DEFINE_WIFIDOG
	return websWrite(wp, T(" "));
#else
	return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int	have_pwd(int eid, webs_t wp, int argc, char_t **argv)
{
#if 0
	if(nvram_match("sysadm_set", "0"))
	{
			return websWrite(wp, T("style=\\\"display: none\\\""));
	}
	else
	{
			return websWrite(wp, T(""));
	}
#endif
	return websWrite(wp, T("style=\"display: none\""));
}

static int	have_neutral(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined(DEFINE_AIRMOBI)
	return websWrite(wp, T(""));
#else
	return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int	have_airmobi_display(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined(DEFINE_AIRMOBI)
	return websWrite(wp, T(""));
#else
	return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int	copyright_company_display(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined(DEFINE_AIRMOBI)
	websWriteWithTranslation(wp, T("<#command#>"));
#elif defined(DEFINE_HXWT)
	websWriteWithTranslation(wp, T("<#command_hxwt#>"));
#elif defined(DEFINE_HARVILON)
	websWriteWithTranslation(wp, T("<#command_harvilon#>"));
#elif defined(DEFINE_SECUTRON)
	websWriteWithTranslation(wp, T("<#command_Secutron#>"));
#else
	websWrite(wp, T(""));
#endif
}

static int isHideModel(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined(DEFINE_HIDE_MODEL)
	return websWrite(wp, T("1"));
#else
	return websWrite(wp, T("0"));
#endif
}

static int	have_allnet_display(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined(DEFINE_ALLNET)
	return websWrite(wp, T(""));
#else
	return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int	have_harvilon_display(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined(DEFINE_HARVILON)
	return websWrite(wp, T(""));
#else
	return websWrite(wp, T("style=\"display: none\""));
#endif
}


static int	have_hxwt_display(int eid, webs_t wp, int argc, char_t **argv)
{
#if defined(DEFINE_HXWT)
	return websWrite(wp, T(""));
#else
	return websWrite(wp, T("style=\"display: none\""));
#endif
}

static int	have_language_en(int eid, webs_t wp, int argc, char_t **argv)  
{
#if defined(HAVE_WEB_LENOVO) || defined(HAVE_WEB_VIVICK) || defined(HAVE_WEB_EDUP) || defined(HAVE_WEB_HXBLUE) || defined(CONFIG_WEB_HWR620)
    return websWrite(wp, T("style=\"display: none\""));
#else
	if(nvram_match("Language", "English"))
	{
		return websWrite(wp, T("style=\"display: none\""));
	}
	else
	{
		return websWrite(wp, T(" "));
	}
#endif    
}


static int have_ntpclient(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef CONFIG_USER_NTPCLIENT_NTPCLIENT
    return websWrite(wp, T("style=\"display: none\""));
#else
    return websWrite(wp, T(""));
#endif
}

#if 0
static int have_sys_statistics(int eid, webs_t wp, int argc, char_t **argv)
{
    return websWrite(wp, T("style=\"display: none\""));
}
#endif

static int language_options(int eid, webs_t wp, int argc, char_t **argv)
{
    char s_html[ 512 ] = { 0 };
#ifdef LANG_ZHCN
	websWriteWithTranslation(wp, T("<option value=\"Chinese\"><#Language_Body_2#></option>\n"));
#endif

#ifdef LANG_ZHTW
	websWriteWithTranslation(wp, T("<option value=\"Taiwan\"><#Language_Body_4#></option>\n"));
#endif

#ifdef LANG_KR
	websWrite(wp, T("<option value=\"Korean\">頃滉竴</option>\n"));
#endif

#ifdef LANG_JP
	websWriteWithTranslation(wp, T("<option value=\"Japanese\"><#Language_Body_3#></option>\n"));
#endif
#ifdef LANG_DE
	websWriteWithTranslation(wp, T("<option value=\"Deutsch\"><#Language_Body_6#></option>\n"));
#endif
#ifdef LANG_PL
	websWriteWithTranslation(wp, T("<option value=\"Polski\">Polski</option>\n"));
#endif

    if( strlen( s_html ) == 0 )
        strcpy( s_html, T("<option value=\"English\">English</option>\n") );

    return websWrite(wp, "%s", s_html);
}

static int wan_type_options(int eid, webs_t wp, int argc, char_t **argv)
{
    char s_html[ 512 ] = { 0 };
#ifdef HAVE_PPPD
    strcat( s_html, T("<option id=\"PPPOE\" value=\"pppoe\">PPPoE</option>\n") );
#endif
#ifdef HAVE_L2TP
    strcat( s_html, T("<option id=\"L2TP\" value=\"l2tp\">L2TP</option>\n") );
#endif
#ifdef HAVE_PPTP_CLIENT
    strcat( s_html, T("<option id=\"PPTP\" value=\"pptp\">PPTP</option>\n") );
#endif

    return websWrite(wp, "%s", s_html);
}

static int wan_type_options_quicksetup(int eid, webs_t wp, int argc, char_t **argv)
{
    char s_html[ 512 ] = { 0 };
#ifdef HAVE_PPPD
    strcat( s_html, T("<option id=\"wConnTypePppoe\" value=\"PPPOE\">PPPoE</option>\n") );
#endif
#ifdef HAVE_L2TP
    strcat( s_html, T("<option id=\"wConnTypeL2tp\" value=\"L2TP\">L2TP</option>\n") );
#endif
#ifdef HAVE_PPTP_CLIENT
    strcat( s_html, T("<option id=\"wConnTypePptp\" value=\"PPTP\">PPTP</option>\n") );
#endif

    return websWrite(wp, "%s", s_html);
}

inline int getOnePortOnly(void)
{
#if defined CONFIG_RAETH_ROUTER || defined CONFIG_MAC_TO_MAC_MODE || defined CONFIG_RT_3052_ESW
	return 0;
#elif defined CONFIG_ICPLUS_PHY
	return 1;
#else
	return 0;
#endif
	return 0;
}

static int isOnePortOnly(int eid, webs_t wp, int argc, char_t **argv)
{
	if( getOnePortOnly() == 1)
		websWrite(wp, T("true"));
	else
		websWrite(wp, T("false"));
	return 0;
}

void redirect_wholepage(webs_t wp, const char *url)
{
	websWrite(wp, T("HTTP/1.1 200 OK\nContent-type: text/html\nPragma: no-cache\nCache-Control: no-cache\n\n"));
	websWrite(wp, T("<html><head><script language=\"JavaScript\">"));
	websWrite(wp, T("parent.location.replace(\"%s\");"), url);
	websWrite(wp, T("</script></head></html>"));
}

int netmask_aton(const char *ip)
{
	int i, a[4], result = 0;
	sscanf(ip, "%d.%d.%d.%d", &a[0], &a[1], &a[2], &a[3]);
	for(i=0; i<4; i++){	//this is dirty
		if(a[i] == 255){
			result += 8;
			continue;
		}
		if(a[i] == 254)
			result += 7;
		if(a[i] == 252)
			result += 6;
		if(a[i] == 248)
			result += 5;
		if(a[i] == 240)
			result += 4;
		if(a[i] == 224)
			result += 3;
		if(a[i] == 192)
			result += 2;
		if(a[i] == 128)
			result += 1;
		//if(a[i] == 0)
		//	result += 0;
		break;
	}
	return result;
}
static void forceMemUpgrade(webs_t wp, char_t *path, char_t *query)
{
	char_t *mode  = websGetVar(wp, T("ForceMemUpgradeSelect"), T("0"));
	if(!mode)
		return;
	if(!strcmp(mode, "1"))
		nvram_set( "Force_mem_upgrade", "1");
	else
		nvram_set( "Force_mem_upgrade", "0");
	nvram_commit();
    websHeader(wp);
    websWrite(wp, T("<h2>force mem upgrade</h2>\n"));
    websWrite(wp, T("mode: %s<br>\n"), mode);
    websFooter(wp);
    websDone(wp, 200);
}

#if defined CONFIG_USB_STORAGE && defined CONFIG_USER_STORAGE
static void ScanUSBFirmware(webs_t wp, char_t *path, char_t *query)
{
	setFirmwarePath();
	printf("%s enter\n", __FUNCTION__);

	websRedirect(wp, "sys_upload.asp");
}
#endif

/* goform/setOpMode */
static void setOpMode(webs_t wp, char_t *path, char_t *query)
{
	char_t	*mode, *nat_en, *tcp_timeout, *udp_timeout;
//	const char *old_dhcp;
	const char	*old_mode = nvram_safe_get( "OperationMode");
	const char	*old_nat = nvram_safe_get( "natEnabled");
#if defined CONFIG_RAETH_ROUTER || defined CONFIG_MAC_TO_MAC_MODE || defined CONFIG_RT_3052_ESW || defined CONFIG_ICPLUS_PHY
#else
	char	*wan_ip, *lan_ip;
#endif
#if defined (CONFIG_RT2860V2_STA_DPB) || defined (CONFIG_RT2860V2_STA_ETH_CONVERT)
	char_t	*econv = "";
#endif
#if defined (RTDEV_SUPPORT)
	char_t	*mii;
#endif

	mode = websGetVar(wp, T("opMode"), T("0"));
	nat_en = websGetVar(wp, T("natEnbl"), T("1"));

#if 0  /*start of (add) in bridge mode ,DHCP service should disable;Mantis: 0000353  porting by huangfeilan 20110325 */
	old_dhcp = nvram_safe_get( "dhcpd_enable");
	if (!strncmp(old_mode, "0", 2))
	{
		if(!strncmp(mode, "0", 2))
		{
		}
		else{
        	if (!strncmp(old_dhcp, "0", 2))
			nvram_set( "dhcpd_enable", "1");
			need_commit = 1;
		}
	}
	else
	{
		if(!strncmp(mode, "0", 2))
		{
        if (!strncmp(old_dhcp, "1", 2))
		nvram_set( "dhcpd_enable", "0");
		need_commit = 1;
		}
	}
#endif /*end of (add) in bridge mode ,DHCP service should disable;Mantis:0000353   porting by huangfeilan 20110325 */

	tcp_timeout = websGetVar(wp, T("tcp_timeout"), T("3600"));
	udp_timeout = websGetVar(wp, T("udp_timeout"), T("180"));
	if (strcmp(tcp_timeout, nvram_safe_get( "ip_conntrack_tcp_timeouts")) ||
		strcmp(udp_timeout, nvram_safe_get( "ip_conntrack_udp_timeouts")))
	{
		nvram_set( "ip_conntrack_tcp_timeouts", tcp_timeout);
		nvram_set( "ip_conntrack_udp_timeouts", udp_timeout);
	}
#if 0
	if (!strncmp(old_mode, "0", 2)) {
	}
	else if (!strncmp(old_mode, "1", 2) || !strncmp(old_mode, "3", 2)) {
		if (!strncmp(mode, "0", 2)) {
#if defined CONFIG_RAETH_ROUTER || defined CONFIG_MAC_TO_MAC_MODE || defined CONFIG_RT_3052_ESW || defined CONFIG_ICPLUS_PHY
			char const *old_dhcp;
			old_dhcp = nvram_safe_get( "dhcpd_enable");
			if (!strncmp(old_dhcp, "1", 2))
				nvram_set( "dhcpd_enable", "0");
			need_commit = 1;
#else
			/*
			 * mode: gateway (or ap-client) -> bridge
			 * config: wan_ip(wired) overwrites lan_ip(bridge)
			 */
			wan_ip = nvram_safe_get( "wan_ipaddr");
			nvram_set( "lan_ipaddr", wan_ip);
			need_commit = 1;
#endif
		}
		if (!strncmp(mode, "2", 2)) {
#if defined CONFIG_RAETH_ROUTER || defined CONFIG_MAC_TO_MAC_MODE || defined CONFIG_RT_3052_ESW || defined CONFIG_ICPLUS_PHY
#else
			/*
			 * mode: gateway (or ap-client) -> ethernet-converter
			 * config: wan_ip(wired) overwrites lan_ip(wired)
			 *         lan_ip(wireless) overwrites wan_ip(wireless)
			 */
			wan_ip = nvram_safe_get( "wan_ipaddr");
			lan_ip = nvram_safe_get( "lan_ipaddr");
			nvram_set( "lan_ipaddr", wan_ip);
			nvram_set( "wan_ipaddr", lan_ip);
			need_commit = 1;
#endif
		}
	}
	else if (!strncmp(old_mode, "2", 2)) {
		if (!strncmp(mode, "0", 2)) {
			/*
			 * mode: wireless-isp -> bridge
			 * config: lan_ip(wired) overwrites lan_ip(bridge) -> the same
			 */
		}
		else if (!strncmp(mode, "1", 2) || !strncmp(mode, "3", 2)) {
#if defined CONFIG_RAETH_ROUTER || defined CONFIG_MAC_TO_MAC_MODE || defined CONFIG_RT_3052_ESW || defined CONFIG_ICPLUS_PHY
#else
			/*
			 * mode: ethernet-converter -> gateway (or ap-client)
			 * config: lan_ip(wired) overwrites wan_ip(wired)
			 *         wan_ip(wireless) overwrites lan_ip(wireless)
			 */
			wan_ip = nvram_safe_get( "wan_ipaddr");
			lan_ip = nvram_safe_get( "lan_ipaddr");
			nvram_set( "lan_ipaddr", wan_ip);
			nvram_set( "wan_ipaddr", lan_ip);
			need_commit = 1;
#endif
		}
	}

#if defined (CONFIG_RT2860V2_STA_DPB) || defined (CONFIG_RT2860V2_STA_ETH_CONVERT)
	if (!strncmp(mode, "0", 2)) {
		const char *old;

		econv = websGetVar(wp, T("ethConv"), T("0"));
		old = nvram_safe_get( "ethConvert");
		if (strncmp(old, econv, 2)) {
			nvram_set( "ethConvert", econv);
			need_commit = 1;
		}
		if (!strncmp(econv, "1", 2)) {
			//disable dhcp server in this mode
			old = nvram_safe_get( "dhcpd_enable");
			if (!strncmp(old, "1", 2)) {
				nvram_set( "dhcpd_enable", "0");
				need_commit = 1;
			}
		}
	}
#endif




	//new OperationMode
	if (strncmp(mode, old_mode, 2)) {
		nvram_set( "OperationMode", mode);

		//from or to ap client mode
		if (!strncmp(mode, "3", 2))
			nvram_set( "ApCliEnable", "1");
		else if (!strncmp(old_mode, "3", 2))
			nvram_set( "ApCliEnable", "0");
		need_commit = 1;
	}
#endif

#ifdef DEFINE_DHCP_AUTO_SWITCH
	//restore wireless info
	if ( nvram_match("sw_mode_ex", "6") //ǰrepeaterģʽͬSSIDлģʽӱȡԭ
		&& nvram_match("SynSsid", "1") 
		&& nvram_invmatch("ApCliSsid","") )
	{		
		nvram_set("EncrypType", nvram_safe_get("res_EncrypType"));
		nvram_set("AuthMode", nvram_safe_get("res_AuthMode"));
		nvram_set("WPAPSK1", nvram_safe_get("res_WPAPSK1"));
		nvram_set("SSID1", nvram_safe_get("res_SSID1"));
	}

	if ( !nvram_match("sw_mode_ex", "6") //ǰrepeaterģʽлģʽᱸ
		//&& !strcmp(mode, "6") 
		&& nvram_invmatch("ApCliSsid","") 
		&& nvram_match("SynSsid", "1"))
	{
		;
	}
	else
	{
		nvram_set( "res_SSID1", nvram_safe_get("SSID1"));
		nvram_set( "res_AuthMode", nvram_safe_get("AuthMode"));
		nvram_set( "res_EncrypType", nvram_safe_get("EncrypType"));
		nvram_set( "res_WPAPSK1", nvram_safe_get("WPAPSK1"));
	}	
#endif

	/* лrepeaterģʽApCliSsidΪApCliEnable. 20131113,lg */
	if(strcmp(mode, "6"))
	{
		if(!strcmp(mode, "3")||!strcmp(mode, "7")||!strcmp(mode, "8"))
			nvram_set( "ApCliEnable", "1");
		else
			nvram_set( "ApCliEnable", "0");
	}
	else if(nvram_invmatch("ApCliSsid", ""))
	{
		nvram_set( "ApCliEnable", "1");
	}
	
	if(!strcmp(mode, "4"))
		nvram_set( "WdsEnable", "2");
	else if (!strcmp(mode, "5"))
		nvram_set( "WdsEnable", "3");
	else
		nvram_set( "WdsEnable", "0");	
	if(!strcmp(mode, "1") || !strcmp(mode, "7") || !strcmp(mode, "8"))
		nvram_set( "wan_nat", nat_en);
	
#ifdef DEFINE_DHCP_AUTO_SWITCH
	//if(!strcmp(mode, "1")|| !strcmp(mode, "9")|| !strcmp(mode, "2")||!strcmp(mode, "5")||!strcmp(mode, "6")||!strcmp(mode, "8"))
	//	nvram_set("WscModeOption","0");
	if(!strcmp(mode, "3")||!strcmp(mode, "4")||!strcmp(mode, "5")||!strcmp(mode, "6")||!strcmp(mode, "7") ||!strcmp(mode, "8"))
		nvram_set("RadioOn","1");
	nvram_set( "dhcpd_enable", "1");
#else
	if(!strcmp(mode, "1")|| !strcmp(mode, "9")|| !strcmp(mode, "2")||!strcmp(mode, "5")||!strcmp(mode, "6")||!strcmp(mode, "8"))
		nvram_set("WscModeOption","0");
	if(!strcmp(mode, "3")||!strcmp(mode, "4")||!strcmp(mode, "5")||!strcmp(mode, "6")||!strcmp(mode, "7") ||!strcmp(mode, "8"))
		nvram_set("RadioOn","1");

	if(!strcmp(mode, "2") ||!strcmp(mode, "3")||!strcmp(mode, "4") ||!strcmp(mode, "5") /* ||!strcmp(mode, "6")*/ )
		nvram_set( "dhcpd_enable", "0");
	
	if(!strcmp(mode, "1") || !strcmp(mode, "9")|| !strcmp(mode, "7") ||!strcmp(mode, "8") || !strcmp(mode, "6"))
		nvram_set( "dhcpd_enable", "1");
#endif


	// For 100PHY  ( Ethernet Convertor with one port only)
	// If this is one port only board(IC+ PHY) then redirect
	// the user browser to our alias ip address.
	if( getOnePortOnly() ){
		//     old mode is Gateway, and new mode is BRIDGE/WirelessISP/Apcli
		if (    (!strcmp(old_mode, "1") && !strcmp(mode, "0"))  ||
				(!strcmp(old_mode, "1") && !strcmp(mode, "2"))  ||
				(!strcmp(old_mode, "1") && !strcmp(mode, "3"))  ){
			char redirect_url[512];
			const char *lan_ip = nvram_safe_get( "lan_ipaddr");

			if(! strlen(lan_ip))
				lan_ip = "10.10.10.254";
			snprintf(redirect_url, 512, "http://%s", lan_ip);
			redirect_wholepage(wp, redirect_url);
			goto final;
        }

		//     old mode is BRIDGE/WirelessISP/Apcli, and new mode is Gateway
		if (    (!strcmp(old_mode, "0") && !strcmp(mode, "1"))  ||
				(!strcmp(old_mode, "2") && !strcmp(mode, "1"))  ||
				(!strcmp(old_mode, "3") && !strcmp(mode, "1"))  ){
			redirect_wholepage(wp, "http://172.32.1.254");
			goto final;
		}
	}

#if defined (RTDEV_SUPPORT)
	mii = websGetVar(wp, T("miiMode"), T("0"));
	if (!strncmp(mode, "0", 2)) {
		const char *old_mii = nvram_safe_get("InicMiiEnable");

		if (strncmp(mii, old_mii, 2)) {
			nvram_set("InicMiiEnable", mii); //force to run initInternet
		}
	}
	else {
		nvram_set("InicMiiEnable", "0");
	}
#endif
	if(!strcmp(mode, "1"))
		nvram_set( "OperationMode", "1");
	else
		nvram_set( "OperationMode", "0");

	if(!strcmp(mode, "9"))	
	{
		if( nvram_invmatch("wan_proto", WAN_PROTO_3G) )
			nvram_set("wan_proto_tmp", nvram_safe_get("wan_proto")); //backup wan_protocol

		nvram_set("sw_mode_ex","1");
		nvram_set("wan_proto", WAN_PROTO_3G);
	}
	else
	{
		nvram_set("sw_mode_ex",mode);
		nvram_set("wan_proto", nvram_safe_get("wan_proto_tmp"));
	}
    nvram_set("wan_set", "1");

	//end of that avoid dhcp_service page can not hide on time by huangfeilan at 20120201
#if 0
#ifdef	GOAHEAD_SHOWSUBMITINFO
	websHeader(wp);
	websWrite(wp, T("<h2>Operation Mode</h2>\n"));
	websWrite(wp, T("mode: %s<br>\n"), mode);
	if (strncmp(mode, "0", 2))
		websWrite(wp, T("NAT enabled: %s<br>\n"), nat_en);
#if defined (CONFIG_RT2860V2_STA_DPB) || defined (CONFIG_RT2860V2_STA_ETH_CONVERT)
	else
		websWrite(wp, T("DPB station: %s<br>\n"), econv);
#endif
#if defined (RTDEV_SUPPORT)
	websWrite(wp, T("INIC MII mode: %s<br>\n"), mii);
#endif
	websFooter(wp);
	websDone(wp, 200);
#else
	websRedirect(wp, "opmode.asp");
#endif
#endif

final:

	//restart internet if any changes
		nvram_commit();
		sleep(1);
		updateFlash8021x();
        /* restart rc by zs 2011/6/10 */
		kill(1, SIGHUP);
	if (strncmp(nat_en, old_nat, 2)) {
		/* if nat changed, flush all nf_conntrack by cat the file. */
		doSystem("cat /proc/net/nf_conntrack_flush");
	}
	//sleep(15);
	//websRedirect(wp, "opmode.asp");
}
static void doReboot(webs_t wp, char_t *path, char_t *query)
{
#if 0       //start of disable to solve reboot can not jump normally at firefox by zengxianzhu
	websWrite(wp, T("HTTP/1.0 200 OK\n"));
	websWrite(wp, T("Server: %s\r\n"), WEBS_NAME);
	websWriteWithTranslation(wp, T("<html>\n<head>\n<title><#WDKWEB#></title>\n"));
	websWrite(wp, T("<meta http-equiv=\"Content-Type\" content=\"application/html; charset=UTF-8\">\n\
<style type=\"text/css\">\n\
#apply_info{ width:800px; height:180px; background:#EEF8FD; border:#FF9900 1px solid; margin-top:70px; color:#666666;}\n\
#apply_info .t_m{background-color:#FF9900; margin:0px; text-align:left; height:30px; line-height:30px; padding-left:10px; color:#FFFFFF;}\n\
#apply_info .mc{text-align:left; margin-top:10px; padding-left:10px;}\n\
body {color: black; font-family: arial, sans-serif; font-size: 9pt }\n\
.title{ font-family: arial, sans-serif; font-size: 13pt; font-weight: bold }\n\
.subtitle { font-family: arial, sans-serif; font-size: 11pt }\n\
.label { color: #306498; font-family: arial, sans-serif; font-size: 7pt }\n\
</style>\n\
</head>\n\
<body>\n\
<center>\n\
<div id=\"apply_info\">\n"));
websWriteWithTranslation(wp, T("<div class=\"title t_m\"><#Reboottitle1#></div>\n"));
websWriteWithTranslation(wp, T("<div class=\"subtitle mc\"><#Rebootintroduct1#></div>\n"));
websWrite(wp, T("<p><width=\"160\" height=\"20\"><br>\n"));
websWriteWithTranslation(wp, T("<br><#Uploadsettingintroduct2#><br>\n</div>\n</center>\n"));

	websWrite(wp, T("<script language=\"JavaScript\" type=\"text/javascript\">\n"));
        /*start of modify reboot funtion web jump problem by zengxianzhu at 20110120*/
	websWrite(wp, T("self.setTimeout(\"(document.location.href ='http://%s\')\",90000);\n"),getLanIP());
        /*end of modify reboot funtion web jump problem by zengxianzhu at 20110120*/

	websWrite(wp, T("</script>\n"));
	websWrite(wp, T("</body></html>\n"));
	websFooter(wp);
	websDone(wp, 200);
#endif
       //end of disable to solve reboot can not jump normally at firefox by zengxianzhu

	system("sleep 3 && reboot &");
}
static void portspeed(webs_t wp, char_t *path, char_t *query)
{
	char_t *prot0speed,*prot1speed,*prot2speed,*prot3speed,*prot4speed;

	/*add by wujunyong for prot speed*/
	prot0speed=websGetVar(wp, T("port0select"), T("0"));
	prot1speed=websGetVar(wp, T("port1select"), T("0"));
	prot2speed=websGetVar(wp, T("port2select"), T("0"));
	prot3speed=websGetVar(wp, T("port3select"), T("0"));
	prot4speed=websGetVar(wp, T("port4select"), T("0"));

	//port0
	if (!strncmp(prot0speed, "0", 2))
		{
	 	nvram_set( "prot0speed", "0");
	 	doSystem("mii_mgr -s -p 0 -r 0 -v 3100");
		}
	else if(!strncmp(prot0speed, "1", 2))//100Mbps
		{
		 nvram_set( "prot0speed", "1");
	 	doSystem("mii_mgr -s -p 0 -r 0 -v 2100");
		}
	else //10Mbps
		{
	 	nvram_set( "prot0speed", "2");
	 	doSystem("mii_mgr -s -p 0 -r 0 -v 0100");
		}

	//port1
	if (!strncmp(prot1speed, "0", 2))
		{
	 	nvram_set( "prot1speed", "0");
	 	doSystem("mii_mgr -s -p 1 -r 0 -v 3100");
		}
	else if(!strncmp(prot1speed, "1", 2))//100Mbps
		{
		 nvram_set( "prot1speed", "1");
		 doSystem("mii_mgr -s -p 1 -r 0 -v 2100");
		}
	else //10Mbps
		{
	 	nvram_set( "prot1speed", "2");
	 	doSystem("mii_mgr -s -p 1 -r 0 -v 0100");
		}

	//port2
	if (!strncmp(prot2speed, "0", 2))
		{
	 	nvram_set( "prot2speed", "0");
	 	doSystem("mii_mgr -s -p 2 -r 0 -v 3100");
		}
	else if(!strncmp(prot2speed, "1", 2))//100Mbps
		{
		 nvram_set( "prot2speed", "1");
	 	doSystem("mii_mgr -s -p 2 -r 0 -v 2100");
		}
	else //10Mbps
		{
		 nvram_set( "prot2speed", "2");
	 	doSystem("mii_mgr -s -p 2 -r 0 -v 0100");
		}

	//port3
	if (!strncmp(prot3speed, "0", 2))
		{
	 	nvram_set( "prot3speed", "0");
	 	doSystem("mii_mgr -s -p 3 -r 0 -v 3100");
		}
	else if(!strncmp(prot3speed, "1", 2))//100Mbps
		{
	 	nvram_set( "prot3speed", "1");
	 	doSystem("mii_mgr -s -p 3 -r 0 -v 2100");
		}
	else //10Mbps
		{
	 	nvram_set( "prot3speed", "2");
	 	doSystem("mii_mgr -s -p 3 -r 0 -v 0100");
		}

	//port4
	if (!strncmp(prot4speed, "0", 2))
		{
	 	nvram_set( "prot4speed", "0");
	 	doSystem("mii_mgr -s -p 4 -r 0 -v 3100");
		}
	else if(!strncmp(prot4speed, "1", 2))//100Mbps
		{
		 nvram_set( "prot4speed", "1");
	 	doSystem("mii_mgr -s -p 4 -r 0 -v 2100");
		}
	else //10Mbps
		{
	 	nvram_set( "prot4speed", "2");
	 	doSystem("mii_mgr -s -p 4 -r 0 -v 0100");
		}

	nvram_commit();
	websRedirect(wp, "adm/portspeed.asp");
}
