/* vi: set sw=4 ts=4 sts=4 fdm=marker: */
/*
 *	vpn.c -- Settings  IPSEC vpn
 *
 *
 *	$Id: vpv.c,v 1.167.2.8 2009-12 add by paul  $
 */

#include	<stdlib.h>
#include	<sys/ioctl.h>
#include	<net/if.h>
#include	<net/route.h>
#include    <string.h>
#include    <dirent.h>
//#include 	<winsock.h>

#include	"internet.h"
#include	"bcmnvram.h"
#include	"webs.h"
#include	"utils.h"
#include 	"firewall.h"

#include	"linux/autoconf.h"  //kernel config
#include	"config/autoconf.h" //user config
#include	"user/busybox/include/autoconf.h" //busybox config

#define IPSEC_MAX_RULES	8
#define CERT_PATH "/var/cert"

	/* vpn handle. paul 200912.  */
static void setipsec_config(webs_t wp, char_t *path, char_t *query);

static int isIpValid(char *str)
{
	struct in_addr addr;	// for examination
	if( (! strcmp(T("any"), str)) || (! strcmp(T("any/0"), str)))
		return 1;

	if(! (inet_aton(str, &addr))){
		printf("isIpValid(): %s is not a valid IP address.\n", str);
		return 0;
	}
	return 1;
}

int rutIPSec_calPrefixLen(char *ipAddr)
{
   unsigned int  addr = inet_addr(ipAddr);
   int i;

   if (addr == INADDR_NONE) {
      return 32;
   } else {
      for (i=0; i<=32;i++) {
         if ((addr>>i) & 1) {
            break;
         }
     }
     return 32 - i;
   }
}



static int getNums(char *value, char delimit)
{
	char *pos = value;
    int count=1;
    if(!pos)
    	return 0;
	while( (pos = strchr(pos, delimit))){
		pos = pos+1;
		count++;
	}
	return count;
}


/*
inline int getRuleNums(char *rules){
	return getNums(rules, ';');
}

*/
/*
 * ASP function
 */
static int getIPSECASPRuleNumsASP(int eid, webs_t wp, int argc, char_t **argv)
{
	int irules = 0;
	char const *en_rules = nvram_safe_get( "IPSECRules");

	if (en_rules && strlen(en_rules))
		irules += getRuleNums(en_rules);
	websWrite(wp, T("%d"),  irules);
	return 0;
}

static int showIPSECASP(int eid, webs_t wp, int argc, char_t **argv)
{
	int i = 0;
	char rec[196];
	char connName[32];
	char remoteGWAddr[32], localIPMode[16], localIP[32], localMask[32],  remoteIP[32];
	char states[8];
	char *pstates = NULL;
	char const *rules = nvram_safe_get( "IPSECRules");
	char const *language = nvram_safe_get( "Language");
	if(!rules)
		return 0;
	if(!strlen(rules))
		return 0;

	while(getNthValueSafe(i++, (char *)rules, ';', rec, sizeof(rec)) != -1 ){
		// get states
		if((getNthValueSafe(0, rec, ',', states, sizeof(states)) == -1)){
			continue;
		}


		// get connname
		if((getNthValueSafe(1, rec, ',', connName, sizeof(connName)) == -1)){
			continue;
		}


		// remoteGWAddr
		if((getNthValueSafe(2, rec, ',', remoteGWAddr, sizeof(remoteGWAddr)) == -1)){
			continue;
		}
		if(!isIpValid(remoteGWAddr)){
			continue;
		}

		// get localIPMode
		if((getNthValueSafe(3, rec, ',', localIPMode, sizeof(localIPMode)) == -1)){
			continue;
		}

		//get localIP
		if((getNthValueSafe(4, rec, ',', localIP, sizeof(localIP)) == -1)){
			continue;
		}
		if(!isIpValid(localIP)){
			continue;
		}

		//get localMask
		if((getNthValueSafe(5, rec, ',', localMask, sizeof(localMask)) == -1)){
			continue;
		}

		//get remoteIP
		if((getNthValueSafe(6, rec, ',', remoteIP, sizeof(remoteIP)) == -1)){
			continue;
		}
		if(!isIpValid(remoteIP)){
			continue;
		}

		websWrite(wp, T("<tr>\n"));
		// output No.

		if(states[1] == '0')
		websWrite(wp, T("<td align=center> %d&nbsp; <input type=\"checkbox\" name=\"actRule%d\"> </td>"), i, i-1 );
		else
		websWrite(wp, T("<td align=center> %d&nbsp; <input type=\"checkbox\" checked=\"checked\" name=\"actRule%d\"> </td>"), i, i-1 );
		// output statues
		//pstates = strchr(states, '|');

		//if (pstates)
			//{
		if(states[1] == '0'){
			if (!strcmp(language,"en"))
				websWrite(wp, T("<td align=center>Disabled</td>"));
			if (!strcmp(language,"zhcn"))
				websWrite(wp, T("<td align=center>ر</td>"));
			if (!strcmp(language,"zhtw"))
				websWrite(wp, T("<td align=center>P]</td>"));
			}
		else
			{
			if (!strcmp(language,"en"))
				websWrite(wp, T("<td align=center>Enabled</td>"));
			if (!strcmp(language,"zhcn"))
				websWrite(wp, T("<td align=center></td>"));
			if (!strcmp(language,"zhtw"))
				websWrite(wp, T("<td align=center>_</td>"));
			}


			//}

		// output name
		websWrite(wp, T("<td align=center> %s </td>"), connName);

		// output remoteGWAddr

		websWrite(wp, T("<td align=center> %s </td>"), remoteGWAddr);

		// output localIP

		websWrite(wp, T("<td align=center> %s </td>"), localIP);

		// output remoteIP

		websWrite(wp, T("<td align=center> %s </td>"), remoteIP);


		websWrite(wp, T("</tr>\n"));
	}
	return 0;



}


/* goform/setipsec_cinfig */
static void setipsec_config(webs_t wp, char_t *path, char_t *query)
{


	char rulesbuf[8192] = {0};
	char const *IPSECRules;

	char *connName;
	char *remoteGWAddr, *localIPMode, *localIP, *localMask, *remoteIPMode, *remoteIP, *remoteMask;
	char *keyExM, *authM, *psk, *certificateName,*perfectFSEn;//key exenge 
	/* ߼ */
	char *manualEncryptionAlgo, *manualEncryptionKey, *manualAuthAlgo, *manualAuthKey, *spi;

	/*   phase 1 */
	char *ph1Mode, *ph1EncryptionAlgo, *ph1IntegrityAlgo, *ph1DHGroup, *ph1KeyTime;

	/*   phase 2 */
	char *ph2EncryptionAlgo, *ph2IntegrityAlgo, *ph2DHGroup, *ph2KeyTime;

	int ph1KeyTimetmp,ph2KeyTimetmp;


 	connName = websGetVar(wp, T("connName"), T(""));
	printf("connName==%s\n",connName);
	remoteGWAddr = websGetVar(wp, T("remoteGWAddr"), T(""));
	printf("remoteGWAddr==%s\n",remoteGWAddr);

	localIPMode = websGetVar(wp, T("localIPMode"), T(""));
		printf("localIPMode==%s\n",localIPMode);
	localIP = websGetVar(wp, T("localIP"), T(""));
		printf("localIP==%s\n",localIP);
	localMask = websGetVar(wp, T("localMask"), T(""));
		printf("localMask==%s\n",localMask);
	remoteIP = websGetVar(wp, T("remoteIP"), T(""));
		printf("remoteIP==%s\n",remoteIP);
	remoteIPMode = websGetVar(wp, T("remoteIPMode"), T(""));
		printf("remoteIPMode==%s\n",remoteIPMode);
	remoteMask = websGetVar(wp, T("remoteMask"), T(""));
		printf("remoteMask==%s\n",remoteMask);
	keyExM = websGetVar(wp, T("keyExM"), T(""));
		printf("keyExM==%s\n",keyExM);

	authM = websGetVar(wp, T("authM"), T(""));
		printf("authM==%s\n",authM);
	psk = websGetVar(wp, T("psk"), T(""));
		printf("psk==%s\n",psk);

	certificateName = websGetVar(wp, T("certificateName"), T(""));//zhengshuwenti
	printf("certificateName==%s\n",certificateName);

	perfectFSEn = websGetVar(wp, T("perfectFSEn"), T(""));
	printf("perfectFSEn==%s\n",perfectFSEn);


	manualEncryptionAlgo = websGetVar(wp, T("manualEncryptionAlgo"), T(""));
	printf("manualEncryptionAlgo==%s\n",manualEncryptionAlgo);
	manualEncryptionKey = websGetVar(wp, T("manualEncryptionKey"), T(""));
	printf("manualEncryptionKey==%s\n",manualEncryptionKey);
	manualAuthAlgo = websGetVar(wp, T("manualAuthAlgo"), T(""));
	printf("manualAuthAlgo==%s\n",manualAuthAlgo);
	manualAuthKey = websGetVar(wp, T("manualAuthKey"), T(""));
	printf("manualAuthKey==%s\n",manualAuthKey);
	spi = websGetVar(wp, T("spi"), T(""));
	printf("spi==%s\n",spi);




	ph1Mode = websGetVar(wp, T("ph1Mode"), T(""));
	printf("ph1Mode==%s\n",ph1Mode);
	ph1EncryptionAlgo = websGetVar(wp, T("ph1EncryptionAlgo"), T(""));
	printf("ph1EncryptionAlgo==%s\n",ph1EncryptionAlgo);
	ph1IntegrityAlgo = websGetVar(wp, T("ph1IntegrityAlgo"), T(""));
	printf("ph1IntegrityAlgo==%s\n",ph1IntegrityAlgo);
	ph1DHGroup = websGetVar(wp, T("ph1DHGroup"), T(""));
	printf("ph1DHGroup==%s\n",ph1DHGroup);
	ph1KeyTime = websGetVar(wp, T("ph1KeyTime"), T(""));
	printf("ph1KeyTime==%s\n",ph1KeyTime);



	ph2EncryptionAlgo = websGetVar(wp, T("ph2EncryptionAlgo"), T(""));
	printf("ph2EncryptionAlgo==%s\n",ph2EncryptionAlgo);
	ph2IntegrityAlgo = websGetVar(wp, T("ph2IntegrityAlgo"), T(""));
	printf("ph2IntegrityAlgo==%s\n",ph2IntegrityAlgo);
	ph2DHGroup = websGetVar(wp, T("ph2DHGroup"), T(""));
	printf("ph2DHGroup==%s\n",ph2DHGroup);
	ph2KeyTime = websGetVar(wp, T("ph2KeyTime"), T(""));
	printf("ph2KeyTime==%s\n",ph2KeyTime);

	ph1KeyTimetmp = atoi (ph1KeyTime);
	printf("ph1KeyTimetmp==%d\n",ph1KeyTimetmp);
	ph2KeyTimetmp = atoi (ph2KeyTime);
	printf("ph2KeyTimetmp==%d\n",ph2KeyTimetmp);




	if(( IPSECRules = nvram_safe_get( "IPSECRules")) && strlen( IPSECRules) )
		snprintf(rulesbuf, sizeof(rulesbuf), "%s;%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%d,%s,%s,%s,%d",
					IPSECRules,"|0|",connName,remoteGWAddr,localIPMode,localIP,localMask,remoteIP,remoteIPMode,remoteMask,keyExM,
					authM,psk,certificateName,perfectFSEn,manualEncryptionAlgo,manualEncryptionKey,manualAuthAlgo,manualAuthKey,spi,ph1Mode,
					ph1EncryptionAlgo,ph1IntegrityAlgo,ph1DHGroup,ph1KeyTimetmp,
					ph2EncryptionAlgo,ph2IntegrityAlgo,ph2DHGroup,ph2KeyTimetmp);

	else
		snprintf(rulesbuf, sizeof(rulesbuf), "%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%d,%s,%s,%s,%d",
					"|0|",connName,remoteGWAddr,localIPMode,localIP,localMask,remoteIP,remoteIPMode,remoteMask,keyExM,
					authM,psk,certificateName,perfectFSEn,manualEncryptionAlgo,manualEncryptionKey,manualAuthAlgo,manualAuthKey,spi,ph1Mode,
					ph1EncryptionAlgo,ph1IntegrityAlgo,ph1DHGroup,ph1KeyTimetmp,
					ph2EncryptionAlgo,ph2IntegrityAlgo,ph2DHGroup,ph2KeyTimetmp);



	//printf("rulesbuf:%s\n", rulesbuf);
	nvram_set( "IPSECRules", rulesbuf);
	nvram_commit();

	/* תipsec ҳ*/
	websRedirect(wp, "firewall/ipsec.asp");
	return;


}


static void ipsec_run(void){

    /*ļ     */
	doSystem("rm -rf /var/racoon.conf");
	doSystem("rm -rf /var/setkey.conf");
	doSystem("rm -rf /var/psk.txt");

	FILE *fp;
   	char wanIP[32];
   	int local_prefixlen;
   	int remote_prefixlen;
  	char cmd[256];
	int i = 0;
	char rec[800];
	char states[8];
	char connName[32];
	char remoteGWAddr[32], localIPMode[8], localIP[32], localMask[32],remoteIPMode[8],remoteIP[32],remoteMask[32];
	char keyExM[8], authM[16], psk[64], certificateName[32],perfectFSEn[8];//key exenge 

	/* ߼ */
	char manualEncryptionAlgo[16], manualEncryptionKey[80], manualAuthAlgo[16], manualAuthKey[60], spi[8];
	/*   phase 1 */
	char ph1Mode[16], ph1EncryptionAlgo[8], ph1IntegrityAlgo[8], ph1DHGroup[10], ph1KeyTime[16];

	/*   phase 2 */
	char ph2EncryptionAlgo[8], ph2IntegrityAlgo[12], ph2DHGroup[12], ph2KeyTime[16];

	char ph1KeyTimetmp[16],ph2KeyTimetmp[16];

	char *pstates = NULL;
	char const *rules = nvram_safe_get( "IPSECRules");


	//жWAN û
	if (-1 == getIfIp(getWanIfNamePPP(), wanIP))
		{
		 //do nothing

		}

	printf("wanIP==%s\n",wanIP);
	printf("getWanIfNamePPP==%s\n",getWanIfNamePPP());

	//ļʼ

	fp = fopen("/var/setkey.conf", "a+");

		 if (fp == NULL) {
				printf("app/ipsec: unable to open setkey file\n");
				return ;
		}
		fprintf(fp, "flush;\n");
		fprintf(fp, "spdflush;\n");
		fclose(fp);//close file


   	fp = fopen("/var/racoon.conf", "a+");
  	 	if (fp == NULL) {
     	 printf("app/ipsec: unable to open racoon file\n");
     	 return ;
  		 }
   		fprintf(fp, "path pre_shared_key \"/var/psk.txt\";\n");
   		fprintf(fp, "path certificate \"%s\";\n\n", CERT_PATH);
		fclose(fp);//close file



	if(!rules)
		return ;
	if(!strlen(rules))
		return ;

	while(getNthValueSafe(i++, (char *)rules, ';', rec, sizeof(rec)) != -1 ){
		// get states
		if((getNthValueSafe(0, rec, ',', states, sizeof(states)) == -1)){
			continue;
		}


		// get connname
		if((getNthValueSafe(1, rec, ',', connName, sizeof(connName)) == -1)){
			continue;
		}


		// remoteGWAddr
		if((getNthValueSafe(2, rec, ',', remoteGWAddr, sizeof(remoteGWAddr)) == -1)){
			continue;
		}
		if(!isIpValid(remoteGWAddr)){
			continue;
		}

		// get localIPMode
		if((getNthValueSafe(3, rec, ',', localIPMode, sizeof(localIPMode)) == -1)){
			continue;
		}

		//get localIP
		if((getNthValueSafe(4, rec, ',', localIP, sizeof(localIP)) == -1)){
			continue;
		}
		if(!isIpValid(localIP)){
			continue;
		}

		//get localMask
		if((getNthValueSafe(5, rec, ',', localMask, sizeof(localMask)) == -1)){
			continue;
		}

		//get remoteIP
		if((getNthValueSafe(6, rec, ',', remoteIP, sizeof(remoteIP)) == -1)){
			continue;
		}
		if(!isIpValid(remoteIP)){
			continue;
		}

		//get remoteIPMode
		if((getNthValueSafe(7, rec, ',', remoteIPMode, sizeof(remoteIPMode)) == -1)){
			continue;
		}

		//get remoteMask
		if((getNthValueSafe(8, rec, ',', remoteMask, sizeof(remoteMask)) == -1)){
			continue;
		}


		//get keyExM
		if((getNthValueSafe(9, rec, ',', keyExM, sizeof(keyExM)) == -1)){
			continue;
		}

		//get authM
		if((getNthValueSafe(10, rec, ',', authM, sizeof(authM)) == -1)){
			continue;
		}

		//get psk
		if((getNthValueSafe(11, rec, ',', psk, sizeof(psk)) == -1)){
			continue;
		}

		//get certificateName
		if((getNthValueSafe(12, rec, ',', certificateName, sizeof(certificateName)) == -1)){
			continue;
		}

		//get perfectFSEn
		if((getNthValueSafe(13, rec, ',', perfectFSEn, sizeof(perfectFSEn)) == -1)){
			continue;
		}

		//get manualEncryptionAlgo
		if((getNthValueSafe(14, rec, ',', manualEncryptionAlgo, sizeof(manualEncryptionAlgo)) == -1)){
			continue;
		}


		//get manualEncryptionKey
		if((getNthValueSafe(15, rec, ',', manualEncryptionKey, sizeof(manualEncryptionKey)) == -1)){
			continue;
		}
		//get manualAuthAlgo
		if((getNthValueSafe(16, rec, ',', manualAuthAlgo, sizeof(manualAuthAlgo)) == -1)){
			continue;
		}

		//get manualAuthKey
		if((getNthValueSafe(17, rec, ',', manualAuthKey, sizeof(manualAuthKey)) == -1)){
			continue;
		}

		//get spi
		if((getNthValueSafe(18, rec, ',', spi, sizeof(spi)) == -1)){
			continue;
		}

		//get ph1Mode
		if((getNthValueSafe(19, rec, ',', ph1Mode, sizeof(ph1Mode)) == -1)){
			continue;
		}

		//get ph1EncryptionAlgo
		if((getNthValueSafe(20, rec, ',', ph1EncryptionAlgo, sizeof(ph1EncryptionAlgo)) == -1)){
			continue;
		}

		//get ph1IntegrityAlgo
		if((getNthValueSafe(21, rec, ',', ph1IntegrityAlgo, sizeof(ph1IntegrityAlgo)) == -1)){
			continue;
		}

		//get ph1DHGroup
		if((getNthValueSafe(22, rec, ',', ph1DHGroup, sizeof(ph1DHGroup)) == -1)){
			continue;
		}

		//get ph1KeyTimetmp
		if((getNthValueSafe(23, rec, ',', ph1KeyTimetmp, sizeof(ph1KeyTimetmp)) == -1)){
			continue;
		}

		//get ph2EncryptionAlgo
		if((getNthValueSafe(24, rec, ',', ph2EncryptionAlgo, sizeof(ph2EncryptionAlgo)) == -1)){
			continue;
		}

		//get ph2IntegrityAlgo
		if((getNthValueSafe(25, rec, ',', ph2IntegrityAlgo, sizeof(ph2IntegrityAlgo)) == -1)){
			continue;
		}

		//get ph2DHGroup
		if((getNthValueSafe(26, rec, ',', ph2DHGroup, sizeof(ph2DHGroup)) == -1)){
			continue;
		}

		//get ph2KeyTimetmp
		if((getNthValueSafe(27, rec, ',', ph2KeyTimetmp, sizeof(ph2KeyTimetmp)) == -1)){
			continue;
		}


		if(states[1] == '0')   //ر״̬ļ
		{
			continue;
		}
		else{


			//setkey.conf
			 fp = fopen("/var/setkey.conf", "a+");

			 if (fp == NULL) {
				printf("app/ipsec: unable to open file\n");
				return ;
			 }

		     if (!strcmp(keyExM, "manual")) {


		         fprintf(fp, "add %s %s esp 0x%s -m tunnel -E %s 0x%s -A %s 0x%s;\n",
                 		wanIP, remoteGWAddr, spi,manualEncryptionAlgo,manualEncryptionKey,manualAuthAlgo, manualAuthKey);
        		 fprintf(fp, "add %s %s esp 0x%s -m tunnel -E %s 0x%s -A %s 0x%s;\n",
                 		remoteGWAddr, wanIP, spi,manualEncryptionAlgo, manualEncryptionKey,manualAuthAlgo,manualAuthKey);

			 	}

			 if (!strcmp(localIPMode, "subnet"))

				 local_prefixlen = rutIPSec_calPrefixLen(localMask);
			 	else
					local_prefixlen = 32;

			 if (!strcmp(remoteIPMode, "subnet"))

				 remote_prefixlen = rutIPSec_calPrefixLen(remoteMask);
			 	else
					remote_prefixlen = 32;


		      	fprintf(fp, "spdadd %s/%d %s/%d any -P out ipsec esp/tunnel/%s-%s/require;\n",
             			localIP, local_prefixlen, remoteIP, remote_prefixlen, wanIP,remoteGWAddr);
     		  	fprintf(fp, "spdadd %s/%d %s/%d any -P in ipsec esp/tunnel/%s-%s/require;\n",
              			remoteIP, remote_prefixlen, localIP, local_prefixlen,remoteGWAddr, wanIP);

				fclose(fp);//close file

				//disable masquegrading for ipsec tunnel
     			sprintf(cmd, "iptables -t nat -I POSTROUTING 1 -s %s/%d -o %s -d %s/%d -j ACCEPT",
                   		localIP, local_prefixlen, getWanIfNamePPP(), remoteIP, remote_prefixlen);
      					doSystem(cmd);

      			printf ("rutIPSec_doSetkeyRacoon: spdadd Done\n");


			 //racoon


      	if (!strcmp(keyExM, "auto") ){


		fp = fopen("/var/racoon.conf", "a+");
  	 		if (fp == NULL) {
     		 printf("app/ipsec: unable to open racoon file\n");
     		 return ;
  			 }

		 if (!strcmp(localIPMode, "subnet")) {
            local_prefixlen = rutIPSec_calPrefixLen(localMask);
          }
			else {
            local_prefixlen = 32;
         	}

		 if (!strcmp(remoteIPMode, "subnet")) {
            remote_prefixlen = rutIPSec_calPrefixLen(remoteMask);
         }
         	else {
            remote_prefixlen = 32;
            }

         fprintf(fp, "remote %s {\n", remoteGWAddr);
         fprintf(fp, "exchange_mode %s;\n", ph1Mode);
         fprintf(fp, "lifetime time %d sec;\n", atoi(ph1KeyTimetmp));
         fprintf(fp, "proposal_check obey;\n");

         if (!strcmp(authM, "pre_shared_key")) {
            fprintf(fp, "proposal {\n");
            fprintf(fp, "encryption_algorithm %s;\n", ph1EncryptionAlgo);
            fprintf(fp, "hash_algorithm %s;\n", ph1IntegrityAlgo);
            fprintf(fp, "authentication_method %s;\n", "pre_shared_key");
            fprintf(fp, "dh_group %s;\n", ph1DHGroup);
            fprintf(fp, "}\n");
            fprintf(fp, "}\n\n");
         }

         else if (strcmp(authM, "certificate")) {
            fprintf(fp, "verify_cert on;\n");
            fprintf(fp, "my_identifier asn1dn;\n");
            fprintf(fp, "peers_identifier asn1dn;\n");
            fprintf(fp, "certificate_type x509 \"%s.cert\" \"%s.priv\";\n",
                    certificateName, certificateName);
            fprintf(fp, "proposal {\n");
            fprintf(fp, "encryption_algorithm %s;\n", ph1EncryptionAlgo);
            fprintf(fp, "hash_algorithm %s;\n", ph1IntegrityAlgo);
            fprintf(fp, "authentication_method %s;\n", "rsasig");
            fprintf(fp, "dh_group %s;\n", ph1DHGroup);
            fprintf(fp, "}\n");
            fprintf(fp, "}\n\n");
         }
		 else {
            // unknown type  do nothing
         }

         fprintf(fp, "sainfo address %s/%d any address %s/%d any {\n",
                 localIP, local_prefixlen, localIP, remote_prefixlen);
         if (!strcmp(perfectFSEn, "enable")) {
            fprintf(fp, "pfs_group %s;\n", ph2DHGroup);
         }
         fprintf(fp, "lifetime time %d sec;\n", atoi(ph2KeyTimetmp));
         fprintf(fp, "encryption_algorithm %s;\n", ph2EncryptionAlgo);
         fprintf(fp, "authentication_algorithm %s;\n", ph2IntegrityAlgo);
         fprintf(fp, "compression_algorithm deflate;\n");
         fprintf(fp, "}\n\n");

		 fclose(fp);//close   racoon

		}


		// psk  psk.txt

   		fp = fopen("/var/psk.txt", "a+");
   		if (fp == NULL) {
      		printf("app/ipsec: unable to open file\n");
     			 return ;}

      	if ((!strcmp(keyExM, "auto")) && (!strcmp(authM, "pre_shared_key"))) {
         	fprintf(fp, "%s %s\n", remoteGWAddr, psk);
     	 	}

   		fclose(fp); //close psk.txt

		}


	}

   	doSystem("chmod 600 /var/psk.txt");

	return;

}

static void IpsecAction(webs_t wp, char_t *path, char_t *query)
{
	int i, j, rule_count;
	char *actionType;
	int *actArray = NULL;
	char_t name_buf[16];
	char flgEnable;
	char *pRule = NULL;
	char_t *value;
	char const *rules = nvram_safe_get( "IPSECRules");
	char *newRules = NULL;
	char *p_ch = NULL;

	if(!rules || !strlen(rules) )
		goto end;

	newRules = strdup(rules);
	if (!newRules){
		perror("strdup");
		return;
	}

	rule_count = getRuleNums(newRules);
	if(!rule_count)
		goto end;

	actionType = websGetVar(wp, T("actionType"), NULL);
	if (!actionType)
		goto end;
	//printf("action:%s\n", actionType);
	/* unknown action */
	if ( (0 != strcmp(actionType, "en"))	//enable
		&& (0 != strcmp(actionType, "dis"))	//disable
		&& (0 != strcmp(actionType, "del")) )	//delete
		goto end;

	actArray = (int *)malloc(rule_count * sizeof(int));
	if (!actArray) {
		perror("malloc");
		goto end;
	}

	for(i=0, j=0; i< rule_count; i++){
		snprintf(name_buf, 16, "actRule%d", i);
		value = websGetVar(wp, T(name_buf), NULL);
		if(value){
			actArray[j++] = i;
		}
	}
	if ( (0 == strcmp(actionType, "en")) || (0 == strcmp(actionType, "dis")) ){
		flgEnable = (actionType[0] == 'e') ? '1' : '0';
		for (i = 0; i < j; i++){
			pRule = getNthValueOffset(actArray[i], (char *)newRules);
			//printf("rule1:%s\n", pRule);
			if (pRule)
				p_ch = strchr(pRule, '|');
			if (p_ch)
				//*(++pRule) = flgEnable;
				*(p_ch+1) = flgEnable;

			//printf("rule2:%s\n", pRule);
		}
	}else if (0 == strcmp(actionType, "del") ){
		deleteNthValueMulti(actArray, rule_count, (char *)newRules, ';');
	}else
		goto end;

	nvram_set( "IPSECRules", (char *)newRules);
	nvram_commit();

	if (actArray)
		free(actArray); //release memory
	if (newRules)
		free(newRules);

	websRedirect(wp, "firewall/ipsec.asp");

	/* make config  file */
	ipsec_run();

	return;

end:
	if (actArray)
		free(actArray); //release memory

	if (newRules)
		free(newRules);

	websRedirect(wp, "firewall/ipsec.asp");
	return;
}


/* call in goahead.c initWebs() */
void formDefineVPN(void)
{


	websFormDefine(T("setipsec_config"), setipsec_config);
	websAspDefine(T("showIPSECASP"), showIPSECASP);
	websAspDefine(T("getIPSECASPRuleNumsASP"), getIPSECASPRuleNumsASP);
	websFormDefine(T("IpsecAction"), IpsecAction);


}

