/* vi: set sw=4 ts=4 sts=4 fdm=marker: */
/*
 *	wizard.c -- Settings Wizard
 *
 *	$Id: wizard.c,v 1.167.2.8 2009-07 radar Exp $
 */

#include	<stdlib.h>
#include	<sys/ioctl.h>
#include	<net/if.h>
#include	<net/route.h>
#include    <string.h>
#include    <dirent.h>
#include    <notify.h>
#include    <oid.h>
#include    <signal.h>

#include	"internet.h"
#include	"bcmnvram.h"
#include	"webs.h"
#include	"utils.h"
#include 	"firewall.h"
#include	"management.h"
#include	"station.h"
#include	"wireless.h"

#include	"linux/autoconf.h"  //kernel config
#include	"config/autoconf.h" //user config
#include	"wps.h"	// add by paul@20091223. for LedReset();
#include	"user/busybox/include/autoconf.h" //busybox config

#define 	NOTIFY_TGT_PID           1 /* init(rc) */


static void wizard_wirelessGetSecurity(webs_t wp, char_t *path, char_t *query);
static void wizard_APSecurity(webs_t wp, char_t *path, char_t *query);
static void	wizard_commit(webs_t wp, char_t *path, char_t *query);
static void	wizard_checkWAN(webs_t wp, char_t *path, char_t *query);

extern void clearRadiusSetting( int mbssid);
//extern int g_wsc_configured;

/* Wizard handle. radar 20090722.  */
struct wizard_buffer{
	char *opMode, *nat_en;	/* OpMode */
	char *time_zone;	/* NTP */

	/* wan */
	char	*ctype;
	char	*ip, *nm, *gw, *pd, *sd;
	char	*eth, *user, *pass;
	char	*clone_en, *clone_mac;
	char 	*pptp_srv, *pptp_mode;
	char 	*l2tp_srv, *l2tp_mode;

	/* wireless */
	char *security_mode;
	char *security_shared_mode;
	char *pass_phrase_str;

	char *PMKCachePeriod;
	char *PreAuth;

	/* wep */
	char_t *DefaultKeyID, *Key1Type, *Key1Str, *Key2Type, *Key2Str, *Key3Type, *Key3Str, *Key4Type, *Key4Str;

	/* WPA */
	char *cipher_str;
	char *key_renewal_interval;
#ifdef CONFIG_USER_3G
	//char_t	*usb3g_dev;
	char_t	net3gOperators[16], net3gMode[16];
	char_t	net3gApn[16], net3gDialNum[32], net3gUserName[32], net3gPassWord[32], net3gAuthenticationType[16], net3gDialStyle[16];
	char_t	net3gPppInterval[16], net3gPppFalutMaxTimes[16], net3gDemandIdleTime[16];
#endif
};
static struct wizard_buffer g_wizardbuf = {NULL};

/* call in goahead.c initWebs() */
void formDefineWizard(void)
{
	static char flg_inited = 0;
	websFormDefine(T("wizard_wirelessGetSecurity"), wizard_wirelessGetSecurity);
	websFormDefine(T("wizard_commit"), wizard_commit);
        websFormDefine(T("wizard_checkWAN"), wizard_checkWAN);

	/* init wizard buffer */
	if ( !flg_inited ) { /* ֹôͬһбִжڴй©*/
		memset(&g_wizardbuf, 0, sizeof(g_wizardbuf));
		flg_inited = 1;
	}
}

static char *
safe_strdup(const char *s)
{
	if (s == NULL) {
		return strdup("");
	}
	return strdup(s);
}

#define	SAFE_STRDUP(dup2, from)	do {	\
	if (dup2){	\
		free(dup2); 	\
		dup2 = NULL;	\
	}	\
	dup2 = safe_strdup(from);	\
	printf("dup %s, set:%s, value:%s\n", dup2?"Success":"Failed", #from, from);	\
}while(0)


static void STFs(int index, char *flash_key, char *value)
{
	char *result;
	char const *tmp = nvram_get(flash_key);
	if(tmp){
		result = setNthValue(index, (char *)tmp, value);
		nvram_set(flash_key, result);
	}
	return ;
}

#define STF( index, flash_key)	STFs( index, #flash_key, flash_key)
#define	PRNT_LN()	printf("func:%s, L%d\n", __FUNCTION__, __LINE__)
#define LFW(x, y)	do{	\
			if(! ( x = websGetVar(wp, T(#y), T(""))))	\
				return;	\
		}while(0)

#define	RELOAD_TIMEOUT	"25000"	/* 25 secs */
#define	RELOAD_TIMEOUT2	"20000"	/* 20 secs */

static void wizard_checkWAN(webs_t wp, char_t *path, char_t *query)
{

        char wan_stat[5] = {0};
        if(is_phyconnected() >0)
        {
                 wan_stat[0]= '1';
        }
        else
        {
                wan_stat[0]= '0';
        }
        websWrite(wp, T("HTTP/1.1 200 OK\nContent-type: text/plain\nPragma: no-cache\nCache-Control: no-cache\n\n"));
        websWrite(wp, T("%s"), wan_stat);
        websDone(wp, 200);
	 return;
}

void wizard_confWPAGeneral(webs_t wp, int mbssid)
{
	//STFs( mbssid, "EncrypType", "TKIPAES"); 
	STFs( mbssid, "DefaultKeyID", "2");	// DefaultKeyID is 2
	STFs( mbssid, "RekeyInterval", "3600");
	STFs( mbssid, "RekeyMethod", "TIME");
	STFs( mbssid, "IEEE8021X", "0");

}

static void wizard_commitSetWan(struct wizard_buffer *wizardBuf)
{
	char_t	*ctype;
	char_t	*ip, *nm, *gw, *pd, *sd;
	char_t	*user, *pass;
	char_t	*clone_en, *clone_mac;
	char_t 	*pptp_srv, *pptp_mode;
	char_t 	*l2tp_srv, *l2tp_mode;

	char	*opmode = wizardBuf->opMode;
	char const *	lan_ip = nvram_safe_get( "lan_ipaddr");
	char const *	lan2enabled = nvram_safe_get( "Lan2Enabled");

	ctype = ip = nm = gw = pd = sd = user = pass =
		clone_en = clone_mac = pptp_srv = pptp_mode = l2tp_srv = l2tp_mode =
		NULL;
        PRNT_LN();
	ctype = wizardBuf->ctype;	
	if (!strncmp(ctype, "STATIC", 7) || !strcmp(opmode, "0")) {
		ip = wizardBuf->ip;	//websGetVar(wp, T("staticIp"), T(""));
		nm = wizardBuf->nm;	//websGetVar(wp, T("staticNetmask"), T("0"));
		gw = wizardBuf->gw;	//websGetVar(wp, T("staticGateway"), T(""));
		pd = wizardBuf->pd;	//websGetVar(wp, T("staticPriDns"), T(""));
		sd = wizardBuf->sd;	//websGetVar(wp, T("staticSecDns"), T(""));
              PRNT_LN();
		nvram_set( "wanConnectionMode", ctype);
		if (-1 == inet_addr(ip)) {
			return;
		}

		if (NULL != opmode && strcmp(opmode, "0") && !strncmp(ip, lan_ip, 15)) {
			return;
		}
		if (!strcmp(lan2enabled, "1"))
		{
			char const *	lan2_ip = nvram_safe_get( "lan2_ipaddr");
			if (NULL != opmode && strcmp(opmode, "0") && !strncmp(ip, lan2_ip, 15)) {
				return;
			}
		}
		nvram_set( "wan_ipaddr", ip);
		nvram_set( "wan_static_ipaddr",  ip );
		if (-1 == inet_addr(nm)) {
			return;
		}
		nvram_set( "wan_netmask", nm);
		nvram_set( "wan_static_netmask",  nm );
		if (NULL != opmode && !strcmp(opmode, "0")) {
			nvram_set( "lan_ipaddr", ip);
			nvram_set( "lan_netmask", nm);
		}
		nvram_set( "wan_gateway", gw);
		nvram_set( "wan_primary_dns", pd);
		nvram_set( "wan_secondary_dns", sd);
		nvram_set( "wan_static_gateway",  gw);
              nvram_set( "wan_static_primary_dns",  pd);
              nvram_set( "wan_static_secondary_dns",  sd);
              nvram_set( "wan_dnsmanual", "1" );
		nvram_set( "wan_proto", "static");
		PRNT_LN();
	}
	else if (!strncmp(ctype, "DHCP", 5)) {
		PRNT_LN();
		nvram_set( "wanConnectionMode", ctype);
		nvram_set( "wan_proto", "dhcp");
        nvram_set( "wan_dnsmanual", "0" );
	}
	else if (!strncmp(ctype, "PPPOE", 6)) {
		PRNT_LN();
		user = wizardBuf->user;
		pass = wizardBuf->pass;	

		nvram_set( "wan_pppoe_user", user);
		nvram_set( "wan_pppoe_pwd", pass);
		nvram_set( "wanConnectionMode", ctype);
		nvram_set( "wan_pppoe_opmode", "keep_alive");
		nvram_set( "wan_pppoe_optime", "60");
		nvram_set( "wan_proto" ,"pppoe");
		 nvram_set( "wan_dnsmanual", "0" );
	}
	/*start of add l2tp and pptp by huangyikun at 20110726*/
	else if (!strncmp(ctype, "L2TP", 5)) {
		PRNT_LN();
		user = wizardBuf->user;
		pass = wizardBuf->pass;
		l2tp_srv = wizardBuf->l2tp_srv;

		nvram_set("wan_l2tp_user", user);
		nvram_set("wan_l2tp_pass", pass);
		nvram_set("wanConnectionMode", ctype);
		nvram_set( "wan_proto", "l2tp");
		nvram_set("wan_l2tp_server", l2tp_srv);
		nvram_set("wan_l2tp_dhcp", "0");
        nvram_set( "wan_dnsmanual", "0" );
	}
	else if (!strncmp(ctype, "PPTP", 5)) {
		PRNT_LN();
		user = wizardBuf->user;
		pass = wizardBuf->pass;
		pptp_srv = wizardBuf->pptp_srv;

		nvram_set("wan_pptp_user", user);
		nvram_set("wan_pptp_pass", pass);
		nvram_set("wanConnectionMode", ctype);
		nvram_set( "wan_proto", "pptp");
		nvram_set("wan_pptp_server", pptp_srv);
		nvram_set("wan_pptp_dhcp", "0");
        nvram_set( "wan_dnsmanual", "0" );
	}
	/*end of add l2tp and pptp by huangyikun at 20110726*/

	else {
		return;
	}
	PRNT_LN();

}


static void wizard_commit(webs_t wp, char_t *path, char_t *query)
{
        char_t	*ctype;
	char_t	*ip, *nm, *gw, *pd, *sd;
	char_t	*user, *pass;
	/*start of add l2tp and pptp by huangyikun at 20110726*/
	char_t  *pptp_srv, *l2tp_srv;
	/*end of add l2tp and pptp by huangyikun at 20110726*/

        int mbssid;
	char *wireless_states;
	char *ssid;
	char *security;
	char *key;
	char_t	*wirelessmode;
	char_t *cur_mode, *old_mode;
	char_t  *newSsid_flag;  //value=1 means we have a new SSID

	mbssid = 0;

	SAFE_STRDUP(g_wizardbuf.opMode, "1");
	SAFE_STRDUP(g_wizardbuf.nat_en, "1");

	char	*opmode = g_wizardbuf.opMode;
	old_mode = nvram_safe_get( "sw_mode_ex");
	cur_mode = websGetVar(wp, T("opMode"), T("0"));
	printf("=========old_mode: %s, cur_mode: %s.\n", old_mode, cur_mode);

	ctype = websGetVar(wp, T("connectionType"), T("0"));
	SAFE_STRDUP(g_wizardbuf.ctype, ctype);
	if (!strncmp(ctype, "STATIC", 7) || !strcmp(opmode, "0")) {
		ip = websGetVar(wp, T("staticIp"), T(""));
		nm = websGetVar(wp, T("staticNetmask"), T("0"));
		gw = websGetVar(wp, T("staticGateway"), T(""));
		pd = websGetVar(wp, T("staticPriDns"), T(""));
		sd = websGetVar(wp, T("staticSecDns"), T(""));

		SAFE_STRDUP(g_wizardbuf.ip, ip);
		SAFE_STRDUP(g_wizardbuf.nm, nm);
		SAFE_STRDUP(g_wizardbuf.gw, gw);
		SAFE_STRDUP(g_wizardbuf.pd, pd);
		SAFE_STRDUP(g_wizardbuf.sd, sd);
	}
	else if (!strncmp(ctype, "PPPOE", 6)) {
		user = websGetVar(wp, T("pppoeUser"), T(""));
		pass = websGetVar(wp, T("pppoePass"), T(""));
		SAFE_STRDUP(g_wizardbuf.user, user);
		SAFE_STRDUP(g_wizardbuf.pass, pass);
	}
	/*start of add l2tp and pptp by huangyikun at 20110726*/
	else if (!strncmp(ctype, "L2TP", 5)) {
		user = websGetVar(wp, T("l2tpUser"), T(""));
		pass = websGetVar(wp, T("l2tpPass"), T(""));
		l2tp_srv = websGetVar(wp, T("l2tpServer"), T(""));
		
		SAFE_STRDUP(g_wizardbuf.user, user);
		SAFE_STRDUP(g_wizardbuf.pass, pass);
		SAFE_STRDUP(g_wizardbuf.l2tp_srv, l2tp_srv);
	}

	else if (!strncmp(ctype, "PPTP", 5)) {
		user = websGetVar(wp, T("pptpUser"), T(""));
		pass = websGetVar(wp, T("pptpPass"), T(""));
		pptp_srv = websGetVar(wp, T("pptpServer"), T(""));
		
		SAFE_STRDUP(g_wizardbuf.user, user);
		SAFE_STRDUP(g_wizardbuf.pass, pass);
		SAFE_STRDUP(g_wizardbuf.pptp_srv, pptp_srv);
	}
	/*end of add l2tp and pptp by huangyikun at 20110726*/

	/* ·ģʽ¸ıwan. lg, 20130530 */
	if(!strncmp(cur_mode,"1",2))
		wizard_commitSetWan(&g_wizardbuf);

        LFW(wireless_states, wireless_states);

	LFW(ssid, ssid);
	LFW(security, security);

	LFW(key, key);

	wirelessmode = websGetVar(wp, T("wirelessmode"), T("9")); 

	if (!strncmp(wireless_states, "1", 2)) {
		system("iwpriv ra0 set RadioOn=0");
		nvram_set( "RadioOn", "0");
		ledAlways(13, LED_OFF); 
		wpsStopTimerAll(); 
		LedReset(); 

		}

	else if (!strncmp(wireless_states, "0", 2)) {
		system("iwpriv ra0 set RadioOn=1");
		nvram_set( "RadioOn", "1");
		}
		//start of to make wireless settings effect by huangfeilan at 20111219
		nvram_set("SetupWizard","1");
		//end of to make wireless settings effect by huangfeilan at 20111219

	if (0 == strlen(ssid)) {
		websError(wp, 403, T("'SSID' should not be empty!"));
		return;
	}

	nvram_set( "WirelessMode", wirelessmode);
	//start of that make wirelessmode effect when change in setup_wizard by huangfeilan at 20120130
	if (!strncmp(wirelessmode, "4", 2) || !strncmp(wirelessmode, "7", 2)) //g, gn
		nvram_set( "BasicRate", "351");
	else if (!strncmp(wirelessmode, "1", 2)) //b
		nvram_set( "BasicRate", "3");
	else //bg,bgn,n
		nvram_set( "BasicRate", "15");
	//end of that make wirelessmode effect when change in setup_wizard by huangfeilan at 20120130
	nvram_set( "SSID1", ssid);
	clearRadiusSetting( mbssid);

#ifdef DEFINE_DHCP_AUTO_SWITCH
	newSsid_flag = atoi(websGetVar(wp, T("newSsid_flag"), T("0")));
	if(newSsid_flag == 1 || strncmp(old_mode, "6", 2)) //We have a new SSID1 or old_mode is not Repeater, backup wireless info for restore 
	{
		nvram_set( "res_SSID1", nvram_safe_get("SSID1")); //we only change res_SSID1
		//backup wireless info for restore	
		/*if(!strncmp(cur_mode,"6",2)){
			nvram_set( "res_SSID1", nvram_safe_get("SSID1"));
			nvram_set( "res_AuthMode", nvram_safe_get("AuthMode"));
			nvram_set( "res_EncrypType", nvram_safe_get("EncrypType"));
			nvram_set( "res_WPAPSK1", nvram_safe_get("WPAPSK1"));
		}*/
	}
	else
	{
		printf("Repeater res_SSID1, do nothing.\n");
	}

#endif	


	if( !strcmp(security, "0")){
		STFs( mbssid, "AuthMode", "OPEN");
		STFs( mbssid, "EncrypType", "NONE");
		STFs( mbssid, "IEEE8021X", "0");
#ifdef DEFINE_DHCP_AUTO_SWITCH			
		nvram_set( "res_AuthMode", "OPEN" );
		nvram_set( "res_EncrypType", "NONE" );
#endif
	}
	else if(!strcmp(security, "1"))	{
		wizard_confWPAGeneral( wp, mbssid);
		if( !strcmp(nvram_safe_get("AuthMode"),"OPEN"))
		{
			STFs( mbssid, "EncrypType", "TKIPAES"); //by wwb 20140412 :if AuthMode=OPEN ,forced change into TKIPAES	
			STFs( mbssid, "AuthMode", "WPAPSKWPA2PSK");  //if AuthMode=OPEN ,forced change into WPAPSKWPA2PSK
		}
		STFs( mbssid, "IEEE8021X", "0");		
#ifdef DEFINE_DHCP_AUTO_SWITCH
		if( !strcmp(nvram_safe_get("res_AuthMode"),"OPEN"))
		{
			nvram_set( "res_EncrypType", "TKIPAES"); //by wwb 20140412 :if res_AuthMode=OPEN ,forced change into TKIPAES 
			nvram_set( "res_AuthMode", "WPAPSKWPA2PSK");  //if res_AuthMode=OPEN ,forced change into WPAPSKWPA2PSK
		}
		
		char *old_key;
		old_key = nvram_safe_get("WPAPSK1");
		printf("old_key= %s   key= %s \n", old_key, key);
		if( old_key != "" && key !="" )
		{		
			if( strcmp(old_key, key) != 0 ) //old_key != key, user change
			{
				printf("nvram_set res_WPAPSK1 \n");
				nvram_set("res_WPAPSK1", key);
			}	
		}		

		//backup wireless info for restore 
		if ( !strncmp(cur_mode, "6", 2) 
			&& nvram_match("SynSsid", "1") 
			&& nvram_invmatch("ApCliSsid",""))
		{		
			;//no backup
		}
		else
		{
			nvram_set( "res_SSID1", nvram_safe_get("SSID1"));
			nvram_set( "res_AuthMode", nvram_safe_get("AuthMode"));
			nvram_set( "res_EncrypType", nvram_safe_get("EncrypType"));
			//nvram_set( "res_WPAPSK1", nvram_safe_get("WPAPSK1"));
		}		
#endif
	    nvram_set(racat("WPAPSK", mbssid+1), key);
		
	}
	else{
		//do nothing
	}
	if(mbssid == 0){
		char const *wordlist= nvram_get( "WscModeOption");
		if(wordlist){
			if (strcmp(wordlist, "0"))
				doSystem("iwpriv ra0 set WscConfStatus=1");
			nvram_set( "WscConfigured", "1");
			//g_wsc_configured = 1;
			nvram_set( "g_wsc_configured", "1");
		}
	}

	/* 3g mode */
	//if(nvram_match("sw_mode_ex", "1") && nvram_match("wan_proto", "3g"))
	//if(!strncmp(cur_mode,"9",2) && nvram_match("wan_proto", "3g"))	
	if(!strncmp(cur_mode,"9",2))
	{
		char *OperatorsName3g = NULL, *pinCode3g = NULL, *dialNum3g = NULL, 
			*APN3g = NULL, *userName3g = NULL, *passWord3g = NULL, *connectMode3g = NULL, *mtu3g = NULL;

		char *idleTime3g = NULL, *state3g = NULL, *net3gMannual = NULL;
		
		OperatorsName3g = websGetVar(wp, T("net3gOperatorsName"), T(""));
        //pinCode3g = websGetVar(wp, T("net3gPinCode"), T(""));
        dialNum3g = websGetVar(wp, T("net3gDialNum"), T(""));
        APN3g = websGetVar(wp, T("net3gAPN"), T(""));
        userName3g = websGetVar(wp, T("net3gUsername"), T(""));
        passWord3g = websGetVar(wp, T("net3gPassword"), T(""));
        //connectMode3g = websGetVar(wp, T("net3gConnectMode"), T(""));
		//mtu3g = websGetVar(wp, T("net3gMtu"), T(""));
		state3g = websGetVar(wp, T("net3gState"), T(""));
		net3gMannual = websGetVar(wp, T("net3gMannual"), T(""));
		printf("=======OperatorsName3g:%s, dialNum3g:%s, APN3g:%s,userName3g:%s,passWord3g:%s.======== \n", 
			OperatorsName3g, dialNum3g, APN3g, userName3g, passWord3g);
#if 0		
		if( !strcmp(connectMode3g, "ConnectOnDemand"))
		{
			idleTime3g = websGetVar(wp, T("net3gIdleTime"), T(""));
			nvram_set( "net3gIdleTime", idleTime3g);
			nvram_set( "net3gConnectMode", "ConnectOnDemand" );
		}
		else if( !strcmp(connectMode3g, "KeepAlive"))
		{
			nvram_set( "net3gIdleTime", "0");
			nvram_set( "net3gConnectMode",	"KeepAlive" );
		}
#endif

		nvram_set( "net3gOperatorsName", OperatorsName3g);
		nvram_set( "net3gUsername", userName3g);
		nvram_set( "net3gPassword", passWord3g);
		//nvram_set( "net3gPinCode", pinCode3g);
		nvram_set( "net3gState", state3g);
		
		if( NULL != net3gMannual && !strncmp( net3gMannual, "1", 2 ))
			nvram_set( "net3gMannual", "1");
		else
			nvram_set( "net3gMannual", "0");
		
        nvram_set( "net3gDialNum", dialNum3g);
		nvram_set( "net3gAPN", APN3g);
        //nvram_set( "net3gMtu", mtu3g );
		//nvram_set( "wan_proto", p_wn_type);
    }	

	/* apclient mode */
	//if(!strncmp(nvram_get("sw_mode_ex"),"6",2))
	if(!strncmp(cur_mode,"6",2))	
	{
		char *apcli_ssid, *apcli_mode, *apcli_enc, *wpapsk, *def_key, *keytype1, *keytype2, *keytype3, *keytype4,
		 	*key1, *key2, *key3, *key4, *Cchannel;
		int curr_channel = 0;
		char* sync_apcli = NULL;
		
		apcli_ssid = websGetVar(wp, T("apcli_ssid"), T(""));
		apcli_mode = websGetVar(wp, T("apcli_mode"), T("OPEN"));
		Cchannel = websGetVar(wp, T("apcli_channel"), T(""));
		apcli_enc = websGetVar(wp, T("apcli_enc"), T("NONE"));
		wpapsk = websGetVar(wp, T("apcli_wpapsk"), T("12345678"));	
		def_key = websGetVar(wp, T("apcli_default_key"), T("1"));
		keytype1 = websGetVar(wp, T("apcli_key1type"), T("1"));
		keytype2 = websGetVar(wp, T("apcli_key2type"), T("1"));
		keytype3 = websGetVar(wp, T("apcli_key3type"), T("1"));
		keytype4 = websGetVar(wp, T("apcli_key4type"), T("1"));
		key1 = websGetVar(wp, T("apcli_key1"), T(""));
		key2 = websGetVar(wp, T("apcli_key2"), T(""));
		key3 = websGetVar(wp, T("apcli_key3"), T(""));
		key4 = websGetVar(wp, T("apcli_key4"), T(""));	

		printf("setup ssid: %s, def_key: %s. \n", apcli_ssid,def_key);
			/*
			if (gstrlen(apcli_ssid) == 0) {
				websError(wp, 200, "SSID is empty");
				return;
			}
			*/	
			int k;
			for(k=0;k<3;k++)
			{
				apcli_t *p_head = NULL, *p_curr = NULL;
				printf("Enter getClientPar\n");
				doSystem( "iwpriv ra0 set SiteSurvey=1" );
			    sleep( 1 );
				get_site_survey_ap( &p_head );

				
				char ex_channel[10]={0};
				p_curr = p_head;
				while (p_curr)
				{
					char strSsid[ 38 ] = { 0 };
					strcpy(strSsid, p_curr->s_ssid);
					if(!strcmp(strSsid,apcli_ssid))
					{
						printf("OK\n");
						curr_channel = p_curr->i_channel;
						strcpy(ex_channel,p_curr->s_extch);
						break;
					}

					p_curr = p_curr->p_next;							
				}
				
				free_apcli_list( p_head );
				p_head = NULL;
				printf("Client Exchannel=%s\n",ex_channel);
				printf("Client curr_channel=%d\n",curr_channel);
				

				if(curr_channel!=0)	
				{
					sprintf(Cchannel,"%d",curr_channel);					
					if(strncmp(ex_channel, "BELOW", 5) == 0)
						nvram_set("HT_EXTCHA","0");
					if(strncmp(ex_channel, "ABOVE", 5) == 0)
						nvram_set("HT_EXTCHA","1");
					break;					
				}
				
			}

			if (gstrlen(apcli_ssid) == 0)
			{
				nvram_set("ApCliEnable", "0");
			}
			else
			{
				nvram_set("ApCliEnable", "1");
			}
			nvram_set("ApCliSsid", apcli_ssid);
			nvram_set("Channel", Cchannel);
			if (!strncmp(Cchannel, "0", 2)) {
				nvram_set("AutoChannelSelect", "2");
			} else {
				nvram_set("AutoChannelSelect", "0");
			}			
			
			nvram_set("ApCliAuthMode", apcli_mode);
			nvram_set("ApCliEncrypType", apcli_enc);
			nvram_set("ApCliWPAPSK", wpapsk);
			nvram_set("ApCliDefaultKeyID", def_key);
			nvram_set("ApCliKey1Type", keytype1);
			nvram_set("ApCliKey1Str", key1);
			nvram_set("ApCliKey2Type", keytype2);
			nvram_set("ApCliKey2Str", key2);
			nvram_set("ApCliKey3Type", keytype3);
			nvram_set("ApCliKey3Str", key3);
			nvram_set("ApCliKey4Type", keytype4);
			nvram_set("ApCliKey4Str", key4);
			nvram_set("SynSsid", "0");
			
			sync_apcli = websGetVar(wp, T("synchronism"), T("off"));			
			/*printf("sync_apcli:%s\n", sync_apcli);*/
			if (!strncmp(sync_apcli, "on", 2)) 
			{
				char* apclienable = nvram_safe_get( "ApCliEnable");
#ifndef DEFINE_DHCP_AUTO_SWITCH
				if (!strcmp(apclienable, "1"))
				{
					/* for ap. liugang, 20130321 */
					nvram_set("SSID1", apcli_ssid);
					nvram_set("AuthMode", apcli_mode);
					nvram_set("EncrypType", apcli_enc);			
					nvram_set("WPAPSK1", wpapsk);
					nvram_set("DefaultKeyID", def_key);			
					nvram_set("Key1Type", keytype1);
					nvram_set("Key1Str1", key1);
					nvram_set("Key2Type", keytype2);
					nvram_set("Key2Str1", key2);
					nvram_set("Key3Type", keytype3);
					nvram_set("Key3Str1", key3);
					nvram_set("Key4Type", keytype4);
					nvram_set("Key4Str1", key4);					
				}
#endif				
				nvram_set("SynSsid", "1");
			}
#ifdef DEFINE_DHCP_AUTO_SWITCH	
			else // not sync_apcli,restore old wireless
			{
				printf("Repeater: not sync_apcli,restore old wireless.\n");
				nvram_set( "SSID1", nvram_safe_get("res_SSID1"));
				nvram_set( "AuthMode", nvram_safe_get("res_AuthMode"));
				nvram_set( "EncrypType", nvram_safe_get("res_EncrypType"));
				nvram_set( "WPAPSK1", nvram_safe_get("res_WPAPSK1"));
			}
#endif
			
	}
	/*
	if(!strcmp(cur_mode, "3")||!strcmp(cur_mode, "6")||!strcmp(cur_mode, "7")||!strcmp(cur_mode, "8"))
		nvram_set( "ApCliEnable", "1");
	else
		nvram_set( "ApCliEnable", "0");
	*/
	if(!strcmp(cur_mode, "1")|| !strcmp(cur_mode, "9")|| !strcmp(cur_mode, "2")
		||!strcmp(cur_mode, "5")||!strcmp(cur_mode, "6")||!strcmp(cur_mode, "8"))
		nvram_set("WscModeOption","0");
	if(!strcmp(cur_mode, "3")||!strcmp(cur_mode, "4")||!strcmp(cur_mode, "5")
		||!strcmp(cur_mode, "6")||!strcmp(cur_mode, "7") ||!strcmp(cur_mode, "8"))
		nvram_set("RadioOn","1");

	if(!strcmp(cur_mode, "2") ||!strcmp(cur_mode, "3")||!strcmp(cur_mode, "4") 
		||!strcmp(cur_mode, "5") ||!strcmp(cur_mode, "6") )
		nvram_set( "dhcpd_enable", "0");

	if(!strcmp(cur_mode, "1") || !strcmp(cur_mode, "9")|| !strcmp(cur_mode, "7") 
		||!strcmp(cur_mode, "8"))
		nvram_set( "dhcpd_enable", "1");

	nvram_set("auto_mode_en", "0");
	if(!strcmp(cur_mode, "9"))	
	{
		if( nvram_invmatch("wan_proto", "3g") )
			nvram_set("wan_proto_tmp", nvram_safe_get("wan_proto")); //backup wan_protocol

		nvram_set("sw_mode_ex","1");
		nvram_set("wan_proto", "3g");
	}
	else if(!strcmp(cur_mode, "10")) /* ѡģʽ */
	{
		nvram_set("auto_mode_en", "1");
	}	
	else
	{
		nvram_set("sw_mode_ex",cur_mode);
		if( nvram_match("wan_proto", "3g") )
			nvram_set("wan_proto", nvram_safe_get("wan_proto_tmp"));		
//		nvram_set("wan_proto", nvram_safe_get("wan_proto_tmp"));
	}

#ifdef DEFINE_DHCP_AUTO_SWITCH
	nvram_set( "dhcpd_enable", "1"); //After quick_setup,Dhcp open anyway
#endif

	nvram_set("wan_set", "1");
    nvram_commit();
    
#ifdef DEFINE_PHONE_WEB //for webapp wwb 2014-4-4
    char_t *phone;
    phone = websGetVar(wp, T("nPhone"), T(""));
#endif

	updateFlash8021x();
	
#if 0//defined (DEFINE_ITRAVEL2) || defined (DEFINE_ISTORAGE2)
	//printf("wan_proto = %s\n",nvram_safe_get("wan_proto"));
	//if( !strcmp(cur_mode, "9") && nvram_match("wan_proto", "3g") ) //3Gģʽ
	//{
		system("sleep 2 && reboot &");
	//}
	//else
#else
	{
		kill(NOTIFY_TGT_PID, SIGHUP);
  		sleep(20);
		
#ifndef DEFINE_PHONE_WEB
			websRedirect(wp, "status.asp");
#else
		if ( 0 == strcmp(phone, "1"))
		{
			websRedirect(wp, "phoneStatus.asp");
		}
		else
			websRedirect(wp, "status.asp");
#endif

		//jsGoPage(wp, "/index.asp");
	}
#endif

}


static void wizard_wirelessGetSecurity(webs_t wp, char_t *path, char_t *query)
{
	return getSecurity(wp, path, query);
}


