//=============================================================================
// Copyright (c) 2009 T&W Corporation.	All Rights Reserved.
//
//	Title:
//		main.c
//	Desc:
//		ap_mgrd : A dameon for accepts commands from the remote setting Utility.
//=============================================================================

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <linux/wireless.h>
#include <errno.h>
#include "bcmnvram.h"


#define LISTEN_PORT 21212                    // the port users will be connecting to
#define SEND_PORT 21213	/* the port is used by win32 program: Discovery */
#define BUFLEN 128                      // length of the buffer
#define MP_TX_PACKET 0x8B71
#define MP_BRX_PACKET 0x8B73

/*
 * arguments: ifname  - interface name
 *            if_addr - a 18-byte buffer to store mac address
 * description: fetch mac address according to given interface name
 */

static char	map64[] = {
	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 62, -1, -1, -1, 63,
	52, 53, 54, 55, 56, 57, 58, 59, 60, 61, -1, -1, -1, -1, -1, -1,
	-1,  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14,
    15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, -1, -1, -1, -1, -1, 
	-1, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
    41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, -1, -1, -1, -1, -1,
	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
};

static char	alphabet64[] = {
	'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
	'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
	'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
	'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
	'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
	'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
	'w', 'x', 'y', 'z', '0', '1', '2', '3',
	'4', '5', '6', '7', '8', '9', '+', '/',
};


/******************************************************************************/
/*
 *	Encode a buffer from "string" into "outbuf"
 */

void websEncode64(char *outbuf, char *string, int outlen)
{
	unsigned long	shiftbuf;
	char			*cp, *op;
	int				x, i, j, shift;

	op = outbuf;
	*op = '\0';
	cp = string;
	while (*cp) {
/*
 *		Take three characters and create a 24 bit number in shiftbuf
 */
		shiftbuf = 0;
		for (j = 2; j >= 0 && *cp; j--, cp++) {
			shiftbuf |= ((*cp & 0xff) << (j * 8));
		}
/*
 *		Now convert shiftbuf to 4 base64 letters.  The i,j magic calculates
 *		how many letters need to be output.
 */
		shift = 18;
		for (i = ++j; i < 4 && op < &outbuf[outlen] ; i++) {
			x = (shiftbuf >> shift) & 0x3f;
			*op++ = alphabet64[(shiftbuf >> shift) & 0x3f];
			shift -= 6;
		}
/*
 *		Pad at the end with '='
 */
		while (j-- > 0) {
			*op++ = '=';
		}
		*op = '\0';
	}
}
/******************************************************************************/


static int getIfMac(char *ifname, char *if_hw)
{
	struct ifreq ifr;
	char *ptr;
	int skfd;

	if((skfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		printf("getIfMac: open socket error\n");
		return -1;
	}

	strncpy(ifr.ifr_name, ifname, 16);
	if(ioctl(skfd, SIOCGIFHWADDR, &ifr) < 0) {
		//error(E_L, E_LOG, T("getIfMac: ioctl SIOCGIFHWADDR error for %s"), ifname);
		return -1;
	}

	ptr = (char *)&ifr.ifr_addr.sa_data;
	sprintf(if_hw, "%02X:%02X:%02X:%02X:%02X:%02X",
			(ptr[0] & 0377), (ptr[1] & 0377), (ptr[2] & 0377),
			(ptr[3] & 0377), (ptr[4] & 0377), (ptr[5] & 0377));

	close(skfd);

	return 0;
}

/* using pipe to get params */
static int pipeGetParams(char *cmd, char *resultbuf, int resultbuflen)
{
	FILE *fp;
	char cmdbuf[BUFLEN] = {0};
	int cmdbuflen  = 0;

	if (!cmd || !resultbuf || (resultbuflen <= 0) )
		return 0;

	//printf("\ncmd:%s\n", cmd);
	if ( (fp = popen(cmd, "r")) != NULL ){
		if ( fgets(cmdbuf, sizeof(cmdbuf), fp) ){
			strtok(cmdbuf, "\n");
			cmdbuflen = strlen(cmdbuf);
			if ( (cmdbuflen > 0) && (cmdbuflen < resultbuflen) ) {
				//printf("cmd:%s. result:%s, cmdbuflen:%d\n", cmd, cmdbuf, cmdbuflen);
				memcpy(resultbuf, cmdbuf, cmdbuflen);
			}else
				fprintf(stderr, "send buf overflow!\n");

		}
		pclose(fp);
	}
	return cmdbuflen;
}

static int nvramGetParams(char *param, char *resultbuf, int resultbuflen)
{
	char cmdbuf[BUFLEN] = {0};
	int cmdbuflen  = 0;
    char *result=NULL;

	if (!param || !resultbuf || (resultbuflen <= 0) )
    {   
        return 0;
    }
    result = nvram_safe_get(param);

    strncpy(cmdbuf, result, strlen(result));
	cmdbuflen = strlen(cmdbuf);
	if ( (cmdbuflen > 0) && (cmdbuflen < resultbuflen) ) 
    {
		memcpy(resultbuf, cmdbuf, cmdbuflen);
	}
    else
    {
		fprintf(stderr, "send buf overflow!\n");
    }
	return cmdbuflen;
}

static in_addr_t getBroadcaseIP(void)
{
	char ipbuf[16] = {0};
	char maskbuf[16] = {0};
	in_addr_t ip = 0;
	in_addr_t netmask = 0;
	in_addr_t ip_masked = 0;
	int ret;
	ret = nvramGetParams("lan_ipaddr", ipbuf, sizeof(ipbuf));
	ret = nvramGetParams("lan_netmask", maskbuf, sizeof(maskbuf));
	if (strlen(ipbuf) && strlen(maskbuf)){
		ip = inet_addr(ipbuf);
		netmask = inet_addr(maskbuf);
	}
	return (ip | (~netmask));
}

/* according the lan_ip and lan_netmask to rebuild a DHCP Pool. */
static void updateDhcpPool(unsigned char flgNewIP, unsigned char flgNewNetmask)
{
	char ipbuf[16] = {0};
	char maskbuf[16] = {0};
	in_addr_t ip = 0;
	in_addr_t netmask = 0;
	in_addr_t ip_masked = 0;
	in_addr_t ipPoolSize = 0;
	int ret;
	char cmdbuf[256] = {0};
	struct in_addr in;

	ret = nvramGetParams("lan_ipaddr", ipbuf, sizeof(ipbuf));
	ret = nvramGetParams("lan_netmask", maskbuf, sizeof(maskbuf));
	if (strlen(ipbuf) && strlen(maskbuf)){
		ip = inet_addr(ipbuf);
		netmask = inet_addr(maskbuf);
	}else {
		printf("nvram get error.\n");
		return;
	}

	if (flgNewIP || flgNewNetmask) {
		ipPoolSize = (~netmask);
		if (ipPoolSize > 0x03000000) {
			/* dhcp start ip */
			in.s_addr = (ip & 0x00ffffff) | 0x02000000;
			//sprintf(cmdbuf, "nvram set dhcp_start=%s", inet_ntoa(in) );
			nvram_set("dhcp_start", inet_ntoa(in));
			printf("dhcp_start:%s\n", inet_ntoa(in));
			//ret = system(cmdbuf);
			/* dhcp end ip */
			in.s_addr = (ip | ipPoolSize) - 0x01000000 ;
			//sprintf(cmdbuf, "nvram set dhcp_end=%s", inet_ntoa(in));
			nvram_set("dhcp_end", inet_ntoa(in));
			printf("dhcp_end:%s\n", inet_ntoa(in));
			//ret = system(cmdbuf);
			/* dhcp gw */
			//sprintf(cmdbuf, "nvram set dhcp_gateway=%s", ipbuf);
			nvram_set("dhcp_gateway", ipbuf);
			//ret = system(cmdbuf);
		}

		if (flgNewNetmask){
			/* dhcp netmask */
			//sprintf(cmdbuf, "nvram set dhcp_netmask=%s", maskbuf);
			//ret = system(cmdbuf);
			nvram_set("dhcp_netmask", maskbuf);
		}
	}
	return;
}

/*
 * nanosleep(2) don't depend on signal SIGALRM and could cooperate with
 * other SIGALRM-related functions(ex. setitimer(2))
 */
unsigned int Sleep(unsigned int secs)
{
	int rc;
	struct timespec ts, remain;
	ts.tv_sec  = secs;
	ts.tv_nsec = 0;

sleep_again:
	rc = nanosleep(&ts, &remain);
	if(rc == -1 && errno == EINTR){
		ts.tv_sec = remain.tv_sec;
		ts.tv_nsec = remain.tv_nsec;
		goto sleep_again;
	}
	return 0;
}


/******************************************************************************/
/*
 *	Set Default should be done by nvram_daemon.
 *	We check the pid file's existence.
 */
int setDefault(void)
{
	FILE *fp;
	int i;

	//retry 15 times (15 seconds)
	for (i = 0; i < 15; i++) {
		fp = fopen("/var/run/nvramd.pid", "r");
		if (fp == NULL) {
			if (i == 0)
				printf("goahead: waiting for nvram_daemon ");
			else
				printf(". ");
		}
		else {
			fclose(fp);

			return 0;
		}
		Sleep(1);
	}
	
	return (-1);
}



#define PRODUCT_TEST_SUPPORT

static const unsigned char g_Data_prefix[] = {0x01, 0x8d, 0x0e, 0x68};
#define	SIZEOF(array)	(sizeof(array)/sizeof(array[0]))
int main(void)
{
	int sockfd;                     				// socket descriptors
	struct sockaddr_in my_addr;     		// my address information
	struct sockaddr_in their_addr;  			// connectors address information
	//struct sockaddr_in broadcast_addr;		// broadcast
	int addr_len, numbytes;
	unsigned char recvbuf[BUFLEN], sendbuf[BUFLEN];				// buffer that stores message
	unsigned char *pSendbuf = NULL;
	int sendbufsize = sizeof(sendbuf);
	int senddatalen = 0;
	int parmdatalen  = 0;
	unsigned int cmd_num; 	//32bit commands number
	unsigned char *pcmdline = NULL;
	char *pAction = NULL;
	char *pRequest = NULL;
	char *saveptr1 = NULL;
	char *token = NULL;
	unsigned char flgFound = 0;
	int prefix_action_len = 0;
	enum{acNone, acReq, acSet} emAction = acNone;
	/* append for broadcast process */
	int broadcast = 1;
	int sendfd;
/*
    if (setDefault() < 0)
    {
        perror("nvram_daemon not init!\n");
		exit(1);
    }
*/    
	/* Broadcast socket setup */
	if ((sendfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
		perror("socket");
		exit(1);
	}

	if ( setsockopt(sendfd, SOL_SOCKET, SO_BROADCAST, &broadcast, sizeof(broadcast)) == -1)
	{
		perror("setsockopt function!\n");
		exit(1);
	}
/*
	broadcast_addr.sin_family = AF_INET;
	broadcast_addr.sin_port = htons(SEND_PORT);
	broadcast_addr.sin_addr.s_addr = INADDR_BROADCAST;
	memset(&(broadcast_addr.sin_zero), '\0', 8); 	// zero the rest of the struct
*/
/* Broadcast socket setup */

	// create a socket
	if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
		perror("socket");
		exit(1);
	}

	my_addr.sin_family = AF_INET;         		// host byte order
	my_addr.sin_port = htons(LISTEN_PORT);     	// short, network byte order
	my_addr.sin_addr.s_addr = INADDR_ANY; 	// automatically fill with my IP
	memset(&(my_addr.sin_zero), '\0', 8); 	// zero the rest of the struct

	// bind the socket with the address
	if (bind(sockfd, (struct sockaddr *)&my_addr,
		sizeof(struct sockaddr)) == -1) {
		perror("bind");
		close(sockfd);
		exit(1);
	}

	addr_len = sizeof(struct sockaddr);

	printf("MP AUTOMATION daemon (ver 1.0)\n");

	// main loop : wait for the client
	while (1) {
		//receive the command from the client
		memset(recvbuf, 0, BUFLEN);
        memset(sendbuf, 0, BUFLEN);
		senddatalen = 0;
		if ((numbytes = recvfrom(sockfd, recvbuf, BUFLEN, MSG_WAITALL,
			(struct sockaddr *)&their_addr, &addr_len)) == -1) {
			printf("Receive failed!!!\n");
			close(sockfd);
			exit(1);
		}
#ifdef PRODUCT_TEST_SUPPORT
		/* For */
#define	PRD_CMD_TELNED	"telnet_open_wdk"
		if ( 0 == strncmp(recvbuf, PRD_CMD_TELNED, strlen(PRD_CMD_TELNED)) ) {
			/* open telnetd */
			printf("telnetd &\n");
			system("telnetd&");
			continue;
		}
#endif

		/* prefix check */
		if ( memcmp(recvbuf, g_Data_prefix, sizeof(g_Data_prefix)) ) {
			//printf("unknown pack?!\n");
			continue;
		}
		pAction = recvbuf + sizeof(g_Data_prefix);

#define	ACTION_GET	"get:"
#define	ACTION_GETACK	"gack:"
		if ( !memcmp(pAction, ACTION_GET, strlen(ACTION_GET)) ){
			/* action flag */
			emAction = acReq;
			/* begin to parse request string */
			pRequest = pAction + strlen(ACTION_GET);
			/* ack get command */
			strcpy(sendbuf, ACTION_GETACK);
			prefix_action_len = strlen(ACTION_GETACK);
			pSendbuf = sendbuf + prefix_action_len;

			for (saveptr1 = pRequest; pRequest = saveptr1; pRequest != NULL, senddatalen < sendbufsize) {
				flgFound = 0;

				token = strtok_r(pRequest, "\n", &saveptr1);
				if (NULL == token)
					break;
				//printf("token:%s, pRequest:%s, saveptr:%s\n", token, pRequest, saveptr1);
#define	REQ_DEV	"device"
#define	REQ_SSID	"ssid"
#define	REQ_IP		"lan_ip"
#define	REQ_NETMASK	"lan_netmask"
#define	REQ_MAC	"lan_mac"
#define	REQ_CHANNEL	"channel"
#define	REQ_USER	"user"
#define	REQ_PASWD	"passwd"
#define	UPDATE_PARAMS()	\
	do {	\
		pSendbuf += parmdatalen;	\
		*pSendbuf++ = '\n';	\
		senddatalen += parmdatalen + 1;	\
		flgFound = 1;	\
	}while(0)

				if (!strncasecmp(token, REQ_DEV, strlen(REQ_DEV))){
#if 0
					parmdatalen = nvramGetParams("hostname", pSendbuf, sendbufsize - senddatalen);
					if (parmdatalen){
						pSendbuf += parmdatalen;
						*pSendbuf++ = '\n';
						senddatalen += parmdatalen + 1;
						flgFound = 1;
					}
#else
					strncpy(pSendbuf, "RT-N11+", strlen("RT-N11+"));
					pSendbuf += strlen("RT-N11+");
					*pSendbuf++ = '\n';
					senddatalen += strlen("RT-N11+");
					flgFound = 1;
#endif
				}else if (!strncasecmp(token, REQ_SSID, strlen(REQ_SSID))){
					parmdatalen = nvramGetParams("SSID1", pSendbuf, sendbufsize - senddatalen);
					if (parmdatalen){
						pSendbuf += parmdatalen;
						*pSendbuf++ = '\n';
						senddatalen += parmdatalen + 1;
						flgFound = 1;
					}

				}else if (!strncasecmp(token, REQ_IP, strlen(REQ_IP))){
					parmdatalen = nvramGetParams("lan_ipaddr", pSendbuf, sendbufsize - senddatalen);
					if (parmdatalen){
						pSendbuf += parmdatalen;
						*pSendbuf++ = '\n';
						senddatalen += parmdatalen + 1;
						flgFound = 1;
					}
				}else if (!strncasecmp(token, REQ_NETMASK, strlen(REQ_NETMASK))){
					parmdatalen = nvramGetParams("lan_netmask", pSendbuf, sendbufsize - senddatalen);
					if (parmdatalen){
						pSendbuf += parmdatalen;
						*pSendbuf++ = '\n';
						senddatalen += parmdatalen + 1;
						flgFound = 1;
					}
				}else if (!strncasecmp(token, REQ_MAC, strlen(REQ_MAC))){
					char if_mac[18];
					if ( 0 == getIfMac("br0", if_mac) ){
						//printf("mac:%s\n", if_mac);
						strncpy(pSendbuf, if_mac, 18);

						pSendbuf += 17;
						*pSendbuf++ = '\n';
						senddatalen += 18;
						flgFound = 1;
					}
					//printf("sendbufsize:%d\n", sendbufsize);
				}else if (!strncasecmp(token, REQ_CHANNEL, strlen(REQ_CHANNEL))){
					//printf("get channel. buflen:%d\n", sendbufsize - senddatalen);
					//printf("bufsend:%s\n", sendbuf + strlen(ACTION_GETACK));
					parmdatalen = nvramGetParams("Channel", pSendbuf, sendbufsize - senddatalen);
					if (parmdatalen){
						pSendbuf += parmdatalen;
						*pSendbuf++ = '\n';
						senddatalen += parmdatalen + 1;
						flgFound = 1;
					}

				}else if (!strncasecmp(token, REQ_USER, strlen(REQ_USER))){
					parmdatalen = nvramGetParams("http_username", pSendbuf, sendbufsize - senddatalen);
					if (parmdatalen)
						UPDATE_PARAMS();

				}else if (!strncasecmp(token, REQ_PASWD, strlen(REQ_PASWD))){
					parmdatalen = nvramGetParams("http_passwd", pSendbuf, sendbufsize - senddatalen);
					if (parmdatalen)
						UPDATE_PARAMS();

				}
				//printf("sendbufsize:%d, senddatalen:%d\n", sendbufsize, senddatalen);

				if (!flgFound){
					int len;
					strcpy(pSendbuf, "?");
					len = strlen("?");
					pSendbuf += len;
					*pSendbuf++ = '\n';
					senddatalen += len + 1;
				}
			}

		}
#if 1
#define	ACTION_SET	"set:"
#define	ACTION_SETACK	"sack:"
		else if ( !memcmp(pAction, ACTION_SET, strlen(ACTION_SET)) ) {
			char cmdbuf[256] = {0};
			int cmdret;
			char *pParam = NULL;
			char userbuf[33] = {0};
			char pswdbuf[33] = {0};
			char *pUsername = NULL;
			char *pPassword = NULL;
			char *pStrEndOffset = NULL;
			int len = 0;
			unsigned char flgNewIP = 0;
			unsigned char flgNewNetmask = 0;

#define	SET_MARK	"mark:"
#define	SET_IP		"lan_ip:"
#define	SET_NETMASK	"lan_netmask:"
#define	SET_DEFAULTGW	"defaultgw:"
#define	SET_AUTH_USER	"user:"
#define	SET_AUTH_PASWD	"pswd:"

			/* begin to parse request string */
			pRequest = pAction + strlen(ACTION_SET);

			/* ack get command */
			strcpy(sendbuf, ACTION_SETACK);
			prefix_action_len = strlen(ACTION_SETACK);
			pSendbuf = sendbuf + prefix_action_len;

			for (saveptr1 = pRequest; pRequest = saveptr1; pRequest != NULL) {

				token = strtok_r(pRequest, "\n", &saveptr1);
				if (NULL == token)
					break;
				//printf("token:%s, pRequest:%s, saveptr:%s\n", token, pRequest, saveptr1);

				/* mark must the first of all. to check if Discovery is talk to me */
				if (!strncasecmp(token, SET_MARK, strlen(SET_MARK))){
					char if_mac[18];
					pParam = token+strlen(SET_MARK);
					if ( 0 == getIfMac("br0", if_mac) ){
						/* mark (MAC) equal. go on parse. */
						if ( !strncmp(if_mac, pParam, 18) )
							continue;

					}else{
						printf("getIfMac Error.\n");
					}
					goto DONOTHING;	//break command parse loop.
				}else if (!strncasecmp(token, SET_IP, strlen(SET_IP))){
					pParam = token+strlen(SET_IP);
					//sprintf(cmdbuf, "nvram set lan_ipaddr=%s", pParam);
					//printf("cmdbuf:%s\n", cmdbuf);
					//cmdret = system(cmdbuf);
					cmdret = nvram_set("lan_ipaddr",pParam);
					flgNewIP = 1;
					//sprintf(cmdbuf, "ifconfig br0 %s", pParam);
					//cmdret = system(cmdbuf);
				}else if (!strncasecmp(token, SET_NETMASK, strlen(SET_NETMASK))){
					pParam = token+strlen(SET_NETMASK);
					//printf("parms:%s\n", pParam);
					//sprintf(cmdbuf, "nvram set lan_netmask=%s", pParam);
					flgNewNetmask = 1;
					cmdret = nvram_set("lan_netmask",pParam);
					//cmdret = system(cmdbuf);
					//sprintf(cmdbuf, "ifconfig br0 netmask %s", pParam);
					//cmdret = system(cmdbuf);
				}else if (!strncasecmp(token, SET_DEFAULTGW, strlen(SET_DEFAULTGW))){
					pParam = token+strlen(SET_DEFAULTGW);
					//printf("parms:%s\n", pParam);
					//sprintf(cmdbuf, "nvram set dhcpGateway %s", pParam);
					sprintf(cmdbuf, "route del default; route add default gw %s", pParam);
					cmdret = system(cmdbuf);
				}else if (!strncasecmp(token, SET_AUTH_USER, strlen(SET_AUTH_USER))){
					pParam = token+strlen(SET_AUTH_USER);
					//printf("parms:%s\n", pParam);
					if ( pParam && strlen(pParam) ) {
						//sprintf(cmdbuf, "nvram set http_username='%s'", pParam);
						//cmdret = system(cmdbuf);
                        cmdret = nvram_set("http_username",pParam);
					}
				}else if (!strncasecmp(token, SET_AUTH_PASWD, strlen(SET_AUTH_PASWD))){
					pParam = token+strlen(SET_AUTH_PASWD);
					//printf("parms:%s\n", pParam);
					if ( pParam && strlen(pParam) ) {
						//sprintf(cmdbuf, "nvram set http_passwd='%s'", pParam);
						//cmdret = system(cmdbuf);
                        char user_passwd[64] = {0};
						websEncode64(user_passwd,pParam,sizeof(user_passwd));
                        cmdret = nvram_set("http_passwd",user_passwd);
					}
				}
			}
			/* update DHCP Pool */
			updateDhcpPool(flgNewIP, flgNewNetmask);
            nvram_commit();
			if (!cmdret)
				strcpy(pSendbuf, "ok");
			else
				strcpy(pSendbuf, "pok");
			/* action flag */
			emAction = acSet;
		}
#endif

SEND_SETACK:
		senddatalen += strlen(pSendbuf) + prefix_action_len;
		/* broadcast package */
        their_addr.sin_port = htons(SEND_PORT);
        
       	if ((numbytes = sendto(sendfd, sendbuf, senddatalen, MSG_WAITALL,
			(struct sockaddr *)&their_addr, sizeof(struct sockaddr))) == -1) {
			fprintf(stderr, "send failed\n");
		}
		if (acSet == emAction){
			/* restart goahead */
			printf("restart goahead\n");
			system("killall goahead; goahead&");
		}
DONOTHING:
		//sometime need to jump here
		emAction = acNone;
	}
	return 0;
}
