#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <linux/autoconf.h>
#include "ralink_gpio.h"

#define GPIO_DEV	"/dev/gpio"

typedef enum {
	BTN_UNKNOWN_ACT,
	BTN_ENABLE,
	BTN_DISABLE
}TemButtonAction;

static struct {
	const char *pActionStr;
	TemButtonAction emAction;
}g_button_action[] = {
	{"enable", BTN_ENABLE},
	{"disable", BTN_DISABLE}
};

#define	ARRAY_SIZEOF(array)	sizeof(array)/sizeof(array[0])
#ifdef	SINGLE_COMMAND
#define	BTN_PARMS_IDX	(1)	/* if command format is: "button <params>", thus params behind offset 0. */
#else
#define	BTN_PARMS_IDX	(2)	/* if command format is: "<prefix_command> button <params>", thus params behind offset 1. */
#endif

#define	RESULT_OK	"ok"
#define	RESULT_FAILED	"fail"

int gpio_enb_irq(void)
{
	int fd;

	fd = open(GPIO_DEV, O_RDONLY);
	if (fd < 0) {
		perror(GPIO_DEV);
		return -1;
	}
	if (ioctl(fd, RALINK_GPIO_ENABLE_INTP) < 0) {
		perror("ioctl");
		close(fd);
		return -1;
	}
	close(fd);
	return 0;
}

int gpio_dis_irq(void)
{
	int fd;

	fd = open(GPIO_DEV, O_RDONLY);
	if (fd < 0) {
		perror(GPIO_DEV);
		return -1;
	}
	if (ioctl(fd, RALINK_GPIO_DISABLE_INTP) < 0) {
		perror("ioctl");
		close(fd);
		return -1;
	}
	close(fd);
	return 0;
}

static void button_action(TemButtonAction emAct)
{
	int ret;
	switch(emAct) {
		case BTN_ENABLE:
			ret = gpio_enb_irq();
			break;
		case BTN_DISABLE:
			ret = gpio_dis_irq();
			break;
		default:;
	}
	if (0 == ret)
		printf(RESULT_OK);
	else
		printf(RESULT_FAILED);
	return;
}

/* Usage: button <enable/disable> */
#ifndef	SINGLE_COMMAND
int button_main(int argc, char *argv[])
#else
int main(int argc, char *argv[])
#endif
{
	TemButtonAction action = BTN_UNKNOWN_ACT;
	int i;

	if (argc < 2)
		return -1;

	for (i = 0; i < ARRAY_SIZEOF(g_button_action); i++){
		if (0 == strncmp(argv[BTN_PARMS_IDX], g_button_action[i].pActionStr,
			strlen(g_button_action[i].pActionStr)) ){
			action = g_button_action[i].emAction;
			break;
		}
	}

	//unknown button action. show result: failed
	if (BTN_UNKNOWN_ACT == action){
		printf(RESULT_FAILED);
		return 0;
	}

	button_action(action);

	return 0;
}

