#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <linux/autoconf.h>
#include "ralink_gpio.h"

#define GPIO_DEV	"/dev/gpio"

typedef enum {
	DETECT_UNKNOWN_OBJ,
	DETECT_BUTTON,
	DETECT_USB
}TemDetectObj;

static struct {
	const char *pDetectObjectStr;
	TemDetectObj emObject;
}g_detect_item[] = {
	{"button", DETECT_BUTTON},
	{"usb", DETECT_USB}
};

#define	ARRAY_SIZEOF(array)	sizeof(array)/sizeof(array[0])
#ifdef	SINGLE_COMMAND
#define	DETECT_OBJ_IDX	(1)	/* if command format is: "detect <params>", thus params behind offset 0. */
#else
#define	DETECT_OBJ_IDX	(2)	/* if command format is: "<prefix_command> detect <params>", thus params behind offset 1. */
#endif

#define	RESULT_OK	"1"
#define	RESULT_FAILED	"0"


int gpio_set_dir(int dir)
{
	int fd;

	fd = open(GPIO_DEV, O_RDONLY);
	if (fd < 0) {
		perror(GPIO_DEV);
		return -1;
	}
	if (ioctl(fd, RALINK_GPIO_SET_DIR, dir) < 0) {
		perror("ioctl");
		close(fd);
		return -1;
	}
	close(fd);
	return 0;
}

void signal_handler(int signum)
{
	if ( (signum == SIGUSR1) || (signum == SIGUSR2) )
		printf(RESULT_OK);
	else
		printf(RESULT_FAILED);
}

#define	BUTTON_WPS	(0)
#ifdef CONFIG_RALINK_RT2880
#define	BUTTON_FACTORY	(0)
#else
	//RT2883, RT3052 use gpio 10 for load-to-default
#define	BUTTON_FACTORY	(10)
#endif

int gpio_reg_info(int gpio_num)
{
	int fd;
	ralink_gpio_reg_info info;

	fd = open(GPIO_DEV, O_RDONLY);
	if (fd < 0) {
		perror(GPIO_DEV);
		return -1;
	}
	info.pid = getpid();
	info.irq = gpio_num;
	if (ioctl(fd, RALINK_GPIO_REG_IRQ, &info) < 0) {
		perror("ioctl");
		close(fd);
		return -1;
	}
	close(fd);
	return 0;
}

static void gpio_test_intr(int gpio_num)
{
	int btn_gpio[] = {BUTTON_WPS, BUTTON_FACTORY};
	int i;

	for (i = 0; i < ARRAY_SIZEOF(btn_gpio); i++) {
		//register my information
		if (0 != gpio_reg_info(btn_gpio[i]) ) {
			/* gpio register failed */
			printf(RESULT_FAILED);
			return;
		}
	}

	//issue a handler to handle SIGUSR1,2
	signal(SIGUSR1, signal_handler);
	signal(SIGUSR2, signal_handler);

	//wait for signal
	pause();
	return;
}

void signal_handler_hotplug(int signum)
{
	/* print result */
	switch(signum){
		case SIGUSR1:
			printf(RESULT_OK);
			break;
		case SIGUSR2:
		default:
			printf(RESULT_FAILED);
			break;
	}
}

static void usb_test_hotplug(void)
{
	/* change hotplug path */
	system("echo '/sbin/pt_hotplug' > /proc/sys/kernel/hotplug");

	//issue a handler to handle SIGUSR1,2
	signal(SIGUSR1, signal_handler_hotplug);	// SIGUSR1 for usb add
	signal(SIGUSR2, signal_handler_hotplug);	// SIGUSR2 for usb remove
	//wait for signal
	pause();
	/* restore default hotplug path.
	But the better way is read before change, and at the end restore the real path. */
	system("echo '/sbin/hotplug' > /proc/sys/kernel/hotplug");
	return;

}

static void detect_obj(TemDetectObj emObj)
{
	switch(emObj){
		case DETECT_BUTTON:
			gpio_test_intr(0);
			break;
		case DETECT_USB:
			usb_test_hotplug();
			break;
		default:;
	}
}

/* Usage: detect <button> */
#ifndef	SINGLE_COMMAND
int detect_main(int argc, char *argv[])
#else
int main(int argc, char *argv[])
#endif
{
	TemDetectObj emObj = DETECT_UNKNOWN_OBJ;
	int i;

	if (argc < 2)
		return -1;
	
	for (i = 0; i < ARRAY_SIZEOF(g_detect_item); i++){
		if (0 == strncmp(argv[DETECT_OBJ_IDX], g_detect_item[i].pDetectObjectStr,
			strlen(g_detect_item[i].pDetectObjectStr)) ){
			emObj = g_detect_item[i].emObject;
			break;
		}
	}
	
	//unknown Detect Object. show result: failed
	if (DETECT_UNKNOWN_OBJ == emObj){
		printf(RESULT_FAILED);
		return 0;
	}
	
	detect_obj(emObj);

	return 0;
}

