#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <stdarg.h>
#include <syslog.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <ifaddrs.h>
#include <sys/sysinfo.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <bcmnvram.h>
#include <bcmdevs.h>
#include <wlutils.h>

#include "shutils.h"
#include "shared.h"
#include "linux/autoconf.h"  //kernel config

#include "ralink.h"
#include "rcl_wps.h"

#ifdef DEFINE_RALINK_MT7620
typedef unsigned int __u32; //without this,include ra_ioctl.h fail
#endif
#include  <ra_ioctl.h>

// -----------------------------------------------------------------------------

#define GATEWAY_TMPFILE "/tmp/.gateway.tmp"
int check_netup(void)
{
	int up = 0;
	if ( access("/tmp/resolv.conf", F_OK) == 0 )
	{
		FILE *f = NULL;
		char s[64] = { 0 }; 
		sprintf(s, "cat /tmp/resolv.conf > %s", GATEWAY_TMPFILE);
		system(s);
		memset(s, 0, 64);
		if ((f = fopen(GATEWAY_TMPFILE, "r")) != NULL) {
			while ( NULL != fgets(s, sizeof(s), f)) {
				if (strstr(s, "nameserver") != NULL) {
					up = 1;
					break;
				}
				else{
					up = 0;
				}
			}
			fclose(f);
		}
		unlink(GATEWAY_TMPFILE);
	}	
	//dprintf("up=%d.\n", up);
	return up;
}

int tcpcheck_net(const char *host, int port)
{
	FILE *f = NULL;
	char s[64] = { 0 }; 
	char cmd[128] = { 0 }; 
	int ok = 0;

	if ((NULL == host) || (strlen(host)<7))/* ַϷԼ */
		return ok;
	
	snprintf(cmd, 127, "tcpcheck %d %s:%d > %s", TCPCHECK_TIMEOUT, host, port, TCPCHECK_TMPFILE);
	do_system(cmd,0);
	
	//memset(s, 0, 32);
	if ((f = fopen(TCPCHECK_TMPFILE, "r")) != NULL) {
		if ( NULL != fgets(s, sizeof(s), f)) {
			if (strstr(s, "alive") != NULL) {
				//dprintf("alive \n");
				ok = 1;
			}
			else if (strstr(s, "timed out") != NULL) {
				//dprintf("timed out \n");
				ok = 0;
			}
			else {
				//dprintf("not alive \n");
				ok = 0;
			}
		}
		fclose(f);
	}
	unlink(TCPCHECK_TMPFILE);
	
	//dprintf("tcpcheck_net [%d]\n", ok);
	return ok;
}

/* check repeater connect is successful */
int check_repeater_connect_state()
{
	int opmode = atoi( nvram_safe_get( "sw_mode_ex" ) );
    int socket_id;
    unsigned int ConnectStatus = 0;
	int connect_flag = STATE_NO;
	struct iwreq wrq;
	
    if( opmode == OPMODE_REPEATER )
	{	
		socket_id = socket(AF_INET, SOCK_DGRAM, 0);
		strcpy(wrq.ifr_name, "apcli0");
		wrq.u.data.length = sizeof(ConnectStatus);
		wrq.u.data.pointer = (caddr_t)(&ConnectStatus);
		wrq.u.data.flags = OID_GEN_MEDIA_CONNECT_STATUS;
		
		if ((ioctl(socket_id, RT_PRIV_IOCTL, &wrq)) < 0)
		{
    		dprintf("###check repeater connect state ioctl error.\n");
			return 0;
    	}
		
        close(socket_id);
    	
    	if(ConnectStatus == 0)		
    		connect_flag = STATE_NO;
    	else
    		connect_flag = STATE_OK;

		//dprintf("###connect flag=%d.\n", connect_flag);
	}

	return connect_flag;
}

void set_action(int a)
{
	int r = 3;
	while (f_write("/var/lock/action", &a, sizeof(a), 0, 0) != sizeof(a)) {
		sleep(1);
		if (--r == 0) return;
	}
	if (a != ACT_IDLE) sleep(2);
}

int check_action(void)
{
	int a;
	int r = 3;

	while (f_read("/var/lock/action", &a, sizeof(a)) != sizeof(a)) {
		sleep(1);
		if (--r == 0) return ACT_UNKNOWN;
	}
	return a;
}

int wait_action_idle(int n)
{
	while (n-- > 0) {
		if (check_action() == ACT_IDLE) return 1;
		sleep(1);
	}
	return 0;
}


void set_net3g_status(int stat)
{
	int r = 3;
	switch(stat){
	case USB3G_INIT:
		nvram_set("net3gStatus", "1"); //Identify
		break;
	case USB3G_DIAL:
		 nvram_set("net3gStatus", "2"); //
		 break;
	case USB3G_REMOVE:
		nvram_set("net3gStatus", "0"); //unplug
		break;
	default:
		break;
	}

#ifdef DEFINE_G4
	set_3g_led(stat);
#endif
	
	while (f_write("/var/net3g_status", &stat, sizeof(stat), 0, 0) != sizeof(stat)) {
		sleep(1);
		if (--r == 0) return;
	}
	//if (a != ACM_NO_ACCESS) sleep(2);
	sleep(1);
}

int check_net3g_status(void)
{
	int a;
	int r = 3;
	while (f_read("/var/net3g_status", &a, sizeof(a)) != sizeof(a)) {
		sleep(1);
		if (--r == 0) return USB3G_UNKNOW;
	}
	return a;
}

int get_file_int(char *pfile)
{
    FILE *fp;
    char buffer[3];
    int ret = -1;
    
    fp = popen(pfile, "r");
	if (fp==NULL)
		goto gfierr;

    if (NULL != fgets(buffer, sizeof(buffer),fp))
    {
        ret = atoi(buffer);
    }
    pclose(fp);

gfierr:
    return ret;
}

// -----------------------------------------------------------------------------
#if defined  (DEFINE_IREADER) || defined (DEFINE_G4)
int gpio_led_off(int gpio)
{
	char cmd[ 128 ] = { 0 };
	sprintf(cmd, "gpio l %d 4000 0 1 0 4000", gpio);
	return system(cmd);
}

int gpio_led_on(int gpio)
{
	char cmd[ 128 ] = { 0 };
	sprintf(cmd, "gpio l %d 0 4000 1 0 4000", gpio);
	return system(cmd);
}
#else
int gpio_led_on(int gpio)
{
	char cmd[ 128 ] = { 0 };
	sprintf(cmd, "gpio l %d 4000 0 1 0 4000", gpio);
	return system(cmd);
}

int gpio_led_off(int gpio)
{
	char cmd[ 128 ] = { 0 };
	sprintf(cmd, "gpio l %d 0 4000 1 0 4000", gpio);
	return system(cmd);
}
#endif
int gpio_led_blinks(int gpio,int ons,int offs)
{
	char cmd[ 128 ] = { 0 };
	sprintf(cmd, "gpio l %d %d %d 4000 0 4000", gpio,ons,offs);
	return system(cmd);
}

/*start of add to make sure the wan port is active */
static int esw_fd;
static int switch_init(void)
{
        esw_fd = socket(AF_INET, SOCK_DGRAM, 0);
        if (esw_fd < 0) {
                perror("socket");
                return -1;
        }
        return 0;
}

static void switch_fini(void)
{
        close(esw_fd);
}

static int ra3052_reg_read(int offset, int *value)
{
        struct ifreq ifr;
        esw_reg reg;

        if (value == NULL)
                return -1;
        reg.off = offset;
        strncpy(ifr.ifr_name, "eth2", 5);
        ifr.ifr_data = &reg;
        if (-1 == ioctl(esw_fd, RAETH_ESW_REG_READ, &ifr)) {
                perror("ioctl");
                close(esw_fd);
                return -1;
        }
        *value = reg.val;
        return 0;
}

int is_phyconnected(void)
{
        int val = 0, idx = 1, ret;
		int off;

        if(switch_init() < 0)
                return 0;

#ifdef DEFINE_RALINK_MT7620
		#ifdef CONFIG_WAN_AT_P0
		off = 0x3008 + 0x100;
		#endif
		#ifdef CONFIG_WAN_AT_P4
		off = 0x3008 + 4*0x100;
		#endif
        if(ra3052_reg_read(off, &val) != 0)
          {
            switch_fini();
            return 0;
              
          }
		ret = val & 0x01;
#elif defined(DEFINE_RALINK_RT5350)
		off = 0x80;
        if(ra3052_reg_read(off, &val) != 0)
          {
            switch_fini();
            return 0;
              
          }
        #ifdef CONFIG_WAN_AT_P0
        ret = ((val & (idx << 25))) >> 25;
        #endif

        #ifdef CONFIG_WAN_AT_P4
        ret = ((val & (idx << 29))) >> 29;
        #endif
#endif
        switch_fini();

        return ret;
}
/*end of add to make sure the wan port is active */

int lock_file(const char * lock_file)
{
	int fl = 0;
	if( access(lock_file, R_OK) == 0 )
	{
		//dprintf("%s had locked .....\n", lock_file);
		return 0;
	}
	else 
	{
		fl = creat(lock_file, S_IREAD);
		//dprintf("%s is locked ..[%d]...\n", lock_file, fl);
		return 1;
	}
}

int unlock_file(const char * lock_file)
{
	unlink(lock_file);
	//dprintf("%s had remove .....\n", lock_file);
	return 1;
}

int get_memory(meminfo_t *m)
{
	FILE *f;
	char s[128];
	int ok = 0;

	memset(m, 0, sizeof(*m));
	if ((f = fopen("/proc/meminfo", "r")) != NULL) {
		while (fgets(s, sizeof(s), f)) {
			if (strncmp(s, "MemTotal:", 9) == 0) {
				m->total = strtoul(s + 12, NULL, 10) * 1024;
				++ok;
			}
			else if (strncmp(s, "MemFree:", 8) == 0) {
				m->free = strtoul(s + 12, NULL, 10) * 1024;
				++ok;
			}
		#if 0
			else if (strncmp(s, "Buffers:", 8) == 0) {
				m->buffers = strtoul(s + 12, NULL, 10) * 1024;
				++ok;
			}
			else if (strncmp(s, "Cached:", 7) == 0) {
				m->cached = strtoul(s + 12, NULL, 10) * 1024;
				++ok;
			}
			else if (strncmp(s, "SwapTotal:", 10) == 0) {
				m->swaptotal = strtoul(s + 12, NULL, 10) * 1024;
				++ok;
			}
			else if (strncmp(s, "SwapFree:", 9) == 0) {
				m->swapfree = strtoul(s + 11, NULL, 10) * 1024;
				++ok;
				break;
			}
		#endif
		}
		fclose(f);
	}
	if (ok == 0) {
		return 0;
	}
#if 0
	m->maxfreeram = m->free;
	if (nvram_match("t_cafree", "1")) m->maxfreeram += (m->cached + m->buffers);
#endif
	return 1;
}
