/*
 * Create by zym. May 16 2011 kmzym@126.com
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <shutils.h>
#include <notify.h>

#define DEBUG_EN
#ifndef DEBUG_EN
#undef  dprintf(fmt, args...)
#define dprintf(fmt, args...)
#endif

#define NOTIFY_MSG_FILE             "/var/notify/msg"
#define NOTIFY_TGT_PID              1 /* init(rc) */
#define NOTIFY_WAITING_TIMEOUT      10 /* seconds */

static int spin_waiting( const int i_wait, const int i_timeout )
{
    int i_ret = -1, i_tiout = i_timeout;

    if( access( NOTIFY_MSG_FILE, F_OK ) < 0 )
        i_ret = 0;
    else
        i_ret = 1;

    if( i_tiout > 0 && i_ret == 1 )
    {
        while( i_wait == NOTIFY_WAIT )
        {
            if( i_tiout < 1 )
                break;

            if( access( NOTIFY_MSG_FILE, F_OK ) < 0 )
            {
                i_ret = 0;
                break;
            }
            sleep( 1 );
            --i_tiout;
        }
    }

    return i_ret;
}

int notification_send_rc( const unsigned int ui_event_id, const int i_wait )
{
    int i_ret = -1, i_rc_pid = NOTIFY_TGT_PID;
    FILE *fp = NULL;

    dprintf( "enter.\n" );
    i_ret = spin_waiting( NOTIFY_WAIT, NOTIFY_WAITING_TIMEOUT );
    if( i_ret != 0 )
    {
        printf( "notification send: wait timeout.\n" );
        return -1;
    }

    fp = fopen( NOTIFY_MSG_FILE, "w" );
    if( fp == NULL )
    {
        printf( "notification send: open file '%s' fail.\n", NOTIFY_MSG_FILE );
        return -1;
    }
    dprintf( "send msg oid '%u' to rc.\n", ui_event_id );
    fprintf( fp, "%u", ui_event_id );
    fclose( fp );

    dprintf( "send signal to rc.\n ");
    kill( i_rc_pid, SIGUSR1 );
    dprintf( "waiting for backing from rc.\n ");
    spin_waiting( i_wait, NOTIFY_WAITING_TIMEOUT );
    dprintf( "done.\n" );

    return 0;
}

unsigned int notification_get_rc( const int i_wait )
{
    unsigned int ui_event_id = 0;
    char s_buf[ 16 ] = { 0 };
    FILE *fp = NULL;

    dprintf( "enter.\n" );
    fp = fopen( NOTIFY_MSG_FILE, "r" );
    if( fp == NULL )
    {
        printf( "notification get: open file '%s' fail.\n", NOTIFY_MSG_FILE );
        return -1;
    }
    fgets( s_buf, sizeof( s_buf ), fp );
    dprintf( "get message '%s'.\n", s_buf );
    ui_event_id = strtoul( s_buf, NULL, 10 );
    fclose( fp );

    dprintf( "if doesn't need waiting, then delete the file of message.\n" );
    if( i_wait == NOTIFY_NOWAIT )
        unlink( NOTIFY_MSG_FILE );

    return ui_event_id;
}

int notification_free_rc( void )
{
    dprintf( "delete the file of message.\n" );
    return unlink( NOTIFY_MSG_FILE );
}


