#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <stdarg.h>
#include <syslog.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <ifaddrs.h>
#include <sys/sysinfo.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <bcmnvram.h>
#include <bcmdevs.h>
#include <wlutils.h>

#include "shutils.h"
#include "shared.h"
#include "user_gpio.h"
//#include "apn.h"


#define VENDOR		"Vendor"
#define PRODID		"ProdID"
#define LINUX_VID_36	"1d6b"
#define LINUX_VID_21  	"0000"
#define LINUX_VIDHUB1 	"058f"
#define LINUX_VIDHUB2 	"8644"

//G4: 05c6:6000 (bus 1, device 2)
#define LINUX_VIDG4     "05c6"

struct systype_info{
	char systype[4];
	char sys_mode[4];
	char ps_status[4];
	char call_status[4];
	char cell_changed[4];
	char sim_status[4];
	char srv_status[4];
};


typedef struct _modem_sim_info_t
{
	const char *mcc;
	const char *mcn;
	const char *ispname;
	const char *apn;
	const char *code;
	const char *username;
	const char *password;
}modem_sim_info_t;

static modem_sim_info_t g_sim_info_table[] = {
	#include "sim_info_table_def.h"
};

// 0: no modem, 1: has modem
int has_usb_modem_from_sys()
{
	return d_exist("/sys/bus/usb-serial/devices/ttyUSB2");
}

static int pre_net3g_st = USB3G_UNKNOW;
void set_3g_led(int net3g_st)
{
#ifndef HAVE_GREEN_LED
	if(pre_net3g_st != net3g_st)
#endif
	{
		dprintf("pre_net3g_st=%d, net3g_st=%d\n", pre_net3g_st, net3g_st);
		switch(net3g_st){
			case USB4G_CONN:
			#ifdef HAVE_GREEN_LED
				gpio_led_off(GPIO_BLUE_LED);
				gpio_led_on(GPIO_GREEN_LED);
			#else
				#ifdef DEFINE_AZTECH
					gpio_led_off(GPIO_BLUE_LED);
				#else
					gpio_led_on(GPIO_BLUE_LED);
				#endif
				gpio_led_on(GPIO_RED_LED);
			#endif
				break;
			case USB3G_CONN:
			#ifdef HAVE_GREEN_LED
				if ( nvram_match("net3gConnectType", "LTE") ) 
				{
					gpio_led_off(GPIO_BLUE_LED);
					gpio_led_on(GPIO_GREEN_LED);
				}
				else
				{
					gpio_led_off(GPIO_GREEN_LED);
					gpio_led_on(GPIO_BLUE_LED);
				}
			#else
				if ( nvram_match("net3gConnectType", "LTE") ) 
				{
					#ifdef DEFINE_AZTECH
						gpio_led_off(GPIO_BLUE_LED);
					#else
						gpio_led_on(GPIO_BLUE_LED);
					#endif
					gpio_led_on(GPIO_RED_LED);
				}
				else
				{
					gpio_led_off(GPIO_RED_LED);
					gpio_led_on(GPIO_BLUE_LED);
				}
			#endif
				break;
			case USB3G_INIT:
			case USB3G_SEARCH:
			#ifdef HAVE_GREEN_LED
				gpio_led_off(GPIO_GREEN_LED);
			#endif
				gpio_led_off(GPIO_RED_LED);
				gpio_led_blinks(GPIO_BLUE_LED,15,5);
				break;
			case USB3G_DIAL:
			#ifdef HAVE_GREEN_LED	
				gpio_led_off(GPIO_GREEN_LED);
			#endif
				gpio_led_off(GPIO_RED_LED);
				gpio_led_blinks(GPIO_BLUE_LED,2,1);
				break;
			case USB3G_SIMERROR:
			#ifdef HAVE_GREEN_LED	
				gpio_led_off(GPIO_GREEN_LED);
			#endif
				gpio_led_off(GPIO_RED_LED);
				gpio_led_off(GPIO_BLUE_LED);
				break;
			case USB3G_REMOVE:
			case USB3G_UNKNOW:
				break;
			case USB3G_DISC:
			default:
				gpio_led_off(GPIO_BLUE_LED);	
				gpio_led_off(GPIO_RED_LED);
			#ifdef HAVE_GREEN_LED	
				gpio_led_off(GPIO_GREEN_LED);
			#endif
				break;
		}
	}

	pre_net3g_st = net3g_st;
}

void set4G_disc(void)
{
	set_net3g_status(USB3G_DISC);
	nvram_set("net3gOperatorsName","");
	nvram_set("net3gConnectType","--");
}

#define AT_BUF_MAX 256
static char ATret[AT_BUF_MAX];

int Atcmd(const char *cmd, char *dev)
{
	int ret=0;
	char CMD[200] = {0};
	int fd;

	if ( !has_usb_modem_from_sys() )
		return -1;
	
	syslog(LOG_INFO,"<<< %s\n",cmd);
	sprintf(CMD, "echo %s > %s", cmd, AT_CMD_FILE);
	system(CMD);
	
	sprintf(CMD, "/bin/nanocom %s 1", dev);
	system(CMD); 
	
	fd = open(AT_RESULT_FILE, O_RDONLY);
	if (fd == -1){
		dprintf("open at result file error\n");
		return -1;
	}

	memset(ATret, '\0', sizeof(ATret));
	read(fd, ATret, sizeof(ATret));
	close(fd);

	dprintf("AT cmd is [%s], result is \n[%s]\n", cmd, ATret);
	syslog(LOG_INFO,">>> %s\n",ATret);

	return ret;

}

int isAtcmdOK(char *atcmd,char *ttyitf)
{
	char *ptr = NULL;
	if ( Atcmd(atcmd, at_ttydev) < 0 )
	{
		return -1;
	}
	ptr = ATret;

	if ((strstr(ptr, "OK")) == NULL ) //NOT receive OK
	{
		dprintf("%s commond ERROR\n",atcmd);
		return 0;
	}
	else
		return 1;  // "OK"
	
}
#ifdef MODEL_MF990
int isAtcmdReset(void)
{
	char *ptr = NULL;
	if ( Atcmd("AT+CFUN=1,1", at_ttydev) < 0 )
	{
		return -1;
	}
	ptr = ATret;

	if ((strstr(ptr, "OK")) == NULL ) //NOT receive OK
	{
		dprintf("Reset commond ERROR\n");
		return 0;
	}
}
#else
int isAtcmdReset(void)
{
	char *ptr = NULL;
	if ( Atcmd("AT+CFUN=7", at_ttydev) < 0 )
	{
		return -1;
	}
	ptr = ATret;

	if ((strstr(ptr, "OK")) == NULL ) //NOT receive OK
	{
		dprintf("Reset commond ERROR\n");
		return 0;
	}
	else
	{
		if ( Atcmd("AT+CFUN=6", at_ttydev) < 0 )
		{
			return -1;
		}
		ptr = ATret;

		if ((strstr(ptr, "OK")) == NULL ) //NOT receive OK
		{
			dprintf("Reset commond ERROR\n");
			return 0;
		}
		
		return 1;  // "OK"
	}
}
#endif

int net_ttyusb_signal(void)
{
	int i_ret = 0;
	char *ptr = NULL;
	
	char *p_tmp = NULL;
	char buf_tmp[256] = {0} ;

	if ( Atcmd("AT+CSQ", at_ttydev) < 0 )
	{
		i_ret = -1;
		goto net_ttyusb_signal_err;
	}
	ptr = ATret;

	if ((p_tmp = strstr(ptr, "+CSQ:")) != NULL )
	{
		//dprintf("===RSSI=%s========\n", p_tmp); //+CSQ: 15,99
		sscanf(p_tmp, "+CSQ:%s,\n", buf_tmp);
		i_ret = atoi(buf_tmp);
		if ( 0 <= i_ret <= 31 )
			i_ret = i_ret*100/31;
	#ifdef MODEM_CLM630	
		else if ( 100 <= i_ret <= 191 ) //TD: 100~191
			i_ret = (i_ret-100)*100/91;
	#endif	
		else
			i_ret = 0;

		syslog(LOG_INFO,">net_ttyusb_signal> i_ret=%d.\n", i_ret);		
        sprintf(buf_tmp, "%d", i_ret);
        nvram_set("net3g_signal", buf_tmp);
	}

net_ttyusb_signal_err:

	return i_ret;
}

int set_coon_type( int modem_mode )
{
	nvram_set("net3gDialNum", "*99#");
	
	switch( modem_mode )
	{
		case SYS_MODE_CDMA:
		case SYS_MODE_EVDO:
			if ( nvram_match("net3gMannual", "0") )	
				nvram_set("net3gDialNum", "#777");
		
			nvram_set("net3gConnectType","EVDO");
			break;
		case SYS_MODE_GSM:
			nvram_set("net3gConnectType","GSM");
			break;

		case SYS_MODE_WCDMA:
			nvram_set("net3gConnectType","WCDMA");
			break;
		case SYS_MODE_LTE:
			
		#ifdef MODEM_CLM630
			if ( nvram_match("net3gMannual", "0") )
			{
				nvram_set( "net3gUsername", "" );
				nvram_set( "net3gPassword", "" );
			}
		#endif	
			nvram_set("net3gConnectType","LTE");
			break;
		case SYS_SYS_MODE_GWL:
			nvram_set("net3gConnectType","GWL");
			break;
		default:
			fprintf(stderr, "err sys_mode = %d\n",modem_mode);
			nvram_set("net3gConnectType","--");
	}
}

int net_ttyusb_mode(int *modem_mode)
{
	int i_ret = 0;
	int srv_status;
	char *ptr = NULL;
	char *p_netmode = NULL, *p_tmp = NULL;
	int i = 0;
	struct systype_info *cardinfo = (struct systype_info *)malloc( sizeof(struct systype_info) );
	memset(cardinfo, 0, sizeof(struct systype_info));

#ifdef MODEM_CLM630
	if ( Atcmd("AT^SYSINFO", at_ttydev) < 0 )
	{
		i_ret = -1;
		goto net_ttyusb_mode_err;
	}
	ptr = ATret;
	//dprintf("ptr=%s\n",ptr);
	//^SYSINFO: 2,3,0,4,1
	if((p_netmode = strstr(ptr, "^SYSINFO:")) != NULL )
	{
		p_netmode = p_netmode+10;
		dprintf("netmode=%s\n",p_netmode);
		sscanf(p_netmode,"%[^,],%[^,],%[^,],%[^,],*",cardinfo->srv_status,cardinfo->call_status,
		cardinfo->ps_status,cardinfo->sys_mode,cardinfo->sim_status);
	}
	else if( strstr(ptr, "ERROR") != NULL )
	{
		dprintf("read card info error\n");
		i_ret = -1;
		goto net_ttyusb_mode_err;
	}
	
#else //
	
	if ( Atcmd("AT+NETMODE?", at_ttydev) < 0 )
	{
		i_ret = -1;
		goto net_ttyusb_mode_err;
	}
	ptr = ATret;
	//dprintf("ptr=%s\n",ptr);
	
	if((p_netmode = strstr(ptr, "+NETMODE:")) != NULL )
	{
	#if 0
		p_netmode = p_netmode+10;
		//dprintf("netmode=%s\n",netmode);
		//11,9,1,0,0,1,2
		sscanf(p_netmode,"%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],*",cardinfo->systype,cardinfo->sys_mode,
		cardinfo->ps_status,cardinfo->call_status,cardinfo->cell_changed,cardinfo->sim_status);
		for(i=0;i<6;i++)
		{
			p_netmode = strchr(p_netmode,',');
			p_netmode++;
		}
		strncpy(cardinfo->srv_status, p_netmode, 1);
	#else
		// +NETMODE: 11,9,1,0,0,1,2    OK
		p_netmode = p_netmode+10;
		if((p_tmp = strchr(p_netmode, ',')) != NULL )
		{
			*p_tmp = '\0';
			strcpy(cardinfo->systype, p_netmode);
			p_netmode = p_tmp+1;
		}
		if((p_tmp = strchr(p_netmode, ',')) != NULL )
		{
			*p_tmp = '\0';
			strcpy(cardinfo->sys_mode, p_netmode);
			p_netmode = p_tmp+1;
		}
		if((p_tmp = strchr(p_netmode, ',')) != NULL )
		{
			*p_tmp = '\0';
			strcpy(cardinfo->ps_status, p_netmode);
			p_netmode = p_tmp+1;
		}
		if((p_tmp = strchr(p_netmode, ',')) != NULL )
		{
			*p_tmp = '\0';
			p_netmode = p_tmp+1;
		}		
		if((p_tmp = strchr(p_netmode, ',')) != NULL )
		{
			*p_tmp = '\0';
			strcpy(cardinfo->cell_changed, p_netmode);
			p_netmode = p_tmp+1;
		}
		if((p_tmp = strchr(p_netmode, ',')) != NULL )
		{
			*p_tmp = '\0';
			strcpy(cardinfo->sim_status, p_netmode);
			p_netmode = p_tmp+1;
		}
		strncpy(cardinfo->srv_status, p_netmode, 1);
	#endif
	}
	else if( strstr(ptr, "ERROR") != NULL )
	{
		if ( Atcmd("AT+SYSTYPE", at_ttydev) < 0 )
		{
			i_ret = -1;
		}
		ptr = ATret;
		
		if((p_netmode = strstr(ptr, "+SYSTYPE:")) != NULL)
		{
			p_netmode = p_netmode+10;
			//dprintf("netmode=%s\n",netmode);
			sscanf(p_netmode,"%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],*",cardinfo->systype,cardinfo->sys_mode,
			cardinfo->ps_status,cardinfo->call_status,cardinfo->cell_changed,cardinfo->sim_status);
			for(i=0;i<6;i++)
			{
				p_netmode = strchr(p_netmode,',');
				p_netmode++;
			}
			strncpy(cardinfo->srv_status, p_netmode, 1);
		}
		if( strstr(ptr, "ERROR")!= NULL )
		{
			dprintf("read card info error\n");
			i_ret = -1;
			//isAtcmdReset();
			goto net_ttyusb_mode_err;
		}
	}
	else
	{
		dprintf("can not get cardinfo \n"); //when get ttyUSB  ^DSDORMANT:1 ,can not get  cardinfo 
		i_ret = -1;
		goto net_ttyusb_mode_err;
	}

#endif	

	*modem_mode = atoi(cardinfo->sys_mode);
	srv_status = atoi(cardinfo->srv_status);
	
	dprintf("srv_status=%d, Modem mode=%d.\n", srv_status, *modem_mode);
	if ( 2 == srv_status )
	{
		nvram_set("3Gsrv_status",cardinfo->srv_status); 
		nvram_set("3Gsrv_mode",cardinfo->sys_mode);
		
		set_coon_type(*modem_mode);
		i_ret = 1;
	}
	else
	{
		i_ret = 0;

	}

net_ttyusb_mode_err:	
	free(cardinfo);

	return i_ret;
}

/****************************************
** return value
** -1:err or unknow   -> does not process
**  0:SIM busy          
**  1:ready
**  2:require PIN Code
**  3:PIN Code error
**  4:PUK 
*****************************************/
int net_ttyusb_cpin(void)
{
	char *ptr = NULL;
	int i_ret = 0;
	
	if ( Atcmd("AT+CPIN?", at_ttydev) < 0 )
	{
		i_ret = -1;
		goto net_ttyusb_cpin_err;
	}
	ptr = ATret;
	
	if(strstr(ptr, "READY") != NULL)
	{
		dprintf("+CPIN: READY\n");
		nvram_set("net3g_sim", "1");
		i_ret = 1;
	}	
	else if(strstr(ptr, "+CPIN: SIM PIN") != NULL)
	{
		char cmd[64] = { 0 };
		dprintf("SIM PIN\n");
		if ( nvram_match("net3gPinSt", "1") )
		{
			nvram_set("net3g_sim", "1");
			sprintf(cmd, "AT+CPIN=\"%s\"", nvram_safe_get("net3gPinCode"));
			Atcmd(cmd, at_ttydev);
			ptr = ATret;

			if(strstr(ptr, "+CME ERROR: incorrect password") != NULL)
			{
				dprintf("PIN incorrect password!\n");
				nvram_set("net3g_sim", "pin_err");
				i_ret = 3;
			}	
			else if(strstr(ptr, "+CPIN: SIM PUK") != NULL)
			{
				dprintf("SIM PUK!\n");
				nvram_set("net3g_sim", "sim_puk");
				i_ret = 4;
			}	
			else if(strstr(ptr, "+CME ERROR: SIM PUK") != NULL)
			{
				dprintf("SIM PUK!\n");
				nvram_set("net3g_sim", "sim_puk");
				i_ret = 4;
			}
			else if(strstr(ptr, "+CME ERROR: SIM PUK") != NULL)
			{
				dprintf("SIM PUK!\n");
				nvram_set("net3g_sim", "sim_puk");
				i_ret = 4;
			}
			else if(strstr(ptr, "OK") != NULL)
			{
				i_ret = 1;
			}
			else
			{
				dprintf("unknow error!\n");
				nvram_set("net3g_sim", "unk_err");
				i_ret = -1;
			}
			
		}
		else
		{
			nvram_set("net3g_sim", "need_pin");
			i_ret = 2;
		}
	}
	else if( strstr(ptr, "SIM not inserted") != NULL )
	{
		dprintf("SIM not inserted\n");
		nvram_set("net3g_sim", "0"); // not insert SIM card OR card not support
		nvram_set("net3g_signal", "");
		i_ret = 0;
	}
	else if( strstr(ptr, "SIM busy") != NULL )
	{
		dprintf("SIM busy\n");
		nvram_set("net3g_sim", "0"); // not insert SIM card OR card not support
		nvram_set("net3g_signal", "");
		i_ret = 0;
	}
	else
	{
		dprintf("SIM Unknow\n");
		i_ret = -1;
	}

net_ttyusb_cpin_err:

	return i_ret;
}

#if 1
int net_ttyusb_cgreg(void)
{
	int i_ret = 0;
	char *ptr = NULL;
	
	char *p_tmp = NULL;
	char buf_tmp[AT_BUF_MAX] = {0} ;

	if ( Atcmd("AT+CGREG?", at_ttydev) < 0 )
	{
		i_ret = -1;
		goto net_ttyusb_cgreg_err;
	}
	ptr = ATret;

	if ((p_tmp = strstr(ptr, "+CGREG:")) != NULL )
	{
		//dprintf("===CGREG=%s========\n", p_tmp); //+CGREG: 0,1
		strncpy(buf_tmp, p_tmp+10, 1);
		//dprintf("===CGREG 2 val=%s========\n", buf_tmp);
		nvram_set("net3g_cgreg", buf_tmp);
		i_ret = atoi(buf_tmp);
	}

net_ttyusb_cgreg_err:

	return i_ret;
}
#endif

//return :1:find , 0:none   ,-1:err
int net_ttyusb_imsi(void)
{
	int i = 0;
	int i_ret = 0;
	char imsi[16] = { 0 };
	char sim_mccmnc[6] = { 0 };
	char *ptr = NULL;
	char s_mcc[] = "460", s_mcn[] = "11x"; //Real MCC, MNC
	modem_sim_info_t *p_sim_id = g_sim_info_table;

	char buf_tmp[AT_BUF_MAX] = {0} ;

//#if defined(MODEL_MF990) ||  defined(MODEM_HVL600)
	if ( Atcmd("AT+CIMI", at_ttydev) < 0 )
//#else
//	if ( Atcmd("AT^CIMI", at_ttydev) < 0 )
//#endif
	{
		i_ret = -1;
		goto net_ttyusb_imsi_err;
	}
	ptr = ATret;

	if ( nvram_match("net3gMannual", "0") )
    {
    	for ( i=0; i < AT_BUF_MAX-16; i++ )
    	{
    		if ( ptr[i] && isdigit( ptr[i] ) )
    		{
    			strncpy(imsi, ptr+i, 15); //copy imsi
    			strncpy(s_mcc, ptr+i, 3); //copy MCC
    			strncpy(s_mcn, ptr+i+3, 3); //copy MNC

    			i_ret = 1;
				break;
    		}

    	}
		
		dprintf( "CIMI Information:[%s].\n\n",imsi) ;
		dprintf("Inserted SIM card of MCC:MNC = %s:%s\n", s_mcc, s_mcn);
	    for( ; i_ret && (p_sim_id->mcc && p_sim_id->mcn); ++p_sim_id )
	    {
	    	if( 0 == strncmp(p_sim_id->mcc, s_mcc, 3)
            	&& 0 == strncmp(p_sim_id->mcn, s_mcn, strlen(p_sim_id->mcn)) )
	    	{
	    		dprintf("find MCC:MNC = %s:%s from sim_info_table.\n", p_sim_id->mcc, p_sim_id->mcn);
				
				sprintf(sim_mccmnc, "%s%s", p_sim_id->mcc, p_sim_id->mcn);
				nvram_set( "net3gMCCMNC", sim_mccmnc);
				
	    		if ( p_sim_id->ispname )
	    			nvram_set( "net3gOperatorsName", p_sim_id->ispname );
				else
					nvram_set( "net3gOperatorsName", "--" );

				if ( p_sim_id->username )
	    			nvram_set( "net3gUsername", p_sim_id->username );
				else
					nvram_set( "net3gUsername", "" );

				if ( p_sim_id->password )
	    			nvram_set( "net3gPassword", p_sim_id->password );
				else
					nvram_set( "net3gPassword", "" );

				if ( p_sim_id->apn )
	    			nvram_set( "net3gAPN", p_sim_id->apn );
				else
					nvram_set( "net3gAPN", "" );

				break;
	    	}
	    }

	}
	else if ( nvram_match("net3gMannual", "1") )
	{
		i_ret = 1;
	}

net_ttyusb_imsi_err:	
	return i_ret;
}


int is_usb_modem_ok(void)
{
	int i_ok = 0;
	
	dprintf("enter....\n");

	if ( !has_usb_modem_from_sys() )
		goto is_usb_modem_ok_err;

	if( 0 == lock_file(lock_usbnet_at) )
		return 0;

	if( isAtcmdOK("AT", at_ttydev) < 1 )
		goto is_usb_modem_ok_err;
	
	if( isAtcmdOK("AT+CFUN?", at_ttydev) < 1 )
		;//goto is_usb_modem_ok_err;
	
#ifdef MODEM_CLM630
	if( isAtcmdOK("ATI", at_ttydev) < 1 )
		;//goto is_usb_modem_ok_err;
	
	if( isAtcmdOK("AT^MODECONFIG?", at_ttydev) < 1 )
		;//goto is_usb_modem_ok_err;
	if( isAtcmdOK("AT^MODECONFIG=2", at_ttydev) < 1 )
		;//goto is_usb_modem_ok_err;
	if( isAtcmdOK("AT^MODECONFIG?", at_ttydev) < 1 )
		;//goto is_usb_modem_ok_err;

#elif defined(MODEM_HVL600)
	;
#else
	if( isAtcmdOK("AT^SWVER", at_ttydev) < 1 )
		;//goto is_usb_modem_ok_err;
#endif	
	//if( isAtcmdOK("ATE0", at_ttydev) < 1 )
	//	goto is_usb_modem_ok_err;

	i_ok = 1;
	
is_usb_modem_ok_err:	  
	dprintf("done ok[%d]\n", i_ok);
	unlock_file(lock_usbnet_at);
	return i_ok;
}


/*
return:
  1:  ok
  0:  fail
*/
int is_usb_modem_ready_ok(int times)
{
	int i_ok = 0;
	int i_ready = 0;
	int modem_mode = USB_MODEM_UNK;
	int i_signal = 0;
	int i_cgreg = 0;
	char *net3g_roaming_en = NULL;
	int i_mode_st = 0;
	int i_search = times;
	
	dprintf("enter....\n");

	if( 0 == lock_file(lock_usbnet_at) )
		return 0;
	
	i_ready = net_ttyusb_cpin();
	while ( ( i_ready <= 0 ) && i_search-- )
	{
		dprintf("!!! WAIT SIM READY !!!\n");
		sleep(1);
		i_ready = net_ttyusb_cpin();
	}

	if ( 1 == i_ready ) 
	{
		set_net3g_status(USB3G_SEARCH);

#if defined(DEFINE_ALLNET)
		net3g_roaming_en = nvram_safe_get( "net3g_roaming_en");
		if ( 0 == strcmp(net3g_roaming_en, "0") )
		{
			i_cgreg = net_ttyusb_cgreg();
			if ( i_cgreg != 1 )
				goto roaming_over;
		}
 #endif
		net_ttyusb_imsi();
		i_search = times+2;

		while( i_search-- )
		{
			i_mode_st = net_ttyusb_mode(&modem_mode);
			if ( i_mode_st == 1 )
			{
				i_signal = net_ttyusb_signal();
				if ( i_signal > 0 )
				{
					i_ok = 1;

					break;
				}
			}
			sleep(1);
		}		
	}
	else if ( 0 == i_ready && 1 == times )
	{
		set_net3g_status(USB3G_SIMERROR);
	}
	else
	{
		
		/****************************************
		** return value
		** -1:err or unknow   -> does not process
		**	0:SIM busy			
		**	1:ready
		**	2:require PIN Code
		**	3:PIN Code error
		**	4:PUK 
		*****************************************/
		i_ok = i_ready;
	}

roaming_over:	
	dprintf("done ok[%d]\n", i_ok);
	unlock_file(lock_usbnet_at);
	return i_ok;
}

