#!/bin/sh

PROFTPD_FILE=/tmp/vsftpd.conf

Usage()
{
	echo "insufficient arguments!"
	echo "Usage: $0 <target> [option]..."
	echo "       $0 server <ServerName> <IP> <TcpPort> <MaxConn>"
	echo "       $0 user <AddDir> <ReName> <ReMove> <Read> <Write> <Download> <Upload>"
	exit 0
}

setFtpServer()
{
	ftpguest=`nvram get ftp_anonymous`
	if [ "$ftpguest" == "1" ]; then
		echo "anonymous_enable=NO" > $PROFTPD_FILE
	else
		echo "anonymous_enable=YES" > $PROFTPD_FILE
		echo "anon_upload_enable=YES" > $PROFTPD_FILE
		echo "anon_mkdir_write_enable=YES" > $PROFTPD_FILE
		echo "anon_other_write_enable=YES" > $PROFTPD_FILE
	fi
	
	#echo "nopriv_user=test" >> $PROFTPD_FILE
	echo "no_anon_password=YES" >> $PROFTPD_FILE
	echo "userlist_enable=NO" >> $PROFTPD_FILE
	echo "write_enable=YES" >> $PROFTPD_FILE
	echo "local_enable=YES" >> $PROFTPD_FILE
	echo "local_umask=022" >> $PROFTPD_FILE
	echo "dirmessage_enable=YES" >> $PROFTPD_FILE
	echo "xferlog_enable=NO" >> $PROFTPD_FILE
	#echo "syslog_enable=NO" >> $PROFTPD_FILE
	echo "connect_from_port_20=YES" >> $PROFTPD_FILE
	echo "listen=YES" >> $PROFTPD_FILE
	echo "pasv_enable=YES" >> $PROFTPD_FILE
	#echo "ssl_enable=NO" >> $PROFTPD_FILE
	echo "tcp_wrappers=NO" >> $PROFTPD_FILE
	echo "max_clients=10" >> $PROFTPD_FILE
	#echo "ftp_username=test" >> $PROFTPD_FILE
	echo "ftpd_banner=Welcome to WDK's FTP service." >> $PROFTPD_FILE
}

setFtpUser()
{
	if [ "$MKDCAP" == "0" ]; then
		echo "<Limit MKD>" >> $PROFTPD_FILE
		echo "  Denyall" >> $PROFTPD_FILE
		echo "</Limit>" >> $PROFTPD_FILE
	fi
	if [ "$RNFRCAP" == "0" ]; then
		echo "<Limit RNFR>" >> $PROFTPD_FILE
		echo "  Denyall" >> $PROFTPD_FILE
		echo "</Limit>" >> $PROFTPD_FILE
	fi
	if [ "$DELECAP" == "0" ]; then
		echo "<Limit DELE>" >> $PROFTPD_FILE
		echo "  Denyall" >> $PROFTPD_FILE
		echo "</Limit>" >> $PROFTPD_FILE
		echo "<Limit RMD>" >> $PROFTPD_FILE
		echo "  Denyall" >> $PROFTPD_FILE
		echo "</Limit>" >> $PROFTPD_FILE
	fi
	if [ "$READCAP" == "0" ]; then
		echo "<Limit READ>" >> $PROFTPD_FILE
		echo "  Denyall" >> $PROFTPD_FILE
		echo "</Limit>" >> $PROFTPD_FILE
	fi
	if [ "$WRITECAP" == "0" ]; then
		echo "<Limit WRITE>" >> $PROFTPD_FILE
		echo "  Denyall" >> $PROFTPD_FILE
		echo "</Limit>" >> $PROFTPD_FILE
	fi
	if [ "$RETRCAP" == "0" ]; then
		echo "<Limit RETR>" >> $PROFTPD_FILE
		echo "  Denyall" >> $PROFTPD_FILE
		echo "</Limit>" >> $PROFTPD_FILE
	fi
	if [ "$STORCAP" == "0" ]; then
		echo "<Limit STOR>" >> $PROFTPD_FILE
		echo "  Denyall" >> $PROFTPD_FILE
		echo "</Limit>" >> $PROFTPD_FILE
		fullcap=0
	fi
	if [ "$STORCAP" == "0" ]; then
		echo "<Limit STOR>" >> $PROFTPD_FILE
		echo "  Denyall" >> $PROFTPD_FILE
		echo "</Limit>" >> $PROFTPD_FILE
	fi
	if [ "$fullcap" == "1" ]; then
		echo "<Limit ALL>" >> $PROFTPD_FILE
		echo "  Allowall" >> $PROFTPD_FILE
		echo "</Limit>" >> $PROFTPD_FILE
	fi
}

if [ "$1" == "server" ]; then
	if [ ! -n "$5" ]; then
		Usage
	else
		SERVER_NAME="$2"
		IP_ADDR="$3"
		TCP_PORT="$4"
		MAX_CONN="$5"
		setFtpServer
	fi
elif [ "$1" == "user" ]; then
	if [ ! -n "$8" ]; then
		Usage
	else
		MKDCAP=$2
		RNFRCAP=$3
		DELECAP=$4
		READCAP=$5
		WRITECAP=$6
		RETRCAP=$7
		STORCAP=$8
		setFtpUser
	fi
elif [ "$1" == "anonymous" ]; then
	if [ ! -n "$2" ]; then
		MAXCLIENTS=10
		
		# start for patch from CNS 2011-02-15
		echo "<Anonymous ~>" >> $PROFTPD_FILE  
		echo "User anonymous Disabled" >> $PROFTPD_FILE
		echo "MaxClients	$MAXCLIENTS" >> $PROFTPD_FILE
  	        # Srini Included this lines in anonymous chroot
		echo "<Limit LOGIN>" >> $PROFTPD_FILE
		echo "  Denyall" >> $PROFTPD_FILE
		echo "</Limit>" >> $PROFTPD_FILE
		echo "</Anonymous>" >> $PROFTPD_FILE
		# end for patch from CNS 2011-02-15
	else
		MAXCLIENTS=$2
	# CNS patch 2011-02-15	
	# fi
		echo "<Anonymous ~>" >> $PROFTPD_FILE  
		echo "User		anonymous" >> $PROFTPD_FILE
		echo "Group		anonymous" >> $PROFTPD_FILE
		echo "MaxClients	$MAXCLIENTS" >> $PROFTPD_FILE
		echo "</Anonymous>" >> $PROFTPD_FILE
        fi
else
	Usage
fi
